@@ -16,7 +16,7 @@
 
 use anyhow::bail;
 use std::fmt;
-use wasmer::{imports, wat2wasm, Function, Instance, Module, Store, TypedFunction};
+use wasmer::{Function, Instance, Module, Store, TypedFunction, imports, wat2wasm};
 
 // First we need to create an error type that we'll use to signal the end of execution.
 #[derive(Debug, Clone, Copy)]
@@ -78,27 +78,24 @@ fn main() -> anyhow::Result<()> {
     // Get the `run` function which we'll use as our entrypoint.
     println!("Calling `run` function...");
     let run_func: TypedFunction<(i32, i32), i32> =
-        instance.exports.get_typed_function(&mut store, "run")?;
+        instance.exports.get_typed_function(&store, "run")?;
 
     // When we call a function it can either succeed or fail. We expect it to fail.
     match run_func.call(&mut store, 1, 7) {
         Ok(result) => {
-            bail!(
-                "Expected early termination with `ExitCode`, found: {}",
-                result
-            );
+            bail!("Expected early termination with `ExitCode`, found: {result}");
         }
         // In case of a failure, which we expect, we attempt to downcast the error into the error
         // type that we were expecting.
         Err(e) => match e.downcast::<ExitCode>() {
             // We found the exit code used to terminate execution.
             Ok(exit_code) => {
-                println!("Exited early with exit code: {}", exit_code);
+                println!("Exited early with exit code: {exit_code}");
 
                 Ok(())
             }
             Err(e) => {
-                bail!("Unknown error `{}` found. expected `ExitCode`", e);
+                bail!("Unknown error `{e}` found. expected `ExitCode`");
             }
         },
     }
