@@ -15,7 +15,7 @@
 //!
 //! Ready?
 
-use wasmer::{imports, wat2wasm, Instance, Module, Mutability, Store, Type, TypedFunction, Value};
+use wasmer::{Instance, Module, Mutability, Store, Type, TypedFunction, Value, imports, wat2wasm};
 
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     // Let's declare the Wasm module with the text representation.
@@ -83,18 +83,16 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     //
     // We will use an exported function for the `one` global
     // and the Global API for `some`.
-    let get_one: TypedFunction<(), f32> = instance
-        .exports
-        .get_function("get_one")?
-        .typed(&mut store)?;
+    let get_one: TypedFunction<(), f32> =
+        instance.exports.get_function("get_one")?.typed(&store)?;
 
     let one_value = get_one.call(&mut store)?;
     let some_value = some.get(&mut store);
 
-    println!("`one` value: {:?}", one_value);
+    println!("`one` value: {one_value:?}");
     assert_eq!(one_value, 1.0);
 
-    println!("`some` value: {:?}", some_value);
+    println!("`some` value: {some_value:?}");
     assert_eq!(some_value, Value::F32(0.0));
 
     println!("Setting global values...");
@@ -109,7 +107,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     // );
 
     let one_result = one.get(&mut store);
-    println!("`one` value after `set`: {:?}", one_result);
+    println!("`one` value after `set`: {one_result:?}");
     assert_eq!(one_result, Value::F32(1.0));
 
     // Setting the values of globals can be done in two ways:
@@ -117,18 +115,16 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     //   2. Using the Global API directly.
     //
     // We will use both for the `some` global.
-    let set_some: TypedFunction<f32, ()> = instance
-        .exports
-        .get_function("set_some")?
-        .typed(&mut store)?;
+    let set_some: TypedFunction<f32, ()> =
+        instance.exports.get_function("set_some")?.typed(&store)?;
     set_some.call(&mut store, 21.0)?;
     let some_result = some.get(&mut store);
-    println!("`some` value after `set_some`: {:?}", some_result);
+    println!("`some` value after `set_some`: {some_result:?}");
     assert_eq!(some_result, Value::F32(21.0));
 
     some.set(&mut store, Value::F32(42.0))?;
     let some_result = some.get(&mut store);
-    println!("`some` value after `set`: {:?}", some_result);
+    println!("`some` value after `set`: {some_result:?}");
     assert_eq!(some_result, Value::F32(42.0));
 
     Ok(())
