@@ -21,8 +21,8 @@
 
 use std::sync::{Arc, Mutex};
 use wasmer::{
-    imports, wat2wasm, Function, FunctionEnv, FunctionEnvMut, Instance, Module, Store,
-    TypedFunction,
+    Function, FunctionEnv, FunctionEnvMut, Instance, Module, Store, TypedFunction, imports,
+    wat2wasm,
 };
 
 fn main() -> Result<(), Box<dyn std::error::Error>> {
@@ -108,10 +108,10 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     let increment_counter_loop: TypedFunction<i32, i32> = instance
         .exports
         .get_function("increment_counter_loop")?
-        .typed(&mut store)?;
+        .typed(&store)?;
 
     let counter_value: i32 = *shared_counter.lock().unwrap();
-    println!("Initial ounter value: {:?}", counter_value);
+    println!("Initial counter value: {counter_value:?}");
 
     println!("Calling `increment_counter_loop` function...");
     // Let's call the `increment_counter_loop` exported function.
@@ -120,10 +120,10 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     let result = increment_counter_loop.call(&mut store, 5)?;
 
     let counter_value: i32 = *shared_counter.lock().unwrap();
-    println!("New counter value (host): {:?}", counter_value);
+    println!("New counter value (host): {counter_value:?}");
     assert_eq!(counter_value, 5);
 
-    println!("New counter value (guest): {:?}", result);
+    println!("New counter value (guest): {result:?}");
     assert_eq!(result, 5);
 
     Ok(())
