@@ -15,7 +15,7 @@
 //!
 //! Ready?
 
-use wasmer::{imports, wat2wasm, Global, Instance, Module, Store, TypedFunction, Value};
+use wasmer::{Global, Instance, Module, Store, TypedFunction, Value, imports, wat2wasm};
 
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     // Let's declare the Wasm module with the text representation.
@@ -60,21 +60,17 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
     //
     // The Wasm module only imports some globals. We'll have to interact
     // with them either using the Global API or exported functions.
-    let get_some: TypedFunction<(), f32> = instance
-        .exports
-        .get_function("get_some")?
-        .typed(&mut store)?;
-    let get_other: TypedFunction<(), f32> = instance
-        .exports
-        .get_function("get_other")?
-        .typed(&mut store)?;
+    let get_some: TypedFunction<(), f32> =
+        instance.exports.get_function("get_some")?.typed(&store)?;
+    let get_other: TypedFunction<(), f32> =
+        instance.exports.get_function("get_other")?.typed(&store)?;
 
     let some_result = get_some.call(&mut store)?;
     let other_result = get_other.call(&mut store)?;
 
-    println!("some value (via `get_some`): {:?}", some_result);
+    println!("some value (via `get_some`): {some_result:?}");
     println!("some value (via Global API): {:?}", some.get(&mut store));
-    println!("other value (via `get_other`): {:?}", other_result);
+    println!("other value (via `get_other`): {other_result:?}");
     println!("other value (via Global API): {:?}", other.get(&mut store));
 
     assert_eq!(some_result, some.get(&mut store).f32().unwrap());
@@ -88,16 +84,14 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     other.set(&mut store, Value::F32(21.0))?;
     let other_result = other.get(&mut store);
-    println!("other value after `set`: {:?}", other_result);
+    println!("other value after `set`: {other_result:?}");
     assert_eq!(other_result, Value::F32(21.0));
 
     println!("Altering global values through exported functions...");
     // Changes made to global through exported functions will
     // be reflected on the host side.
-    let set_other: TypedFunction<f32, ()> = instance
-        .exports
-        .get_function("set_other")?
-        .typed(&mut store)?;
+    let set_other: TypedFunction<f32, ()> =
+        instance.exports.get_function("set_other")?.typed(&store)?;
     set_other.call(&mut store, 42.0)?;
 
     println!("other value (via Global API): {:?}", other.get(&mut store));
