@@ -1,37 +1,34 @@
 #![no_main]
 
-use libfuzzer_sys::{arbitrary, arbitrary::Arbitrary, fuzz_target};
-use wasm_smith::{Config, ConfiguredModule};
-use wasmer::{CompilerConfig, EngineBuilder, Module, Store};
+use libfuzzer_sys::fuzz_target;
+use wasmer::{Module, Store};
 use wasmer_compiler::Engine;
+use wasmer_compiler::{CompilerConfig, EngineBuilder};
 use wasmer_compiler_cranelift::Cranelift;
 use wasmer_compiler_llvm::LLVM;
 use wasmer_compiler_singlepass::Singlepass;
+mod misc;
+use misc::{SinglePassFuzzModule, ignore_compilation_error, save_wasm_file};
 
-#[derive(Arbitrary, Debug, Default, Copy, Clone)]
-struct NoImportsConfig;
-impl Config for NoImportsConfig {
-    fn max_imports(&self) -> usize {
-        0
-    }
-    fn max_memory_pages(&self) -> u32 {
-        // https://github.com/wasmerio/wasmer/issues/2187
-        65535
-    }
-    fn allow_start_export(&self) -> bool {
-        false
-    }
-}
-
-fn compile_and_compare(name: &str, engine: Engine, wasm: &[u8]) {
+fn compile_and_compare(name: &str, engine: Engine, wasm_bytes: &[u8]) {
     let store = Store::new(engine);
 
     // compile for first time
-    let module = Module::new(&store, wasm).unwrap();
+    let module = Module::new(&store, wasm_bytes);
+    let module = match module {
+        Ok(m) => m,
+        Err(e) => {
+            if ignore_compilation_error(&e.to_string()) {
+                return;
+            }
+            save_wasm_file(wasm_bytes);
+            panic!("{}", e);
+        }
+    };
     let first = module.serialize().unwrap();
 
     // compile for second time
-    let module = Module::new(&store, wasm).unwrap();
+    let module = Module::new(&store, wasm_bytes).unwrap();
     let second = module.serialize().unwrap();
 
     if first != second {
@@ -39,8 +36,8 @@ fn compile_and_compare(name: &str, engine: Engine, wasm: &[u8]) {
     }
 }
 
-fuzz_target!(|module: ConfiguredModule<NoImportsConfig>| {
-    let wasm_bytes = module.to_bytes();
+fuzz_target!(|module: SinglePassFuzzModule| {
+    let wasm_bytes = module.0.to_bytes();
 
     let mut compiler = Cranelift::default();
     compiler.canonicalize_nans(true);
