@@ -14,9 +14,9 @@ use wasmer_types::{
 
 use wasmparser::{
     self, BinaryReaderError, Export, ExportSectionReader, ExternalKind, FunctionSectionReader,
-    GlobalSectionReader, GlobalType as WPGlobalType, ImportSectionReader, MemorySectionReader,
-    MemoryType as WPMemoryType, NameSectionReader, Parser, Payload, TableSectionReader,
-    TagType as WPTagType, TypeRef, TypeSectionReader,
+    GlobalSectionReader, GlobalType as WPGlobalType, ImportSectionReader, Imports,
+    MemorySectionReader, MemoryType as WPMemoryType, NameSectionReader, Parser, Payload,
+    TableSectionReader, TagType as WPTagType, TypeRef, TypeSectionReader,
 };
 
 pub type WasmResult<T> = Result<T, String>;
@@ -84,6 +84,39 @@ impl ModuleInfoPolyfill {
         Ok(())
     }
 
+    pub(crate) fn declare_tag_import(
+        &mut self,
+        t: wasmparser::TagType,
+        module_name: &str,
+        field_name: &str,
+    ) -> WasmResult<()> {
+        debug_assert_eq!(
+            self.info.tags.len(),
+            self.info.num_imported_tags,
+            "Imported tags must be declared first"
+        );
+
+        let tag = SignatureIndex::from_u32(t.func_type_idx);
+        debug_assert!(
+            self.info
+                .signatures
+                .get(SignatureIndex::from_u32(t.func_type_idx))
+                .is_some(),
+            "Imported tags must mach a declared signature!"
+        );
+
+        self.declare_import(
+            ImportIndex::Tag(TagIndex::from_u32(self.info.num_imported_tags as _)),
+            module_name,
+            field_name,
+        )?;
+
+        self.info.num_imported_tags += 1;
+        self.info.tags.push(tag);
+
+        Ok(())
+    }
+
     pub(crate) fn declare_table_import(
         &mut self,
         table: TableType,
@@ -340,7 +373,7 @@ pub fn wpreftype_to_type(ty: wasmparser::RefType) -> WasmResult<Type> {
     } else if ty.is_func_ref() {
         Ok(Type::FuncRef)
     } else {
-        Err(format!("Unsupported ref type: {:?}", ty))
+        Err(format!("Unsupported ref type: {ty:?}"))
     }
 }
 
@@ -395,6 +428,10 @@ pub fn parse_import_section(
 
     for entry in imports {
         let import = entry.map_err(transform_err)?;
+        let Imports::Single(_index, import) = import else {
+            return Err("non-Single section Imports not implemented yet".to_string());
+        };
+
         let module_name = import.module;
         let field_name = import.name;
 
@@ -406,8 +443,8 @@ pub fn parse_import_section(
                     field_name,
                 )?;
             }
-            TypeRef::Tag(_) => {
-                unimplemented!("exception handling not implemented yet")
+            TypeRef::Tag(t) => {
+                module_info.declare_tag_import(t, module_name, field_name)?;
             }
             TypeRef::Memory(WPMemoryType {
                 shared,
@@ -450,6 +487,7 @@ pub fn parse_import_section(
                     field_name,
                 )?;
             }
+            TypeRef::FuncExact(_) => unimplemented!("custom-descriptors not implemented yet"),
         }
     }
     Ok(())
@@ -588,6 +626,9 @@ pub fn parse_export_section(
                 module_info.declare_global_export(GlobalIndex::new(index), name)?
             }
             ExternalKind::Tag => module_info.declare_tag_export(TagIndex::new(index), name)?,
+            ExternalKind::FuncExact => {
+                return Err("custom-descriptors not implemented yet".to_string());
+            }
         }
     }
     Ok(())
