@@ -1,6 +1,6 @@
 use std::{collections::HashSet, time::Duration};
 
-use anyhow::{bail, Context};
+use anyhow::{Context, bail};
 use cynic::{MutationBuilder, QueryBuilder};
 use futures::StreamExt;
 use merge_streams::MergeStreams;
@@ -12,8 +12,8 @@ use wasmer_package::utils::from_bytes;
 use webc::Container;
 
 use crate::{
-    types::{self, *},
     GraphQLApiFailure, WasmerClient,
+    types::{self, *},
 };
 
 /// Rotate the s3 secrets tied to an app given its id.
@@ -260,6 +260,34 @@ pub async fn get_app_volumes(
     Ok(volumes)
 }
 
+/// Retrieve volumes for an app.
+pub async fn get_app_databases(
+    client: &WasmerClient,
+    owner: impl Into<String>,
+    name: impl Into<String>,
+) -> Result<Vec<types::AppDatabase>, anyhow::Error> {
+    let vars = types::GetAppDatabasesVars {
+        owner: owner.into(),
+        name: name.into(),
+        after: None,
+    };
+    let res = client
+        .run_graphql_strict(types::GetAppDatabases::build(vars))
+        .await?;
+
+    let app = res.get_deploy_app.context("app not found")?;
+    let dbs = app.databases;
+    let _ = dbs.page_info;
+
+    let dbs = dbs
+        .edges
+        .into_iter()
+        .flatten()
+        .flat_map(|edge| edge.node)
+        .collect::<Vec<_>>();
+    Ok(dbs)
+}
+
 /// Load the S3 credentials.
 ///
 /// S3 can be used to get access to an apps volumes.
@@ -421,7 +449,7 @@ pub async fn fetch_webc_package(
         ))?,
         PackageIdent::Hash(h) => match get_package_release(client, &h.to_string()).await? {
             Some(webc) => Url::parse(&webc.webc_url)?,
-            None => anyhow::bail!("Could not find package with hash '{}'", h),
+            None => anyhow::bail!("Could not find package with hash '{h}'"),
         },
     };
 
@@ -813,6 +841,20 @@ pub fn fetch_all_app_template_frameworks(
     )
 }
 
+/// Upload method.
+#[derive(Debug)]
+pub enum UploadMethod {
+    R2,
+}
+
+impl UploadMethod {
+    pub fn as_str(&self) -> &'static str {
+        match self {
+            UploadMethod::R2 => "R2",
+        }
+    }
+}
+
 /// Get a signed URL to upload packages.
 pub async fn get_signed_url_for_package_upload(
     client: &WasmerClient,
@@ -820,6 +862,7 @@ pub async fn get_signed_url_for_package_upload(
     filename: Option<&str>,
     name: Option<&str>,
     version: Option<&str>,
+    method: Option<UploadMethod>,
 ) -> Result<Option<SignedUrl>, anyhow::Error> {
     client
         .run_graphql_strict(types::GetSignedUrlForPackageUpload::build(
@@ -828,11 +871,64 @@ pub async fn get_signed_url_for_package_upload(
                 filename,
                 name,
                 version,
+                method: method.map(|m| m.as_str()),
             },
         ))
         .await
         .map(|r| r.get_signed_url_for_package_upload)
 }
+
+/// Request a signed URL for uploading an app archive via the autobuild flow.
+pub async fn generate_upload_url(
+    client: &WasmerClient,
+    filename: &str,
+    name: Option<&str>,
+    version: Option<&str>,
+    expires_after_seconds: Option<i32>,
+    method: Option<UploadMethod>,
+) -> Result<SignedUrl, anyhow::Error> {
+    let payload = client
+        .run_graphql_strict(types::GenerateUploadUrl::build(
+            GenerateUploadUrlVariables {
+                expires_after_seconds,
+                filename,
+                name,
+                version,
+                method: method.map(|m| m.as_str()),
+            },
+        ))
+        .await
+        .and_then(|res| {
+            res.generate_upload_url
+                .context("generateUploadUrl mutation did not return data")
+        })?;
+
+    Ok(payload.signed_url)
+}
+
+/// Retrieve autobuild metadata derived from a previously uploaded archive.
+pub async fn autobuild_config_for_zip_upload(
+    client: &WasmerClient,
+    upload_url: &str,
+) -> Result<Option<types::AutobuildConfigForZipUploadPayload>, anyhow::Error> {
+    client
+        .run_graphql_strict(types::AutobuildConfigForZipUpload::build(
+            AutobuildConfigForZipUploadVariables { upload_url },
+        ))
+        .await
+        .map(|res| res.autobuild_config_for_zip_upload)
+}
+
+/// Trigger an autobuild deployment for an uploaded archive or repository.
+pub async fn deploy_via_autobuild(
+    client: &WasmerClient,
+    vars: DeployViaAutobuildVars,
+) -> Result<Option<types::DeployViaAutobuildPayload>, anyhow::Error> {
+    client
+        .run_graphql_strict(types::DeployViaAutobuild::build(vars))
+        .await
+        .map(|res| res.deploy_via_autobuild)
+}
 /// Push a package to the registry.
 pub async fn push_package_release(
     client: &WasmerClient,
@@ -1334,6 +1430,39 @@ pub async fn get_app_version_by_id_with_app(
     Ok((app, version))
 }
 
+pub async fn user_apps_page(
+    client: &WasmerClient,
+    sort: types::DeployAppsSortBy,
+    cursor: Option<String>,
+) -> Result<Paginated<types::DeployApp>, anyhow::Error> {
+    let user = client
+        .run_graphql(types::GetCurrentUserWithApps::build(
+            GetCurrentUserWithAppsVars {
+                after: cursor,
+                first: Some(10),
+                sort: Some(sort),
+            },
+        ))
+        .await?
+        .viewer
+        .context("not logged in")?;
+
+    let apps: Vec<_> = user
+        .apps
+        .edges
+        .into_iter()
+        .flatten()
+        .filter_map(|x| x.node)
+        .collect();
+
+    let out = Paginated {
+        items: apps,
+        next_cursor: user.apps.page_info.end_cursor,
+    };
+
+    Ok(out)
+}
+
 /// List all apps that are accessible by the current user.
 ///
 /// NOTE: this will only include the first pages and does not provide pagination.
@@ -1345,6 +1474,7 @@ pub async fn user_apps(
         let user = client
             .run_graphql(types::GetCurrentUserWithApps::build(
                 GetCurrentUserWithAppsVars {
+                    first: Some(10),
                     after: cursor,
                     sort: Some(sort),
                 },
@@ -1408,6 +1538,43 @@ pub async fn user_accessible_apps(
     Ok((user_apps, ns_apps.merge()).merge())
 }
 
+/// Get apps for a specific namespace.
+///
+/// NOTE: only retrieves the first page and does not do pagination.
+pub async fn namespace_apps_page(
+    client: &WasmerClient,
+    namespace: String,
+    sort: types::DeployAppsSortBy,
+    cursor: Option<String>,
+) -> Result<Paginated<types::DeployApp>, anyhow::Error> {
+    let namespace = namespace.clone();
+
+    let res = client
+        .run_graphql(types::GetNamespaceApps::build(GetNamespaceAppsVars {
+            name: namespace.to_string(),
+            after: cursor,
+            sort: Some(sort),
+        }))
+        .await?
+        .get_namespace
+        .context("namespace not found")?
+        .apps;
+
+    let apps: Vec<_> = res
+        .edges
+        .into_iter()
+        .flatten()
+        .filter_map(|x| x.node)
+        .collect();
+
+    let out = Paginated {
+        items: apps,
+        next_cursor: res.page_info.end_cursor,
+    };
+
+    Ok(out)
+}
+
 /// Get apps for a specific namespace.
 ///
 /// NOTE: only retrieves the first page and does not do pagination.
@@ -1692,6 +1859,27 @@ pub async fn generate_deploy_config_token_raw(
         .context("no token returned")
 }
 
+/// Generate an SSH token for accessing Edge over SSH or SFTP.
+///
+/// If an app id is provided, the token will be scoped to that app,
+/// and using the token will open an ssh context for that app.
+pub async fn generate_ssh_token(
+    client: &WasmerClient,
+    app_id: Option<String>,
+) -> Result<String, anyhow::Error> {
+    let res = client
+        .run_graphql_strict(types::GenerateSshToken::build(
+            types::GenerateSshTokenVariables {
+                app_id: app_id.map(cynic::Id::new),
+            },
+        ))
+        .await?;
+
+    res.generate_ssh_token
+        .map(|x| x.token)
+        .context("no token returned")
+}
+
 /// Get pages of logs associated with an application that lie within the
 /// specified date range.
 // NOTE: this is not public due to severe usability issues.
@@ -1790,7 +1978,7 @@ fn get_app_logs(
 /// Get pages of logs associated with an application that lie within the
 /// specified date range.
 ///
-/// In contrast to [`get_app_logs`], this function collects the stream into a
+/// In contrast to `get_app_logs`, this function collects the stream into a
 /// final vector.
 #[tracing::instrument(skip_all, level = "debug")]
 #[allow(clippy::let_with_type_underscore)]
@@ -1827,7 +2015,7 @@ pub async fn get_app_logs_paginated(
 /// Get pages of logs associated with an application that lie within the
 /// specified date range with a specific instance identifier.
 ///
-/// In contrast to [`get_app_logs`], this function collects the stream into a
+/// In contrast to `get_app_logs`, this function collects the stream into a
 /// final vector.
 #[tracing::instrument(skip_all, level = "debug")]
 #[allow(clippy::let_with_type_underscore)]
@@ -1874,7 +2062,7 @@ pub async fn get_app_logs_paginated_filter_instance(
 /// Get pages of logs associated with an specific request for application that lie within the
 /// specified date range.
 ///
-/// In contrast to [`get_app_logs`], this function collects the stream into a
+/// In contrast to `get_app_logs`, this function collects the stream into a
 /// final vector.
 #[tracing::instrument(skip_all, level = "debug")]
 #[allow(clippy::let_with_type_underscore)]
@@ -1929,8 +2117,7 @@ pub async fn get_domain(
 
     let opt = client
         .run_graphql(types::GetDomain::build(vars))
-        .await
-        .map_err(anyhow::Error::from)?
+        .await?
         .get_domain;
     Ok(opt)
 }
@@ -1946,8 +2133,7 @@ pub async fn get_domain_zone_file(
 
     let opt = client
         .run_graphql(types::GetDomainWithZoneFile::build(vars))
-        .await
-        .map_err(anyhow::Error::from)?
+        .await?
         .get_domain;
     Ok(opt)
 }
@@ -1961,8 +2147,7 @@ pub async fn get_domain_with_records(
 
     let opt = client
         .run_graphql(types::GetDomainWithRecords::build(vars))
-        .await
-        .map_err(anyhow::Error::from)?
+        .await?
         .get_domain;
     Ok(opt)
 }
@@ -1981,8 +2166,7 @@ pub async fn register_domain(
     };
     let opt = client
         .run_graphql_strict(types::RegisterDomain::build(vars))
-        .await
-        .map_err(anyhow::Error::from)?
+        .await?
         .register_domain
         .context("Domain registration failed")?
         .domain
@@ -2000,7 +2184,6 @@ pub async fn get_all_dns_records(
     client
         .run_graphql_strict(types::GetAllDnsRecords::build(vars))
         .await
-        .map_err(anyhow::Error::from)
         .map(|x| x.get_all_dnsrecords)
 }
 
@@ -2012,7 +2195,6 @@ pub async fn get_all_domains(
     let connection = client
         .run_graphql_strict(types::GetAllDomains::build(vars))
         .await
-        .map_err(anyhow::Error::from)
         .map(|x| x.get_all_domains)
         .context("no domains returned")?;
     Ok(connection
@@ -2065,7 +2247,6 @@ pub async fn purge_cache_for_app_version(
     client
         .run_graphql_strict(types::PurgeCacheForAppVersion::build(vars))
         .await
-        .map_err(anyhow::Error::from)
         .map(|x| x.purge_cache_for_app_version)
         .context("backend did not return data")?;
 
