@@ -2,6 +2,12 @@ pub use queries::*;
 
 pub use cynic::Id;
 
+#[derive(serde::Serialize, serde::Deserialize, Clone, Debug)]
+pub struct Paginated<T> {
+    pub items: Vec<T>,
+    pub next_cursor: Option<String>,
+}
+
 #[cynic::schema_for_derives(file = r#"schema.graphql"#, module = "schema")]
 mod queries {
     use serde::Serialize;
@@ -297,7 +303,7 @@ mod queries {
         pub cursor: String,
     }
 
-    #[derive(serde::Serialize, cynic::QueryFragment, PartialEq, Eq, Debug)]
+    #[derive(serde::Serialize, cynic::QueryFragment, PartialEq, Eq, Debug, Clone)]
     pub struct AppTemplate {
         #[serde(rename = "demoUrl")]
         pub demo_url: String,
@@ -320,6 +326,10 @@ mod queries {
         pub updated_at: DateTime,
         #[serde(rename = "useCases")]
         pub use_cases: Jsonstring,
+        #[serde(rename = "branch")]
+        pub branch: Option<String>,
+        #[serde(rename = "rootDir")]
+        pub root_dir: Option<String>,
     }
 
     #[derive(cynic::QueryVariables, Debug, Clone)]
@@ -574,6 +584,7 @@ mod queries {
         pub filename: Option<&'a str>,
         pub name: Option<&'a str>,
         pub version: Option<&'a str>,
+        pub method: Option<&'a str>,
     }
 
     #[derive(cynic::QueryFragment, Debug)]
@@ -582,7 +593,7 @@ mod queries {
         variables = "GetSignedUrlForPackageUploadVariables"
     )]
     pub struct GetSignedUrlForPackageUpload {
-        #[arguments(name: $name, version: $version, filename: $filename, expiresAfterSeconds: $expires_after_seconds)]
+        #[arguments(name: $name, version: $version, filename: $filename, expiresAfterSeconds: $expires_after_seconds, method: $method)]
         pub get_signed_url_for_package_upload: Option<SignedUrl>,
     }
 
@@ -591,6 +602,29 @@ mod queries {
         pub url: String,
     }
 
+    #[derive(cynic::QueryVariables, Debug)]
+    pub struct GenerateUploadUrlVariables<'a> {
+        pub expires_after_seconds: Option<i32>,
+        pub filename: &'a str,
+        pub name: Option<&'a str>,
+        pub version: Option<&'a str>,
+        pub method: Option<&'a str>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Mutation", variables = "GenerateUploadUrlVariables")]
+    pub struct GenerateUploadUrl {
+        #[arguments(input: { expiresAfterSeconds: $expires_after_seconds, filename: $filename, name: $name, version: $version, method: $method })]
+        pub generate_upload_url: Option<GenerateUploadUrlPayload>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub struct GenerateUploadUrlPayload {
+        #[cynic(rename = "signedUrl")]
+        pub signed_url: SignedUrl,
+        pub method: String,
+    }
+
     #[derive(cynic::QueryFragment, Debug)]
     pub struct PackageWebcConnection {
         pub page_info: PageInfo,
@@ -632,6 +666,7 @@ mod queries {
 
     #[derive(cynic::QueryVariables, Debug)]
     pub struct GetCurrentUserWithAppsVars {
+        pub first: Option<i32>,
         pub after: Option<String>,
         pub sort: Option<DeployAppsSortBy>,
     }
@@ -648,7 +683,7 @@ mod queries {
     pub struct UserWithApps {
         pub id: cynic::Id,
         pub username: String,
-        #[arguments(after: $after, sortBy: $sort)]
+        #[arguments(after: $after, sortBy: $sort, first: $first)]
         pub apps: DeployAppConnection,
     }
 
@@ -848,6 +883,51 @@ mod queries {
         pub used_size: Option<BigInt>,
     }
 
+    #[derive(cynic::QueryVariables, Debug)]
+    pub(crate) struct GetAppDatabasesVars {
+        pub name: String,
+        pub owner: String,
+        pub after: Option<String>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Query", variables = "GetAppDatabasesVars")]
+    pub(crate) struct GetAppDatabases {
+        #[arguments(owner: $owner, name: $name)]
+        pub get_deploy_app: Option<AppDatabases>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub(crate) struct AppDatabaseConnection {
+        pub page_info: PageInfo,
+        pub edges: Vec<Option<AppDatabaseEdge>>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "DeployApp")]
+    pub(crate) struct AppDatabases {
+        pub databases: AppDatabaseConnection,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub(crate) struct AppDatabaseEdge {
+        pub node: Option<AppDatabase>,
+    }
+
+    #[derive(serde::Serialize, cynic::QueryFragment, Debug)]
+    pub struct AppDatabase {
+        pub id: cynic::Id,
+        pub name: String,
+        pub created_at: DateTime,
+        pub updated_at: DateTime,
+        pub deleted_at: Option<DateTime>,
+        pub username: String,
+        pub db_explorer_url: Option<String>,
+        pub host: String,
+        pub port: String,
+        pub password: Option<String>,
+    }
+
     #[derive(cynic::QueryFragment, Debug)]
     pub struct RegisterDomainPayload {
         pub success: bool,
@@ -1251,7 +1331,7 @@ mod queries {
     pub enum StatusEnum {
         Success,
         Working,
-        Failure,
+        Failed,
         Queued,
         Timeout,
         InternalError,
@@ -1264,7 +1344,7 @@ mod queries {
             match self {
                 Self::Success => "success",
                 Self::Working => "working",
-                Self::Failure => "failure",
+                Self::Failed => "failed",
                 Self::Queued => "queued",
                 Self::Timeout => "timeout",
                 Self::InternalError => "internal_error",
@@ -1274,6 +1354,99 @@ mod queries {
         }
     }
 
+    #[derive(cynic::QueryVariables, Debug)]
+    pub struct AutobuildConfigForZipUploadVariables<'a> {
+        pub upload_url: &'a str,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(
+        graphql_type = "Mutation",
+        variables = "AutobuildConfigForZipUploadVariables"
+    )]
+    pub struct AutobuildConfigForZipUpload {
+        #[arguments(input: { uploadUrl: $upload_url })]
+        pub autobuild_config_for_zip_upload: Option<AutobuildConfigForZipUploadPayload>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug, Clone, Serialize)]
+    pub struct AutobuildConfigForZipUploadPayload {
+        pub build_config: Option<BuildConfig>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug, Clone, Serialize)]
+    pub struct BuildConfig {
+        pub build_cmd: Option<String>,
+        pub install_cmd: Option<String>,
+        pub start_cmd: Option<String>,
+        pub setup_db: bool,
+        pub preset_name: String,
+        pub app_name: String,
+        pub completion_time_in_seconds: i32,
+        pub branch: Option<String>,
+    }
+
+    #[derive(cynic::InputObject, Debug, Clone)]
+    pub struct WordpressDeploymentExtraData {
+        pub site_name: String,
+        pub admin_username: String,
+        pub admin_password: String,
+        pub admin_email: String,
+        pub language: Option<String>,
+    }
+
+    #[derive(cynic::InputObject, Debug, Clone)]
+    pub struct AutobuildDeploymentExtraData {
+        pub wordpress: Option<WordpressDeploymentExtraData>,
+    }
+
+    #[derive(cynic::InputObject, Debug, Clone)]
+    pub struct JobDefinitionInput {
+        pub name: Option<String>,
+        pub package: Option<String>,
+        pub command: String,
+        pub cli_args: Option<Vec<Option<String>>>,
+        pub env: Option<Vec<Option<String>>>,
+        pub timeout: Option<String>,
+    }
+
+    #[derive(cynic::QueryVariables, Debug, Clone)]
+    pub struct DeployViaAutobuildVars {
+        pub repo_url: Option<String>,
+        pub upload_url: Option<String>,
+        pub app_name: Option<String>,
+        pub app_id: Option<cynic::Id>,
+        pub owner: Option<String>,
+        pub build_cmd: Option<String>,
+        pub install_cmd: Option<String>,
+        pub enable_database: Option<bool>,
+        pub secrets: Option<Vec<SecretInput>>,
+        pub extra_data: Option<AutobuildDeploymentExtraData>,
+        pub params: Option<AutobuildDeploymentExtraData>,
+        pub managed: Option<bool>,
+        pub kind: Option<String>,
+        pub wait_for_screenshot_generation: Option<bool>,
+        pub region: Option<String>,
+        pub branch: Option<String>,
+        pub allow_existing_app: Option<bool>,
+        pub jobs: Option<Vec<JobDefinitionInput>>,
+        pub domains: Option<Vec<Option<String>>>,
+        pub client_mutation_id: Option<String>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Mutation", variables = "DeployViaAutobuildVars")]
+    pub struct DeployViaAutobuild {
+        #[arguments(input: { repoUrl: $repo_url, uploadUrl: $upload_url, appName: $app_name, appId: $app_id, owner: $owner, buildCmd: $build_cmd, installCmd: $install_cmd, enableDatabase: $enable_database, secrets: $secrets, extraData: $extra_data, params: $params, managed: $managed, kind: $kind, waitForScreenshotGeneration: $wait_for_screenshot_generation, region: $region, branch: $branch, allowExistingApp: $allow_existing_app, jobs: $jobs, domains: $domains, clientMutationId: $client_mutation_id })]
+        pub deploy_via_autobuild: Option<DeployViaAutobuildPayload>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub struct DeployViaAutobuildPayload {
+        pub success: bool,
+        pub build_id: Uuid,
+    }
+
     #[derive(cynic::Scalar, Debug, Clone)]
     #[cynic(graphql_type = "UUID")]
     pub struct Uuid(pub String);
@@ -1377,6 +1550,42 @@ mod queries {
         pub instance_id: String,
     }
 
+    #[derive(cynic::Enum, Clone, Copy, Debug, PartialEq, Eq)]
+    pub enum AutoBuildDeployAppLogKind {
+        Log,
+        PreparingToDeployStatus,
+        FetchingPlanStatus,
+        BuildStatus,
+        DeployStatus,
+        Complete,
+        Failed,
+    }
+
+    #[derive(cynic::QueryVariables, Debug)]
+    pub struct AutobuildDeploymentSubscriptionVariables {
+        pub build_id: Uuid,
+    }
+
+    #[derive(cynic::QueryFragment, Debug, Clone, Serialize)]
+    #[cynic(
+        graphql_type = "Subscription",
+        variables = "AutobuildDeploymentSubscriptionVariables"
+    )]
+    pub struct AutobuildDeploymentSubscription {
+        #[arguments(buildId: $build_id)]
+        pub autobuild_deployment: Option<AutobuildLog>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug, Clone, Serialize)]
+    pub struct AutobuildLog {
+        pub kind: AutoBuildDeployAppLogKind,
+        pub message: Option<String>,
+        pub app_version: Option<DeployAppVersion>,
+        pub timestamp: String,
+        pub datetime: DateTime,
+        pub stream: Option<LogStream>,
+    }
+
     #[derive(cynic::QueryVariables, Debug)]
     pub struct GenerateDeployConfigTokenVars {
         pub input: String,
@@ -1393,6 +1602,23 @@ mod queries {
         pub token: String,
     }
 
+    #[derive(cynic::QueryVariables, Debug)]
+    pub struct GenerateSshTokenVariables {
+        pub app_id: Option<cynic::Id>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Mutation", variables = "GenerateSshTokenVariables")]
+    pub struct GenerateSshToken {
+        #[arguments(input: { appId: $app_id })]
+        pub generate_ssh_token: Option<GenerateSshTokenPayload>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub struct GenerateSshTokenPayload {
+        pub token: String,
+    }
+
     #[derive(cynic::QueryVariables, Debug)]
     pub struct GetNodeVars {
         pub id: cynic::Id,
@@ -1559,7 +1785,7 @@ mod queries {
         pub success: bool,
     }
 
-    #[derive(cynic::InputObject, Debug)]
+    #[derive(cynic::InputObject, Debug, Clone)]
     pub struct SecretInput {
         pub name: String,
         pub value: String,
