@@ -29,7 +29,7 @@
 // For the same reason, we also turn off the warning for camel_case types.
 #![allow(clippy::missing_safety_doc)]
 #![allow(non_camel_case_types)]
-#![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 pub mod error;
 pub mod tracing;
