@@ -4,7 +4,7 @@
 
 use crate::hash::Hash;
 use std::error::Error;
-use wasmer::{Module, Store};
+use wasmer::{AsEngineRef, Module};
 
 /// A generic cache for storing and loading compiled wasm modules.
 pub trait Cache {
@@ -13,12 +13,16 @@ pub trait Cache {
     /// The deserialization error for the implementation
     type DeserializeError: Error + Send + Sync;
 
-    /// Loads a module using the provided [`Store`] and [`Hash`].
+    /// Loads a module using the provided [`wasmer::Store`] and [`crate::Hash`].
     ///
     /// # Safety
     /// This function is unsafe as the cache store could be tampered with.
-    unsafe fn load(&self, store: &Store, key: Hash) -> Result<Module, Self::DeserializeError>;
+    unsafe fn load(
+        &self,
+        engine: &impl AsEngineRef,
+        key: Hash,
+    ) -> Result<Module, Self::DeserializeError>;
 
-    /// Store a [`Module`] into the cache with the given [`Hash`].
+    /// Store a [`Module`] into the cache with the given [`crate::Hash`].
     fn store(&mut self, key: Hash, module: &Module) -> Result<(), Self::SerializeError>;
 }
