@@ -11,11 +11,11 @@ use std::string::ToString;
 use std::sync::Arc;
 use std::{path::PathBuf, str::FromStr};
 
-use anyhow::{bail, Result};
+use anyhow::{Context, Result, bail};
 #[cfg(feature = "sys")]
 use wasmer::sys::*;
 use wasmer::*;
-use wasmer_types::{target::Target, Features};
+use wasmer_types::{Features, target::Target};
 
 #[cfg(feature = "compiler")]
 use wasmer_compiler::CompilerConfig;
@@ -88,7 +88,7 @@ pub struct WasmFeatures {
 pub struct RuntimeOptions {
     /// Use Singlepass compiler.
     #[cfg(feature = "singlepass")]
-    #[clap(long, conflicts_with_all = &Vec::<&str>::from_iter([
+    #[clap(short, long, conflicts_with_all = &Vec::<&str>::from_iter([
         #[cfg(feature = "llvm")]
         "llvm", 
         #[cfg(feature = "v8")]
@@ -104,7 +104,7 @@ pub struct RuntimeOptions {
 
     /// Use Cranelift compiler.
     #[cfg(feature = "cranelift")]
-    #[clap(long, conflicts_with_all = &Vec::<&str>::from_iter([
+    #[clap(short, long, conflicts_with_all = &Vec::<&str>::from_iter([
         #[cfg(feature = "llvm")]
         "llvm", 
         #[cfg(feature = "v8")]
@@ -120,7 +120,7 @@ pub struct RuntimeOptions {
 
     /// Use LLVM compiler.
     #[cfg(feature = "llvm")]
-    #[clap(long, conflicts_with_all = &Vec::<&str>::from_iter([
+    #[clap(short, long, conflicts_with_all = &Vec::<&str>::from_iter([
         #[cfg(feature = "cranelift")]
         "cranelift", 
         #[cfg(feature = "v8")]
@@ -188,27 +188,27 @@ pub struct RuntimeOptions {
     #[clap(long)]
     enable_verifier: bool,
 
+    /// Debug directory, where IR and object files will be written to.
+    ///
+    /// Available for cranelift, LLVM and singlepass.
+    #[clap(long, alias = "llvm-debug-dir")]
+    compiler_debug_dir: Option<PathBuf>,
+
     /// Enable a profiler.
     ///
     /// Available for cranelift, LLVM and singlepass.
     #[clap(long, value_enum)]
     profiler: Option<Profiler>,
 
-    /// LLVM debug directory, where IR and object files will be written to.
-    ///
-    /// Only available for the LLVM compiler.
-    #[clap(long)]
-    llvm_debug_dir: Option<PathBuf>,
-
     /// Only available for the LLVM compiler. Enable the "pass-params" optimization, where the first (#0)
     /// global and the first (#0) memory passed between guest functions as explicit parameters.
+    #[cfg(feature = "llvm")]
     #[clap(long)]
     enable_pass_params_opt: bool,
 
-    /// Only available for the LLVM compiler. Sets the number of threads used to compile the
-    /// input module(s).
-    #[clap(long)]
-    llvm_num_threads: Option<NonZero<usize>>,
+    /// Sets the number of threads used to compile the input module(s).
+    #[clap(long, alias = "llvm-num-threads")]
+    compiler_threads: Option<NonZero<usize>>,
 
     #[clap(flatten)]
     features: WasmFeatures,
@@ -298,7 +298,7 @@ impl RuntimeOptions {
 
     pub fn get_engine(&self, target: &Target) -> Result<Engine> {
         let backends = self.get_available_backends()?;
-        let backend = backends.first().unwrap();
+        let backend = backends.first().context("no compiler backend enabled")?;
         let backend_kind = wasmer::BackendKind::from(backend);
         let required_features = wasmer::Engine::default_features_for_backend(&backend_kind, target);
         backend.get_engine(target, &required_features, self)
@@ -338,7 +338,9 @@ impl RuntimeOptions {
                     extra_text
                 );
             } else {
-                bail!("No backends support the required features for the Wasm module. Feel free to open an issue at https://github.com/wasmerio/wasmer/issues");
+                bail!(
+                    "No backends support the required features for the Wasm module. Feel free to open an issue at https://github.com/wasmerio/wasmer/issues"
+                );
             }
         }
         filtered_backends
@@ -348,25 +350,29 @@ impl RuntimeOptions {
     }
 
     #[cfg(feature = "compiler")]
-    /// Get the enaled Wasm features.
-    pub fn get_features(&self, features: &Features) -> Result<Features> {
-        let mut result = features.clone();
-        if !self.features.disable_threads || self.features.all {
+    /// Get the enabled Wasm features.
+    pub fn get_features(&self, default_features: &Features) -> Result<Features> {
+        if self.features.all {
+            return Ok(Features::all());
+        }
+
+        let mut result = default_features.clone();
+        if !self.features.disable_threads {
             result.threads(true);
         }
-        if self.features.disable_threads && !self.features.all {
+        if self.features.disable_threads {
             result.threads(false);
         }
-        if self.features.multi_value || self.features.all {
+        if self.features.multi_value {
             result.multi_value(true);
         }
-        if self.features.simd || self.features.all {
+        if self.features.simd {
             result.simd(true);
         }
-        if self.features.bulk_memory || self.features.all {
+        if self.features.bulk_memory {
             result.bulk_memory(true);
         }
-        if self.features.reference_types || self.features.all {
+        if self.features.reference_types {
             result.reference_types(true);
         }
         Ok(result)
@@ -384,37 +390,41 @@ impl RuntimeOptions {
         &self,
         wasm_bytes: &[u8],
     ) -> Result<Features, wasmparser::BinaryReaderError> {
+        if self.features.all {
+            return Ok(Features::all());
+        }
+
         let mut features = Features::detect_from_wasm(wasm_bytes)?;
 
         // Merge with user-configured features
-        if !self.features.disable_threads || self.features.all {
+        if !self.features.disable_threads {
             features.threads(true);
         }
-        if self.features.reference_types || self.features.all {
+        if self.features.reference_types {
             features.reference_types(true);
         }
-        if self.features.simd || self.features.all {
+        if self.features.simd {
             features.simd(true);
         }
-        if self.features.bulk_memory || self.features.all {
+        if self.features.bulk_memory {
             features.bulk_memory(true);
         }
-        if self.features.multi_value || self.features.all {
+        if self.features.multi_value {
             features.multi_value(true);
         }
-        if self.features.tail_call || self.features.all {
+        if self.features.tail_call {
             features.tail_call(true);
         }
-        if self.features.module_linking || self.features.all {
+        if self.features.module_linking {
             features.module_linking(true);
         }
-        if self.features.multi_memory || self.features.all {
+        if self.features.multi_memory {
             features.multi_memory(true);
         }
-        if self.features.memory64 || self.features.all {
+        if self.features.memory64 {
             features.memory64(true);
         }
-        if self.features.exceptions || self.features.all {
+        if self.features.exceptions {
             features.exceptions(true);
         }
 
@@ -456,7 +466,15 @@ impl RuntimeOptions {
                         Profiler::Perfmap => config.enable_perfmap(),
                     }
                 }
+                if let Some(mut debug_dir) = self.compiler_debug_dir.clone() {
+                    use wasmer_compiler_singlepass::SinglepassCallbacks;
 
+                    debug_dir.push("singlepass");
+                    config.callbacks(Some(SinglepassCallbacks::new(debug_dir)?));
+                }
+                if let Some(num_threads) = self.compiler_threads {
+                    config.num_threads(num_threads);
+                }
                 Box::new(config)
             }
             #[cfg(feature = "cranelift")]
@@ -470,15 +488,20 @@ impl RuntimeOptions {
                         Profiler::Perfmap => config.enable_perfmap(),
                     }
                 }
+                if let Some(mut debug_dir) = self.compiler_debug_dir.clone() {
+                    use wasmer_compiler_cranelift::CraneliftCallbacks;
+
+                    debug_dir.push("cranelift");
+                    config.callbacks(Some(CraneliftCallbacks::new(debug_dir)?));
+                }
+                if let Some(num_threads) = self.compiler_threads {
+                    config.num_threads(num_threads);
+                }
                 Box::new(config)
             }
             #[cfg(feature = "llvm")]
             BackendType::LLVM => {
-                use std::{fmt, fs::File, io::Write};
-
-                use wasmer_compiler_llvm::{
-                    CompiledKind, InkwellMemoryBuffer, InkwellModule, LLVMCallbacks, LLVM,
-                };
+                use wasmer_compiler_llvm::LLVMCallbacks;
                 use wasmer_types::entity::EntityRef;
                 let mut config = LLVM::new();
 
@@ -486,98 +509,14 @@ impl RuntimeOptions {
                     config.enable_pass_params_opt();
                 }
 
-                if let Some(num_threads) = self.llvm_num_threads {
+                if let Some(num_threads) = self.compiler_threads {
                     config.num_threads(num_threads);
                 }
 
-                struct Callbacks {
-                    debug_dir: PathBuf,
-                }
-                impl Callbacks {
-                    fn new(debug_dir: PathBuf) -> Result<Self> {
-                        // Create the debug dir in case it doesn't exist
-                        std::fs::create_dir_all(&debug_dir)?;
-                        Ok(Self { debug_dir })
-                    }
-                }
-                // Converts a kind into a filename, that we will use to dump
-                // the contents of the IR object file to.
-                fn types_to_signature(types: &[Type]) -> String {
-                    types
-                        .iter()
-                        .map(|ty| match ty {
-                            Type::I32 => "i".to_string(),
-                            Type::I64 => "I".to_string(),
-                            Type::F32 => "f".to_string(),
-                            Type::F64 => "F".to_string(),
-                            Type::V128 => "v".to_string(),
-                            Type::ExternRef => "e".to_string(),
-                            Type::FuncRef => "r".to_string(),
-                            Type::ExceptionRef => "x".to_string(),
-                        })
-                        .collect::<Vec<_>>()
-                        .join("")
-                }
-                // Converts a kind into a filename, that we will use to dump
-                // the contents of the IR object file to.
-                fn function_kind_to_filename(kind: &CompiledKind) -> String {
-                    match kind {
-                        CompiledKind::Local(local_index) => {
-                            format!("function_{}", local_index.index())
-                        }
-                        CompiledKind::FunctionCallTrampoline(func_type) => format!(
-                            "trampoline_call_{}_{}",
-                            types_to_signature(func_type.params()),
-                            types_to_signature(func_type.results())
-                        ),
-                        CompiledKind::DynamicFunctionTrampoline(func_type) => format!(
-                            "trampoline_dynamic_{}_{}",
-                            types_to_signature(func_type.params()),
-                            types_to_signature(func_type.results())
-                        ),
-                        CompiledKind::Module => "module".into(),
-                    }
-                }
-                impl LLVMCallbacks for Callbacks {
-                    fn preopt_ir(&self, kind: &CompiledKind, module: &InkwellModule) {
-                        let mut path = self.debug_dir.clone();
-                        path.push(format!("{}.preopt.ll", function_kind_to_filename(kind)));
-                        module
-                            .print_to_file(&path)
-                            .expect("Error while dumping pre optimized LLVM IR");
-                    }
-                    fn postopt_ir(&self, kind: &CompiledKind, module: &InkwellModule) {
-                        let mut path = self.debug_dir.clone();
-                        path.push(format!("{}.postopt.ll", function_kind_to_filename(kind)));
-                        module
-                            .print_to_file(&path)
-                            .expect("Error while dumping post optimized LLVM IR");
-                    }
-                    fn obj_memory_buffer(
-                        &self,
-                        kind: &CompiledKind,
-                        memory_buffer: &InkwellMemoryBuffer,
-                    ) {
-                        let mut path = self.debug_dir.clone();
-                        path.push(format!("{}.o", function_kind_to_filename(kind)));
-                        let mem_buf_slice = memory_buffer.as_slice();
-                        let mut file = File::create(path)
-                            .expect("Error while creating debug object file from LLVM IR");
-                        let mut pos = 0;
-                        while pos < mem_buf_slice.len() {
-                            pos += file.write(&mem_buf_slice[pos..]).unwrap();
-                        }
-                    }
-                }
-
-                impl fmt::Debug for Callbacks {
-                    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-                        write!(f, "LLVMCallbacks")
-                    }
-                }
-
-                if let Some(ref llvm_debug_dir) = self.llvm_debug_dir {
-                    config.callbacks(Some(Arc::new(Callbacks::new(llvm_debug_dir.clone())?)));
+                if let Some(mut debug_dir) = self.compiler_debug_dir.clone() {
+                    debug_dir.push("llvm");
+                    config.callbacks(Some(LLVMCallbacks::new(debug_dir)?));
+                    config.verbose_asm(true);
                 }
                 if self.enable_verifier {
                     config.enable_verifier();
@@ -593,10 +532,7 @@ impl RuntimeOptions {
             BackendType::V8 | BackendType::Wamr | BackendType::Wasmi => unreachable!(),
             #[cfg(not(all(feature = "singlepass", feature = "cranelift", feature = "llvm")))]
             compiler => {
-                bail!(
-                    "The `{}` compiler is not included in this binary.",
-                    compiler.to_string()
-                )
+                bail!("The `{compiler}` compiler is not included in this binary.")
             }
         };
 
@@ -670,6 +606,15 @@ impl BackendType {
                         Profiler::Perfmap => config.enable_perfmap(),
                     }
                 }
+                if let Some(mut debug_dir) = runtime_opts.compiler_debug_dir.clone() {
+                    use wasmer_compiler_singlepass::SinglepassCallbacks;
+
+                    debug_dir.push("singlepass");
+                    config.callbacks(Some(SinglepassCallbacks::new(debug_dir)?));
+                }
+                if let Some(num_threads) = runtime_opts.compiler_threads {
+                    config.num_threads(num_threads);
+                }
                 let engine = wasmer_compiler::EngineBuilder::new(config)
                     .set_features(Some(features.clone()))
                     .set_target(Some(target.clone()))
@@ -688,6 +633,15 @@ impl BackendType {
                         Profiler::Perfmap => config.enable_perfmap(),
                     }
                 }
+                if let Some(mut debug_dir) = runtime_opts.compiler_debug_dir.clone() {
+                    use wasmer_compiler_cranelift::CraneliftCallbacks;
+
+                    debug_dir.push("cranelift");
+                    config.callbacks(Some(CraneliftCallbacks::new(debug_dir)?));
+                }
+                if let Some(num_threads) = runtime_opts.compiler_threads {
+                    config.num_threads(num_threads);
+                }
                 let engine = wasmer_compiler::EngineBuilder::new(config)
                     .set_features(Some(features.clone()))
                     .set_target(Some(target.clone()))
@@ -697,103 +651,15 @@ impl BackendType {
             }
             #[cfg(feature = "llvm")]
             Self::LLVM => {
-                use std::{fmt, fs::File, io::Write};
-
-                use wasmer_compiler_llvm::{
-                    CompiledKind, InkwellMemoryBuffer, InkwellModule, LLVMCallbacks, LLVM,
-                };
+                use wasmer_compiler_llvm::LLVMCallbacks;
                 use wasmer_types::entity::EntityRef;
 
                 let mut config = wasmer_compiler_llvm::LLVM::new();
 
-                struct Callbacks {
-                    debug_dir: PathBuf,
-                }
-                impl Callbacks {
-                    fn new(debug_dir: PathBuf) -> Result<Self> {
-                        // Create the debug dir in case it doesn't exist
-                        std::fs::create_dir_all(&debug_dir)?;
-                        Ok(Self { debug_dir })
-                    }
-                }
-                // Converts a kind into a filename, that we will use to dump
-                // the contents of the IR object file to.
-                fn types_to_signature(types: &[Type]) -> String {
-                    types
-                        .iter()
-                        .map(|ty| match ty {
-                            Type::I32 => "i".to_string(),
-                            Type::I64 => "I".to_string(),
-                            Type::F32 => "f".to_string(),
-                            Type::F64 => "F".to_string(),
-                            Type::V128 => "v".to_string(),
-                            Type::ExternRef => "e".to_string(),
-                            Type::FuncRef => "r".to_string(),
-                            Type::ExceptionRef => "x".to_string(),
-                        })
-                        .collect::<Vec<_>>()
-                        .join("")
-                }
-                // Converts a kind into a filename, that we will use to dump
-                // the contents of the IR object file to.
-                fn function_kind_to_filename(kind: &CompiledKind) -> String {
-                    match kind {
-                        CompiledKind::Local(local_index) => {
-                            format!("function_{}", local_index.index())
-                        }
-                        CompiledKind::FunctionCallTrampoline(func_type) => format!(
-                            "trampoline_call_{}_{}",
-                            types_to_signature(func_type.params()),
-                            types_to_signature(func_type.results())
-                        ),
-                        CompiledKind::DynamicFunctionTrampoline(func_type) => format!(
-                            "trampoline_dynamic_{}_{}",
-                            types_to_signature(func_type.params()),
-                            types_to_signature(func_type.results())
-                        ),
-                        CompiledKind::Module => "module".into(),
-                    }
-                }
-                impl LLVMCallbacks for Callbacks {
-                    fn preopt_ir(&self, kind: &CompiledKind, module: &InkwellModule) {
-                        let mut path = self.debug_dir.clone();
-                        path.push(format!("{}.preopt.ll", function_kind_to_filename(kind)));
-                        module
-                            .print_to_file(&path)
-                            .expect("Error while dumping pre optimized LLVM IR");
-                    }
-                    fn postopt_ir(&self, kind: &CompiledKind, module: &InkwellModule) {
-                        let mut path = self.debug_dir.clone();
-                        path.push(format!("{}.postopt.ll", function_kind_to_filename(kind)));
-                        module
-                            .print_to_file(&path)
-                            .expect("Error while dumping post optimized LLVM IR");
-                    }
-                    fn obj_memory_buffer(
-                        &self,
-                        kind: &CompiledKind,
-                        memory_buffer: &InkwellMemoryBuffer,
-                    ) {
-                        let mut path = self.debug_dir.clone();
-                        path.push(format!("{}.o", function_kind_to_filename(kind)));
-                        let mem_buf_slice = memory_buffer.as_slice();
-                        let mut file = File::create(path)
-                            .expect("Error while creating debug object file from LLVM IR");
-                        let mut pos = 0;
-                        while pos < mem_buf_slice.len() {
-                            pos += file.write(&mem_buf_slice[pos..]).unwrap();
-                        }
-                    }
-                }
-
-                impl fmt::Debug for Callbacks {
-                    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-                        write!(f, "LLVMCallbacks")
-                    }
-                }
-
-                if let Some(ref llvm_debug_dir) = runtime_opts.llvm_debug_dir {
-                    config.callbacks(Some(Arc::new(Callbacks::new(llvm_debug_dir.clone())?)));
+                if let Some(mut debug_dir) = runtime_opts.compiler_debug_dir.clone() {
+                    debug_dir.push("llvm");
+                    config.callbacks(Some(LLVMCallbacks::new(debug_dir)?));
+                    config.verbose_asm(true);
                 }
                 if runtime_opts.enable_verifier {
                     config.enable_verifier();
@@ -803,7 +669,7 @@ impl BackendType {
                     config.enable_pass_params_opt();
                 }
 
-                if let Some(num_threads) = runtime_opts.llvm_num_threads {
+                if let Some(num_threads) = runtime_opts.compiler_threads {
                     config.num_threads(num_threads);
                 }
 
