@@ -1,12 +1,17 @@
-use std::process::{Command, Stdio};
-
+use super::AsyncCliCommand;
+use crate::config::WasmerEnv;
 use anyhow::{Context, Error};
 use clap::Parser;
-use wasmer_registry::{wasmer_env::WasmerEnv, Bindings, ProgrammingLanguage};
+use std::process::{Command, Stdio};
+use wasmer_backend_api::{
+    WasmerClient,
+    types::{Bindings, ProgrammingLanguage},
+};
+use wasmer_config::package::NamedPackageIdent;
 
 /// Add a Wasmer package's bindings to your application.
 #[derive(Debug, Parser)]
-pub struct Add {
+pub struct CmdAdd {
     #[clap(flatten)]
     env: WasmerEnv,
     /// Add the JavaScript bindings using "npm install".
@@ -25,20 +30,19 @@ pub struct Add {
     #[clap(long, groups = &["bindings", "py"])]
     pip: bool,
     /// The packages to add (e.g. "wasmer/wasmer-pack@0.5.0" or "python/python")
-    packages: Vec<wasmer_registry::Package>,
+    packages: Vec<NamedPackageIdent>,
 }
 
-impl Add {
+#[async_trait::async_trait]
+impl AsyncCliCommand for CmdAdd {
+    type Output = ();
+
     /// Execute [`Add`].
-    pub fn execute(&self) -> Result<(), Error> {
+    async fn run_async(self) -> Result<Self::Output, anyhow::Error> {
         anyhow::ensure!(!self.packages.is_empty(), "No packages specified");
 
-        let registry = self
-            .env
-            .registry_endpoint()
-            .context("Unable to determine which registry to use")?;
-
-        let bindings = self.lookup_bindings(registry.as_str())?;
+        let client = self.env.client_unauthennticated()?;
+        let bindings = self.lookup_bindings(&client).await?;
 
         let mut cmd = self.target()?.command(&bindings)?;
         cmd.stdin(Stdio::null())
@@ -54,19 +58,22 @@ impl Add {
             )
         })?;
 
-        anyhow::ensure!(status.success(), "Command failed: {:?}", cmd);
+        anyhow::ensure!(status.success(), "Command failed: {cmd:?}");
 
         Ok(())
     }
+}
 
-    fn lookup_bindings(&self, registry: &str) -> Result<Vec<Bindings>, Error> {
+impl CmdAdd {
+    async fn lookup_bindings(&self, client: &WasmerClient) -> Result<Vec<Bindings>, Error> {
         println!("Querying Wasmer for package bindings");
 
         let mut bindings_to_add = Vec::new();
         let language = self.target()?.language();
 
         for pkg in &self.packages {
-            let bindings = lookup_bindings_for_package(registry, pkg, &language)
+            let bindings = lookup_bindings_for_package(client, pkg, &language)
+                .await
                 .with_context(|| format!("Unable to find bindings for {pkg}"))?;
             bindings_to_add.push(bindings);
         }
@@ -90,18 +97,22 @@ impl Add {
     }
 }
 
-fn lookup_bindings_for_package(
-    registry: &str,
-    pkg: &wasmer_registry::Package,
+async fn lookup_bindings_for_package(
+    client: &WasmerClient,
+    pkg: &NamedPackageIdent,
     language: &ProgrammingLanguage,
 ) -> Result<Bindings, Error> {
-    let all_bindings =
-        wasmer_registry::list_bindings(registry, &pkg.package(), pkg.version.as_deref())?;
+    let all_bindings = wasmer_backend_api::query::list_bindings(
+        client,
+        &pkg.name,
+        pkg.version_opt().map(|v| v.to_string()).as_deref(),
+    )
+    .await?;
 
     match all_bindings.iter().find(|b| b.language == *language) {
         Some(b) => {
             let Bindings { url, generator, .. } = b;
-            log::debug!("Found {pkg} bindings generated by {generator} at {url}");
+            log::debug!("Found {pkg} bindings generated by {generator:?} at {url}");
 
             Ok(b.clone())
         }
@@ -126,9 +137,9 @@ enum Target {
 impl Target {
     fn language(self) -> ProgrammingLanguage {
         match self {
-            Target::Pip => ProgrammingLanguage::PYTHON,
+            Target::Pip => ProgrammingLanguage::Python,
             Target::Pnpm { .. } | Target::Yarn { .. } | Target::Npm { .. } => {
-                ProgrammingLanguage::JAVASCRIPT
+                ProgrammingLanguage::Javascript
             }
         }
     }
@@ -162,11 +173,7 @@ impl Target {
                 if Command::new("yarn").arg("--version").output().is_err() {
                     return Err(anyhow::anyhow!("yarn not installed"));
                 }
-                if dev {
-                    "yarn add --dev"
-                } else {
-                    "yarn add"
-                }
+                if dev { "yarn add --dev" } else { "yarn add" }
             }
             Target::Npm { dev } => {
                 if Command::new("npm").arg("--version").output().is_err() {
@@ -182,11 +189,7 @@ impl Target {
                 if Command::new("pnpm").arg("--version").output().is_err() {
                     return Err(anyhow::anyhow!("pnpm not installed"));
                 }
-                if dev {
-                    "pnpm add --dev"
-                } else {
-                    "pnpm add"
-                }
+                if dev { "pnpm add --dev" } else { "pnpm add" }
             }
         };
         let mut command_line = command_line.to_string();
