@@ -1,10 +1,10 @@
-use super::utils::{get_secrets, BackendSecretWrapper};
+use super::utils::{BackendSecretWrapper, get_secrets};
 use crate::{
-    commands::{app::util::AppIdentFlag, AsyncCliCommand},
+    commands::{AsyncCliCommand, app::util::AppIdentFlag},
     config::WasmerEnv,
     opts::ListFormatOpts,
 };
-use is_terminal::IsTerminal;
+use std::io::IsTerminal as _;
 use std::path::PathBuf;
 
 /// Retrieve the value of an existing app secret.
