@@ -2,12 +2,8 @@
 
 use self::utils::normalize_atom_name;
 use super::CliCommand;
-use crate::{
-    backend::RuntimeOptions,
-    common::{normalize_path, HashAlgorithm},
-    config::WasmerEnv,
-};
-use anyhow::{anyhow, bail, Context, Result};
+use crate::{backend::RuntimeOptions, common::normalize_path, config::WasmerEnv};
+use anyhow::{Context, Result, anyhow, bail};
 use clap::Parser;
 use object::ObjectSection;
 use serde::{Deserialize, Serialize};
@@ -49,8 +45,8 @@ pub struct CreateExe {
     #[clap(name = "OUTPUT PATH", short = 'o')]
     output: PathBuf,
 
-    /// Optional directorey used for debugging: if present, will output the zig command
-    /// for reproducing issues in a debug directory
+    /// Optional directory used for debugging: if present, will output the zig command
+    /// for reproducing issues in a debug directory.
     #[clap(long, name = "DEBUG PATH")]
     debug_dir: Option<PathBuf>,
 
@@ -102,10 +98,6 @@ pub struct CreateExe {
 
     #[clap(flatten)]
     compiler: RuntimeOptions,
-
-    /// Hashing algorithm to be used for module hash
-    #[clap(long, value_enum)]
-    hash_algorithm: Option<HashAlgorithm>,
 }
 
 /// Url or version to download the release from
@@ -237,10 +229,7 @@ impl CliCommand for CreateExe {
         }
 
         let _backends = self.compiler.get_available_backends()?;
-        let mut engine = self.compiler.get_engine(&target)?;
-
-        let hash_algorithm = self.hash_algorithm.unwrap_or_default().into();
-        engine.set_hash_algorithm(Some(hash_algorithm));
+        let engine = self.compiler.get_engine(&target)?;
 
         println!("Compiler: {}", engine.deterministic_id());
         println!("Target: {}", target.triple());
@@ -635,7 +624,8 @@ impl PrefixMapCompilation {
         if prefixes.len() != atoms.len() {
             println!(
                 "WARNING: invalid mapping of prefix and atoms: expected prefixes for {} atoms, got {} prefixes",
-                atoms.len(), prefixes.len()
+                atoms.len(),
+                prefixes.len()
             );
         }
 
@@ -692,7 +682,9 @@ impl PrefixMapCompilation {
                     manual_prefixes.insert(normalize_atom_name(&atoms[0].0), prefix.to_string());
                 }
                 _ => {
-                    return Err(anyhow::anyhow!("invalid --precompiled-atom {p:?} - correct format is ATOM:PREFIX:PATH or ATOM:PATH"));
+                    return Err(anyhow::anyhow!(
+                        "invalid --precompiled-atom {p:?} - correct format is ATOM:PREFIX:PATH or ATOM:PATH"
+                    ));
                 }
             }
         }
@@ -1304,7 +1296,11 @@ fn link_exe_from_dir(
     include_path.push("include");
     if !include_path.exists() {
         // Can happen when we got the wrong library_path
-        return Err(anyhow::anyhow!("Wasmer include path {} does not exist, maybe library path {} is wrong (expected /lib/libwasmer.a)?", include_path.display(), library_path.display()));
+        return Err(anyhow::anyhow!(
+            "Wasmer include path {} does not exist, maybe library path {} is wrong (expected /lib/libwasmer.a)?",
+            include_path.display(),
+            library_path.display()
+        ));
     }
     cmd.arg("-I");
     cmd.arg(normalize_path(&format!("{}", include_path.display())));
@@ -1388,10 +1384,9 @@ fn link_exe_from_dir(
         .context(anyhow!("Could not execute `zig`: {cmd:?}"))?;
 
     if !compilation.status.success() {
-        return Err(anyhow::anyhow!(String::from_utf8_lossy(
-            &compilation.stderr
-        )
-        .to_string()));
+        return Err(anyhow::anyhow!(
+            String::from_utf8_lossy(&compilation.stderr).to_string()
+        ));
     }
 
     // remove file if it exists - if not done, can lead to errors on copy
@@ -1450,7 +1445,11 @@ fn link_objects_system_linker(
     include_path.push("include");
     if !include_path.exists() {
         // Can happen when we got the wrong library_path
-        return Err(anyhow::anyhow!("Wasmer include path {} does not exist, maybe library path {} is wrong (expected /lib/libwasmer.a)?", include_path.display(), libwasmer_path.display()));
+        return Err(anyhow::anyhow!(
+            "Wasmer include path {} does not exist, maybe library path {} is wrong (expected /lib/libwasmer.a)?",
+            include_path.display(),
+            libwasmer_path.display()
+        ));
     }
     command = command.arg("-I");
     command = command.arg(normalize_path(&format!("{}", include_path.display())));
@@ -1508,9 +1507,9 @@ fn generate_wasmer_main_c(
         let prefix = prefixes
             .get_prefix_for_atom(&utils::normalize_atom_name(a))
             .ok_or_else(|| {
+                let formatted_prefixes = format!("{prefixes:#?}");
                 anyhow::anyhow!(
-                    "cannot find prefix for atom {a} when generating wasmer_main.c ({:#?})",
-                    prefixes
+                    "cannot find prefix for atom {a} when generating wasmer_main.c ({formatted_prefixes})"
                 )
             })?;
         let atom_name = prefix.clone();
@@ -1564,10 +1563,10 @@ fn generate_wasmer_main_c(
         let prefix = prefixes
             .get_prefix_for_atom(&utils::normalize_atom_name(atom_names[0]))
             .ok_or_else(|| {
+                let formatted_prefixes = format!("{prefixes:#?}");
                 anyhow::anyhow!(
-                    "cannot find prefix for atom {} when generating wasmer_main.c ({:#?})",
-                    &atom_names[0],
-                    prefixes
+                    "cannot find prefix for atom {} when generating wasmer_main.c ({formatted_prefixes})",
+                    &atom_names[0]
                 )
             })?;
         write!(c_code_to_instantiate, "module = atom_{prefix};")?;
@@ -1576,9 +1575,9 @@ fn generate_wasmer_main_c(
             let prefix = prefixes
                 .get_prefix_for_atom(&utils::normalize_atom_name(a))
                 .ok_or_else(|| {
+                    let formatted_prefixes = format!("{prefixes:#?}");
                     anyhow::anyhow!(
-                        "cannot find prefix for atom {a} when generating wasmer_main.c ({:#?})",
-                        prefixes
+                        "cannot find prefix for atom {a} when generating wasmer_main.c ({formatted_prefixes})"
                     )
                 })?;
             writeln!(
@@ -1617,7 +1616,7 @@ pub(super) mod utils {
         path::{Path, PathBuf},
     };
 
-    use anyhow::{anyhow, Context};
+    use anyhow::{Context, anyhow};
     use target_lexicon::{Architecture, Environment, OperatingSystem, Triple};
     use wasmer_types::target::{CpuFeature, Target};
 
@@ -1647,20 +1646,20 @@ pub(super) mod utils {
     ) -> Result<CrossCompileSetup, anyhow::Error> {
         let target = target_triple;
 
-        if let Some(tarball_path) = cross_subc.tarball.as_mut() {
-            if tarball_path.is_relative() {
-                *tarball_path = starting_cd.join(&tarball_path);
-                if !tarball_path.exists() {
-                    return Err(anyhow!(
-                        "Tarball path `{}` does not exist.",
-                        tarball_path.display()
-                    ));
-                } else if tarball_path.is_dir() {
-                    return Err(anyhow!(
-                        "Tarball path `{}` is a directory.",
-                        tarball_path.display()
-                    ));
-                }
+        if let Some(tarball_path) = cross_subc.tarball.as_mut()
+            && tarball_path.is_relative()
+        {
+            *tarball_path = starting_cd.join(&tarball_path);
+            if !tarball_path.exists() {
+                return Err(anyhow!(
+                    "Tarball path `{}` does not exist.",
+                    tarball_path.display()
+                ));
+            } else if tarball_path.is_dir() {
+                return Err(anyhow!(
+                    "Tarball path `{}` is a directory.",
+                    tarball_path.display()
+                ));
             }
         }
 
@@ -1767,10 +1766,10 @@ pub(super) mod utils {
             return None;
         }
 
-        if let Architecture::Aarch64(_) = target.architecture {
-            if !(filename.contains("aarch64") || filename.contains("arm64")) {
-                return None;
-            }
+        if let Architecture::Aarch64(_) = target.architecture
+            && !(filename.contains("aarch64") || filename.contains("arm64"))
+        {
+            return None;
         }
 
         if let Architecture::X86_64 = target.architecture {
@@ -1783,22 +1782,22 @@ pub(super) mod utils {
             }
         }
 
-        if let OperatingSystem::Windows = target.operating_system {
-            if !filename.contains("windows") {
-                return None;
-            }
+        if let OperatingSystem::Windows = target.operating_system
+            && !filename.contains("windows")
+        {
+            return None;
         }
 
-        if let OperatingSystem::Darwin = target.operating_system {
-            if !(filename.contains("apple") || filename.contains("darwin")) {
-                return None;
-            }
+        if let OperatingSystem::Darwin(_) = target.operating_system
+            && !(filename.contains("apple") || filename.contains("darwin"))
+        {
+            return None;
         }
 
-        if let OperatingSystem::Linux = target.operating_system {
-            if !filename.contains("linux") {
-                return None;
-            }
+        if let OperatingSystem::Linux = target.operating_system
+            && !filename.contains("linux")
+        {
+            return None;
         }
 
         Some(true)
@@ -1851,7 +1850,9 @@ pub(super) mod utils {
         })
         .cloned()
         .ok_or_else(|| {
-            anyhow!("Could not find libwasmer.a for {} target in the provided tarball path (files = {files:#?})", target)
+            anyhow!(
+                "Could not find libwasmer.a for {target} target in the provided tarball path (files = {files:#?})"
+            )
         })
     }
 
@@ -1879,7 +1880,7 @@ pub(super) mod utils {
         };
         let os = match target_triple.operating_system {
             OperatingSystem::Linux => "linux".into(),
-            OperatingSystem::Darwin => "macos".into(),
+            OperatingSystem::Darwin(_) => "macos".into(),
             OperatingSystem::Windows => "windows".into(),
             v => v.to_string(),
         };
@@ -1948,14 +1949,18 @@ pub(super) mod utils {
 
     pub(super) fn find_zig_binary(path: Option<PathBuf>) -> Result<PathBuf, anyhow::Error> {
         use std::env::split_paths;
+        #[cfg(not(unix))]
+        use std::ffi::OsStr;
         #[cfg(unix)]
-        use std::os::unix::ffi::OsStrExt;
+        use std::ffi::OsString;
+        #[cfg(unix)]
+        use std::os::unix::ffi::OsStringExt;
         let path_var = std::env::var("PATH").unwrap_or_default();
         #[cfg(unix)]
         let system_path_var = std::process::Command::new("getconf")
             .args(["PATH"])
             .output()
-            .map(|output| output.stdout)
+            .map(|output| OsString::from_vec(output.stdout))
             .unwrap_or_default();
         let retval = if let Some(p) = path {
             if p.exists() {
@@ -1965,16 +1970,12 @@ pub(super) mod utils {
             }
         } else {
             let mut retval = None;
-            for mut p in split_paths(&path_var).chain(split_paths(
-                #[cfg(unix)]
-                {
-                    &OsStr::from_bytes(&system_path_var[..])
-                },
-                #[cfg(not(unix))]
-                {
-                    OsStr::new("")
-                },
-            )) {
+            #[cfg(unix)]
+            let combined_paths =
+                split_paths(&path_var).chain(split_paths(system_path_var.as_os_str()));
+            #[cfg(not(unix))]
+            let combined_paths = split_paths(&path_var).chain(split_paths(OsStr::new("")));
+            for mut p in combined_paths {
                 p.push(get_zig_exe_str());
                 if p.exists() {
                     retval = Some(p);
@@ -2008,7 +2009,10 @@ pub(super) mod utils {
             .map_err(|e| anyhow!("could not parse zig version: {version_slice}: {e}"))?;
 
         if version_semver < semver::Version::parse("0.10.0").unwrap() {
-            Err(anyhow!("`zig` binary in PATH (`{}`) is not a new enough version (`{version_slice}`): please use version `0.10.0` or newer.", retval.display()))
+            Err(anyhow!(
+                "`zig` binary in PATH (`{}`) is not a new enough version (`{version_slice}`): please use version `0.10.0` or newer.",
+                retval.display()
+            ))
         } else {
             Ok(retval)
         }
@@ -2143,7 +2147,7 @@ pub(super) mod utils {
 mod http_fetch {
     use std::path::Path;
 
-    use anyhow::{anyhow, Context, Result};
+    use anyhow::{Context, Result, anyhow};
 
     pub(super) fn get_release(
         release_version: Option<semver::Version>,
@@ -2290,7 +2294,7 @@ mod http_fetch {
     ) -> Result<std::path::PathBuf, anyhow::Error> {
         let filename = browser_download_url
             .split('/')
-            .last()
+            .next_back()
             .unwrap_or("output")
             .to_string();
 
