@@ -1,6 +1,6 @@
 use super::WasmerCmd;
 use clap::CommandFactory;
-use clap_complete::{generate, Shell};
+use clap_complete::{Shell, generate};
 use std::fs::OpenOptions;
 
 #[derive(Debug, Clone, clap::Parser)]
