@@ -6,8 +6,6 @@ mod extract;
 mod filter;
 mod import;
 mod inspect;
-#[cfg(feature = "fuse")]
-mod mount;
 
 pub use compact::*;
 pub use export::*;
@@ -15,8 +13,6 @@ pub use extract::*;
 pub use filter::*;
 pub use import::*;
 pub use inspect::*;
-#[cfg(feature = "fuse")]
-pub use mount::*;
 
 /// Manage Journal files.
 #[derive(clap::Subcommand, Debug)]
@@ -31,9 +27,6 @@ pub enum CmdJournal {
     Inspect(CmdJournalInspect),
     /// Filters out certain events from a journal
     Filter(CmdJournalFilter),
-    /// Mounts the journal at a particular directory
-    #[cfg(feature = "fuse")]
-    Mount(CmdJournalMount),
     /// Extracts an element of a journal
     Extract(CmdJournalExtract),
 }
@@ -48,8 +41,6 @@ impl CliCommand for CmdJournal {
             Self::Export(cmd) => cmd.run(),
             Self::Inspect(cmd) => cmd.run(),
             Self::Filter(cmd) => cmd.run(),
-            #[cfg(feature = "fuse")]
-            Self::Mount(cmd) => cmd.run(),
             Self::Extract(cmd) => cmd.run(),
         }
     }
