@@ -1,10 +1,13 @@
 #![allow(missing_docs, unused)]
 
 mod capabilities;
+mod package_source;
+mod runtime;
+mod target;
 mod wasi;
 
 use std::{
-    collections::{hash_map::DefaultHasher, BTreeMap},
+    collections::{BTreeMap, hash_map::DefaultHasher},
     fmt::{Binary, Display},
     fs::File,
     hash::{BuildHasherDefault, Hash, Hasher},
@@ -16,8 +19,10 @@ use std::{
     time::{Duration, SystemTime, UNIX_EPOCH},
 };
 
-use anyhow::{anyhow, bail, Context, Error};
+use anyhow::{Context, Error, anyhow, bail};
 use clap::{Parser, ValueEnum};
+use colored::Colorize;
+use futures::future::BoxFuture;
 use indicatif::{MultiProgress, ProgressBar};
 use once_cell::sync::Lazy;
 use tempfile::NamedTempFile;
@@ -25,42 +30,51 @@ use url::Url;
 #[cfg(feature = "sys")]
 use wasmer::sys::NativeEngineExt;
 use wasmer::{
-    AsStoreMut, DeserializeError, Engine, Function, Imports, Instance, Module, Store, Type,
-    TypedFunction, Value,
+    AsStoreMut, DeserializeError, Engine, Function, Imports, Instance, Module, RuntimeError, Store,
+    Type, TypedFunction, Value, wat2wasm,
 };
 
-use wasmer_types::{target::Target, Features};
+use wasmer_types::{Features, target::Target};
 
 #[cfg(feature = "compiler")]
 use wasmer_compiler::ArtifactBuild;
-use wasmer_config::package::PackageSource as PackageSpecifier;
+use wasmer_config::package::PackageSource;
 use wasmer_package::utils::from_disk;
 use wasmer_types::ModuleHash;
 
 #[cfg(feature = "journal")]
 use wasmer_wasix::journal::{LogFileJournal, SnapshotTrigger};
 use wasmer_wasix::{
-    bin_factory::BinaryPackage,
+    Runtime, SpawnError, WasiError,
+    bin_factory::{BinaryPackage, BinaryPackageCommand},
     journal::CompactingLogFileJournal,
     runners::{
+        MappedCommand, MappedDirectory, Runner,
         dcgi::{DcgiInstanceFactory, DcgiRunner},
         dproxy::DProxyRunner,
-        wasi::WasiRunner,
+        wasi::{RuntimeOrEngine, WasiRunner},
         wcgi::{self, AbortHandle, NoOpWcgiCallbacks, WcgiRunner},
-        MappedCommand, MappedDirectory, Runner,
     },
     runtime::{
-        module_cache::CacheError, package_loader::PackageLoader, resolver::QueryError,
+        module_cache::{CacheError, HashedModuleData},
+        package_loader::PackageLoader,
+        resolver::QueryError,
         task_manager::VirtualTaskManagerExt,
     },
-    Runtime, WasiError,
 };
-use webc::metadata::Manifest;
 use webc::Container;
+use webc::metadata::Manifest;
 
 use crate::{
-    backend::RuntimeOptions, commands::run::wasi::Wasi, common::HashAlgorithm, config::WasmerEnv,
-    error::PrettyError, logging::Output,
+    backend::RuntimeOptions,
+    commands::run::{target::TargetOnDisk, wasi::Wasi},
+    config::WasmerEnv,
+    error::PrettyError,
+    logging::Output,
+};
+
+use self::{
+    package_source::CliPackageSource, runtime::MonitoringRuntime, target::ExecutableTarget,
 };
 
 const TICK: Duration = Duration::from_millis(250);
@@ -89,13 +103,10 @@ pub struct Run {
     #[clap(name = "COREDUMP_PATH", long)]
     coredump_on_trap: Option<PathBuf>,
     /// The file, URL, or package to run.
-    #[clap(value_parser = PackageSource::infer)]
-    input: PackageSource,
+    #[clap(value_parser = CliPackageSource::infer)]
+    input: CliPackageSource,
     /// Command-line arguments passed to the package
     args: Vec<String>,
-    /// Hashing algorithm to be used for module hash
-    #[clap(long, value_enum)]
-    hash_algorithm: Option<HashAlgorithm>,
 }
 
 impl Run {
@@ -106,6 +117,8 @@ impl Run {
 
     #[tracing::instrument(level = "debug", name = "wasmer_run", skip_all)]
     fn execute_inner(mut self, output: Output) -> Result<(), Error> {
+        self.print_option_warnings();
+
         let pb = ProgressBar::new_spinner();
         pb.set_draw_target(output.draw_target());
         pb.enable_steady_tick(TICK);
@@ -135,36 +148,43 @@ impl Run {
 
         // Try to detect WebAssembly features before selecting a backend
         tracing::info!("Input source: {:?}", self.input);
-        if let PackageSource::File(path) = &self.input {
+        if let CliPackageSource::File(path) = &self.input {
             tracing::info!("Input file path: {}", path.display());
 
             // Try to read and detect any file that exists, regardless of extension
-            if path.exists() {
-                tracing::info!("Found file: {}", path.display());
-                match std::fs::read(path) {
-                    Ok(bytes) => {
-                        tracing::info!("Read {} bytes from file", bytes.len());
-
-                        // Check if it's a WebAssembly module by looking for magic bytes
-                        let magic = [0x00, 0x61, 0x73, 0x6D]; // "\0asm"
-                        if bytes.len() >= 4 && bytes[0..4] == magic {
-                            // Looks like a valid WebAssembly module, save the bytes for feature detection
-                            tracing::info!(
-                                "Valid WebAssembly module detected, magic header verified"
-                            );
-                            wasm_bytes = Some(bytes);
+            let target = TargetOnDisk::from_file(path);
+            if let Ok(target) = target {
+                match target {
+                    TargetOnDisk::WebAssemblyBinary => {
+                        if let Ok(data) = std::fs::read(path) {
+                            wasm_bytes = Some(data);
                         } else {
-                            tracing::info!("File does not have valid WebAssembly magic number, will try to run it anyway");
-                            // Still provide the bytes so the engine can attempt to run it
-                            wasm_bytes = Some(bytes);
+                            tracing::info!("Failed to read file: {}", path.display());
                         }
                     }
-                    Err(e) => {
-                        tracing::info!("Failed to read file for feature detection: {}", e);
-                    }
+                    TargetOnDisk::Wat => match std::fs::read(path) {
+                        Ok(data) => match wat2wasm(&data) {
+                            Ok(wasm) => {
+                                wasm_bytes = Some(wasm.to_vec());
+                            }
+                            Err(e) => {
+                                tracing::info!(
+                                    "Failed to convert WAT to Wasm for {}: {e}",
+                                    path.display()
+                                );
+                            }
+                        },
+                        Err(e) => {
+                            tracing::info!("Failed to read WAT file {}: {e}", path.display());
+                        }
+                    },
+                    _ => {}
                 }
             } else {
-                tracing::info!("File does not exist: {}", path.display());
+                tracing::info!(
+                    "Failed to read file for feature detection: {}",
+                    path.display()
+                );
             }
         } else {
             tracing::info!("Input is not a file, skipping WebAssembly feature detection");
@@ -180,7 +200,7 @@ impl Run {
             }
             None => {
                 // No WebAssembly file available for analysis, check if we have a webc package
-                if let PackageSource::Package(ref pkg_source) = &self.input {
+                if let CliPackageSource::Package(pkg_source) = &self.input {
                     tracing::info!("Checking package for WebAssembly features: {}", pkg_source);
                     self.rt.get_engine(&Target::default())?
                 } else {
@@ -194,12 +214,8 @@ impl Run {
         tracing::info!("Executing on backend {engine_kind:?}");
 
         #[cfg(feature = "sys")]
-        if engine.is_sys() {
-            if self.stack_size.is_some() {
-                wasmer_vm::set_stack_size(self.stack_size.unwrap());
-            }
-            let hash_algorithm = self.hash_algorithm.unwrap_or_default().into();
-            engine.set_hash_algorithm(Some(hash_algorithm));
+        if engine.is_sys() && self.stack_size.is_some() {
+            wasmer_vm::set_stack_size(self.stack_size.unwrap());
         }
 
         let engine = engine.clone();
@@ -214,7 +230,11 @@ impl Run {
 
         // This is a slow operation, so let's temporarily wrap the runtime with
         // something that displays progress
-        let monitoring_runtime = Arc::new(MonitoringRuntime::new(runtime, pb.clone()));
+        let monitoring_runtime = Arc::new(MonitoringRuntime::new(
+            runtime,
+            pb.clone(),
+            output.is_quiet_or_no_tty(),
+        ));
         let runtime: Arc<dyn Runtime + Send + Sync> = monitoring_runtime.runtime.clone();
         let monitoring_runtime: Arc<dyn Runtime + Send + Sync> = monitoring_runtime;
 
@@ -222,7 +242,7 @@ impl Run {
 
         if let ExecutableTarget::Package(ref pkg) = target {
             self.wasi
-                .mapped_dirs
+                .volumes
                 .extend(pkg.additional_host_mapped_directories.clone());
         }
 
@@ -240,69 +260,74 @@ impl Run {
                 } => self.execute_wasm(&path, module, module_hash, runtime.clone()),
                 ExecutableTarget::Package(pkg) => {
                     // Check if we should update the engine based on the WebC package features
-                    if let Some(cmd) = pkg.get_entrypoint_command() {
-                        if let Some(features) = cmd.wasm_features() {
-                            // Get the right engine for these features
-                            let backends = self.rt.get_available_backends()?;
-                            let available_engines = backends
-                                .iter()
-                                .map(|b| b.to_string())
-                                .collect::<Vec<_>>()
-                                .join(", ");
+                    if let Some(cmd) = pkg.get_entrypoint_command()
+                        && let Some(features) = cmd.wasm_features()
+                    {
+                        // Get the right engine for these features
+                        let backends = self.rt.get_available_backends()?;
+                        let available_engines = backends
+                            .iter()
+                            .map(|b| b.to_string())
+                            .collect::<Vec<_>>()
+                            .join(", ");
 
-                            let filtered_backends = RuntimeOptions::filter_backends_by_features(
-                                backends.clone(),
-                                &features,
+                        let filtered_backends = RuntimeOptions::filter_backends_by_features(
+                            backends.clone(),
+                            &features,
+                            &Target::default(),
+                        );
+
+                        if !filtered_backends.is_empty() {
+                            let engine_id = filtered_backends[0].to_string();
+
+                            // Get a new engine that's compatible with the required features
+                            if let Ok(new_engine) = filtered_backends[0].get_engine(
                                 &Target::default(),
-                            );
-
-                            if !filtered_backends.is_empty() {
-                                let engine_id = filtered_backends[0].to_string();
-
-                                // Get a new engine that's compatible with the required features
-                                if let Ok(new_engine) = filtered_backends[0].get_engine(
-                                    &Target::default(),
-                                    &features,
-                                    &self.rt,
-                                ) {
-                                    tracing::info!(
-                                        "The command '{}' requires to run the Wasm module with the features {:?}. The backends available are {}. Choosing {}.",
-                                        cmd.name(),
-                                        features,
-                                        available_engines,
-                                        engine_id
-                                    );
-                                    // Create a new runtime with the updated engine
-                                    let new_runtime = self.wasi.prepare_runtime(
-                                        new_engine,
+                                &features,
+                                &self.rt,
+                            ) {
+                                tracing::info!(
+                                    "The command '{}' requires to run the Wasm module with the features {:?}. The backends available are {}. Choosing {}.",
+                                    cmd.name(),
+                                    features,
+                                    available_engines,
+                                    engine_id
+                                );
+                                // Create a new runtime with the updated engine
+                                let capability_cache_path =
+                                    capabilities::get_capability_cache_path(
                                         &self.env,
-                                        &capabilities::get_capability_cache_path(
-                                            &self.env,
-                                            &self.input,
-                                        )?,
-                                        tokio::runtime::Builder::new_multi_thread()
-                                            .enable_all()
-                                            .build()?,
-                                        preferred_webc_version,
+                                        &self.input,
                                     )?;
+                                let new_runtime = self.wasi.prepare_runtime(
+                                    new_engine,
+                                    &self.env,
+                                    &capability_cache_path,
+                                    tokio::runtime::Builder::new_multi_thread()
+                                        .enable_all()
+                                        .build()?,
+                                    preferred_webc_version,
+                                )?;
 
-                                    let new_runtime =
-                                        Arc::new(MonitoringRuntime::new(new_runtime, pb.clone()));
-                                    return self.execute_webc(&pkg, new_runtime);
-                                }
+                                let new_runtime = Arc::new(MonitoringRuntime::new(
+                                    new_runtime,
+                                    pb.clone(),
+                                    output.is_quiet_or_no_tty(),
+                                ));
+                                return self.execute_webc(&pkg, new_runtime);
                             }
                         }
                     }
-                    self.execute_webc(&pkg, runtime.clone())
+                    self.execute_webc(&pkg, monitoring_runtime)
                 }
             }
         };
 
         // restore the TTY state as the execution may have changed it
-        if let Some(state) = tty {
-            if let Some(tty) = runtime.tty() {
-                tty.tty_set(state);
-            }
+        if let Some(state) = tty
+            && let Some(tty) = runtime.tty()
+        {
+            tty.tty_set(state);
         }
 
         if let Err(e) = &result {
@@ -367,7 +392,8 @@ impl Run {
         let mut dependencies = Vec::new();
 
         for name in &self.wasi.uses {
-            let specifier = PackageSpecifier::from_str(name)
+            let specifier = name
+                .parse::<PackageSource>()
                 .with_context(|| format!("Unable to parse \"{name}\" as a package specifier"))?;
             let pkg = {
                 let specifier = specifier.clone();
@@ -392,8 +418,9 @@ impl Run {
         uses: Vec<BinaryPackage>,
         runtime: Arc<dyn Runtime + Send + Sync>,
     ) -> Result<(), Error> {
-        let mut runner = self.build_wasi_runner(&runtime)?;
-        runner.run_command(command_name, pkg, runtime)
+        // Assume webcs are always WASIX
+        let mut runner = self.build_wasi_runner(&runtime, true)?;
+        Runner::run_command(&mut runner, command_name, pkg, runtime)
     }
 
     fn run_wcgi(
@@ -417,7 +444,7 @@ impl Run {
             .args(self.args.clone())
             .addr(self.wcgi.addr)
             .envs(self.wasi.env_vars.clone())
-            .map_directories(self.wasi.mapped_dirs.clone())
+            .map_directories(self.wasi.all_volumes())
             .callbacks(Callbacks::new(self.wcgi.addr))
             .inject_packages(uses);
         *config.capabilities() = self.wasi.capabilities();
@@ -475,7 +502,7 @@ impl Run {
         pkg: &BinaryPackage,
         runtime: Arc<dyn Runtime + Send + Sync>,
     ) -> Result<(), Error> {
-        let mut inner = self.build_wasi_runner(&runtime)?;
+        let mut inner = self.build_wasi_runner(&runtime, true)?;
         let mut runner = wasmer_wasix::runners::dproxy::DProxyRunner::new(inner, pkg);
         runner.run_command(command_name, pkg, runtime)
     }
@@ -501,42 +528,54 @@ impl Run {
             }
         };
 
-        let return_values = invoke_function(&instance, &mut store, entry_function, &self.args)?;
+        let result = invoke_function(&instance, &mut store, entry_function, &self.args)?;
 
-        println!(
-            "{}",
-            return_values
-                .iter()
-                .map(|val| val.to_string())
-                .collect::<Vec<String>>()
-                .join(" ")
-        );
-
-        Ok(())
+        match result {
+            Ok(return_values) => {
+                println!(
+                    "{}",
+                    return_values
+                        .iter()
+                        .map(|val| val.to_string())
+                        .collect::<Vec<String>>()
+                        .join(" ")
+                );
+                Ok(())
+            }
+            Err(err) => {
+                bail!("{}", err.display(&mut store));
+            }
+        }
     }
 
     fn build_wasi_runner(
         &self,
         runtime: &Arc<dyn Runtime + Send + Sync>,
+        is_wasix: bool,
     ) -> Result<WasiRunner, anyhow::Error> {
         let packages = self.load_injected_packages(runtime)?;
 
         let mut runner = WasiRunner::new();
 
-        let (is_home_mapped, is_tmp_mapped, mapped_diretories) =
-            self.wasi.build_mapped_directories()?;
+        let (is_home_mapped, mapped_directories) = self.wasi.build_mapped_directories(is_wasix)?;
 
         runner
             .with_args(&self.args)
             .with_injected_packages(packages)
             .with_envs(self.wasi.env_vars.clone())
             .with_mapped_host_commands(self.wasi.build_mapped_commands()?)
-            .with_mapped_directories(mapped_diretories)
+            .with_mapped_directories(mapped_directories)
             .with_home_mapped(is_home_mapped)
-            .with_tmp_mapped(is_tmp_mapped)
             .with_forward_host_env(self.wasi.forward_host_env)
             .with_capabilities(self.wasi.capabilities());
 
+        if let Some(cwd) = self.wasi.cwd.as_ref() {
+            if !cwd.starts_with("/") {
+                bail!("The argument to --cwd must be an absolute path");
+            }
+            runner.with_current_dir(cwd.clone());
+        }
+
         if let Some(ref entry_function) = self.invoke {
             runner.with_entry_function(entry_function);
         }
@@ -583,60 +622,46 @@ impl Run {
     ) -> Result<(), Error> {
         let program_name = wasm_path.display().to_string();
 
-        let runner = self.build_wasi_runner(&runtime)?;
-        runner.run_wasm(runtime, &program_name, module, module_hash)
+        let runner = self.build_wasi_runner(&runtime, wasmer_wasix::is_wasix_module(&module))?;
+        runner.run_wasm(
+            RuntimeOrEngine::Runtime(runtime),
+            &program_name,
+            module,
+            module_hash,
+        )
     }
 
     #[allow(unused_variables)]
     fn maybe_save_coredump(&self, e: &Error) {
         #[cfg(feature = "coredump")]
-        if let Some(coredump) = &self.coredump_on_trap {
-            if let Err(e) = generate_coredump(e, self.input.to_string(), coredump) {
-                tracing::warn!(
-                    error = &*e as &dyn std::error::Error,
-                    coredump_path=%coredump.display(),
-                    "Unable to generate a coredump",
-                );
-            }
+        if let Some(coredump) = &self.coredump_on_trap
+            && let Err(e) = generate_coredump(e, self.input.to_string(), coredump)
+        {
+            tracing::warn!(
+                error = &*e as &dyn std::error::Error,
+                coredump_path=%coredump.display(),
+                "Unable to generate a coredump",
+            );
         }
     }
 
-    /// Create Run instance for arguments/env, assuming we're being run from a
-    /// CFP binfmt interpreter.
-    pub fn from_binfmt_args() -> Self {
-        Run::from_binfmt_args_fallible().unwrap_or_else(|e| {
-            crate::error::PrettyError::report::<()>(
-                Err(e).context("Failed to set up wasmer binfmt invocation"),
-            )
-        })
-    }
-
-    fn from_binfmt_args_fallible() -> Result<Self, Error> {
-        if cfg!(not(target_os = "linux")) {
-            bail!("binfmt_misc is only available on linux.");
+    fn print_option_warnings(&self) {
+        if !self.wasi.mapped_dirs.is_empty() {
+            eprintln!(
+                "{}The `{}` option is deprecated and will be removed in the next major release. Please use `{}` instead.",
+                "warning: ".yellow(),
+                "--mapdir".yellow(),
+                "--volume".green()
+            );
+        }
+        if !self.wasi.pre_opened_directories.is_empty() {
+            eprintln!(
+                "{}The `{}` option is deprecated and will be removed in the next major release. Please use `{}` instead.",
+                "warning: ".yellow(),
+                "--dir".yellow(),
+                "--volume".green()
+            );
         }
-
-        let argv = std::env::args().collect::<Vec<_>>();
-        let (_interpreter, executable, original_executable, args) = match &argv[..] {
-            [a, b, c, rest @ ..] => (a, b, c, rest),
-            _ => {
-                bail!("Wasmer binfmt interpreter needs at least three arguments (including $0) - must be registered as binfmt interpreter with the CFP flags. (Got arguments: {:?})", argv);
-            }
-        };
-        let rt = RuntimeOptions::default();
-        Ok(Run {
-            env: WasmerEnv::default(),
-            rt,
-            wasi: Wasi::for_binfmt_interpreter()?,
-            wcgi: WcgiOptions::default(),
-            stack_size: None,
-            entrypoint: Some(original_executable.to_string()),
-            invoke: None,
-            coredump_on_trap: None,
-            input: PackageSource::infer(executable)?,
-            args: args.to_vec(),
-            hash_algorithm: None,
-        })
     }
 }
 
@@ -645,16 +670,14 @@ fn invoke_function(
     store: &mut Store,
     func: &Function,
     args: &[String],
-) -> Result<Box<[Value]>, Error> {
+) -> anyhow::Result<Result<Box<[Value]>, RuntimeError>> {
     let func_ty = func.ty(store);
     let required_arguments = func_ty.params().len();
     let provided_arguments = args.len();
 
     anyhow::ensure!(
         required_arguments == provided_arguments,
-        "Function expected {} arguments, but received {}",
-        required_arguments,
-        provided_arguments,
+        "Function expected {required_arguments} arguments, but received {provided_arguments}"
     );
 
     let invoke_args = args
@@ -666,9 +689,7 @@ fn invoke_function(
         })
         .collect::<Result<Vec<_>, _>>()?;
 
-    let return_values = func.call(store, &invoke_args)?;
-
-    Ok(return_values)
+    Ok(func.call(store, &invoke_args))
 }
 
 fn parse_value(s: &str, ty: wasmer_types::Type) -> Result<Value, Error> {
@@ -683,202 +704,6 @@ fn parse_value(s: &str, ty: wasmer_types::Type) -> Result<Value, Error> {
     Ok(value)
 }
 
-/// The input that was passed in via the command-line.
-#[derive(Debug, Clone, PartialEq)]
-enum PackageSource {
-    /// A file on disk (`*.wasm`, `*.webc`, etc.).
-    File(PathBuf),
-    /// A directory containing a `wasmer.toml` file
-    Dir(PathBuf),
-    /// A package to be downloaded (a URL, package name, etc.)
-    Package(PackageSpecifier),
-}
-
-impl PackageSource {
-    fn infer(s: &str) -> Result<PackageSource, Error> {
-        let path = Path::new(s);
-        if path.is_file() {
-            return Ok(PackageSource::File(path.to_path_buf()));
-        } else if path.is_dir() {
-            return Ok(PackageSource::Dir(path.to_path_buf()));
-        }
-
-        if let Ok(pkg) = PackageSpecifier::from_str(s) {
-            return Ok(PackageSource::Package(pkg));
-        }
-
-        Err(anyhow::anyhow!(
-            "Unable to resolve \"{s}\" as a URL, package name, or file on disk"
-        ))
-    }
-
-    /// Try to resolve the [`PackageSource`] to an executable artifact.
-    ///
-    /// This will try to automatically download and cache any resources from the
-    /// internet.
-    #[tracing::instrument(level = "debug", skip_all)]
-    fn resolve_target(
-        &self,
-        rt: &Arc<dyn Runtime + Send + Sync>,
-        pb: &ProgressBar,
-    ) -> Result<ExecutableTarget, Error> {
-        match self {
-            PackageSource::File(path) => ExecutableTarget::from_file(path, rt, pb),
-            PackageSource::Dir(d) => ExecutableTarget::from_dir(d, rt, pb),
-            PackageSource::Package(pkg) => {
-                pb.set_message("Loading from the registry");
-                let inner_pck = pkg.clone();
-                let inner_rt = rt.clone();
-                let pkg = rt.task_manager().spawn_and_block_on(async move {
-                    BinaryPackage::from_registry(&inner_pck, inner_rt.as_ref()).await
-                })??;
-                Ok(ExecutableTarget::Package(pkg))
-            }
-        }
-    }
-}
-
-impl Display for PackageSource {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self {
-            PackageSource::File(path) | PackageSource::Dir(path) => write!(f, "{}", path.display()),
-            PackageSource::Package(p) => write!(f, "{p}"),
-        }
-    }
-}
-
-/// We've been given the path for a file... What does it contain and how should
-/// that be run?
-#[derive(Debug, Clone)]
-enum TargetOnDisk {
-    WebAssemblyBinary,
-    Wat,
-    LocalWebc,
-    Artifact,
-}
-
-impl TargetOnDisk {
-    fn from_file(path: &Path) -> Result<TargetOnDisk, Error> {
-        // Normally the first couple hundred bytes is enough to figure
-        // out what type of file this is.
-        let mut buffer = [0_u8; 512];
-
-        let mut f = File::open(path)
-            .with_context(|| format!("Unable to open \"{}\" for reading", path.display()))?;
-        let bytes_read = f.read(&mut buffer)?;
-
-        let leading_bytes = &buffer[..bytes_read];
-
-        if wasmer::is_wasm(leading_bytes) {
-            return Ok(TargetOnDisk::WebAssemblyBinary);
-        }
-
-        if webc::detect(leading_bytes).is_ok() {
-            return Ok(TargetOnDisk::LocalWebc);
-        }
-
-        #[cfg(feature = "compiler")]
-        if ArtifactBuild::is_deserializable(leading_bytes) {
-            return Ok(TargetOnDisk::Artifact);
-        }
-
-        // If we can't figure out the file type based on its content, fall back
-        // to checking the extension.
-
-        match path.extension().and_then(|s| s.to_str()) {
-            Some("wat") => Ok(TargetOnDisk::Wat),
-            Some("wasm") => Ok(TargetOnDisk::WebAssemblyBinary),
-            Some("webc") => Ok(TargetOnDisk::LocalWebc),
-            Some("wasmu") => Ok(TargetOnDisk::WebAssemblyBinary),
-            _ => bail!("Unable to determine how to execute \"{}\"", path.display()),
-        }
-    }
-}
-
-#[derive(Debug, Clone)]
-enum ExecutableTarget {
-    WebAssembly {
-        module: Module,
-        module_hash: ModuleHash,
-        path: PathBuf,
-    },
-    Package(BinaryPackage),
-}
-
-impl ExecutableTarget {
-    /// Try to load a Wasmer package from a directory containing a `wasmer.toml`
-    /// file.
-    #[tracing::instrument(level = "debug", skip_all)]
-    fn from_dir(
-        dir: &Path,
-        runtime: &Arc<dyn Runtime + Send + Sync>,
-        pb: &ProgressBar,
-    ) -> Result<Self, Error> {
-        pb.set_message(format!("Loading \"{}\" into memory", dir.display()));
-        pb.set_message("Resolving dependencies");
-        let inner_runtime = runtime.clone();
-        let pkg = runtime.task_manager().spawn_and_block_on({
-            let path = dir.to_path_buf();
-
-            async move { BinaryPackage::from_dir(&path, inner_runtime.as_ref()).await }
-        })??;
-
-        Ok(ExecutableTarget::Package(pkg))
-    }
-
-    /// Try to load a file into something that can be used to run it.
-    #[tracing::instrument(level = "debug", skip_all)]
-    fn from_file(
-        path: &Path,
-        runtime: &Arc<dyn Runtime + Send + Sync>,
-        pb: &ProgressBar,
-    ) -> Result<Self, Error> {
-        pb.set_message(format!("Loading from \"{}\"", path.display()));
-
-        match TargetOnDisk::from_file(path)? {
-            TargetOnDisk::WebAssemblyBinary | TargetOnDisk::Wat => {
-                let wasm = std::fs::read(path)?;
-
-                pb.set_message("Compiling to WebAssembly");
-                let module = runtime
-                    .load_module_sync(&wasm)
-                    .with_context(|| format!("Unable to compile \"{}\"", path.display()))?;
-
-                Ok(ExecutableTarget::WebAssembly {
-                    module,
-                    module_hash: ModuleHash::xxhash(&wasm),
-                    path: path.to_path_buf(),
-                })
-            }
-            TargetOnDisk::Artifact => {
-                let engine = runtime.engine();
-                pb.set_message("Deserializing pre-compiled WebAssembly module");
-                let module = unsafe { Module::deserialize_from_file(&engine, path)? };
-
-                let module_hash = module.info().hash.ok_or_else(|| {
-                    anyhow::Error::msg("module hash is not present in the artifact")
-                })?;
-
-                Ok(ExecutableTarget::WebAssembly {
-                    module,
-                    module_hash,
-                    path: path.to_path_buf(),
-                })
-            }
-            TargetOnDisk::LocalWebc => {
-                let container = from_disk(path)?;
-                pb.set_message("Resolving dependencies");
-
-                let inner_runtime = runtime.clone();
-                let pkg = runtime.task_manager().spawn_and_block_on(async move {
-                    BinaryPackage::from_webc(&container, inner_runtime.as_ref()).await
-                })??;
-                Ok(ExecutableTarget::Package(pkg))
-            }
-        }
-    }
-}
-
 #[cfg(feature = "coredump")]
 fn generate_coredump(err: &Error, source_name: String, coredump_path: &Path) -> Result<(), Error> {
     let err: &wasmer::RuntimeError = match err.downcast_ref() {
@@ -1000,134 +825,3 @@ fn get_exit_code(
 
     None
 }
-
-#[derive(Debug)]
-struct MonitoringRuntime<R> {
-    runtime: Arc<R>,
-    progress: ProgressBar,
-}
-
-impl<R> MonitoringRuntime<R> {
-    fn new(runtime: R, progress: ProgressBar) -> Self {
-        MonitoringRuntime {
-            runtime: Arc::new(runtime),
-            progress,
-        }
-    }
-}
-
-impl<R: wasmer_wasix::Runtime + Send + Sync> wasmer_wasix::Runtime for MonitoringRuntime<R> {
-    fn networking(&self) -> &virtual_net::DynVirtualNetworking {
-        self.runtime.networking()
-    }
-
-    fn task_manager(&self) -> &Arc<dyn wasmer_wasix::VirtualTaskManager> {
-        self.runtime.task_manager()
-    }
-
-    fn package_loader(
-        &self,
-    ) -> Arc<dyn wasmer_wasix::runtime::package_loader::PackageLoader + Send + Sync> {
-        let inner = self.runtime.package_loader();
-        Arc::new(MonitoringPackageLoader {
-            inner,
-            progress: self.progress.clone(),
-        })
-    }
-
-    fn module_cache(
-        &self,
-    ) -> Arc<dyn wasmer_wasix::runtime::module_cache::ModuleCache + Send + Sync> {
-        self.runtime.module_cache()
-    }
-
-    fn source(&self) -> Arc<dyn wasmer_wasix::runtime::resolver::Source + Send + Sync> {
-        let inner = self.runtime.source();
-        Arc::new(MonitoringSource {
-            inner,
-            progress: self.progress.clone(),
-        })
-    }
-
-    fn engine(&self) -> wasmer::Engine {
-        self.runtime.engine()
-    }
-
-    fn new_store(&self) -> wasmer::Store {
-        self.runtime.new_store()
-    }
-
-    fn http_client(&self) -> Option<&wasmer_wasix::http::DynHttpClient> {
-        self.runtime.http_client()
-    }
-
-    fn tty(&self) -> Option<&(dyn wasmer_wasix::os::TtyBridge + Send + Sync)> {
-        self.runtime.tty()
-    }
-
-    #[cfg(feature = "journal")]
-    fn read_only_journals<'a>(
-        &'a self,
-    ) -> Box<dyn Iterator<Item = Arc<wasmer_wasix::journal::DynReadableJournal>> + 'a> {
-        self.runtime.read_only_journals()
-    }
-
-    #[cfg(feature = "journal")]
-    fn writable_journals<'a>(
-        &'a self,
-    ) -> Box<dyn Iterator<Item = Arc<wasmer_wasix::journal::DynJournal>> + 'a> {
-        self.runtime.writable_journals()
-    }
-
-    #[cfg(feature = "journal")]
-    fn active_journal(&self) -> Option<&'_ wasmer_wasix::journal::DynJournal> {
-        self.runtime.active_journal()
-    }
-}
-
-#[derive(Debug)]
-struct MonitoringSource {
-    inner: Arc<dyn wasmer_wasix::runtime::resolver::Source + Send + Sync>,
-    progress: ProgressBar,
-}
-
-#[async_trait::async_trait]
-impl wasmer_wasix::runtime::resolver::Source for MonitoringSource {
-    async fn query(
-        &self,
-        package: &PackageSpecifier,
-    ) -> Result<Vec<wasmer_wasix::runtime::resolver::PackageSummary>, QueryError> {
-        self.progress.set_message(format!("Looking up {package}"));
-        self.inner.query(package).await
-    }
-}
-
-#[derive(Debug)]
-struct MonitoringPackageLoader {
-    inner: Arc<dyn wasmer_wasix::runtime::package_loader::PackageLoader + Send + Sync>,
-    progress: ProgressBar,
-}
-
-#[async_trait::async_trait]
-impl wasmer_wasix::runtime::package_loader::PackageLoader for MonitoringPackageLoader {
-    async fn load(
-        &self,
-        summary: &wasmer_wasix::runtime::resolver::PackageSummary,
-    ) -> Result<Container, Error> {
-        let pkg_id = summary.package_id();
-        self.progress.set_message(format!("Downloading {pkg_id}"));
-
-        self.inner.load(summary).await
-    }
-
-    async fn load_package_tree(
-        &self,
-        root: &Container,
-        resolution: &wasmer_wasix::runtime::resolver::Resolution,
-        root_is_local_dir: bool,
-    ) -> Result<BinaryPackage, Error> {
-        self.inner
-            .load_package_tree(root, resolution, root_is_local_dir)
-            .await
-    }
-}
