@@ -32,6 +32,8 @@ impl SelfUpdate {
 
     #[cfg(target_os = "windows")]
     fn inner_execute(&self) -> Result<(), anyhow::Error> {
-        anyhow::bail!("Self update is not supported on Windows. Use install instructions on the Wasmer homepage: https://wasmer.io");
+        anyhow::bail!(
+            "Self update is not supported on Windows. Use install instructions on the Wasmer homepage: https://wasmer.io"
+        );
     }
 }
