@@ -1,33 +1,29 @@
 //! Edge SSH command.
 
 use anyhow::Context;
-use wasmer_backend_api::WasmerClient;
+use wasmer_backend_api::{WasmerClient, types::DeployApp};
 
-use super::AsyncCliCommand;
-use crate::{config::WasmerEnv, edge_config::EdgeConfig};
+use super::{AsyncCliCommand, app::AppIdentArgOpts};
+use crate::{config::WasmerEnv, edge_config::LoadedEdgeConfig};
 
 /// Start a remote SSH session.
 #[derive(clap::Parser, Debug)]
 pub struct CmdSsh {
+    #[clap(flatten)]
+    pub app: AppIdentArgOpts,
+
     #[clap(flatten)]
     env: WasmerEnv,
+
     /// SSH port to use.
     #[clap(long, default_value = "22")]
     pub ssh_port: u16,
+
     /// Local port mapping to the package that's running, this allows
     /// for instance a HTTP server to be tested remotely while giving
     /// instant logs over stderr channelled via SSH.
     #[clap(long)]
     pub map_port: Vec<u16>,
-    /// Package to run on the Deploy servers
-    ///
-    /// By default a bash session will be started.
-    #[clap(index = 1)]
-    pub run: Option<String>,
-
-    /// Additional arguments to pass to the package.
-    #[clap(index = 2, trailing_var_arg = true)]
-    pub run_args: Vec<String>,
 
     /// The Edge SSH server host to connect to.
     ///
@@ -38,11 +34,10 @@ pub struct CmdSsh {
     /// Prints the SSH command rather than executing it.
     #[clap(short, long)]
     pub print: bool,
-}
 
-impl CmdSsh {
-    /// The default package to run.
-    const DEFAULT_PACKAGE: &'static str = "wasmer/bash";
+    // All extra arguments.
+    #[clap(trailing_var_arg = true)]
+    pub extra: Vec<String>,
 }
 
 #[async_trait::async_trait]
@@ -53,10 +48,22 @@ impl AsyncCliCommand for CmdSsh {
         let mut config = crate::edge_config::load_config(None)?;
         let client = self.env.client()?;
 
-        let (token, is_new) = acquire_ssh_token(&client, &config.config).await?;
+        let token = acquire_ssh_token(&client, &config, &self.app).await?;
+        let app_id = token.app.as_ref().map(|a| a.id.inner().to_owned());
 
         let host = if let Some(host) = self.host {
             host
+        } else if let Some(app) = &token.app {
+            // For apps, use the app domain to ensure proper routing.
+            let url = app.url.clone();
+            if url.starts_with("http://") || url.starts_with("https://") {
+                let url = url.parse::<url::Url>().context("Could not parse app url")?;
+                url.host_str()
+                    .context("Could not determine host from app url")?
+                    .to_string()
+            } else {
+                url
+            }
         } else {
             // No custom host specified, use an appropriate one based on the
             // environment.
@@ -66,14 +73,18 @@ impl AsyncCliCommand for CmdSsh {
         };
         let port = self.ssh_port;
 
-        if is_new {
+        if token.is_new {
             eprintln!("Acquired new SSH token");
-            config.set_ssh_token(token.clone())?;
+            config.config.add_ssh_token(app_id, token.token.clone());
             if let Err(err) = config.save() {
                 eprintln!("WARNING: failed to save config: {err}");
             }
         }
 
+        if let Some(app) = &token.app {
+            eprintln!("Connecting to app '{}' at {}", app.name, host);
+        }
+
         let mut cmd = std::process::Command::new("ssh");
         let mut cmd = cmd
             .args([
@@ -98,16 +109,14 @@ impl AsyncCliCommand for CmdSsh {
             cmd = cmd.args(["-L", format!("{map_port}:localhost:{map_port}").as_str()]);
         }
 
-        let package = self.run.as_deref().unwrap_or(Self::DEFAULT_PACKAGE);
-        cmd = cmd.arg(format!("{token}@{host}")).arg(package);
-        for run_arg in self.run_args {
-            cmd = cmd.arg(&run_arg);
-        }
+        cmd = cmd
+            .arg(format!("{}@{}", token.token, host))
+            .args(&self.extra);
 
         if self.print {
             print!("ssh");
             for arg in cmd.get_args() {
-                print!(" {}", arg.to_string_lossy().as_ref());
+                print!(" {}", arg.to_string_lossy());
             }
             println!();
             return Ok(());
@@ -117,33 +126,58 @@ impl AsyncCliCommand for CmdSsh {
         if exit.success() {
             Ok(())
         } else {
-            Err(anyhow::anyhow!("ssh failed with status {}", exit))
+            Err(anyhow::anyhow!("ssh failed with status {exit}"))
         }
     }
 }
 
-type IsNew = bool;
 type RawToken = String;
 
+struct AcquiredToken {
+    token: RawToken,
+    app: Option<DeployApp>,
+    is_new: bool,
+}
+
 async fn acquire_ssh_token(
     client: &WasmerClient,
-    config: &EdgeConfig,
-) -> Result<(RawToken, IsNew), anyhow::Error> {
-    if let Some(token) = &config.ssh_token {
-        // TODO: validate that token is still valid.
-        return Ok((token.clone(), false));
+    config: &LoadedEdgeConfig,
+    app_opts: &AppIdentArgOpts,
+) -> Result<AcquiredToken, anyhow::Error> {
+    let app_opts = app_opts.clone().to_opts();
+
+    let app = if let Some((_, app)) = app_opts.load_app_opt(client).await? {
+        Some(app)
+    } else {
+        None
+    };
+
+    // Check cached.
+    let app_id = app.as_ref().map(|a| a.id.inner());
+    if let Some(token) = config.config.get_valid_ssh_token(app_id) {
+        return Ok(AcquiredToken {
+            token: token.to_string(),
+            app,
+            is_new: false,
+        });
     }
 
-    let token = create_ssh_token(client).await?;
-    Ok((token, true))
+    // Create new.
+    let token = create_ssh_token(client, app_id.map(|x| x.to_owned())).await?;
+
+    Ok(AcquiredToken {
+        token,
+        app,
+        is_new: true,
+    })
 }
 
 /// Create a new token for SSH access through the backend API.
-async fn create_ssh_token(client: &WasmerClient) -> Result<RawToken, anyhow::Error> {
-    wasmer_backend_api::query::generate_deploy_config_token_raw(
-        client,
-        wasmer_backend_api::query::TokenKind::SSH,
-    )
-    .await
-    .context("Could not create token for ssh access")
+async fn create_ssh_token(
+    client: &WasmerClient,
+    app: Option<String>,
+) -> Result<RawToken, anyhow::Error> {
+    wasmer_backend_api::query::generate_ssh_token(client, app)
+        .await
+        .context("Could not create token for ssh access")
 }
