@@ -7,17 +7,19 @@
 use super::func_state::FuncTranslationState;
 use super::translation_utils::reference_type;
 use crate::heap::{Heap, HeapData};
+use crate::translator::func_state::LandingPad;
 use core::convert::From;
 use cranelift_codegen::cursor::FuncCursor;
 use cranelift_codegen::ir::immediates::Offset32;
 use cranelift_codegen::ir::{self, InstBuilder};
 use cranelift_codegen::isa::TargetFrontendConfig;
 use cranelift_frontend::FunctionBuilder;
-use wasmer_compiler::wasmparser::{HeapType, Operator};
+use smallvec::SmallVec;
+use wasmer_compiler::wasmparser::{AbstractHeapType, HeapType, Operator};
 use wasmer_types::entity::PrimaryMap;
 use wasmer_types::{
     FunctionIndex, FunctionType, GlobalIndex, LocalFunctionIndex, MemoryIndex, SignatureIndex,
-    TableIndex, Type as WasmerType, WasmResult,
+    TableIndex, TagIndex, Type as WasmerType, WasmResult,
 };
 
 /// The value of a WebAssembly global variable.
@@ -70,7 +72,7 @@ pub trait TargetEnvironment {
 
     /// Get the Cranelift reference type to use for native references.
     ///
-    /// This returns `R64` for 64-bit architectures and `R32` for 32-bit architectures.
+    /// This returns the target pointer type for both `funcref` and `externref`.
     fn reference_type(&self) -> ir::Type {
         reference_type(self.target_config()).expect("expected reference type")
     }
@@ -189,9 +191,8 @@ pub trait FuncEnvironment: TargetEnvironment {
         _callee_index: FunctionIndex,
         callee: ir::FuncRef,
         call_args: &[ir::Value],
-    ) -> WasmResult<ir::Inst> {
-        Ok(builder.ins().call(callee, call_args))
-    }
+        landing_pad: Option<LandingPad>,
+    ) -> WasmResult<SmallVec<[ir::Value; 4]>>;
 
     /// Translate a `call_indirect` WebAssembly instruction at `pos`.
     ///
@@ -204,6 +205,7 @@ pub trait FuncEnvironment: TargetEnvironment {
     /// Return the call instruction whose results are the WebAssembly return values.
     /// Returns `None` if this statically traps instead of creating a call
     /// instruction.
+    #[allow(clippy::too_many_arguments)]
     fn translate_call_indirect(
         &mut self,
         builder: &mut FunctionBuilder,
@@ -212,7 +214,57 @@ pub trait FuncEnvironment: TargetEnvironment {
         sig_ref: ir::SigRef,
         callee: ir::Value,
         call_args: &[ir::Value],
-    ) -> WasmResult<ir::Inst>;
+        landing_pad: Option<LandingPad>,
+    ) -> WasmResult<SmallVec<[ir::Value; 4]>>;
+
+    /// Return the number of WebAssembly values contained in the payload for the given exception tag.
+    fn tag_param_arity(&self, tag_index: TagIndex) -> usize;
+
+    /// Get the exception reference from the raw exception pointer (used by libunwind).
+    fn translate_exn_pointer_to_ref(
+        &mut self,
+        builder: &mut FunctionBuilder,
+        exn_ptr: ir::Value,
+    ) -> ir::Value;
+
+    /// Extract the payload values from an exception reference produced by the given tag.
+    fn translate_exn_unbox(
+        &mut self,
+        builder: &mut FunctionBuilder,
+        tag_index: TagIndex,
+        exnref: ir::Value,
+    ) -> WasmResult<SmallVec<[ir::Value; 4]>>;
+
+    /// Emit IR to allocate and throw a new exception with the specified tag.
+    fn translate_exn_throw(
+        &mut self,
+        builder: &mut FunctionBuilder,
+        tag_index: TagIndex,
+        args: &[ir::Value],
+        landing_pad: Option<LandingPad>,
+    ) -> WasmResult<()>;
+
+    /// Emit IR to rethrow an existing exception reference.
+    fn translate_exn_throw_ref(
+        &mut self,
+        builder: &mut FunctionBuilder,
+        exnref: ir::Value,
+        landing_pad: Option<LandingPad>,
+    ) -> WasmResult<()>;
+
+    /// Invoke the runtime personality helper to choose the matching catch tag for an exception.
+    fn translate_exn_personality_selector(
+        &mut self,
+        builder: &mut FunctionBuilder,
+        exn_ptr: ir::Value,
+    ) -> WasmResult<ir::Value>;
+
+    /// Reraise an exception when no catch clause within the current handler matches.
+    fn translate_exn_reraise_unmatched(
+        &mut self,
+        builder: &mut FunctionBuilder,
+        exnref: ir::Value,
+    ) -> WasmResult<()>;
 
     /// Translate a `memory.grow` WebAssembly instruction.
     ///
@@ -296,7 +348,7 @@ pub trait FuncEnvironment: TargetEnvironment {
 
     /// Translate a `table.size` WebAssembly instruction.
     fn translate_table_size(&mut self, pos: FuncCursor, index: TableIndex)
-        -> WasmResult<ir::Value>;
+    -> WasmResult<ir::Value>;
 
     /// Translate a `table.grow` WebAssembly instruction.
     fn translate_table_grow(
@@ -370,11 +422,16 @@ pub trait FuncEnvironment: TargetEnvironment {
     /// null sentinel is not a null reference type pointer for your type. If you
     /// override this method, then you should also override
     /// `translate_ref_is_null` as well.
-    fn translate_ref_null(&mut self, pos: FuncCursor, ty: HeapType) -> WasmResult<ir::Value>;
-    // {
-    //     let _ = ty;
-    //     Ok(pos.ins().null(self.reference_type(ty)))
-    // }
+    fn translate_ref_null(&mut self, mut pos: FuncCursor, ty: HeapType) -> WasmResult<ir::Value> {
+        let ty = match ty {
+            HeapType::Abstract {
+                ty: AbstractHeapType::Exn,
+                ..
+            } => ir::types::I32,
+            _ => self.reference_type(),
+        };
+        Ok(pos.ins().iconst(ty, 0))
+    }
 
     /// Translate a `ref.is_null` WebAssembly instruction.
     ///
@@ -389,8 +446,10 @@ pub trait FuncEnvironment: TargetEnvironment {
         mut pos: FuncCursor,
         value: ir::Value,
     ) -> WasmResult<ir::Value> {
-        let is_null = pos.ins().is_null(value);
-        Ok(pos.ins().uextend(ir::types::I64, is_null))
+        let is_null = pos
+            .ins()
+            .icmp_imm(cranelift_codegen::ir::condcodes::IntCC::Equal, value, 0);
+        Ok(pos.ins().uextend(ir::types::I32, is_null))
     }
 
     /// Translate a `ref.func` WebAssembly instruction.
