@@ -7,16 +7,20 @@
 //! function to Cranelift IR guided by a `FuncEnvironment` which provides information about the
 //! WebAssembly module and the runtime environment.
 
-use super::code_translator::{bitcast_arguments, translate_operator, wasm_param_types};
+use super::code_translator::translate_operator;
 use super::func_environ::{FuncEnvironment, ReturnMode};
 use super::func_state::FuncTranslationState;
 use super::translation_utils::get_vmctx_value_label;
+use crate::translator::EXN_REF_TYPE;
+use crate::translator::code_translator::bitcast_wasm_returns;
+use core::convert::TryFrom;
 use cranelift_codegen::entity::EntityRef;
 use cranelift_codegen::ir::{self, Block, InstBuilder, ValueLabel};
 use cranelift_codegen::timing;
-use cranelift_frontend::{FunctionBuilder, FunctionBuilderContext, Variable};
+use cranelift_frontend::{FunctionBuilder, FunctionBuilderContext};
+use wasmer_compiler::wasmparser::RefType;
+use wasmer_compiler::{FunctionBinaryReader, ModuleTranslationState, wptype_to_type};
 use wasmer_compiler::{wasm_unsupported, wasmparser};
-use wasmer_compiler::{wptype_to_type, FunctionBinaryReader, ModuleTranslationState};
 use wasmer_types::{LocalFunctionIndex, WasmResult};
 
 /// WebAssembly to Cranelift IR function translator.
@@ -136,8 +140,10 @@ fn declare_wasm_parameters<FE: FuncEnvironment + ?Sized>(
         // signature parameters. For example, a `vmctx` pointer.
         if environ.is_wasm_parameter(&builder.func.signature, i) {
             // This is a normal WebAssembly signature parameter, so create a local for it.
-            let local = Variable::new(next_local);
-            builder.declare_var(local, param_type.value_type);
+            let local = builder.declare_var(param_type.value_type);
+            let local_index = local.index();
+            debug_assert_eq!(local_index, next_local);
+            debug_assert!(u32::try_from(local_index).is_ok());
             next_local += 1;
 
             let param_value = builder.block_params(entry_block)[i];
@@ -196,7 +202,10 @@ fn declare_locals<FE: FuncEnvironment + ?Sized>(
         }
         Ref(ty) => {
             if ty.is_func_ref() || ty.is_extern_ref() {
-                builder.ins().null(environ.reference_type())
+                builder.ins().iconst(environ.reference_type(), 0)
+            } else if ty == RefType::EXNREF || ty == RefType::EXN {
+                // no `.is_exnref` yet
+                builder.ins().iconst(EXN_REF_TYPE, 0)
             } else {
                 return Err(wasm_unsupported!("unsupported reference type: {:?}", ty));
             }
@@ -206,8 +215,10 @@ fn declare_locals<FE: FuncEnvironment + ?Sized>(
     let wasmer_ty = wptype_to_type(wasm_type).unwrap();
     let ty = builder.func.dfg.value_type(zeroval);
     for _ in 0..count {
-        let local = Variable::new(*next_local);
-        builder.declare_var(local, ty);
+        let local = builder.declare_var(ty);
+        let local_index = local.index();
+        debug_assert_eq!(local_index, *next_local);
+        debug_assert!(u32::try_from(local_index).is_ok());
         builder.def_var(local, zeroval);
         builder.set_val_label(zeroval, ValueLabel::new(*next_local));
         environ.push_local_decl_on_stack(wasmer_ty);
@@ -249,10 +260,7 @@ fn parse_function_body<FE: FuncEnvironment + ?Sized>(
         if !builder.is_unreachable() {
             match environ.return_mode() {
                 ReturnMode::NormalReturns => {
-                    let return_types = wasm_param_types(&builder.func.signature.returns, |i| {
-                        environ.is_wasm_return(&builder.func.signature, i)
-                    });
-                    bitcast_arguments(&mut state.stack, &return_types, builder);
+                    bitcast_wasm_returns(environ, &mut state.stack, builder);
                     builder.ins().return_(&state.stack)
                 }
             };
