@@ -1,83 +1,26 @@
 use crate::abi::Abi;
 use crate::error::{err, err_nt};
-use crate::translator::intrinsics::{type_to_llvm, Intrinsics};
+use crate::translator::intrinsics::{Intrinsics, type_to_llvm};
+use inkwell::values::BasicValue;
 use inkwell::{
+    AddressSpace,
     attributes::{Attribute, AttributeLoc},
     builder::Builder,
     context::Context,
     types::{AnyType, BasicMetadataTypeEnum, BasicType, FunctionType, StructType},
-    values::{
-        BasicValue, BasicValueEnum, CallSiteValue, FloatValue, FunctionValue, IntValue,
-        PointerValue, VectorValue,
-    },
-    AddressSpace,
+    values::{BasicValueEnum, CallSiteValue, FloatValue, IntValue, VectorValue},
 };
 use wasmer_types::{CompileError, FunctionType as FuncSig, Type};
 use wasmer_vm::VMOffsets;
 
 use std::convert::TryInto;
 
-use super::{G0M0FunctionKind, LocalFunctionG0M0params};
+use super::G0M0FunctionKind;
 
 /// Implementation of the [`Abi`] trait for the AMD64 SystemV ABI.
 pub struct X86_64SystemV {}
 
 impl Abi for X86_64SystemV {
-    /// Given a function definition, retrieve the parameter that is the vmctx pointer.
-    fn get_vmctx_ptr_param<'ctx>(&self, func_value: &FunctionValue<'ctx>) -> PointerValue<'ctx> {
-        let param = func_value
-            .get_nth_param(u32::from(
-                func_value
-                    .get_enum_attribute(
-                        AttributeLoc::Param(0),
-                        Attribute::get_named_enum_kind_id("sret"),
-                    )
-                    .is_some(),
-            ))
-            .unwrap();
-        //param.set_name("vmctx");
-
-        param.into_pointer_value()
-    }
-
-    /// Given a function definition, retrieve the parameter that is the pointer to the first --
-    /// number 0 -- local global.
-    fn get_g0_ptr_param<'ctx>(&self, func_value: &FunctionValue<'ctx>) -> IntValue<'ctx> {
-        // g0 is always after the vmctx.
-        let vmctx_idx = u32::from(
-            func_value
-                .get_enum_attribute(
-                    AttributeLoc::Param(0),
-                    Attribute::get_named_enum_kind_id("sret"),
-                )
-                .is_some(),
-        );
-
-        let param = func_value.get_nth_param(vmctx_idx + 1).unwrap();
-        param.set_name("g0");
-
-        param.into_int_value()
-    }
-
-    /// Given a function definition, retrieve the parameter that is the pointer to the first --
-    /// number 0 -- local memory.
-    fn get_m0_ptr_param<'ctx>(&self, func_value: &FunctionValue<'ctx>) -> PointerValue<'ctx> {
-        // m0 is always after g0.
-        let vmctx_idx = u32::from(
-            func_value
-                .get_enum_attribute(
-                    AttributeLoc::Param(0),
-                    Attribute::get_named_enum_kind_id("sret"),
-                )
-                .is_some(),
-        );
-
-        let param = func_value.get_nth_param(vmctx_idx + 2).unwrap();
-        param.set_name("m0_base_ptr");
-
-        param.into_pointer_value()
-    }
-
     // Given a wasm function type, produce an llvm function declaration.
     fn func_type_to_llvm<'ctx>(
         &self,
@@ -134,10 +77,10 @@ impl Abi for X86_64SystemV {
             .results()
             .iter()
             .map(|ty| match ty {
-                Type::I32 | Type::F32 => 32,
+                Type::I32 | Type::F32 | Type::ExceptionRef => 32,
                 Type::I64 | Type::F64 => 64,
                 Type::V128 => 128,
-                Type::ExternRef | Type::FuncRef | Type::ExceptionRef => 64, /* pointer */
+                Type::ExternRef | Type::FuncRef => 64, /* pointer */
             })
             .collect::<Vec<i32>>();
 
@@ -203,25 +146,27 @@ impl Abi for X86_64SystemV {
                 ),
                 vmctx_attributes(0),
             ),
-            [32, 32, _] if sig.results()[0] == Type::F32 && sig.results()[1] == Type::F32 => (
-                context
-                    .struct_type(
-                        &[
-                            intrinsics.f32_ty.vec_type(2).as_basic_type_enum(),
-                            type_to_llvm(intrinsics, sig.results()[2])?,
-                        ],
-                        false,
-                    )
-                    .fn_type(
-                        param_types
-                            .map(|v| v.map(Into::into))
-                            .collect::<Result<Vec<BasicMetadataTypeEnum>, _>>()?
-                            .as_slice(),
-                        false,
-                    ),
-                vmctx_attributes(0),
-            ),
-            [32, 32, _] => (
+            [32, 32, 32 | 64] if sig.results()[0] == Type::F32 && sig.results()[1] == Type::F32 => {
+                (
+                    context
+                        .struct_type(
+                            &[
+                                intrinsics.f32_ty.vec_type(2).as_basic_type_enum(),
+                                type_to_llvm(intrinsics, sig.results()[2])?,
+                            ],
+                            false,
+                        )
+                        .fn_type(
+                            param_types
+                                .map(|v| v.map(Into::into))
+                                .collect::<Result<Vec<BasicMetadataTypeEnum>, _>>()?
+                                .as_slice(),
+                            false,
+                        ),
+                    vmctx_attributes(0),
+                )
+            }
+            [32, 32, 32 | 64] => (
                 context
                     .struct_type(
                         &[
@@ -337,52 +282,6 @@ impl Abi for X86_64SystemV {
         })
     }
 
-    // Marshall wasm stack values into function parameters.
-    fn args_to_call<'ctx>(
-        &self,
-        alloca_builder: &Builder<'ctx>,
-        func_sig: &FuncSig,
-        llvm_fn_ty: &FunctionType<'ctx>,
-        ctx_ptr: PointerValue<'ctx>,
-        values: &[BasicValueEnum<'ctx>],
-        intrinsics: &Intrinsics<'ctx>,
-        g0m0: LocalFunctionG0M0params<'ctx>,
-    ) -> Result<Vec<BasicValueEnum<'ctx>>, CompileError> {
-        // If it's an sret, allocate the return space.
-        let sret = if llvm_fn_ty.get_return_type().is_none() && func_sig.results().len() > 1 {
-            let llvm_params: Vec<_> = func_sig
-                .results()
-                .iter()
-                .map(|x| type_to_llvm(intrinsics, *x).unwrap())
-                .collect();
-            let llvm_params = llvm_fn_ty
-                .get_context()
-                .struct_type(llvm_params.as_slice(), false);
-            Some(err!(alloca_builder.build_alloca(llvm_params, "sret")))
-        } else {
-            None
-        };
-
-        let mut args = vec![ctx_ptr.as_basic_value_enum()];
-
-        if let Some((g0, m0)) = g0m0 {
-            args.push(g0.into());
-            args.push(m0.into());
-        }
-
-        let args = args.into_iter().chain(values.iter().copied());
-
-        let ret = if let Some(sret) = sret {
-            std::iter::once(sret.as_basic_value_enum())
-                .chain(args)
-                .collect()
-        } else {
-            args.collect()
-        };
-
-        Ok(ret)
-    }
-
     // Given a CallSite, extract the returned values and return them in a Vec.
     fn rets_from_call<'ctx>(
         &self,
@@ -420,7 +319,7 @@ impl Abi for X86_64SystemV {
         let casted =
             |value: BasicValueEnum<'ctx>, ty: Type| -> Result<BasicValueEnum<'ctx>, CompileError> {
                 match ty {
-                    Type::I32 => {
+                    Type::I32 | Type::ExceptionRef => {
                         assert!(
                             value.get_type() == intrinsics.i32_ty.as_basic_type_enum()
                                 || value.get_type() == intrinsics.f32_ty.as_basic_type_enum()
@@ -438,7 +337,7 @@ impl Abi for X86_64SystemV {
                 }
             };
 
-        if let Some(basic_value) = call_site.try_as_basic_value().left() {
+        if let Some(basic_value) = call_site.try_as_basic_value().basic() {
             if func_sig.results().len() > 1 {
                 if basic_value.get_type() == intrinsics.i64_ty.as_basic_type_enum() {
                     assert!(func_sig.results().len() == 2);
@@ -461,10 +360,10 @@ impl Abi for X86_64SystemV {
                     .results()
                     .iter()
                     .map(|ty| match ty {
-                        Type::I32 | Type::F32 => 32,
+                        Type::I32 | Type::F32 | Type::ExceptionRef => 32,
                         Type::I64 | Type::F64 => 64,
                         Type::V128 => 128,
-                        Type::ExternRef | Type::FuncRef | Type::ExceptionRef => 64, /* pointer */
+                        Type::ExternRef | Type::FuncRef => 64, /* pointer */
                     })
                     .collect::<Vec<i32>>();
 
@@ -548,12 +447,10 @@ impl Abi for X86_64SystemV {
             {
                 let sret_ty = call_site
                     .try_as_basic_value()
-                    .right()
-                    .unwrap()
+                    .unwrap_instruction()
                     .get_operand(0)
                     .unwrap()
-                    .left()
-                    .unwrap();
+                    .unwrap_value();
                 let sret = sret_ty.into_pointer_value();
                 // re-build the llvm-type struct holding the return values
                 let llvm_results: Vec<_> = func_sig
@@ -582,32 +479,6 @@ impl Abi for X86_64SystemV {
         }
     }
 
-    fn is_sret(&self, func_sig: &FuncSig) -> Result<bool, CompileError> {
-        let func_sig_returns_bitwidths = func_sig
-            .results()
-            .iter()
-            .map(|ty| match ty {
-                Type::I32 | Type::F32 => 32,
-                Type::I64 | Type::F64 => 64,
-                Type::V128 => 128,
-                Type::ExternRef | Type::FuncRef | Type::ExceptionRef => 64, /* pointer */
-            })
-            .collect::<Vec<i32>>();
-
-        Ok(!matches!(
-            func_sig_returns_bitwidths.as_slice(),
-            [] | [_]
-                | [32, 32]
-                | [32, 64]
-                | [64, 32]
-                | [64, 64]
-                | [32, 32, 32]
-                | [32, 32, 64]
-                | [64, 32, 32]
-                | [32, 32, 32, 32]
-        ))
-    }
-
     fn pack_values_for_register_return<'ctx>(
         &self,
         intrinsics: &Intrinsics<'ctx>,
@@ -621,7 +492,8 @@ impl Abi for X86_64SystemV {
                     && value.into_float_value().get_type() == intrinsics.f32_ty)
         };
         let is_64 = |value: BasicValueEnum| {
-            (value.is_int_value() && value.into_int_value().get_type() == intrinsics.i64_ty)
+            value.is_pointer_value()
+                || (value.is_int_value() && value.into_int_value().get_type() == intrinsics.i64_ty)
                 || (value.is_float_value()
                     && value.into_float_value().get_type() == intrinsics.f64_ty)
         };
@@ -637,9 +509,11 @@ impl Abi for X86_64SystemV {
             let high = err!(builder.build_int_z_extend(high, intrinsics.i64_ty, ""));
             let high =
                 err!(builder.build_left_shift(high, intrinsics.i64_ty.const_int(32, false), ""));
-            err_nt!(builder
-                .build_or(low, high, "")
-                .map(|v| v.as_basic_value_enum()))
+            err_nt!(
+                builder
+                    .build_or(low, high, "")
+                    .map(|v| v.as_basic_value_enum())
+            )
         };
 
         let pack_f32s = |first: BasicValueEnum<'ctx>,
@@ -655,9 +529,11 @@ impl Abi for X86_64SystemV {
                 intrinsics.i32_zero,
                 ""
             ));
-            err_nt!(builder
-                .build_insert_element(vec, second, intrinsics.i32_ty.const_int(1, false), "")
-                .map(|v| v.as_basic_value_enum()))
+            err_nt!(
+                builder
+                    .build_insert_element(vec, second, intrinsics.i32_ty.const_int(1, false), "")
+                    .map(|v| v.as_basic_value_enum())
+            )
         };
 
         let build_struct = |ty: StructType<'ctx>, values: &[BasicValueEnum<'ctx>]| {
