@@ -1,31 +1,38 @@
 use crate::config::LLVM;
 use crate::trampoline::FuncTrampoline;
 use crate::translator::FuncTranslator;
-use crate::CompiledKind;
+use inkwell::DLLStorageClass;
 use inkwell::context::Context;
 use inkwell::memory_buffer::MemoryBuffer;
 use inkwell::module::{Linkage, Module};
 use inkwell::targets::FileType;
-use inkwell::DLLStorageClass;
+use rayon::ThreadPoolBuilder;
 use rayon::iter::ParallelBridge;
 use rayon::prelude::{IntoParallelIterator, IntoParallelRefIterator, ParallelIterator};
-use rayon::ThreadPoolBuilder;
-use std::collections::{HashMap, HashSet};
-use std::sync::Arc;
+use std::{
+    borrow::Cow,
+    collections::{HashMap, HashSet},
+    sync::Arc,
+};
+use wasmer_compiler::misc::CompiledKind;
+use wasmer_compiler::progress::ProgressContext;
 use wasmer_compiler::types::function::{Compilation, UnwindInfo};
 use wasmer_compiler::types::module::CompileModuleInfo;
 use wasmer_compiler::types::relocation::RelocationKind;
 use wasmer_compiler::{
+    Compiler, FunctionBodyData, ModuleMiddleware, ModuleTranslationState,
     types::{
         relocation::RelocationTarget,
         section::{CustomSection, CustomSectionProtection, SectionBody, SectionIndex},
         symbols::{Symbol, SymbolRegistry},
     },
-    Compiler, FunctionBodyData, ModuleMiddleware, ModuleTranslationState,
 };
 use wasmer_types::entity::{EntityRef, PrimaryMap};
 use wasmer_types::target::Target;
-use wasmer_types::{CompileError, FunctionIndex, LocalFunctionIndex, ModuleInfo, SignatureIndex};
+use wasmer_types::{
+    CompilationProgressCallback, CompileError, FunctionIndex, LocalFunctionIndex, ModuleInfo,
+    SignatureIndex,
+};
 use wasmer_vm::LibCall;
 
 /// A compiler that compiles a WebAssembly module with LLVM, translating the Wasm to LLVM IR,
@@ -91,6 +98,11 @@ struct ModuleBasedSymbolRegistry {
 }
 
 impl ModuleBasedSymbolRegistry {
+    const PROBLEMATIC_PREFIXES: &[&'static str] = &[
+        ".L",    // .L is used for local symbols
+        "llvm.", // llvm. is used for LLVM's own intrinsics
+    ];
+
     fn new(wasm_module: Arc<ModuleInfo>) -> Self {
         let local_func_names = HashMap::from_iter(
             wasm_module
@@ -98,7 +110,7 @@ impl ModuleBasedSymbolRegistry {
                 .iter()
                 .map(|(f, v)| (wasm_module.local_func_index(*f), v))
                 .filter(|(f, _)| f.is_some())
-                .map(|(f, v)| (v.clone(), f.unwrap())),
+                .map(|(f, v)| (format!("{}_{}", v.clone(), f.unwrap().as_u32()), f.unwrap())),
         );
         Self {
             wasm_module,
@@ -106,6 +118,31 @@ impl ModuleBasedSymbolRegistry {
             short_names: ShortNames {},
         }
     }
+
+    // If the name starts with a problematic prefix, we prefix it with an underscore.
+    fn fixup_problematic_name(name: &str) -> Cow<'_, str> {
+        for prefix in Self::PROBLEMATIC_PREFIXES {
+            if name.starts_with(prefix) {
+                return format!("_{name}").into();
+            }
+        }
+        name.into()
+    }
+
+    // If the name starts with an underscore and the rest starts with a problematic prefix,
+    // remove the underscore to get back the original name. This is necessary to be able
+    // to match the name back to the original name in the wasm module.
+    fn unfixup_problematic_name(name: &str) -> &str {
+        if let Some(stripped_name) = name.strip_prefix('_') {
+            for prefix in Self::PROBLEMATIC_PREFIXES {
+                if stripped_name.starts_with(prefix) {
+                    return stripped_name;
+                }
+            }
+        }
+
+        name
+    }
 }
 
 impl SymbolRegistry for ModuleBasedSymbolRegistry {
@@ -115,13 +152,14 @@ impl SymbolRegistry for ModuleBasedSymbolRegistry {
                 .wasm_module
                 .function_names
                 .get(&self.wasm_module.func_index(index))
-                .cloned()
+                .map(|name| format!("{}_{}", Self::fixup_problematic_name(name), index.as_u32()))
                 .unwrap_or(self.short_names.symbol_to_name(symbol)),
             _ => self.short_names.symbol_to_name(symbol),
         }
     }
 
     fn name_to_symbol(&self, name: &str) -> Option<Symbol> {
+        let name = Self::unfixup_problematic_name(name);
         if let Some(idx) = self.local_func_names.get(name) {
             Some(Symbol::LocalFunction(*idx))
         } else {
@@ -150,7 +188,16 @@ impl LLVMCompiler {
         let merged_bitcode = function_body_inputs.into_iter().par_bridge().map_init(
             || {
                 let target_machine = self.config().target_machine(target);
-                FuncTranslator::new(target_machine, binary_format).unwrap()
+                let target_machine_no_opt = self.config().target_machine_with_opt(target, false);
+                let pointer_width = target.triple().pointer_width().unwrap().bytes();
+                FuncTranslator::new(
+                    target.triple().clone(),
+                    target_machine,
+                    Some(target_machine_no_opt),
+                    binary_format,
+                    pointer_width,
+                )
+                .unwrap()
             },
             |func_translator, (i, input)| {
                 let module = func_translator.translate_to_module(
@@ -162,6 +209,7 @@ impl LLVMCompiler {
                     &compile_info.memory_styles,
                     &compile_info.table_styles,
                     symbol_registry,
+                    target.triple(),
                 )?;
 
                 Ok(module.write_bitcode_to_memory().as_slice().to_vec())
@@ -171,7 +219,7 @@ impl LLVMCompiler {
         let trampolines_bitcode = compile_info.module.signatures.iter().par_bridge().map_init(
             || {
                 let target_machine = self.config().target_machine(target);
-                FuncTrampoline::new(target_machine, binary_format).unwrap()
+                FuncTrampoline::new(target_machine, target.triple().clone(), binary_format).unwrap()
             },
             |func_trampoline, (i, sig)| {
                 let name = symbol_registry.symbol_to_name(Symbol::FunctionCallTrampoline(i));
@@ -185,20 +233,26 @@ impl LLVMCompiler {
             },
         );
 
+        let module_hash = compile_info.module.hash_string();
         let dynamic_trampolines_bitcode =
             compile_info.module.functions.iter().par_bridge().map_init(
                 || {
                     let target_machine = self.config().target_machine(target);
                     (
-                        FuncTrampoline::new(target_machine, binary_format).unwrap(),
+                        FuncTrampoline::new(target_machine, target.triple().clone(), binary_format)
+                            .unwrap(),
                         &compile_info.module.signatures,
                     )
                 },
                 |(func_trampoline, signatures), (i, sig)| {
                     let sig = &signatures[*sig];
                     let name = symbol_registry.symbol_to_name(Symbol::DynamicFunctionTrampoline(i));
-                    let module =
-                        func_trampoline.dynamic_trampoline_to_module(sig, self.config(), &name)?;
+                    let module = func_trampoline.dynamic_trampoline_to_module(
+                        sig,
+                        self.config(),
+                        &name,
+                        &module_hash,
+                    )?;
                     Ok(module.write_bitcode_to_memory().as_slice().to_vec())
                 },
             );
@@ -250,7 +304,11 @@ impl LLVMCompiler {
             .write_to_memory_buffer(&merged_module, FileType::Object)
             .unwrap();
         if let Some(ref callbacks) = self.config.callbacks {
-            callbacks.obj_memory_buffer(&CompiledKind::Module, &memory_buffer);
+            callbacks.obj_memory_buffer(
+                &CompiledKind::Module,
+                &compile_info.module.hash_string(),
+                &memory_buffer,
+            );
         }
 
         tracing::trace!("Finished compling the module!");
@@ -320,6 +378,7 @@ impl Compiler for LLVMCompiler {
         compile_info: &CompileModuleInfo,
         module_translation: &ModuleTranslationState,
         function_body_inputs: PrimaryMap<LocalFunctionIndex, FunctionBodyData<'_>>,
+        progress_callback: Option<&CompilationProgressCallback>,
     ) -> Result<Compilation, CompileError> {
         //let data = Arc::new(Mutex::new(0));
 
@@ -328,6 +387,15 @@ impl Compiler for LLVMCompiler {
         let binary_format = self.config.target_binary_format(target);
 
         let module = &compile_info.module;
+        let module_hash = module.hash_string();
+        let total_functions = function_body_inputs.len() as u64;
+        let total_function_call_trampolines = module.signatures.len() as u64;
+        let total_dynamic_trampolines = module.num_imported_functions as u64;
+        let total_steps =
+            total_functions + total_function_call_trampolines + total_dynamic_trampolines;
+        let progress = progress_callback
+            .cloned()
+            .map(|cb| ProgressContext::new(cb, total_steps, "Compiling functions"));
 
         // TODO: merge constants in sections.
 
@@ -350,64 +418,56 @@ impl Compiler for LLVMCompiler {
 
         let symbol_registry = ModuleBasedSymbolRegistry::new(module.clone());
 
-        let functions = if self.config.num_threads.get() > 1 {
-            let pool = ThreadPoolBuilder::new()
-                .num_threads(self.config.num_threads.get())
-                .build()
-                .map_err(|e| CompileError::Resource(e.to_string()))?;
-            pool.install(|| {
-                function_body_inputs
-                    .iter()
-                    .collect::<Vec<(LocalFunctionIndex, &FunctionBodyData<'_>)>>()
-                    .par_iter()
-                    .map_init(
-                        || {
-                            let target_machine = self.config().target_machine(target);
-                            FuncTranslator::new(target_machine, binary_format).unwrap()
-                        },
-                        |func_translator, (i, input)| {
-                            // TODO: remove (to serialize)
-                            //let _data = data.lock().unwrap();
-
-                            func_translator.translate(
-                                module,
-                                module_translation,
-                                i,
-                                input,
-                                self.config(),
-                                memory_styles,
-                                table_styles,
-                                &symbol_registry,
-                            )
-                        },
-                    )
-                    .collect::<Result<Vec<_>, CompileError>>()
-            })?
-        } else {
-            let target_machine = self.config().target_machine(target);
-            let func_translator = FuncTranslator::new(target_machine, binary_format).unwrap();
-
+        let progress = progress.clone();
+        let pool = ThreadPoolBuilder::new()
+            .num_threads(self.config.num_threads.get())
+            .build()
+            .map_err(|e| CompileError::Resource(e.to_string()))?;
+        let functions = pool.install(|| {
             function_body_inputs
                 .iter()
                 .collect::<Vec<(LocalFunctionIndex, &FunctionBodyData<'_>)>>()
-                .into_iter()
-                .map(|(i, input)| {
-                    // TODO: remove (to serialize)
-                    //let _data = data.lock().unwrap();
+                .par_iter()
+                .map_init(
+                    || {
+                        let target_machine = self.config().target_machine_with_opt(target, true);
+                        let target_machine_no_opt =
+                            self.config().target_machine_with_opt(target, false);
+                        let pointer_width = target.triple().pointer_width().unwrap().bytes();
+                        FuncTranslator::new(
+                            target.triple().clone(),
+                            target_machine,
+                            Some(target_machine_no_opt),
+                            binary_format,
+                            pointer_width,
+                        )
+                        .unwrap()
+                    },
+                    |func_translator, (i, input)| {
+                        // TODO: remove (to serialize)
+                        //let _data = data.lock().unwrap();
 
-                    func_translator.translate(
-                        module,
-                        module_translation,
-                        &i,
-                        input,
-                        self.config(),
-                        memory_styles,
-                        table_styles,
-                        &symbol_registry,
-                    )
-                })
-                .collect::<Result<Vec<_>, CompileError>>()?
-        };
+                        let translated = func_translator.translate(
+                            module,
+                            module_translation,
+                            i,
+                            input,
+                            self.config(),
+                            memory_styles,
+                            table_styles,
+                            &symbol_registry,
+                            target.triple(),
+                        );
+
+                        if let Some(progress) = progress.as_ref() {
+                            progress.notify()?;
+                        }
+
+                        translated
+                    },
+                )
+                .collect::<Result<Vec<_>, CompileError>>()
+        })?;
 
         let functions = functions
             .into_iter()
@@ -437,6 +497,8 @@ impl Compiler for LLVMCompiler {
                             reloc.offset += offset;
                         }
                         eh_frame_section_bytes.extend_from_slice(custom_section.bytes.as_slice());
+                        // Terminate the eh_frame info with a zero-length CIE.
+                        eh_frame_section_bytes.extend_from_slice(&[0, 0, 0, 0]);
                         eh_frame_section_relocations.extend(custom_section.relocations);
                         // TODO: we do this to keep the count right, remove it.
                         module_custom_sections.push(CustomSection {
@@ -482,13 +544,75 @@ impl Compiler for LLVMCompiler {
             })
             .collect::<PrimaryMap<LocalFunctionIndex, _>>();
 
+        let progress = progress.clone();
+        let function_call_trampolines = pool.install(|| {
+            module
+                .signatures
+                .values()
+                .collect::<Vec<_>>()
+                .par_iter()
+                .map_init(
+                    || {
+                        let target_machine = self.config().target_machine(target);
+                        FuncTrampoline::new(target_machine, target.triple().clone(), binary_format)
+                            .unwrap()
+                    },
+                    |func_trampoline, sig| {
+                        let trampoline =
+                            func_trampoline.trampoline(sig, self.config(), "", compile_info);
+                        if let Some(progress) = progress.as_ref() {
+                            progress.notify()?;
+                        }
+                        trampoline
+                    },
+                )
+                .collect::<Vec<_>>()
+                .into_iter()
+                .collect::<Result<PrimaryMap<_, _>, CompileError>>()
+        })?;
+
+        // TODO: I removed the parallel processing of dynamic trampolines because we're passing
+        // the sections bytes and relocations directly into the trampoline generation function.
+        // We can move that logic out and re-enable parallel processing. Hopefully, there aren't
+        // enough dynamic trampolines to actually cause a noticeable performance degradation.
+        let dynamic_function_trampolines = {
+            let progress = progress.clone();
+            let target_machine = self.config().target_machine(target);
+            let func_trampoline =
+                FuncTrampoline::new(target_machine, target.triple().clone(), binary_format)
+                    .unwrap();
+            module
+                .imported_function_types()
+                .collect::<Vec<_>>()
+                .into_iter()
+                .enumerate()
+                .map(|(index, func_type)| {
+                    let trampoline = func_trampoline.dynamic_trampoline(
+                        &func_type,
+                        self.config(),
+                        "",
+                        index as u32,
+                        &mut module_custom_sections,
+                        &mut eh_frame_section_bytes,
+                        &mut eh_frame_section_relocations,
+                        &mut compact_unwind_section_bytes,
+                        &mut compact_unwind_section_relocations,
+                        &module_hash,
+                    )?;
+                    if let Some(progress) = progress.as_ref() {
+                        progress.notify()?;
+                    }
+                    Ok(trampoline)
+                })
+                .collect::<Vec<_>>()
+                .into_iter()
+                .collect::<Result<PrimaryMap<_, _>, CompileError>>()?
+        };
+
         let mut unwind_info = UnwindInfo::default();
 
         if !eh_frame_section_bytes.is_empty() {
             let eh_frame_idx = SectionIndex::from_u32(module_custom_sections.len() as u32);
-            // Do not terminate dwarf info with a zero-length CIE.
-            // Because more info will be added later
-            // in lib/object/src/module.rs emit_compilation
             module_custom_sections.push(CustomSection {
                 protection: CustomSectionProtection::Read,
                 alignment: None,
@@ -509,80 +633,6 @@ impl Compiler for LLVMCompiler {
             unwind_info.compact_unwind = Some(cu_index);
         }
 
-        let function_call_trampolines = if self.config.num_threads.get() > 1 {
-            let pool = ThreadPoolBuilder::new()
-                .num_threads(self.config.num_threads.get())
-                .build()
-                .map_err(|e| CompileError::Resource(e.to_string()))?;
-            pool.install(|| {
-                module
-                    .signatures
-                    .values()
-                    .collect::<Vec<_>>()
-                    .par_iter()
-                    .map_init(
-                        || {
-                            let target_machine = self.config().target_machine(target);
-                            FuncTrampoline::new(target_machine, binary_format).unwrap()
-                        },
-                        |func_trampoline, sig| {
-                            func_trampoline.trampoline(sig, self.config(), "", compile_info)
-                        },
-                    )
-                    .collect::<Vec<_>>()
-                    .into_iter()
-                    .collect::<Result<PrimaryMap<_, _>, CompileError>>()
-            })?
-        } else {
-            let target_machine = self.config().target_machine(target);
-            let func_trampoline = FuncTrampoline::new(target_machine, binary_format).unwrap();
-            module
-                .signatures
-                .values()
-                .collect::<Vec<_>>()
-                .into_iter()
-                .map(|sig| func_trampoline.trampoline(sig, self.config(), "", compile_info))
-                .collect::<Vec<_>>()
-                .into_iter()
-                .collect::<Result<PrimaryMap<_, _>, CompileError>>()?
-        };
-
-        let dynamic_function_trampolines = if self.config.num_threads.get() > 1 {
-            let pool = ThreadPoolBuilder::new()
-                .num_threads(self.config.num_threads.get())
-                .build()
-                .map_err(|e| CompileError::Resource(e.to_string()))?;
-            pool.install(|| {
-                module
-                    .imported_function_types()
-                    .collect::<Vec<_>>()
-                    .par_iter()
-                    .map_init(
-                        || {
-                            let target_machine = self.config().target_machine(target);
-                            FuncTrampoline::new(target_machine, binary_format).unwrap()
-                        },
-                        |func_trampoline, func_type| {
-                            func_trampoline.dynamic_trampoline(func_type, self.config(), "")
-                        },
-                    )
-                    .collect::<Vec<_>>()
-                    .into_iter()
-                    .collect::<Result<PrimaryMap<_, _>, CompileError>>()
-            })?
-        } else {
-            let target_machine = self.config().target_machine(target);
-            let func_trampoline = FuncTrampoline::new(target_machine, binary_format).unwrap();
-            module
-                .imported_function_types()
-                .collect::<Vec<_>>()
-                .into_iter()
-                .map(|func_type| func_trampoline.dynamic_trampoline(&func_type, self.config(), ""))
-                .collect::<Vec<_>>()
-                .into_iter()
-                .collect::<Result<PrimaryMap<_, _>, CompileError>>()?
-        };
-
         let mut got = wasmer_compiler::types::function::GOT::empty();
 
         if !got_targets.is_empty() {
