@@ -1,17 +1,21 @@
 use crate::compiler::LLVMCompiler;
+pub use inkwell::OptimizationLevel as LLVMOptLevel;
 use inkwell::targets::{
     CodeModel, InitializationConfig, RelocMode, Target as InkwellTarget, TargetMachine,
-    TargetTriple,
+    TargetMachineOptions, TargetTriple,
 };
-pub use inkwell::OptimizationLevel as LLVMOptLevel;
 use itertools::Itertools;
+use std::fs::File;
+use std::io::{self, Write};
+use std::path::PathBuf;
 use std::sync::Arc;
 use std::{fmt::Debug, num::NonZero};
 use target_lexicon::BinaryFormat;
+use wasmer_compiler::misc::{CompiledKind, function_kind_to_filename};
 use wasmer_compiler::{Compiler, CompilerConfig, Engine, EngineBuilder, ModuleMiddleware};
 use wasmer_types::{
+    Features,
     target::{Architecture, OperatingSystem, Target, Triple},
-    Features, FunctionType, LocalFunctionIndex,
 };
 
 /// The InkWell ModuleInfo type
@@ -20,24 +24,80 @@ pub type InkwellModule<'ctx> = inkwell::module::Module<'ctx>;
 /// The InkWell MemoryBuffer type
 pub type InkwellMemoryBuffer = inkwell::memory_buffer::MemoryBuffer;
 
-/// The compiled function kind, used for debugging in the `LLVMCallbacks`.
+/// Callbacks to the different LLVM compilation phases.
 #[derive(Debug, Clone)]
-pub enum CompiledKind {
-    // A locally-defined function in the Wasm file.
-    Local(LocalFunctionIndex),
-    // A function call trampoline for a given signature.
-    FunctionCallTrampoline(FunctionType),
-    // A dynamic function trampoline for a given signature.
-    DynamicFunctionTrampoline(FunctionType),
-    // An entire Wasm module.
-    Module,
+pub struct LLVMCallbacks {
+    debug_dir: PathBuf,
 }
 
-/// Callbacks to the different LLVM compilation phases.
-pub trait LLVMCallbacks: Debug + Send + Sync {
-    fn preopt_ir(&self, function: &CompiledKind, module: &InkwellModule);
-    fn postopt_ir(&self, function: &CompiledKind, module: &InkwellModule);
-    fn obj_memory_buffer(&self, function: &CompiledKind, memory_buffer: &InkwellMemoryBuffer);
+impl LLVMCallbacks {
+    pub fn new(debug_dir: PathBuf) -> Result<Self, io::Error> {
+        // Create the debug dir in case it doesn't exist
+        std::fs::create_dir_all(&debug_dir)?;
+        Ok(Self { debug_dir })
+    }
+
+    fn base_path(&self, module_hash: &Option<String>) -> PathBuf {
+        let mut path = self.debug_dir.clone();
+        if let Some(hash) = module_hash {
+            path.push(hash);
+        }
+        std::fs::create_dir_all(&path)
+            .unwrap_or_else(|_| panic!("cannot create debug directory: {}", path.display()));
+        path
+    }
+
+    pub fn preopt_ir(
+        &self,
+        kind: &CompiledKind,
+        module_hash: &Option<String>,
+        module: &InkwellModule,
+    ) {
+        let mut path = self.base_path(module_hash);
+        path.push(function_kind_to_filename(kind, ".preopt.ll"));
+        module
+            .print_to_file(&path)
+            .expect("Error while dumping pre optimized LLVM IR");
+    }
+    pub fn postopt_ir(
+        &self,
+        kind: &CompiledKind,
+        module_hash: &Option<String>,
+        module: &InkwellModule,
+    ) {
+        let mut path = self.base_path(module_hash);
+        path.push(function_kind_to_filename(kind, ".postopt.ll"));
+        module
+            .print_to_file(&path)
+            .expect("Error while dumping post optimized LLVM IR");
+    }
+    pub fn obj_memory_buffer(
+        &self,
+        kind: &CompiledKind,
+        module_hash: &Option<String>,
+        memory_buffer: &InkwellMemoryBuffer,
+    ) {
+        let mut path = self.base_path(module_hash);
+        path.push(function_kind_to_filename(kind, ".o"));
+        let mem_buf_slice = memory_buffer.as_slice();
+        let mut file =
+            File::create(path).expect("Error while creating debug object file from LLVM IR");
+        file.write_all(mem_buf_slice).unwrap();
+    }
+
+    pub fn asm_memory_buffer(
+        &self,
+        kind: &CompiledKind,
+        module_hash: &Option<String>,
+        asm_memory_buffer: &InkwellMemoryBuffer,
+    ) {
+        let mut path = self.base_path(module_hash);
+        path.push(function_kind_to_filename(kind, ".s"));
+        let mem_buf_slice = asm_memory_buffer.as_slice();
+        let mut file =
+            File::create(path).expect("Error while creating debug assembly file from LLVM IR");
+        file.write_all(mem_buf_slice).unwrap();
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -48,11 +108,12 @@ pub struct LLVM {
     pub(crate) enable_perfmap: bool,
     pub(crate) opt_level: LLVMOptLevel,
     is_pic: bool,
-    pub(crate) callbacks: Option<Arc<dyn LLVMCallbacks>>,
+    pub(crate) callbacks: Option<LLVMCallbacks>,
     /// The middleware chain.
     pub(crate) middlewares: Vec<Arc<dyn ModuleMiddleware>>,
     /// Number of threads to use when compiling a module.
     pub(crate) num_threads: NonZero<usize>,
+    pub(crate) verbose_asm: bool,
 }
 
 impl LLVM {
@@ -68,6 +129,7 @@ impl LLVM {
             callbacks: None,
             middlewares: vec![],
             enable_g0m0_opt: false,
+            verbose_asm: false,
             num_threads: std::thread::available_parallelism().unwrap_or(NonZero::new(1).unwrap()),
         }
     }
@@ -91,9 +153,14 @@ impl LLVM {
         self
     }
 
+    pub fn verbose_asm(&mut self, verbose_asm: bool) -> &mut Self {
+        self.verbose_asm = verbose_asm;
+        self
+    }
+
     /// Callbacks that will triggered in the different compilation
     /// phases in LLVM.
-    pub fn callbacks(&mut self, callbacks: Option<Arc<dyn LLVMCallbacks>>) -> &mut Self {
+    pub fn callbacks(&mut self, callbacks: Option<LLVMCallbacks>) -> &mut Self {
         self.callbacks = callbacks;
         self
     }
@@ -128,21 +195,22 @@ impl LLVM {
     }
 
     pub(crate) fn target_operating_system(&self, target: &Target) -> OperatingSystem {
-        if target.triple().operating_system == OperatingSystem::Darwin && !self.is_pic {
-            // LLVM detects static relocation + darwin + 64-bit and
-            // force-enables PIC because MachO doesn't support that
-            // combination. They don't check whether they're targeting
-            // MachO, they check whether the OS is set to Darwin.
-            //
-            // Since both linux and darwin use SysV ABI, this should work.
-            //  but not in the case of Aarch64, there the ABI is slightly different
-            #[allow(clippy::match_single_binding)]
-            match target.triple().architecture {
-                Architecture::Aarch64(_) => OperatingSystem::Darwin,
-                _ => OperatingSystem::Linux,
+        match target.triple().operating_system {
+            OperatingSystem::Darwin(deployment) if !self.is_pic => {
+                // LLVM detects static relocation + darwin + 64-bit and
+                // force-enables PIC because MachO doesn't support that
+                // combination. They don't check whether they're targeting
+                // MachO, they check whether the OS is set to Darwin.
+                //
+                // Since both linux and darwin use SysV ABI, this should work.
+                //  but not in the case of Aarch64, there the ABI is slightly different
+                #[allow(clippy::match_single_binding)]
+                match target.triple().architecture {
+                    Architecture::Aarch64(_) => OperatingSystem::Darwin(deployment),
+                    _ => OperatingSystem::Linux,
+                }
             }
-        } else {
-            target.triple().operating_system
+            other => other,
         }
     }
 
@@ -151,7 +219,7 @@ impl LLVM {
             target.triple().binary_format
         } else {
             match self.target_operating_system(target) {
-                OperatingSystem::Darwin => target_lexicon::BinaryFormat::Macho,
+                OperatingSystem::Darwin(_) => target_lexicon::BinaryFormat::Macho,
                 _ => target_lexicon::BinaryFormat::Elf,
             }
         }
@@ -183,6 +251,14 @@ impl LLVM {
 
     /// Generates the target machine for the current target
     pub fn target_machine(&self, target: &Target) -> TargetMachine {
+        self.target_machine_with_opt(target, true)
+    }
+
+    pub(crate) fn target_machine_with_opt(
+        &self,
+        target: &Target,
+        enable_optimization: bool,
+    ) -> TargetMachine {
         let triple = target.triple();
         let cpu_features = &target.cpu_features();
 
@@ -205,14 +281,16 @@ impl LLVM {
                 info: true,
                 machine_code: true,
             }),
-            Architecture::Riscv64(_) => InkwellTarget::initialize_riscv(&InitializationConfig {
-                asm_parser: true,
-                asm_printer: true,
-                base: true,
-                disassembler: true,
-                info: true,
-                machine_code: true,
-            }),
+            Architecture::Riscv64(_) | Architecture::Riscv32(_) => {
+                InkwellTarget::initialize_riscv(&InitializationConfig {
+                    asm_parser: true,
+                    asm_printer: true,
+                    base: true,
+                    disassembler: true,
+                    info: true,
+                    machine_code: true,
+                })
+            }
             Architecture::LoongArch64 => {
                 InkwellTarget::initialize_loongarch(&InitializationConfig {
                     asm_parser: true,
@@ -223,14 +301,6 @@ impl LLVM {
                     machine_code: true,
                 })
             }
-            // Architecture::Arm(_) => InkwellTarget::initialize_arm(&InitializationConfig {
-            //     asm_parser: true,
-            //     asm_printer: true,
-            //     base: true,
-            //     disassembler: true,
-            //     info: true,
-            //     machine_code: true,
-            // }),
             _ => unimplemented!("target {} not yet supported in Wasmer", triple),
         }
 
@@ -244,61 +314,39 @@ impl LLVM {
 
         let target_triple = self.target_triple(target);
         let llvm_target = InkwellTarget::from_triple(&target_triple).unwrap();
-        let llvm_target_machine = llvm_target
-            .create_target_machine(
-                &target_triple,
-                match triple.architecture {
-                    Architecture::Riscv64(_) => "generic-rv64",
-                    Architecture::LoongArch64 => "generic-la64",
-                    _ => "generic",
-                },
-                match triple.architecture {
-                    Architecture::Riscv64(_) => "+m,+a,+c,+d,+f",
-                    Architecture::LoongArch64 => "+f,+d",
-                    _ => &llvm_cpu_features,
-                },
-                self.opt_level,
-                self.reloc_mode(self.target_binary_format(target)),
-                match triple.architecture {
-                    Architecture::LoongArch64 | Architecture::Riscv64(_) => CodeModel::Medium,
-                    _ => self.code_model(self.target_binary_format(target)),
-                },
-            )
-            .unwrap();
-
-        if let Architecture::Riscv64(_) = triple.architecture {
-            // TODO: totally non-portable way to change ABI
-            unsafe {
-                // This structure mimic the internal structure from inkwell
-                // that is defined as
-                //  #[derive(Debug)]
-                //  pub struct TargetMachine {
-                //    pub(crate) target_machine: LLVMTargetMachineRef,
-                //  }
-                pub struct MyTargetMachine {
-                    pub target_machine: *const u8,
+        let mut llvm_target_machine_options = TargetMachineOptions::new()
+            .set_cpu(match triple.architecture {
+                Architecture::Riscv64(_) => "generic-rv64",
+                Architecture::Riscv32(_) => "generic-rv32",
+                Architecture::LoongArch64 => "generic-la64",
+                _ => "generic",
+            })
+            .set_features(match triple.architecture {
+                Architecture::Riscv64(_) => "+m,+a,+c,+d,+f",
+                Architecture::Riscv32(_) => "+m,+a,+c,+d,+f",
+                Architecture::LoongArch64 => "+f,+d",
+                _ => &llvm_cpu_features,
+            })
+            .set_level(if enable_optimization {
+                self.opt_level
+            } else {
+                LLVMOptLevel::None
+            })
+            .set_reloc_mode(self.reloc_mode(self.target_binary_format(target)))
+            .set_code_model(match triple.architecture {
+                Architecture::LoongArch64 | Architecture::Riscv64(_) | Architecture::Riscv32(_) => {
+                    CodeModel::Medium
                 }
-                // It is use to live patch the create LLVMTargetMachine
-                // to hard change the ABI and force "-mabi=lp64d" ABI
-                // instead of the default that don't use float registers
-                // because there is no current way to do this change
-
-                let my_target_machine: MyTargetMachine = std::mem::transmute(llvm_target_machine);
-
-                *((my_target_machine.target_machine as *mut u8).offset(0x410) as *mut u64) = 5;
-                std::ptr::copy_nonoverlapping(
-                    "lp64d\0".as_ptr(),
-                    (my_target_machine.target_machine as *mut u8).offset(0x418),
-                    6,
-                );
-
-                std::mem::transmute::<MyTargetMachine, inkwell::targets::TargetMachine>(
-                    my_target_machine,
-                )
-            }
-        } else {
-            llvm_target_machine
+                _ => self.code_model(self.target_binary_format(target)),
+            });
+        if let Architecture::Riscv64(_) = triple.architecture {
+            llvm_target_machine_options = llvm_target_machine_options.set_abi("lp64d");
         }
+        let target_machine = llvm_target
+            .create_target_machine_from_options(&target_triple, llvm_target_machine_options)
+            .unwrap();
+        target_machine.set_asm_verbosity(self.verbose_asm);
+        target_machine
     }
 }
 
