@@ -9,7 +9,7 @@
 //! runtime performance.
 
 #![allow(clippy::unnecessary_cast)]
-#![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 mod address_map;
 mod arm64_decl;
@@ -20,15 +20,18 @@ mod config;
 #[cfg(feature = "unwind")]
 mod dwarf;
 mod emitter_arm64;
+mod emitter_riscv;
 mod emitter_x64;
 mod location;
 mod machine;
 mod machine_arm64;
+mod machine_riscv;
 mod machine_x64;
+mod riscv_decl;
 mod unwind;
 #[cfg(feature = "unwind")]
 mod unwind_winx64;
 mod x64_decl;
 
 pub use crate::compiler::SinglepassCompiler;
-pub use crate::config::Singlepass;
+pub use {crate::config::Singlepass, crate::config::SinglepassCallbacks};
