@@ -4,22 +4,22 @@
 #[cfg(feature = "compiler")]
 use super::trampoline::{libcall_trampoline_len, make_libcall_trampolines};
 
-#[cfg(feature = "compiler")]
-use crate::{
-    serialize::SerializableCompilation, EngineInner, ModuleEnvironment, ModuleMiddlewareChain,
-};
 use crate::{
+    ArtifactCreate, Features,
     serialize::{
         ArchivedSerializableCompilation, ArchivedSerializableModule, MetadataHeader,
         SerializableModule,
     },
     types::{
-        function::{CompiledFunctionFrameInfo, FunctionBody, UnwindInfo, GOT},
+        function::{CompiledFunctionFrameInfo, FunctionBody, GOT, UnwindInfo},
         module::CompileModuleInfo,
         relocation::Relocation,
         section::{CustomSection, SectionIndex},
     },
-    ArtifactCreate, Features,
+};
+#[cfg(feature = "compiler")]
+use crate::{
+    EngineInner, ModuleEnvironment, ModuleMiddlewareChain, serialize::SerializableCompilation,
 };
 #[cfg(feature = "compiler")]
 use wasmer_types::target::Target;
@@ -31,9 +31,9 @@ use self_cell::self_cell;
 use shared_buffer::OwnedBuffer;
 use std::sync::Arc;
 use wasmer_types::{
+    CompilationProgressCallback, DeserializeError,
     entity::{ArchivedPrimaryMap, PrimaryMap},
     target::CpuFeature,
-    DeserializeError,
 };
 
 // Not every compiler backend uses these.
@@ -63,7 +63,7 @@ impl ArtifactBuild {
         target: &Target,
         memory_styles: PrimaryMap<MemoryIndex, MemoryStyle>,
         table_styles: PrimaryMap<TableIndex, TableStyle>,
-        hash_algorithm: Option<HashAlgorithm>,
+        progress_callback: Option<&CompilationProgressCallback>,
     ) -> Result<Self, CompileError> {
         let environ = ModuleEnvironment::new();
         let features = inner_engine.features().clone();
@@ -78,15 +78,7 @@ impl ArtifactBuild {
         middlewares
             .apply_on_module_info(&mut module)
             .map_err(|err| CompileError::MiddlewareError(err.to_string()))?;
-
-        if let Some(hash_algorithm) = hash_algorithm {
-            let hash = match hash_algorithm {
-                HashAlgorithm::Sha256 => ModuleHash::sha256(data),
-                HashAlgorithm::XXHash => ModuleHash::xxhash(data),
-            };
-
-            module.hash = Some(hash);
-        }
+        module.hash = Some(ModuleHash::new(data));
 
         let compile_info = CompileModuleInfo {
             module: Arc::new(module),
@@ -104,6 +96,7 @@ impl ArtifactBuild {
             // `module_translation_state`.
             translation.module_translation_state.as_ref().unwrap(),
             translation.function_body_inputs,
+            progress_callback,
         )?;
 
         let data_initializers = translation
@@ -226,7 +219,7 @@ impl<'a> ArtifactCreate<'a> for ArtifactBuild {
     }
 
     fn set_module_info_name(&mut self, name: String) -> bool {
-        Arc::get_mut(&mut self.serializable.compile_info.module).map_or(false, |module_info| {
+        Arc::get_mut(&mut self.serializable.compile_info.module).is_some_and(|module_info| {
             module_info.name = Some(name.to_string());
             true
         })
@@ -462,7 +455,7 @@ impl<'a> ArtifactCreate<'a> for ArtifactBuildFromArchive {
     }
 
     fn set_module_info_name(&mut self, name: String) -> bool {
-        Arc::get_mut(&mut self.compile_info.module).map_or(false, |module_info| {
+        Arc::get_mut(&mut self.compile_info.module).is_some_and(|module_info| {
             module_info.name = Some(name.to_string());
             true
         })
