@@ -3,17 +3,17 @@
 
 use crate::types::{module::CompileModuleInfo, symbols::SymbolRegistry};
 use crate::{
+    FunctionBodyData, ModuleTranslationState,
     lib::std::{boxed::Box, sync::Arc},
     translator::ModuleMiddleware,
     types::function::Compilation,
-    FunctionBodyData, ModuleTranslationState,
 };
 use enumset::EnumSet;
 use wasmer_types::{
+    CompilationProgressCallback, Features, LocalFunctionIndex,
     entity::PrimaryMap,
     error::CompileError,
     target::{CpuFeature, Target, UserCompilerOptimizations},
-    Features, LocalFunctionIndex,
 };
 #[cfg(feature = "translator")]
 use wasmparser::{Validator, WasmFeatures};
@@ -134,8 +134,8 @@ pub trait Compiler: Send + std::fmt::Debug {
         wasm_features.set(WasmFeatures::FUNCTION_REFERENCES, false);
         wasm_features.set(WasmFeatures::MEMORY_CONTROL, false);
         wasm_features.set(WasmFeatures::GC, false);
-        wasm_features.set(WasmFeatures::COMPONENT_MODEL_VALUES, false);
-        wasm_features.set(WasmFeatures::COMPONENT_MODEL_NESTED_NAMES, false);
+        wasm_features.set(WasmFeatures::CM_VALUES, false);
+        wasm_features.set(WasmFeatures::CM_NESTED_NAMES, false);
 
         let mut validator = Validator::new_with_features(wasm_features);
         validator
@@ -154,6 +154,7 @@ pub trait Compiler: Send + std::fmt::Debug {
         module_translation: &ModuleTranslationState,
         // The list of function bodies
         function_body_inputs: PrimaryMap<LocalFunctionIndex, FunctionBodyData<'_>>,
+        progress_callback: Option<&CompilationProgressCallback>,
     ) -> Result<Compilation, CompileError>;
 
     /// Compiles a module into a native object file.
