@@ -3,10 +3,8 @@ mod cu_entry;
 
 use core::ops::Range;
 pub(crate) use cu_entry::CompactUnwindEntry;
-use std::{
-    collections::HashMap,
-    sync::{LazyLock, Mutex},
-};
+use rangemap::RangeMap;
+use std::sync::{LazyLock, Mutex};
 use wasmer_types::CompileError;
 
 type CUResult<T> = Result<T, CompileError>;
@@ -51,7 +49,7 @@ pub struct UnwDynamicUnwindSections {
 type UnwFindDynamicUnwindSections =
     unsafe extern "C" fn(addr: usize, info: *mut UnwDynamicUnwindSections) -> u32;
 
-extern "C" {
+unsafe extern "C" {
     // Register a dynamic unwind-info lookup callback. If libunwind does not find
     // unwind info for a given frame in the executable program or normal dynamic
     // shared objects then it will call all registered dynamic lookup functions
@@ -77,7 +75,6 @@ extern "C" {
 
     pub fn __unw_add_dynamic_eh_frame_section(eh_frame_start: usize);
     pub fn __unw_remove_dynamic_eh_frame_section(eh_frame_start: usize);
-
 }
 
 trait ToBytes {
@@ -107,11 +104,10 @@ pub struct CompactUnwindManager {
     maybe_eh_personality_addr_in_got: Option<usize>,
 }
 
-static mut UNWIND_INFO: LazyLock<Mutex<Option<UnwindInfo>>> = LazyLock::new(|| Mutex::new(None));
+static UNWIND_INFO: LazyLock<Mutex<RangeMap<usize, UnwindInfoEntry>>> =
+    LazyLock::new(|| Mutex::new(RangeMap::new()));
 
-type UnwindInfo = HashMap<Range<usize>, UnwindInfoEntry>;
-
-#[derive(Debug)]
+#[derive(Debug, Clone, PartialEq)]
 struct UnwindInfoEntry {
     dso_base: usize,
     section_ptr: usize,
@@ -122,41 +118,31 @@ unsafe extern "C" fn find_dynamic_unwind_sections(
     addr: usize,
     info: *mut UnwDynamicUnwindSections,
 ) -> u32 {
-    unsafe {
-        if let Ok(uw_info) = UNWIND_INFO.try_lock() {
-            if uw_info.is_none() {
-                (*info).compact_unwind_section = 0;
-                (*info).compact_unwind_section_length = 0;
-                (*info).dwarf_section = 0;
-                (*info).dwarf_section_length = 0;
-                (*info).dso_base = 0;
+    let Some(info) = (unsafe { info.as_mut() }) else {
+        return 0;
+    };
 
-                return 0;
-            }
+    if let Some(entry) = UNWIND_INFO
+        .lock()
+        .expect("cannot lock UNWIND_INFO")
+        .get(&addr)
+    {
+        info.compact_unwind_section = entry.section_ptr as u64;
+        info.compact_unwind_section_length = entry.section_len as u64;
+        info.dwarf_section = 0;
+        info.dwarf_section_length = 0;
+        info.dso_base = entry.dso_base as u64;
 
-            let uw_info = uw_info.as_ref().unwrap();
+        1
+    } else {
+        info.compact_unwind_section = 0;
+        info.compact_unwind_section_length = 0;
+        info.dwarf_section = 0;
+        info.dwarf_section_length = 0;
+        info.dso_base = 0;
 
-            for (range, u) in uw_info.iter() {
-                if range.contains(&addr) {
-                    (*info).compact_unwind_section = u.section_ptr as _;
-                    (*info).compact_unwind_section_length = u.section_len as _;
-                    (*info).dwarf_section = 0;
-                    (*info).dwarf_section_length = 0;
-                    (*info).dso_base = u.dso_base as u64;
-
-                    return 1;
-                }
-            }
-        }
+        0
     }
-
-    (*info).compact_unwind_section = 0;
-    (*info).compact_unwind_section_length = 0;
-    (*info).dwarf_section = 0;
-    (*info).dwarf_section_length = 0;
-    (*info).dso_base = 0;
-
-    0
 }
 
 impl CompactUnwindManager {
@@ -187,10 +173,12 @@ impl CompactUnwindManager {
         }
         let mut offset = 0;
         while offset < len {
-            let entry = CompactUnwindEntry::from_ptr_and_len(
-                compact_unwind_section_ptr.wrapping_add(offset),
-                len,
-            );
+            let entry = unsafe {
+                CompactUnwindEntry::from_ptr_and_len(
+                    compact_unwind_section_ptr.wrapping_add(offset),
+                    len,
+                )
+            };
             self.compact_unwind_entries.push(entry);
             offset += size_of::<CompactUnwindEntry>();
         }
@@ -234,9 +222,7 @@ impl CompactUnwindManager {
         }
         self.dso_base = info.dli_fbase as usize;
 
-        unsafe {
-            self.write_unwind_info()?;
-        }
+        self.write_unwind_info()?;
 
         let ranges: Vec<Range<usize>> = self
             .compact_unwind_entries
@@ -249,39 +235,16 @@ impl CompactUnwindManager {
         let section_len = data.len();
         let dso_base = self.dso_base;
 
-        unsafe {
-            let mut uw_info = UNWIND_INFO.lock().map_err(|_| {
-                CompileError::Codegen("Failed to acquire lock for UnwindInfo!".into())
-            })?;
-
-            match uw_info.as_mut() {
-                Some(r) => {
-                    for range in ranges {
-                        r.insert(
-                            range,
-                            UnwindInfoEntry {
-                                dso_base,
-                                section_ptr,
-                                section_len,
-                            },
-                        );
-                    }
-                }
-                None => {
-                    let mut map = HashMap::new();
-                    for range in ranges {
-                        map.insert(
-                            range,
-                            UnwindInfoEntry {
-                                dso_base,
-                                section_ptr,
-                                section_len,
-                            },
-                        );
-                    }
-                    _ = uw_info.insert(map);
-                }
-            }
+        let mut uw_info = UNWIND_INFO.lock().expect("cannot lock UNWIND_INFO");
+        for range in ranges {
+            (*uw_info).insert(
+                range,
+                UnwindInfoEntry {
+                    dso_base,
+                    section_ptr,
+                    section_len,
+                },
+            );
         }
 
         Ok(())
@@ -309,9 +272,10 @@ impl CompactUnwindManager {
             }
         }
 
-        self.num_second_level_pages =
-            (self.compact_unwind_entries.len() + Self::NUM_RECORDS_PER_SECOND_LEVEL_PAGE - 1)
-                / Self::NUM_RECORDS_PER_SECOND_LEVEL_PAGE;
+        self.num_second_level_pages = self
+            .compact_unwind_entries
+            .len()
+            .div_ceil(Self::NUM_RECORDS_PER_SECOND_LEVEL_PAGE);
 
         self.compact_unwind_entries
             .sort_by(|l, r| l.function_addr.cmp(&r.function_addr));
@@ -328,7 +292,7 @@ impl CompactUnwindManager {
         Ok(())
     }
 
-    unsafe fn write_unwind_info(&mut self) -> CUResult<()> {
+    fn write_unwind_info(&mut self) -> CUResult<()> {
         self.write_header()?;
         self.write_personalities()?;
         self.write_indices()?;
@@ -357,9 +321,10 @@ impl CompactUnwindManager {
             }
         }
 
-        self.num_second_level_pages =
-            (self.compact_unwind_entries.len() + Self::NUM_RECORDS_PER_SECOND_LEVEL_PAGE - 1)
-                / Self::NUM_RECORDS_PER_SECOND_LEVEL_PAGE;
+        self.num_second_level_pages = self
+            .compact_unwind_entries
+            .len()
+            .div_ceil(Self::NUM_RECORDS_PER_SECOND_LEVEL_PAGE);
     }
 
     #[inline(always)]
@@ -381,7 +346,7 @@ impl CompactUnwindManager {
         Ok(())
     }
 
-    unsafe fn write_header(&mut self) -> CUResult<()> {
+    fn write_header(&mut self) -> CUResult<()> {
         //#[derive(Debug, Default)]
         //#[repr(C)]
         //#[allow(non_snake_case, non_camel_case_types)]
@@ -550,7 +515,15 @@ impl CompactUnwindManager {
 
     pub(crate) fn deregister(&self) {
         if self.dso_base != 0 {
-            unsafe { __unw_remove_find_dynamic_unwind_sections(find_dynamic_unwind_sections) };
+            let ranges: Vec<Range<usize>> = self
+                .compact_unwind_entries
+                .iter()
+                .map(|v| v.function_addr..v.function_addr + (v.length as usize))
+                .collect();
+            let mut uw_info = UNWIND_INFO.lock().expect("cannot lock UNWIND_INFO");
+            for range in ranges {
+                (*uw_info).remove(range);
+            }
         }
     }
 
