@@ -18,7 +18,19 @@
     clippy::unicode_not_nfc,
     clippy::use_self
 )]
-#![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
+
+macro_rules! cfg_std_or_core {
+    ($($item:item)*) => {
+        $(
+            #[cfg(any(
+                feature = "std",
+                feature = "core",
+            ))]
+            $item
+        )*
+    };
+}
 
 #[cfg(all(feature = "std", feature = "core"))]
 compile_error!(
@@ -32,6 +44,7 @@ compile_error!("Both the `std` and `core` features are disabled. Please enable o
 extern crate alloc;
 
 #[allow(unused_imports)]
+#[cfg(any(feature = "std", feature = "core"))]
 mod lib {
     #[cfg(feature = "core")]
     pub mod std {
@@ -46,34 +59,43 @@ mod lib {
     }
 }
 
-mod engine;
-mod traits;
+cfg_std_or_core! {
+    mod engine;
+    mod traits;
 
-pub mod object;
-pub mod serialize;
-pub mod types;
+    pub mod misc;
+    pub mod object;
+    pub mod progress;
+    pub mod serialize;
+    pub mod types;
 
-pub use crate::engine::*;
-pub use crate::traits::*;
+    pub use crate::engine::*;
+    pub use crate::traits::*;
 
-mod artifact_builders;
+    mod artifact_builders;
 
-pub use self::artifact_builders::*;
+    pub use self::artifact_builders::*;
+}
 
 #[cfg(feature = "compiler")]
 mod compiler;
 #[cfg(feature = "compiler")]
 pub use crate::compiler::{Compiler, CompilerConfig};
 
+#[cfg(feature = "compiler")]
+mod constants;
+#[cfg(feature = "compiler")]
+pub use crate::constants::*;
+
 #[cfg(feature = "translator")]
 #[macro_use]
 mod translator;
 #[cfg(feature = "translator")]
 pub use crate::translator::{
-    from_binaryreadererror_wasmerror, translate_module, wpheaptype_to_type, wptype_to_type,
     FunctionBinaryReader, FunctionBodyData, FunctionMiddleware, MiddlewareBinaryReader,
     MiddlewareReaderState, ModuleEnvironment, ModuleMiddleware, ModuleMiddlewareChain,
-    ModuleTranslationState,
+    ModuleTranslationState, from_binaryreadererror_wasmerror, translate_module, wpheaptype_to_type,
+    wptype_to_type,
 };
 
 pub use wasmer_types::{Addend, CodeOffset, Features};
