@@ -14,11 +14,11 @@
 use super::environ::ModuleEnvironment;
 use super::error::from_binaryreadererror_wasmerror;
 use super::state::ModuleTranslationState;
-use crate::wasm_unsupported;
 use std::boxed::Box;
+use std::collections::{HashMap, HashSet};
 use std::vec::Vec;
-use wasmer_types::entity::packed_option::ReservedValue;
 use wasmer_types::entity::EntityRef;
+use wasmer_types::entity::packed_option::ReservedValue;
 use wasmer_types::{
     DataIndex, ElemIndex, FunctionIndex, FunctionType, GlobalIndex, GlobalInit, GlobalType,
     MemoryIndex, MemoryType, Pages, SignatureIndex, TableIndex, TableType, TagIndex, Type, V128,
@@ -27,9 +27,9 @@ use wasmer_types::{WasmError, WasmResult};
 use wasmparser::{
     self, Data, DataKind, DataSectionReader, Element, ElementItems, ElementKind,
     ElementSectionReader, Export, ExportSectionReader, ExternalKind, FunctionSectionReader,
-    GlobalSectionReader, GlobalType as WPGlobalType, ImportSectionReader, MemorySectionReader,
-    MemoryType as WPMemoryType, NameSectionReader, Operator, TableSectionReader,
-    TagType as WPTagType, TypeRef, TypeSectionReader,
+    GlobalSectionReader, GlobalType as WPGlobalType, ImportSectionReader, Imports,
+    MemorySectionReader, MemoryType as WPMemoryType, NameSectionReader, Operator,
+    TableSectionReader, TagType as WPTagType, TypeRef, TypeSectionReader,
 };
 
 /// Helper function translating wasmparser types to Wasm Type.
@@ -118,6 +118,11 @@ pub fn parse_import_section<'data>(
 
     for entry in imports {
         let import = entry.map_err(from_binaryreadererror_wasmerror)?;
+        let Imports::Single(_index, import) = import else {
+            return Err(WasmError::Generic(
+                "non-Single section Imports not implemented yet".to_string(),
+            ));
+        };
         let module_name = import.module;
         let field_name = import.name;
 
@@ -129,6 +134,11 @@ pub fn parse_import_section<'data>(
                     field_name,
                 )?;
             }
+            TypeRef::FuncExact(_) => {
+                return Err(WasmError::Generic(
+                    "custom-descriptors not implemented yet".to_string(),
+                ));
+            }
             TypeRef::Tag(t) => {
                 environ.declare_tag_import(t, module_name, field_name)?;
             }
@@ -280,9 +290,9 @@ pub fn parse_global_section(
                 },
             init_expr,
         } = entry.map_err(from_binaryreadererror_wasmerror)?;
-        let mut init_expr_reader = init_expr.get_binary_reader();
+        let mut init_expr_reader = init_expr.get_operators_reader();
         let initializer = match init_expr_reader
-            .read_operator()
+            .read()
             .map_err(from_binaryreadererror_wasmerror)?
         {
             Operator::I32Const { value } => GlobalInit::I32Const(value),
@@ -345,6 +355,7 @@ pub fn parse_export_section<'data>(
                 environ.declare_global_export(GlobalIndex::new(index), field)?
             }
             ExternalKind::Tag => environ.declare_tag_export(TagIndex::new(index), field)?,
+            ExternalKind::FuncExact => unimplemented!("custom-descriptors not implemented yet"),
         }
     }
 
@@ -381,8 +392,8 @@ fn read_elems(items: &ElementItems) -> WasmResult<Box<[FunctionIndex]>> {
                 let expr = res.map_err(from_binaryreadererror_wasmerror)?;
 
                 let op = expr
-                    .get_binary_reader()
-                    .read_operator()
+                    .get_operators_reader()
+                    .read()
                     .map_err(from_binaryreadererror_wasmerror)?;
                 match op {
                     Operator::RefNull { .. } => out.push(FunctionIndex::reserved_value()),
@@ -424,9 +435,9 @@ pub fn parse_element_section(
             } => {
                 let table_index = TableIndex::from_u32(table_index.unwrap_or(0));
 
-                let mut init_expr_reader = offset_expr.get_binary_reader();
+                let mut init_expr_reader = offset_expr.get_operators_reader();
                 let (base, offset) = match init_expr_reader
-                    .read_operator()
+                    .read()
                     .map_err(from_binaryreadererror_wasmerror)?
                 {
                     Operator::I32Const { value } => (None, value as u32 as usize),
@@ -470,9 +481,9 @@ pub fn parse_data_section<'data>(
                 memory_index,
                 offset_expr,
             } => {
-                let mut init_expr_reader = offset_expr.get_binary_reader();
+                let mut init_expr_reader = offset_expr.get_operators_reader();
                 let (base, offset) = match init_expr_reader
-                    .read_operator()
+                    .read()
                     .map_err(from_binaryreadererror_wasmerror)?
                 {
                     Operator::I32Const { value } => (None, value as u32 as usize),
@@ -483,7 +494,7 @@ pub fn parse_data_section<'data>(
                         return Err(wasm_unsupported!(
                             "unsupported init expr in data section: {:?}",
                             s
-                        ))
+                        ));
                     }
                 };
                 environ.declare_data_initialization(
@@ -508,6 +519,9 @@ pub fn parse_name_section<'data>(
     names: NameSectionReader<'data>,
     environ: &mut ModuleEnvironment<'data>,
 ) -> WasmResult<()> {
+    let mut functions = HashMap::new();
+    let mut names_set = HashSet::new();
+
     for res in names {
         let subsection = if let Ok(subsection) = res {
             subsection
@@ -517,12 +531,21 @@ pub fn parse_name_section<'data>(
         };
         match subsection {
             wasmparser::Name::Function(function_subsection) => {
+                let mut unique_name_map = HashMap::new();
                 for naming in function_subsection.into_iter().flatten() {
                     if naming.index != u32::MAX {
-                        environ.declare_function_name(
-                            FunctionIndex::from_u32(naming.index),
-                            naming.name,
-                        )?;
+                        let mut name = naming.name.to_string();
+                        // In very rare cases a function name can have duplicates.
+                        if names_set.contains(&name) {
+                            let index = unique_name_map
+                                .entry(name.clone())
+                                .and_modify(|e| *e += 1)
+                                .or_insert(0);
+                            let alternative = format!("{name}.{index}");
+                            name = alternative;
+                        }
+                        names_set.insert(name.clone());
+                        functions.insert(FunctionIndex::from_u32(naming.index), name);
                     }
                 }
             }
@@ -546,5 +569,5 @@ pub fn parse_name_section<'data>(
         }
     }
 
-    Ok(())
+    environ.declare_function_names(functions)
 }
