@@ -92,7 +92,7 @@ pub trait CustomSectionLike<'a> {
     fn protection(&self) -> CustomSectionProtection;
     fn alignment(&self) -> Option<u64>;
     fn bytes(&self) -> &[u8];
-    fn relocations(&'a self) -> &[Self::Relocations];
+    fn relocations(&'a self) -> &'a [Self::Relocations];
 }
 
 impl<'a> CustomSectionLike<'a> for CustomSection {
@@ -110,7 +110,7 @@ impl<'a> CustomSectionLike<'a> for CustomSection {
         self.bytes.0.as_ref()
     }
 
-    fn relocations(&'a self) -> &[Self::Relocations] {
+    fn relocations(&'a self) -> &'a [Self::Relocations] {
         self.relocations.as_slice()
     }
 }
@@ -132,7 +132,7 @@ impl<'a> CustomSectionLike<'a> for ArchivedCustomSection {
         self.bytes.0.as_ref()
     }
 
-    fn relocations(&'a self) -> &[Self::Relocations] {
+    fn relocations(&'a self) -> &'a [Self::Relocations] {
         self.relocations.as_slice()
     }
 }
