@@ -1,7 +1,7 @@
 extern crate proc_macro;
 
 use proc_macro_error2::proc_macro_error;
-use syn::{parse_macro_input, DeriveInput};
+use syn::{DeriveInput, parse_macro_input};
 
 mod value_type;
 
@@ -9,6 +9,6 @@ mod value_type;
 #[proc_macro_derive(ValueType)]
 pub fn derive_value_type(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
     let input = parse_macro_input!(input as DeriveInput);
-    let gen = value_type::impl_value_type(&input);
-    gen.into()
+    let r#gen = value_type::impl_value_type(&input);
+    r#gen.into()
 }
