@@ -13,6 +13,7 @@ impl From<wasi::Snapshot0Clockid> for JournalSnapshot0ClockidV1 {
             wasi::Snapshot0Clockid::ProcessCputimeId => JournalSnapshot0ClockidV1::ProcessCputimeId,
             wasi::Snapshot0Clockid::ThreadCputimeId => JournalSnapshot0ClockidV1::ThreadCputimeId,
             wasi::Snapshot0Clockid::Unknown => JournalSnapshot0ClockidV1::Unknown,
+            _ => panic!("Unsupported Snapshot0Clockid variant"),
         }
     }
 }
@@ -52,6 +53,7 @@ impl From<wasi::Whence> for JournalWhenceV1 {
             wasi::Whence::Cur => JournalWhenceV1::Cur,
             wasi::Whence::End => JournalWhenceV1::End,
             wasi::Whence::Unknown => JournalWhenceV1::Unknown,
+            _ => panic!("Unsupported Whence variant"),
         }
     }
 }
@@ -88,6 +90,7 @@ impl From<wasi::Advice> for JournalAdviceV1 {
             wasi::Advice::Dontneed => JournalAdviceV1::Dontneed,
             wasi::Advice::Noreuse => JournalAdviceV1::Noreuse,
             wasi::Advice::Unknown => JournalAdviceV1::Unknown,
+            _ => panic!("Unsupported Advice variant"),
         }
     }
 }
@@ -339,6 +342,7 @@ impl From<wasi::Addressfamily> for JournalAddressfamilyV1 {
             wasi::Addressfamily::Inet4 => JournalAddressfamilyV1::Inet4,
             wasi::Addressfamily::Inet6 => JournalAddressfamilyV1::Inet6,
             wasi::Addressfamily::Unix => JournalAddressfamilyV1::Unix,
+            _ => panic!("Unsupported Addressfamily variant"),
         }
     }
 }
@@ -373,6 +377,7 @@ impl From<wasi::Socktype> for JournalSocktypeV1 {
             wasi::Socktype::Raw => JournalSocktypeV1::Raw,
             wasi::Socktype::Seqpacket => JournalSocktypeV1::Seqpacket,
             wasi::Socktype::Unknown => JournalSocktypeV1::Unknown,
+            _ => panic!("Unsupported Socktype variant"),
         }
     }
 }
@@ -431,6 +436,7 @@ impl From<wasi::Sockoption> for JournalSockoptionV1 {
             wasi::Sockoption::MulticastTtlV4 => JournalSockoptionV1::MulticastTtlV4,
             wasi::Sockoption::Type => JournalSockoptionV1::Type,
             wasi::Sockoption::Proto => JournalSockoptionV1::Proto,
+            _ => panic!("Unsupported Sockoption variant"),
         }
     }
 }
@@ -614,6 +620,7 @@ impl From<JournalWasiMemoryLayout> for WasiMemoryLayout {
             stack_lower: value.stack_lower,
             guard_size: value.guard_size,
             stack_size: value.stack_size,
+            tls_base: None,
         }
     }
 }
@@ -625,6 +632,7 @@ impl From<&'_ ArchivedJournalWasiMemoryLayout> for WasiMemoryLayout {
             stack_lower: value.stack_lower.to_native(),
             guard_size: value.guard_size.to_native(),
             stack_size: value.stack_size.to_native(),
+            tls_base: None,
         }
     }
 }
@@ -706,11 +714,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
                 is_64bit: *is_64bit,
             },
             ArchivedJournalEntry::FileDescriptorSeekV1(
-                ArchivedJournalEntryFileDescriptorSeekV1 {
-                    fd,
-                    offset,
-                    ref whence,
-                },
+                ArchivedJournalEntryFileDescriptorSeekV1 { fd, offset, whence },
             ) => Self::FileDescriptorSeekV1 {
                 fd: fd.to_native(),
                 offset: offset.to_native(),
@@ -793,7 +797,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
             },
             ArchivedJournalEntry::SnapshotV1(ArchivedJournalEntrySnapshotV1 {
                 since_epoch,
-                ref trigger,
+                trigger,
             }) => Self::SnapshotV1 {
                 when: SystemTime::UNIX_EPOCH
                     .checked_add((*since_epoch).into())
@@ -801,7 +805,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
                 trigger: trigger.into(),
             },
             ArchivedJournalEntry::SetClockTimeV1(ArchivedJournalEntrySetClockTimeV1 {
-                ref clock_id,
+                clock_id,
                 time,
             }) => Self::SetClockTimeV1 {
                 clock_id: clock_id.into(),
@@ -904,7 +908,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
                     fd,
                     offset,
                     len,
-                    ref advice,
+                    advice,
                 },
             ) => Self::FileDescriptorAdviseV1 {
                 fd: fd.to_native(),
@@ -953,9 +957,9 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
             }
             ArchivedJournalEntry::EpollCtlV1(ArchivedJournalEntryEpollCtlV1 {
                 epfd,
-                ref op,
+                op,
                 fd,
-                ref event,
+                event,
             }) => Self::EpollCtlV1 {
                 epfd: epfd.to_native(),
                 op: op.into(),
@@ -1012,7 +1016,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
             ArchivedJournalEntry::PortBridgeV1(ArchivedJournalEntryPortBridgeV1 {
                 network,
                 token,
-                ref security,
+                security,
             }) => Self::PortBridgeV1 {
                 network: String::from_utf8_lossy(network.as_ref()),
                 token: String::from_utf8_lossy(token.as_ref()),
@@ -1043,8 +1047,8 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
                 Self::PortRouteDelV1 { ip: ip.as_ipaddr() }
             }
             ArchivedJournalEntry::SocketOpenV1(ArchivedJournalEntrySocketOpenV1 {
-                ref af,
-                ref ty,
+                af,
+                ty,
                 pt,
                 fd,
             }) => Self::SocketOpenV1 {
@@ -1177,7 +1181,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
             },
             ArchivedJournalEntry::SocketSetOptFlagV1(ArchivedJournalEntrySocketSetOptFlagV1 {
                 fd,
-                ref opt,
+                opt,
                 flag,
             }) => Self::SocketSetOptFlagV1 {
                 fd: fd.to_native(),
@@ -1186,7 +1190,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
             },
             ArchivedJournalEntry::SocketSetOptSizeV1(ArchivedJournalEntrySocketSetOptSizeV1 {
                 fd,
-                ref opt,
+                opt,
                 size,
             }) => Self::SocketSetOptSizeV1 {
                 fd: fd.to_native(),
@@ -1195,7 +1199,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
             },
             ArchivedJournalEntry::SocketSetOptTimeV1(ArchivedJournalEntrySocketSetOptTimeV1 {
                 fd,
-                ref ty,
+                ty,
                 time,
             }) => Self::SocketSetOptTimeV1 {
                 fd: fd.to_native(),
@@ -1204,7 +1208,7 @@ impl<'a> TryFrom<ArchivedJournalEntry<'a>> for JournalEntry<'a> {
             },
             ArchivedJournalEntry::SocketShutdownV1(ArchivedJournalEntrySocketShutdownV1 {
                 fd,
-                ref how,
+                how,
             }) => Self::SocketShutdownV1 {
                 fd: fd.to_native(),
                 how: how.into(),
