@@ -3,11 +3,13 @@ mod aligned_cow_vec;
 mod arc;
 mod archived;
 mod archived_from;
+mod auto_consistent;
 mod boxed;
 mod buffered;
 mod compacting;
 #[cfg(feature = "log-file")]
 mod compacting_log_file;
+mod compacting_transaction;
 mod counting;
 mod filter;
 #[cfg(feature = "log-file")]
@@ -19,6 +21,7 @@ mod printing;
 mod recombined;
 #[cfg(test)]
 mod tests;
+mod transaction;
 mod unsupported;
 
 pub(super) use super::*;
@@ -26,10 +29,12 @@ pub(super) use super::*;
 pub use aligned_cow_str::*;
 pub use aligned_cow_vec::*;
 pub use archived::*;
+pub use auto_consistent::*;
 pub use buffered::*;
 pub use compacting::*;
 #[cfg(feature = "log-file")]
 pub use compacting_log_file::*;
+pub use compacting_transaction::*;
 pub use counting::*;
 pub use filter::*;
 #[cfg(feature = "log-file")]
@@ -39,4 +44,5 @@ pub use null::*;
 pub use pipe::*;
 pub use printing::*;
 pub use recombined::*;
+pub use transaction::*;
 pub use unsupported::*;
