@@ -1,11 +1,9 @@
 use std::{fmt::Debug, path::Path};
 
-use anyhow::Error;
-
 use super::ManifestError;
 
 /// The strictness to use when working with a
-/// [`crate::wasmer_package::Package`].
+/// `Package` values from this crate.
 ///
 /// This can be useful when loading a package that may be edited interactively
 /// or if you just want to use a package and don't care if the manifest is
@@ -24,13 +22,6 @@ impl Strictness {
         matches!(self, Strictness::Strict)
     }
 
-    pub(crate) fn on_error(&self, _path: &Path, error: Error) -> Result<(), Error> {
-        match self {
-            Strictness::Lossy => Ok(()),
-            Strictness::Strict => Err(error),
-        }
-    }
-
     pub(crate) fn outside_base_directory(
         &self,
         path: &Path,
