@@ -1,3 +1,8 @@
+#![allow(
+    clippy::result_large_err,
+    reason = "WasmerPackageError is large, but not often used"
+)]
+
 use bytes::{Buf, Bytes};
 use std::{
     fs::File,
@@ -62,6 +67,11 @@ pub fn from_disk(path: impl AsRef<Path>) -> Result<Container, WasmerPackageError
     }
 }
 
+/// Check if the data looks like a webc.
+pub fn is_container(bytes: &[u8]) -> bool {
+    is_tarball(std::io::Cursor::new(bytes)) || webc::detect(bytes).is_ok()
+}
+
 pub fn from_bytes(bytes: impl Into<Bytes>) -> Result<Container, WasmerPackageError> {
     let bytes: Bytes = bytes.into();
 
