@@ -356,6 +356,11 @@ impl ModuleInfo {
         self.hash
     }
 
+    /// Returns the module hash as String if available
+    pub fn hash_string(&self) -> Option<String> {
+        self.hash.map(|m| m.to_string())
+    }
+
     /// Get the given passive element, if it exists.
     pub fn get_passive_element(&self, index: ElemIndex) -> Option<&[FunctionIndex]> {
         self.passive_elements.get(&index).map(|es| &**es)
@@ -575,6 +580,14 @@ impl ModuleInfo {
             .take(self.num_imported_functions)
             .map(move |sig_index| self.signatures[*sig_index].clone())
     }
+
+    /// Get the name of a function by its index.
+    pub fn get_function_name(&self, func_index: FunctionIndex) -> String {
+        self.function_names
+            .get(&func_index)
+            .cloned()
+            .unwrap_or_else(|| format!("function_{}", func_index.as_u32()))
+    }
 }
 
 impl fmt::Display for ModuleInfo {
