@@ -9,7 +9,6 @@ use crate::tmp_fs::TmpFileSystem;
 
 pub struct RootFileSystemBuilder {
     default_root_dirs: bool,
-    create_tmp: bool,
     default_dev_files: bool,
     add_wasmer_command: bool,
     stdin: Option<Box<dyn VirtualFile + Send + Sync>>,
@@ -22,7 +21,6 @@ impl Default for RootFileSystemBuilder {
     fn default() -> Self {
         Self {
             default_root_dirs: true,
-            create_tmp: true,
             default_dev_files: true,
             add_wasmer_command: true,
             stdin: None,
@@ -63,24 +61,18 @@ impl RootFileSystemBuilder {
         self
     }
 
-    pub fn with_tmp(mut self, val: bool) -> Self {
-        self.create_tmp = val;
-        self
+    pub fn build(self) -> TmpFileSystem {
+        self.build_ext(&[])
     }
 
-    pub fn build(self) -> TmpFileSystem {
+    pub fn build_ext(self, mapped_dirs: &[&str]) -> TmpFileSystem {
         let tmp = TmpFileSystem::new();
 
         if self.default_root_dirs {
-            let default_dirs = {
-                let mut dirs = vec!["/.app", "/.private", "/bin", "/dev", "/etc"];
-
-                if self.create_tmp {
-                    dirs.push("/tmp");
-                }
-
-                dirs
-            };
+            let default_dirs = ["/.app", "/.private", "/bin", "/dev", "/etc", "/tmp"]
+                .into_iter()
+                .filter(|d| !mapped_dirs.contains(d))
+                .collect::<Vec<_>>();
 
             for root_dir in &default_dirs {
                 if let Err(err) = tmp.create_dir(Path::new(root_dir)) {
@@ -122,6 +114,8 @@ impl RootFileSystemBuilder {
                 PathBuf::from("/dev/tty"),
                 self.tty.unwrap_or_else(|| Box::<NullFile>::default()),
             );
+
+            let _ = tmp.create_dir(Path::new("/dev/shm"));
         }
         tmp
     }
@@ -130,6 +124,7 @@ impl RootFileSystemBuilder {
 #[cfg(test)]
 mod test_builder {
     use crate::{FileSystem, RootFileSystemBuilder};
+    use std::path::Path;
     use tokio::io::{AsyncReadExt, AsyncWriteExt};
 
     #[tokio::test]
@@ -198,5 +193,8 @@ mod test_builder {
             .open("/dev/stderr")
             .unwrap();
         assert_eq!(dev_stderr.get_special_fd().unwrap(), 2);
+
+        let dev_shm_metadata = root_fs.metadata(Path::new("/dev/shm")).unwrap();
+        assert!(dev_shm_metadata.is_dir());
     }
 }
