@@ -13,8 +13,8 @@ use replace_with::replace_with_or_abort;
 use tokio::io::{AsyncRead, AsyncSeek, AsyncWrite, ReadBuf};
 
 use crate::{
-    ops, FileOpener, FileSystem, FileSystems, FsError, Metadata, OpenOptions, OpenOptionsConfig,
-    ReadDir, VirtualFile,
+    FileOpener, FileSystem, FileSystems, FsError, Metadata, OpenOptions, OpenOptionsConfig,
+    ReadDir, VirtualFile, ops,
 };
 
 /// A primary filesystem and chain of secondary filesystems that are overlayed
@@ -210,10 +210,10 @@ where
         // Make sure the parent tree is in place on the primary, this is to cover the
         // scenario where the secondaries has a parent structure that is not yet in the
         // primary and the primary needs it to create a sub-directory
-        if let Some(parent) = path.parent() {
-            if self.read_dir(parent).is_ok() {
-                ops::create_dir_all(&self.primary, parent).ok();
-            }
+        if let Some(parent) = path.parent()
+            && self.read_dir(parent).is_ok()
+        {
+            ops::create_dir_all(&self.primary, parent).ok();
         }
 
         // Create the directory in the primary
@@ -526,10 +526,10 @@ where
         // If we are creating the file then do so
         if conf.create {
             // Create the parent structure and remove any whiteouts
-            if let Some(parent) = path.parent() {
-                if ops::exists(self, parent) {
-                    ops::create_dir_all(&self.primary, parent)?;
-                }
+            if let Some(parent) = path.parent()
+                && ops::exists(self, parent)
+            {
+                ops::create_dir_all(&self.primary, parent)?;
             }
             ops::remove_white_out(&self.primary, path);
 
@@ -975,7 +975,12 @@ where
                 Poll::Ready(Err(err)) => return Poll::Ready(Err(err)),
                 Poll::Pending => return Poll::Pending,
             }
-            Pin::new(self.state.as_mut()).poll_flush(cx)
+            // The file may actually be read-only and not support flush operations
+            // at all, and there's nothing to flush in read-only state anyway.
+            match self.state {
+                CowState::ReadOnly(_) => Poll::Ready(Ok(())),
+                _ => Pin::new(self.state.as_mut()).poll_flush(cx),
+            }
         }
 
         fn poll_shutdown(
@@ -987,7 +992,11 @@ where
                 Poll::Ready(Err(err)) => return Poll::Ready(Err(err)),
                 Poll::Pending => return Poll::Pending,
             }
-            Pin::new(self.state.as_mut()).poll_shutdown(cx)
+            // Same deal as flush above
+            match self.state {
+                CowState::ReadOnly(_) => Poll::Ready(Ok(())),
+                _ => Pin::new(self.state.as_mut()).poll_shutdown(cx),
+            }
         }
     }
 
@@ -1084,7 +1093,7 @@ where
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         struct IterFilesystems<'a, S>(&'a S);
-        impl<'a, S> Debug for IterFilesystems<'a, S>
+        impl<S> Debug for IterFilesystems<'_, S>
         where
             S: for<'b> FileSystems<'b>,
         {
@@ -1122,9 +1131,44 @@ mod tests {
     use std::path::PathBuf;
 
     use super::*;
-    use crate::mem_fs::FileSystem as MemFS;
+    use crate::{TmpFileSystem, mem_fs::FileSystem as MemFS};
     use tokio::io::{AsyncReadExt, AsyncSeekExt, AsyncWriteExt};
 
+    #[test]
+    fn overlay_read_dir_rebases_mounted_entries() {
+        let primary = TmpFileSystem::new();
+        ops::create_dir_all(&primary, "/app").unwrap();
+
+        let volume = MemFS::default();
+        ops::create_dir_all(&volume, "/themes/twentytwentyfour").unwrap();
+
+        let container = MemFS::default();
+        ops::create_dir_all(&container, "/app/wp-content/themes/twentytwentyfour").unwrap();
+
+        let volume: Arc<dyn FileSystem + Send + Sync> = Arc::new(volume);
+        primary
+            .mount(
+                PathBuf::from("/app/wp-content"),
+                &volume,
+                PathBuf::from("/"),
+            )
+            .unwrap();
+
+        let overlay = OverlayFileSystem::new(primary, [container]);
+
+        let mut entries: Vec<_> = overlay
+            .read_dir(Path::new("/app/wp-content/themes"))
+            .unwrap()
+            .map(|entry| entry.unwrap().path)
+            .collect();
+        entries.sort();
+
+        assert_eq!(
+            entries,
+            vec![PathBuf::from("/app/wp-content/themes/twentytwentyfour")],
+        );
+    }
+
     #[tokio::test]
     async fn remove_directory() {
         let primary = MemFS::default();
