@@ -10,8 +10,8 @@ use std::{
 use futures::future::BoxFuture;
 use tokio::io::{AsyncRead, AsyncSeek, AsyncWrite};
 use webc::{
-    compat::SharedBytes, Container, Metadata as WebcMetadata, PathSegmentError, PathSegments,
-    ToPathSegments, Volume,
+    Container, Metadata as WebcMetadata, PathSegmentError, PathSegments, ToPathSegments, Volume,
+    compat::SharedBytes,
 };
 
 use crate::{
@@ -152,7 +152,7 @@ impl FileSystem for WebcVolumeFileSystem {
         Err(FsError::PermissionDenied)
     }
 
-    fn new_open_options(&self) -> crate::OpenOptions {
+    fn new_open_options(&self) -> crate::OpenOptions<'_> {
         crate::OpenOptions::new(self)
     }
 
@@ -251,6 +251,10 @@ impl VirtualFile for File {
     ) -> Poll<std::io::Result<usize>> {
         Poll::Ready(Err(std::io::ErrorKind::PermissionDenied.into()))
     }
+
+    fn as_owned_buffer(&self) -> Option<SharedBytes> {
+        Some(self.content.get_ref().clone())
+    }
 }
 
 impl AsyncRead for File {
@@ -300,12 +304,14 @@ impl AsyncWrite for File {
     }
 }
 
-// HACK: timestamps are not present in webc v2, so we have to return
-// a stub modified time. previously we used to just return 0, but that
-// proved to cause problems with programs that interpret the value 0.
-// to circumvent this problem, we decided to return a non-zero value.
+// HACK: WebC v2 doesn't have timestamps, and WebC v3 files sometimes
+// have directories with a zero timestamp as well. Since some programs
+// interpret a zero timestamp as the absence of a value, we return
+// 1 second past epoch instead.
 fn get_modified(timestamps: Option<webc::Timestamps>) -> u64 {
-    timestamps.map(|t| t.modified()).unwrap_or(1)
+    let modified = timestamps.map(|t| t.modified()).unwrap_or_default();
+    // 1 billion nanoseconds = 1 second
+    modified.max(1_000_000_000)
 }
 
 fn compat_meta(meta: WebcMetadata) -> Metadata {
