@@ -1,12 +1,12 @@
 #![allow(unused_variables)]
 use crate::ruleset::{Direction, Ruleset};
-use crate::{io_err_into_net_error, VirtualIoSource};
 #[allow(unused_imports)]
 use crate::{
     IpCidr, IpRoute, NetworkError, Result, SocketStatus, StreamSecurity, VirtualConnectedSocket,
     VirtualConnectionlessSocket, VirtualIcmpSocket, VirtualNetworking, VirtualRawSocket,
     VirtualSocket, VirtualTcpListener, VirtualTcpSocket, VirtualUdpSocket,
 };
+use crate::{VirtualIoSource, io_err_into_net_error};
 use bytes::{Buf, BytesMut};
 use std::collections::VecDeque;
 use std::io::{self, Read, Write};
@@ -26,7 +26,7 @@ use tokio::runtime::Handle;
 #[allow(unused_imports, dead_code)]
 use tracing::{debug, error, info, trace, warn};
 use virtual_mio::{
-    state_as_waker_map, HandlerGuardState, InterestGuard, InterestHandler, InterestType, Selector,
+    HandlerGuardState, InterestGuard, InterestHandler, InterestType, Selector, state_as_waker_map,
 };
 
 #[derive(Debug)]
@@ -76,11 +76,11 @@ impl VirtualNetworking for LocalNetworking {
         reuse_port: bool,
         reuse_addr: bool,
     ) -> Result<Box<dyn VirtualTcpListener + Sync>> {
-        if let Some(ruleset) = self.ruleset.as_ref() {
-            if !ruleset.allows_socket(addr, Direction::Inbound) {
-                tracing::warn!(%addr, "listen_tcp blocked by firewall rule");
-                return Err(NetworkError::PermissionDenied);
-            }
+        if let Some(ruleset) = self.ruleset.as_ref()
+            && !ruleset.allows_socket(addr, Direction::Inbound)
+        {
+            tracing::warn!(%addr, "listen_tcp blocked by firewall rule");
+            return Err(NetworkError::PermissionDenied);
         }
 
         let listener = std::net::TcpListener::bind(addr)
@@ -106,19 +106,27 @@ impl VirtualNetworking for LocalNetworking {
         reuse_port: bool,
         reuse_addr: bool,
     ) -> Result<Box<dyn VirtualUdpSocket + Sync>> {
+        #[cfg(not(windows))]
         use socket2::{Domain, Socket, Type};
 
-        if let Some(ruleset) = self.ruleset.as_ref() {
-            if !ruleset.allows_socket(addr, Direction::Inbound) {
-                tracing::warn!(%addr, "bind_udp blocked by firewall rule");
-                return Err(NetworkError::PermissionDenied);
-            }
+        if let Some(ruleset) = self.ruleset.as_ref()
+            && !ruleset.allows_socket(addr, Direction::Inbound)
+        {
+            tracing::warn!(%addr, "bind_udp blocked by firewall rule");
+            return Err(NetworkError::PermissionDenied);
         }
 
         #[cfg(not(windows))]
         let socket = {
-            let std_sock =
-                Socket::new(Domain::IPV4, Type::DGRAM, None).map_err(io_err_into_net_error)?;
+            let domain = if addr.is_ipv4() {
+                Domain::IPV4
+            } else {
+                Domain::IPV6
+            };
+            let std_sock = Socket::new(domain, Type::DGRAM, None).map_err(io_err_into_net_error)?;
+            std_sock
+                .set_nonblocking(true)
+                .map_err(io_err_into_net_error)?;
             std_sock
                 .set_reuse_address(reuse_addr)
                 .map_err(io_err_into_net_error)?;
@@ -160,11 +168,11 @@ impl VirtualNetworking for LocalNetworking {
         _addr: SocketAddr,
         mut peer: SocketAddr,
     ) -> Result<Box<dyn VirtualTcpSocket + Sync>> {
-        if let Some(ruleset) = self.ruleset.as_ref() {
-            if !ruleset.allows_socket(peer, Direction::Outbound) {
-                tracing::warn!(%peer, "connect_tcp blocked by firewall rule");
-                return Err(NetworkError::PermissionDenied);
-            }
+        if let Some(ruleset) = self.ruleset.as_ref()
+            && !ruleset.allows_socket(peer, Direction::Outbound)
+        {
+            tracing::warn!(%peer, "connect_tcp blocked by firewall rule");
+            return Err(NetworkError::PermissionDenied);
         }
 
         let stream = mio::net::TcpStream::connect(peer).map_err(io_err_into_net_error)?;
@@ -182,11 +190,11 @@ impl VirtualNetworking for LocalNetworking {
         port: Option<u16>,
         dns_server: Option<IpAddr>,
     ) -> Result<Vec<IpAddr>> {
-        if let Some(ruleset) = self.ruleset.as_ref() {
-            if !ruleset.allows_domain(host) {
-                tracing::warn!(%host, "dns resolve blocked by firewall rule");
-                return Err(NetworkError::PermissionDenied);
-            }
+        if let Some(ruleset) = self.ruleset.as_ref()
+            && !ruleset.allows_domain(host)
+        {
+            tracing::warn!(%host, "dns resolve blocked by firewall rule");
+            return Err(NetworkError::PermissionDenied);
         }
 
         let host_to_lookup = if host.contains(':') {
@@ -229,11 +237,11 @@ impl LocalTcpListener {
     fn try_accept_internal(&mut self) -> Result<(Box<dyn VirtualTcpSocket + Sync>, SocketAddr)> {
         match self.stream.accept().map_err(io_err_into_net_error) {
             Ok((stream, addr)) => {
-                if let Some(ruleset) = self.ruleset.as_ref() {
-                    if !ruleset.allows_socket(addr, Direction::Outbound) {
-                        tracing::warn!(%addr, "try_accept blocked by firewall rule");
-                        return Err(NetworkError::PermissionDenied);
-                    }
+                if let Some(ruleset) = self.ruleset.as_ref()
+                    && !ruleset.allows_socket(addr, Direction::Outbound)
+                {
+                    tracing::warn!(%addr, "try_accept blocked by firewall rule");
+                    return Err(NetworkError::PermissionDenied);
                 }
 
                 let mut socket = LocalTcpStream::new(self.selector.clone(), stream, addr);
@@ -563,16 +571,23 @@ impl VirtualConnectedSocket for LocalTcpStream {
         Ok(())
     }
 
-    fn try_recv(&mut self, buf: &mut [MaybeUninit<u8>]) -> Result<usize> {
+    fn try_recv(&mut self, buf: &mut [MaybeUninit<u8>], peek: bool) -> Result<usize> {
         let buf: &mut [u8] = unsafe { std::mem::transmute(buf) };
         if !self.buffer.is_empty() {
             let amt = buf.len().min(self.buffer.len());
             buf[..amt].copy_from_slice(&self.buffer[..amt]);
-            self.buffer.advance(amt);
+            if !peek {
+                self.buffer.advance(amt);
+            }
             return Ok(amt);
         }
 
-        self.stream.read(buf).map_err(io_err_into_net_error)
+        if peek {
+            self.stream.peek(buf)
+        } else {
+            self.stream.read(buf)
+        }
+        .map_err(io_err_into_net_error)
     }
 }
 
@@ -839,11 +854,11 @@ impl VirtualUdpSocket for LocalUdpSocket {
 
 impl VirtualConnectionlessSocket for LocalUdpSocket {
     fn try_send_to(&mut self, data: &[u8], addr: SocketAddr) -> Result<usize> {
-        if let Some(ruleset) = self.ruleset.as_ref() {
-            if !ruleset.allows_socket(addr, Direction::Outbound) {
-                tracing::warn!(%addr, "try_send blocked by firewall rule");
-                return Err(NetworkError::PermissionDenied);
-            }
+        if let Some(ruleset) = self.ruleset.as_ref()
+            && !ruleset.allows_socket(addr, Direction::Outbound)
+        {
+            tracing::warn!(%addr, "try_send blocked by firewall rule");
+            return Err(NetworkError::PermissionDenied);
         }
 
         let ret = self
@@ -861,9 +876,18 @@ impl VirtualConnectionlessSocket for LocalUdpSocket {
         ret
     }
 
-    fn try_recv_from(&mut self, buf: &mut [MaybeUninit<u8>]) -> Result<(usize, SocketAddr)> {
+    fn try_recv_from(
+        &mut self,
+        buf: &mut [MaybeUninit<u8>],
+        peek: bool,
+    ) -> Result<(usize, SocketAddr)> {
         let buf: &mut [u8] = unsafe { std::mem::transmute(buf) };
-        self.socket.recv_from(buf).map_err(io_err_into_net_error)
+        if peek {
+            self.socket.peek_from(buf)
+        } else {
+            self.socket.recv_from(buf)
+        }
+        .map_err(io_err_into_net_error)
     }
 }
 
