@@ -8,21 +8,20 @@
 
 mod allocator;
 
-use crate::export::VMExtern;
+use crate::LinearMemory;
 use crate::imports::Imports;
 use crate::store::{InternalStoreHandle, StoreObjects};
 use crate::table::TableElement;
 use crate::trap::{Trap, TrapCode};
 use crate::vmcontext::{
-    memory32_atomic_check32, memory32_atomic_check64, memory_copy, memory_fill,
     VMBuiltinFunctionsArray, VMCallerCheckedAnyfunc, VMContext, VMFunctionContext,
     VMFunctionImport, VMFunctionKind, VMGlobalDefinition, VMGlobalImport, VMMemoryDefinition,
-    VMMemoryImport, VMSharedSignatureIndex, VMTableDefinition, VMTableImport, VMTagImport,
-    VMTrampoline,
+    VMMemoryImport, VMSharedSignatureIndex, VMSharedTagIndex, VMTableDefinition, VMTableImport,
+    VMTrampoline, memory_copy, memory_fill, memory32_atomic_check32, memory32_atomic_check64,
 };
-use crate::{wasmer_call_trampoline, FunctionBodyPtr, MaybeInstanceOwned, TrapHandlerFn, VMTag};
-use crate::{LinearMemory, NotifyLocation};
+use crate::{FunctionBodyPtr, MaybeInstanceOwned, TrapHandlerFn, VMTag, wasmer_call_trampoline};
 use crate::{VMConfig, VMFuncRef, VMFunction, VMGlobal, VMMemory, VMTable};
+use crate::{export::VMExtern, threadconditions::ExpectedValue};
 pub use allocator::InstanceAllocator;
 use memoffset::offset_of;
 use more_asserts::assert_lt;
@@ -35,12 +34,12 @@ use std::mem;
 use std::ptr::{self, NonNull};
 use std::slice;
 use std::sync::Arc;
-use wasmer_types::entity::{packed_option::ReservedValue, BoxedSlice, EntityRef, PrimaryMap};
+use wasmer_types::entity::{BoxedSlice, EntityRef, PrimaryMap, packed_option::ReservedValue};
 use wasmer_types::{
     DataIndex, DataInitializer, ElemIndex, ExportIndex, FunctionIndex, GlobalIndex, GlobalInit,
-    LocalFunctionIndex, LocalGlobalIndex, LocalMemoryIndex, LocalTableIndex, LocalTagIndex,
-    MemoryError, MemoryIndex, ModuleInfo, Pages, SignatureIndex, TableIndex, TableInitializer,
-    TagIndex, VMOffsets,
+    LocalFunctionIndex, LocalGlobalIndex, LocalMemoryIndex, LocalTableIndex, MemoryError,
+    MemoryIndex, ModuleInfo, Pages, SignatureIndex, TableIndex, TableInitializer, TagIndex,
+    VMOffsets,
 };
 
 /// A WebAssembly instance.
@@ -70,8 +69,8 @@ pub(crate) struct Instance {
     /// WebAssembly global data.
     globals: BoxedSlice<LocalGlobalIndex, InternalStoreHandle<VMGlobal>>,
 
-    /// WebAssembly global data.
-    tags: BoxedSlice<LocalTagIndex, InternalStoreHandle<VMTag>>,
+    /// WebAssembly tag data. Notably, this stores *all* tags, not just local ones.
+    tags: BoxedSlice<TagIndex, InternalStoreHandle<VMTag>>,
 
     /// Pointers to functions in executable memory.
     functions: BoxedSlice<LocalFunctionIndex, FunctionBodyPtr>,
@@ -113,9 +112,11 @@ impl Instance {
     /// Helper function to access various locations offset from our `*mut
     /// VMContext` object.
     unsafe fn vmctx_plus_offset<T>(&self, offset: u32) -> *mut T {
-        (self.vmctx_ptr() as *mut u8)
-            .add(usize::try_from(offset).unwrap())
-            .cast()
+        unsafe {
+            (self.vmctx_ptr() as *mut u8)
+                .add(usize::try_from(offset).unwrap())
+                .cast()
+        }
     }
 
     fn module(&self) -> &Arc<ModuleInfo> {
@@ -126,11 +127,11 @@ impl Instance {
         &self.module
     }
 
-    fn context(&self) -> &StoreObjects {
+    pub(crate) fn context(&self) -> &StoreObjects {
         unsafe { &*self.context }
     }
 
-    fn context_mut(&mut self) -> &mut StoreObjects {
+    pub(crate) fn context_mut(&mut self) -> &mut StoreObjects {
         unsafe { &mut *self.context }
     }
 
@@ -188,15 +189,16 @@ impl Instance {
         unsafe { self.vmctx_plus_offset(self.offsets.vmctx_imported_globals_begin()) }
     }
 
-    /// Return the indexed `VMTagImport`.
-    fn imported_tag(&self, index: TagIndex) -> &VMTagImport {
+    /// Return the indexed `VMSharedTagIndex`.
+    #[cfg_attr(target_os = "windows", allow(dead_code))]
+    pub(crate) fn shared_tag_ptr(&self, index: TagIndex) -> &VMSharedTagIndex {
         let index = usize::try_from(index.as_u32()).unwrap();
-        unsafe { &*self.imported_tags_ptr().add(index) }
+        unsafe { &*self.shared_tags_ptr().add(index) }
     }
 
-    /// Return a pointer to the `VMTagImport`s.
-    fn imported_tags_ptr(&self) -> *mut VMTagImport {
-        unsafe { self.vmctx_plus_offset(self.offsets.vmctx_imported_tags_begin()) }
+    /// Return a pointer to the `VMSharedTagIndex`s.
+    pub(crate) fn shared_tags_ptr(&self) -> *mut VMSharedTagIndex {
+        unsafe { self.vmctx_plus_offset(self.offsets.vmctx_tag_ids_begin()) }
     }
 
     /// Return the indexed `VMTableDefinition`.
@@ -373,7 +375,7 @@ impl Instance {
 
         let sig = self.module.functions[start_index];
         let trampoline = self.function_call_trampolines[sig];
-        let values_vec = vec![].as_mut_ptr();
+        let mut values_vec = vec![];
 
         unsafe {
             // Even though we already know the type of the function we need to call, in certain
@@ -385,7 +387,7 @@ impl Instance {
                 callee_vmctx,
                 trampoline,
                 callee_address,
-                values_vec,
+                values_vec.as_mut_ptr(),
             )
         }
     }
@@ -624,10 +626,8 @@ impl Instance {
             .get(&elem_index)
             .map_or::<&[Option<VMFuncRef>], _>(&[], |e| &**e);
 
-        if src
-            .checked_add(len)
-            .map_or(true, |n| n as usize > elem.len())
-            || dst.checked_add(len).map_or(true, |m| m > table.size())
+        if src.checked_add(len).is_none_or(|n| n as usize > elem.len())
+            || dst.checked_add(len).is_none_or(|m| m > table.size())
         {
             return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
         }
@@ -660,7 +660,7 @@ impl Instance {
 
         if start_index
             .checked_add(len)
-            .map_or(true, |n| n as usize > table_size)
+            .is_none_or(|n| n as usize > table_size)
         {
             return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
         }
@@ -775,12 +775,10 @@ impl Instance {
         let data = passive_data.get(&data_index).map_or(&[][..], |d| &**d);
 
         let current_length = unsafe { memory.vmmemory().as_ref().current_length };
-        if src
-            .checked_add(len)
-            .map_or(true, |n| n as usize > data.len())
+        if src.checked_add(len).is_none_or(|n| n as usize > data.len())
             || dst
                 .checked_add(len)
-                .map_or(true, |m| usize::try_from(m).unwrap() > current_length)
+                .is_none_or(|m| usize::try_from(m).unwrap() > current_length)
         {
             return Err(Trap::lib(TrapCode::HeapAccessOutOfBounds));
         }
@@ -830,14 +828,20 @@ impl Instance {
         }
     }
 
-    fn memory_wait(memory: &mut VMMemory, dst: u32, timeout: i64) -> Result<u32, Trap> {
-        let location = NotifyLocation { address: dst };
+    /// # Safety
+    /// See [`LinearMemory::do_wait`].
+    unsafe fn memory_wait(
+        memory: &mut VMMemory,
+        dst: u32,
+        expected: ExpectedValue,
+        timeout: i64,
+    ) -> Result<u32, Trap> {
         let timeout = if timeout < 0 {
             None
         } else {
             Some(std::time::Duration::from_nanos(timeout as u64))
         };
-        match memory.do_wait(location, timeout) {
+        match unsafe { memory.do_wait(dst, expected, timeout) } {
             Ok(count) => Ok(count),
             Err(_err) => {
                 // ret is None if there is more than 2^32 waiter in queue or some other error
@@ -859,12 +863,14 @@ impl Instance {
         // We should trap according to spec, but official test rely on not trapping...
         //}
 
+        // Do a fast-path check of the expected value, and also ensure proper alignment
         let ret = unsafe { memory32_atomic_check32(&memory, dst, val) };
 
         if let Ok(mut ret) = ret {
             if ret == 0 {
                 let memory = self.get_local_vmmemory_mut(memory_index);
-                ret = Self::memory_wait(memory, dst, timeout)?;
+                // Safety: we have already checked alignment and bounds in memory32_atomic_check32
+                ret = unsafe { Self::memory_wait(memory, dst, ExpectedValue::U32(val), timeout)? };
             }
             Ok(ret)
         } else {
@@ -886,11 +892,14 @@ impl Instance {
         // We should trap according to spec, but official test rely on not trapping...
         //}
 
+        // Do a fast-path check of the expected value, and also ensure proper alignment
         let ret = unsafe { memory32_atomic_check32(memory, dst, val) };
+
         if let Ok(mut ret) = ret {
             if ret == 0 {
                 let memory = self.get_vmmemory_mut(memory_index);
-                ret = Self::memory_wait(memory, dst, timeout)?;
+                // Safety: we have already checked alignment and bounds in memory32_atomic_check32
+                ret = unsafe { Self::memory_wait(memory, dst, ExpectedValue::U32(val), timeout)? };
             }
             Ok(ret)
         } else {
@@ -911,12 +920,14 @@ impl Instance {
         // We should trap according to spec, but official test rely on not trapping...
         //}
 
+        // Do a fast-path check of the expected value, and also ensure proper alignment
         let ret = unsafe { memory32_atomic_check64(&memory, dst, val) };
 
         if let Ok(mut ret) = ret {
             if ret == 0 {
                 let memory = self.get_local_vmmemory_mut(memory_index);
-                ret = Self::memory_wait(memory, dst, timeout)?;
+                // Safety: we have already checked alignment and bounds in memory32_atomic_check64
+                ret = unsafe { Self::memory_wait(memory, dst, ExpectedValue::U64(val), timeout)? };
             }
             Ok(ret)
         } else {
@@ -938,12 +949,14 @@ impl Instance {
         // We should trap according to spec, but official test rely on not trapping...
         //}
 
+        // Do a fast-path check of the expected value, and also ensure proper alignment
         let ret = unsafe { memory32_atomic_check64(memory, dst, val) };
 
         if let Ok(mut ret) = ret {
             if ret == 0 {
                 let memory = self.get_vmmemory_mut(memory_index);
-                ret = Self::memory_wait(memory, dst, timeout)?;
+                // Safety: we have already checked alignment and bounds in memory32_atomic_check64
+                ret = unsafe { Self::memory_wait(memory, dst, ExpectedValue::U64(val), timeout)? };
             }
             Ok(ret)
         } else {
@@ -959,9 +972,7 @@ impl Instance {
         count: u32,
     ) -> Result<u32, Trap> {
         let memory = self.get_local_vmmemory_mut(memory_index);
-        // fetch the notifier
-        let location = NotifyLocation { address: dst };
-        Ok(memory.do_notify(location, count))
+        Ok(memory.do_notify(dst, count))
     }
 
     /// Perform an Atomic.Notify
@@ -972,9 +983,7 @@ impl Instance {
         count: u32,
     ) -> Result<u32, Trap> {
         let memory = self.get_vmmemory_mut(memory_index);
-        // fetch the notifier
-        let location = NotifyLocation { address: dst };
-        Ok(memory.do_notify(location, count))
+        Ok(memory.do_notify(dst, count))
     }
 }
 
@@ -1016,7 +1025,7 @@ impl Drop for VMInstance {
 }
 
 impl VMInstance {
-    /// Create a new `VMInstance` pointing at a new [`Instance`].
+    /// Create a new `VMInstance` pointing at freshly allocated instance data.
     ///
     /// # Safety
     ///
@@ -1046,113 +1055,125 @@ impl VMInstance {
         finished_function_call_trampolines: BoxedSlice<SignatureIndex, VMTrampoline>,
         finished_memories: BoxedSlice<LocalMemoryIndex, InternalStoreHandle<VMMemory>>,
         finished_tables: BoxedSlice<LocalTableIndex, InternalStoreHandle<VMTable>>,
-        finished_tags: BoxedSlice<LocalTagIndex, InternalStoreHandle<VMTag>>,
         finished_globals: BoxedSlice<LocalGlobalIndex, InternalStoreHandle<VMGlobal>>,
+        tags: BoxedSlice<TagIndex, InternalStoreHandle<VMTag>>,
         imports: Imports,
         vmshared_signatures: BoxedSlice<SignatureIndex, VMSharedSignatureIndex>,
     ) -> Result<Self, Trap> {
-        let vmctx_globals = finished_globals
-            .values()
-            .map(|m| m.get(context).vmglobal())
-            .collect::<PrimaryMap<LocalGlobalIndex, _>>()
-            .into_boxed_slice();
-        let passive_data = RefCell::new(
-            module
-                .passive_data
-                .clone()
-                .into_iter()
-                .map(|(idx, bytes)| (idx, Arc::from(bytes)))
-                .collect::<HashMap<_, _>>(),
-        );
+        unsafe {
+            let vmctx_tags = tags
+                .values()
+                .map(|m: &InternalStoreHandle<VMTag>| VMSharedTagIndex::new(m.index() as u32))
+                .collect::<PrimaryMap<TagIndex, VMSharedTagIndex>>()
+                .into_boxed_slice();
+            let vmctx_globals = finished_globals
+                .values()
+                .map(|m: &InternalStoreHandle<VMGlobal>| m.get(context).vmglobal())
+                .collect::<PrimaryMap<LocalGlobalIndex, NonNull<VMGlobalDefinition>>>()
+                .into_boxed_slice();
+            let passive_data = RefCell::new(
+                module
+                    .passive_data
+                    .clone()
+                    .into_iter()
+                    .map(|(idx, bytes)| (idx, Arc::from(bytes)))
+                    .collect::<HashMap<_, _>>(),
+            );
 
-        let handle = {
-            let offsets = allocator.offsets().clone();
-            // use dummy value to create an instance so we can get the vmctx pointer
-            let funcrefs = PrimaryMap::new().into_boxed_slice();
-            let imported_funcrefs = PrimaryMap::new().into_boxed_slice();
-            // Create the `Instance`. The unique, the One.
-            let instance = Instance {
-                module,
-                context,
-                offsets,
-                memories: finished_memories,
-                tables: finished_tables,
-                tags: finished_tags,
-                globals: finished_globals,
-                functions: finished_functions,
-                function_call_trampolines: finished_function_call_trampolines,
-                passive_elements: Default::default(),
-                passive_data,
-                funcrefs,
-                imported_funcrefs,
-                vmctx: VMContext {},
-            };
-
-            let mut instance_handle = allocator.into_vminstance(instance);
-
-            // Set the funcrefs after we've built the instance
-            {
-                let instance = instance_handle.instance_mut();
-                let vmctx_ptr = instance.vmctx_ptr();
-                (instance.funcrefs, instance.imported_funcrefs) = build_funcrefs(
-                    &instance.module,
+            let handle = {
+                let offsets = allocator.offsets().clone();
+                // use dummy value to create an instance so we can get the vmctx pointer
+                let funcrefs = PrimaryMap::new().into_boxed_slice();
+                let imported_funcrefs = PrimaryMap::new().into_boxed_slice();
+                // Create the `Instance`. The unique, the One.
+                let instance = Instance {
+                    module,
                     context,
-                    &imports,
-                    &instance.functions,
-                    &vmshared_signatures,
-                    &instance.function_call_trampolines,
-                    vmctx_ptr,
-                );
-            }
+                    offsets,
+                    memories: finished_memories,
+                    tables: finished_tables,
+                    tags,
+                    globals: finished_globals,
+                    functions: finished_functions,
+                    function_call_trampolines: finished_function_call_trampolines,
+                    passive_elements: Default::default(),
+                    passive_data,
+                    funcrefs,
+                    imported_funcrefs,
+                    vmctx: VMContext {},
+                };
 
-            instance_handle
-        };
-        let instance = handle.instance();
+                let mut instance_handle = allocator.into_vminstance(instance);
 
-        ptr::copy(
-            vmshared_signatures.values().as_slice().as_ptr(),
-            instance.signature_ids_ptr(),
-            vmshared_signatures.len(),
-        );
-        ptr::copy(
-            imports.functions.values().as_slice().as_ptr(),
-            instance.imported_functions_ptr(),
-            imports.functions.len(),
-        );
-        ptr::copy(
-            imports.tables.values().as_slice().as_ptr(),
-            instance.imported_tables_ptr(),
-            imports.tables.len(),
-        );
-        ptr::copy(
-            imports.memories.values().as_slice().as_ptr(),
-            instance.imported_memories_ptr(),
-            imports.memories.len(),
-        );
-        ptr::copy(
-            imports.globals.values().as_slice().as_ptr(),
-            instance.imported_globals_ptr(),
-            imports.globals.len(),
-        );
-        // these should already be set, add asserts here? for:
-        // - instance.tables_ptr() as *mut VMTableDefinition
-        // - instance.memories_ptr() as *mut VMMemoryDefinition
-        ptr::copy(
-            vmctx_globals.values().as_slice().as_ptr(),
-            instance.globals_ptr() as *mut NonNull<VMGlobalDefinition>,
-            vmctx_globals.len(),
-        );
-        ptr::write(
-            instance.builtin_functions_ptr(),
-            VMBuiltinFunctionsArray::initialized(),
-        );
+                // Set the funcrefs after we've built the instance
+                {
+                    let instance = instance_handle.instance_mut();
+                    let vmctx_ptr = instance.vmctx_ptr();
+                    (instance.funcrefs, instance.imported_funcrefs) = build_funcrefs(
+                        &instance.module,
+                        context,
+                        &imports,
+                        &instance.functions,
+                        &vmshared_signatures,
+                        &instance.function_call_trampolines,
+                        vmctx_ptr,
+                    );
+                }
 
-        // Perform infallible initialization in this constructor, while fallible
-        // initialization is deferred to the `initialize` method.
-        initialize_passive_elements(instance);
-        initialize_globals(instance);
+                instance_handle
+            };
+            let instance = handle.instance();
 
-        Ok(handle)
+            ptr::copy(
+                vmctx_tags.values().as_slice().as_ptr(),
+                instance.shared_tags_ptr(),
+                vmctx_tags.len(),
+            );
+            ptr::copy(
+                vmshared_signatures.values().as_slice().as_ptr(),
+                instance.signature_ids_ptr(),
+                vmshared_signatures.len(),
+            );
+            ptr::copy(
+                imports.functions.values().as_slice().as_ptr(),
+                instance.imported_functions_ptr(),
+                imports.functions.len(),
+            );
+            ptr::copy(
+                imports.tables.values().as_slice().as_ptr(),
+                instance.imported_tables_ptr(),
+                imports.tables.len(),
+            );
+            ptr::copy(
+                imports.memories.values().as_slice().as_ptr(),
+                instance.imported_memories_ptr(),
+                imports.memories.len(),
+            );
+            ptr::copy(
+                imports.globals.values().as_slice().as_ptr(),
+                instance.imported_globals_ptr(),
+                imports.globals.len(),
+            );
+            // these should already be set, add asserts here? for:
+            // - instance.tables_ptr() as *mut VMTableDefinition
+            // - instance.memories_ptr() as *mut VMMemoryDefinition
+            ptr::copy(
+                vmctx_globals.values().as_slice().as_ptr(),
+                instance.globals_ptr() as *mut NonNull<VMGlobalDefinition>,
+                vmctx_globals.len(),
+            );
+            ptr::write(
+                instance.builtin_functions_ptr(),
+                VMBuiltinFunctionsArray::initialized(),
+            );
+
+            // Perform infallible initialization in this constructor, while fallible
+            // initialization is deferred to the `initialize` method.
+            initialize_passive_elements(instance);
+            initialize_globals(instance);
+
+            Ok(handle)
+        }
     }
 
     /// Return a reference to the contained `Instance`.
@@ -1282,12 +1303,7 @@ impl VMInstance {
             }
 
             ExportIndex::Tag(index) => {
-                let handle = if let Some(def_index) = instance.module.local_tag_index(index) {
-                    instance.tags[def_index]
-                } else {
-                    let import = instance.imported_tag(index);
-                    import.handle
-                };
+                let handle = instance.tags[index];
                 VMExtern::Tag(handle)
             }
         }
@@ -1298,7 +1314,7 @@ impl VMInstance {
     /// Specifically, it provides access to the key-value pairs, where the keys
     /// are export names, and the values are export declarations which can be
     /// resolved `lookup_by_declaration`.
-    pub fn exports(&self) -> indexmap::map::Iter<String, ExportIndex> {
+    pub fn exports(&self) -> indexmap::map::Iter<'_, String, ExportIndex> {
         self.module().exports.iter()
     }
 
@@ -1391,16 +1407,18 @@ unsafe fn get_memory_slice<'instance>(
     init: &DataInitializer<'_>,
     instance: &'instance Instance,
 ) -> &'instance mut [u8] {
-    let memory = if let Some(local_memory_index) = instance
-        .module
-        .local_memory_index(init.location.memory_index)
-    {
-        instance.memory(local_memory_index)
-    } else {
-        let import = instance.imported_memory(init.location.memory_index);
-        *import.definition.as_ref()
-    };
-    slice::from_raw_parts_mut(memory.base, memory.current_length)
+    unsafe {
+        let memory = if let Some(local_memory_index) = instance
+            .module
+            .local_memory_index(init.location.memory_index)
+        {
+            instance.memory(local_memory_index)
+        } else {
+            let import = instance.imported_memory(init.location.memory_index);
+            *import.definition.as_ref()
+        };
+        slice::from_raw_parts_mut(memory.base, memory.current_length)
+    }
 }
 
 /// Compute the offset for a table element initializer.
@@ -1431,7 +1449,7 @@ fn initialize_tables(instance: &mut Instance) -> Result<(), Trap> {
 
         if start
             .checked_add(init.elements.len())
-            .map_or(true, |end| end > table.size() as usize)
+            .is_none_or(|end| end > table.size() as usize)
         {
             return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
         }
@@ -1471,19 +1489,21 @@ fn initialize_passive_elements(instance: &Instance) {
         "should only be called once, at initialization time"
     );
 
-    passive_elements.extend(
-        instance
-            .module
-            .passive_elements
-            .iter()
-            .filter(|(_, segments)| !segments.is_empty())
-            .map(|(idx, segments)| {
-                (
-                    *idx,
-                    segments.iter().map(|s| instance.func_ref(*s)).collect(),
-                )
-            }),
-    );
+    passive_elements.extend(instance.module.passive_elements.iter().filter_map(
+        |(&idx, segments)| -> Option<(ElemIndex, Box<[Option<VMFuncRef>]>)> {
+            if segments.is_empty() {
+                None
+            } else {
+                Some((
+                    idx,
+                    segments
+                        .iter()
+                        .map(|s| instance.func_ref(*s))
+                        .collect::<Box<[Option<VMFuncRef>]>>(),
+                ))
+            }
+        },
+    ));
 }
 
 /// Initialize the table memory from the provided initializers.
@@ -1499,7 +1519,7 @@ fn initialize_memories(
             let current_length = memory.vmmemory().as_ref().current_length;
             if start
                 .checked_add(init.data.len())
-                .map_or(true, |end| end > current_length)
+                .is_none_or(|end| end > current_length)
             {
                 return Err(Trap::lib(TrapCode::HeapAccessOutOfBounds));
             }
