@@ -5,11 +5,15 @@
 //!
 //! `Memory` is to WebAssembly linear memories what `Table` is to WebAssembly tables.
 
-use crate::mmap::MmapType;
 use crate::threadconditions::ThreadConditions;
 pub use crate::threadconditions::{NotifyLocation, WaiterError};
 use crate::trap::Trap;
-use crate::{mmap::Mmap, store::MaybeInstanceOwned, vmcontext::VMMemoryDefinition};
+use crate::{
+    mmap::{Mmap, MmapType},
+    store::MaybeInstanceOwned,
+    threadconditions::ExpectedValue,
+    vmcontext::VMMemoryDefinition,
+};
 use more_asserts::assert_ge;
 use std::cell::UnsafeCell;
 use std::convert::TryInto;
@@ -59,13 +63,13 @@ impl WasmMmap {
             })?;
         let prev_pages = self.size;
 
-        if let Some(maximum) = conf.maximum {
-            if new_pages > maximum {
-                return Err(MemoryError::CouldNotGrow {
-                    current: self.size,
-                    attempted_delta: delta,
-                });
-            }
+        if let Some(maximum) = conf.maximum
+            && new_pages > maximum
+        {
+            return Err(MemoryError::CouldNotGrow {
+                current: self.size,
+                attempted_delta: delta,
+            });
         }
 
         // Wasm linear memories are never allowed to grow beyond what is
@@ -139,6 +143,12 @@ impl WasmMmap {
     /// Resets the memory down to a zero size
     fn reset(&mut self) -> Result<(), MemoryError> {
         self.size.0 = 0;
+        // update memory definition
+        unsafe {
+            let mut md_ptr = self.vm_memory_definition.as_ptr();
+            let md = md_ptr.as_mut();
+            md.current_length = 0;
+        }
         Ok(())
     }
 
@@ -239,13 +249,15 @@ impl VMOwnedMemory {
         style: &MemoryStyle,
         vm_memory_location: NonNull<VMMemoryDefinition>,
     ) -> Result<Self, MemoryError> {
-        Self::new_internal(
-            memory,
-            style,
-            Some(vm_memory_location),
-            None,
-            MmapType::Private,
-        )
+        unsafe {
+            Self::new_internal(
+                memory,
+                style,
+                Some(vm_memory_location),
+                None,
+                MmapType::Private,
+            )
+        }
     }
 
     /// Create a new linear memory instance with specified minimum and maximum number of wasm pages
@@ -264,13 +276,15 @@ impl VMOwnedMemory {
         backing_file: Option<std::path::PathBuf>,
         memory_type: MmapType,
     ) -> Result<Self, MemoryError> {
-        Self::new_internal(
-            memory,
-            style,
-            Some(vm_memory_location),
-            backing_file,
-            memory_type,
-        )
+        unsafe {
+            Self::new_internal(
+                memory,
+                style,
+                Some(vm_memory_location),
+                backing_file,
+                memory_type,
+            )
+        }
     }
 
     /// Build a `Memory` with either self-owned or VM owned metadata.
@@ -281,82 +295,84 @@ impl VMOwnedMemory {
         backing_file: Option<std::path::PathBuf>,
         memory_type: MmapType,
     ) -> Result<Self, MemoryError> {
-        if memory.minimum > Pages::max_value() {
-            return Err(MemoryError::MinimumMemoryTooLarge {
-                min_requested: memory.minimum,
-                max_allowed: Pages::max_value(),
-            });
-        }
-        // `maximum` cannot be set to more than `65536` pages.
-        if let Some(max) = memory.maximum {
-            if max > Pages::max_value() {
-                return Err(MemoryError::MaximumMemoryTooLarge {
-                    max_requested: max,
+        unsafe {
+            if memory.minimum > Pages::max_value() {
+                return Err(MemoryError::MinimumMemoryTooLarge {
+                    min_requested: memory.minimum,
                     max_allowed: Pages::max_value(),
                 });
             }
-            if max < memory.minimum {
-                return Err(MemoryError::InvalidMemory {
-                    reason: format!(
-                        "the maximum ({} pages) is less than the minimum ({} pages)",
-                        max.0, memory.minimum.0
-                    ),
-                });
-            }
-        }
-
-        let offset_guard_bytes = style.offset_guard_size() as usize;
-
-        let minimum_pages = match style {
-            MemoryStyle::Dynamic { .. } => memory.minimum,
-            MemoryStyle::Static { bound, .. } => {
-                assert_ge!(*bound, memory.minimum);
-                *bound
-            }
-        };
-        let minimum_bytes = minimum_pages.bytes().0;
-        let request_bytes = minimum_bytes.checked_add(offset_guard_bytes).unwrap();
-        let mapped_pages = memory.minimum;
-        let mapped_bytes = mapped_pages.bytes();
-
-        let mut alloc =
-            Mmap::accessible_reserved(mapped_bytes.0, request_bytes, backing_file, memory_type)
-                .map_err(MemoryError::Region)?;
-
-        let base_ptr = alloc.as_mut_ptr();
-        let mem_length = memory
-            .minimum
-            .bytes()
-            .0
-            .max(alloc.as_slice_accessible().len());
-        let mmap = WasmMmap {
-            vm_memory_definition: if let Some(mem_loc) = vm_memory_location {
-                {
-                    let mut ptr = mem_loc;
-                    let md = ptr.as_mut();
-                    md.base = base_ptr;
-                    md.current_length = mem_length;
+            // `maximum` cannot be set to more than `65536` pages.
+            if let Some(max) = memory.maximum {
+                if max > Pages::max_value() {
+                    return Err(MemoryError::MaximumMemoryTooLarge {
+                        max_requested: max,
+                        max_allowed: Pages::max_value(),
+                    });
                 }
-                MaybeInstanceOwned::Instance(mem_loc)
-            } else {
-                MaybeInstanceOwned::Host(Box::new(UnsafeCell::new(VMMemoryDefinition {
-                    base: base_ptr,
-                    current_length: mem_length,
-                })))
-            },
-            alloc,
-            size: Bytes::from(mem_length).try_into().unwrap(),
-        };
+                if max < memory.minimum {
+                    return Err(MemoryError::InvalidMemory {
+                        reason: format!(
+                            "the maximum ({} pages) is less than the minimum ({} pages)",
+                            max.0, memory.minimum.0
+                        ),
+                    });
+                }
+            }
 
-        Ok(Self {
-            mmap,
-            config: VMMemoryConfig {
-                maximum: memory.maximum,
-                offset_guard_size: offset_guard_bytes,
-                memory: *memory,
-                style: *style,
-            },
-        })
+            let offset_guard_bytes = style.offset_guard_size() as usize;
+
+            let minimum_pages = match style {
+                MemoryStyle::Dynamic { .. } => memory.minimum,
+                MemoryStyle::Static { bound, .. } => {
+                    assert_ge!(*bound, memory.minimum);
+                    *bound
+                }
+            };
+            let minimum_bytes = minimum_pages.bytes().0;
+            let request_bytes = minimum_bytes.checked_add(offset_guard_bytes).unwrap();
+            let mapped_pages = memory.minimum;
+            let mapped_bytes = mapped_pages.bytes();
+
+            let mut alloc =
+                Mmap::accessible_reserved(mapped_bytes.0, request_bytes, backing_file, memory_type)
+                    .map_err(MemoryError::Region)?;
+
+            let base_ptr = alloc.as_mut_ptr();
+            let mem_length = memory
+                .minimum
+                .bytes()
+                .0
+                .max(alloc.as_slice_accessible().len());
+            let mmap = WasmMmap {
+                vm_memory_definition: if let Some(mem_loc) = vm_memory_location {
+                    {
+                        let mut ptr = mem_loc;
+                        let md = ptr.as_mut();
+                        md.base = base_ptr;
+                        md.current_length = mem_length;
+                    }
+                    MaybeInstanceOwned::Instance(mem_loc)
+                } else {
+                    MaybeInstanceOwned::Host(Box::new(UnsafeCell::new(VMMemoryDefinition {
+                        base: base_ptr,
+                        current_length: mem_length,
+                    })))
+                },
+                alloc,
+                size: Bytes::from(mem_length).try_into().unwrap(),
+            };
+
+            Ok(Self {
+                mmap,
+                config: VMMemoryConfig {
+                    maximum: memory.maximum,
+                    offset_guard_size: offset_guard_bytes,
+                    memory: *memory,
+                    style: *style,
+                },
+            })
+        }
     }
 
     /// Converts this owned memory into shared memory
@@ -481,7 +497,9 @@ impl VMSharedMemory {
         style: &MemoryStyle,
         vm_memory_location: NonNull<VMMemoryDefinition>,
     ) -> Result<Self, MemoryError> {
-        Ok(VMOwnedMemory::from_definition(memory, style, vm_memory_location)?.to_shared())
+        unsafe {
+            Ok(VMOwnedMemory::from_definition(memory, style, vm_memory_location)?.to_shared())
+        }
     }
 
     /// Create a new linear memory instance with specified minimum and maximum number of wasm pages
@@ -500,14 +518,16 @@ impl VMSharedMemory {
         backing_file: Option<std::path::PathBuf>,
         memory_type: MmapType,
     ) -> Result<Self, MemoryError> {
-        Ok(VMOwnedMemory::from_definition_with_file(
-            memory,
-            style,
-            vm_memory_location,
-            backing_file,
-            memory_type,
-        )?
-        .to_shared())
+        unsafe {
+            Ok(VMOwnedMemory::from_definition_with_file(
+                memory,
+                style,
+                vm_memory_location,
+                backing_file,
+                memory_type,
+            )?
+            .to_shared())
+        }
     }
 
     /// Copies this memory to a new memory
@@ -583,16 +603,21 @@ impl LinearMemory for VMSharedMemory {
     }
 
     // Add current thread to waiter list
-    fn do_wait(
+    unsafe fn do_wait(
         &mut self,
-        dst: NotifyLocation,
+        dst: u32,
+        expected: ExpectedValue,
         timeout: Option<Duration>,
     ) -> Result<u32, WaiterError> {
-        self.conditions.do_wait(dst, timeout)
+        let dst = NotifyLocation {
+            address: dst,
+            memory_base: self.mmap.read().unwrap().alloc.as_ptr() as *mut _,
+        };
+        unsafe { self.conditions.do_wait(dst, expected, timeout) }
     }
 
     /// Notify waiters from the wait list. Return the number of waiters notified
-    fn do_notify(&mut self, dst: NotifyLocation, count: u32) -> u32 {
+    fn do_notify(&mut self, dst: u32, count: u32) -> u32 {
         self.conditions.do_notify(dst, count)
     }
 
@@ -671,7 +696,7 @@ impl LinearMemory for VMMemory {
 
     /// Initialize memory with data
     unsafe fn initialize_with_data(&self, start: usize, data: &[u8]) -> Result<(), Trap> {
-        self.0.initialize_with_data(start, data)
+        unsafe { self.0.initialize_with_data(start, data) }
     }
 
     /// Copies this memory to a new memory
@@ -680,16 +705,17 @@ impl LinearMemory for VMMemory {
     }
 
     // Add current thread to waiter list
-    fn do_wait(
+    unsafe fn do_wait(
         &mut self,
-        dst: NotifyLocation,
+        dst: u32,
+        expected: ExpectedValue,
         timeout: Option<Duration>,
     ) -> Result<u32, WaiterError> {
-        self.0.do_wait(dst, timeout)
+        unsafe { self.0.do_wait(dst, expected, timeout) }
     }
 
     /// Notify waiters from the wait list. Return the number of waiters notified
-    fn do_notify(&mut self, dst: NotifyLocation, count: u32) -> u32 {
+    fn do_notify(&mut self, dst: u32, count: u32) -> u32 {
         self.0.do_notify(dst, count)
     }
 
@@ -729,19 +755,21 @@ impl VMMemory {
         style: &MemoryStyle,
         vm_memory_location: NonNull<VMMemoryDefinition>,
     ) -> Result<Self, MemoryError> {
-        Ok(if memory.shared {
-            Self(Box::new(VMSharedMemory::from_definition(
-                memory,
-                style,
-                vm_memory_location,
-            )?))
-        } else {
-            Self(Box::new(VMOwnedMemory::from_definition(
-                memory,
-                style,
-                vm_memory_location,
-            )?))
-        })
+        unsafe {
+            Ok(if memory.shared {
+                Self(Box::new(VMSharedMemory::from_definition(
+                    memory,
+                    style,
+                    vm_memory_location,
+                )?))
+            } else {
+                Self(Box::new(VMOwnedMemory::from_definition(
+                    memory,
+                    style,
+                    vm_memory_location,
+                )?))
+            })
+        }
     }
 
     /// Creates VMMemory from a custom implementation - the following into implementations
@@ -768,12 +796,14 @@ pub unsafe fn initialize_memory_with_data(
     start: usize,
     data: &[u8],
 ) -> Result<(), Trap> {
-    let mem_slice = slice::from_raw_parts_mut(memory.base, memory.current_length);
-    let end = start + data.len();
-    let to_init = &mut mem_slice[start..end];
-    to_init.copy_from_slice(data);
+    unsafe {
+        let mem_slice = slice::from_raw_parts_mut(memory.base, memory.current_length);
+        let end = start + data.len();
+        let to_init = &mut mem_slice[start..end];
+        to_init.copy_from_slice(data);
 
-    Ok(())
+        Ok(())
+    }
 }
 
 /// Represents memory that is used by the WebAsssembly module
@@ -822,26 +852,35 @@ where
     /// This function is unsafe because WebAssembly specification requires that data is always set at initialization time.
     /// It should be the implementors responsibility to make sure this respects the spec
     unsafe fn initialize_with_data(&self, start: usize, data: &[u8]) -> Result<(), Trap> {
-        let memory = self.vmmemory().as_ref();
+        unsafe {
+            let memory = self.vmmemory().as_ref();
 
-        initialize_memory_with_data(memory, start, data)
+            initialize_memory_with_data(memory, start, data)
+        }
     }
 
     /// Copies this memory to a new memory
     fn copy(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError>;
 
-    /// Add current thread to the waiter hash, and wait until notified or timout.
-    /// Return 0 if the waiter has been notified, 2 if the timeout occured, or None if en error happened
-    fn do_wait(
+    /// Add current thread to the waiter hash, and wait until notified or timeout.
+    /// Return 0 if the waiter has been notified, 1 if there was a value mismatch,
+    /// or 2 if the timeout occurred.
+    ///
+    /// # Safety
+    /// the destination address must be a valid offset within this memory. It must also
+    /// be properly aligned for the expected value type; either 4-byte aligned for
+    /// `ExpectedValue::U32` or 8-byte aligned for `ExpectedValue::u64`.
+    unsafe fn do_wait(
         &mut self,
-        _dst: NotifyLocation,
+        _dst: u32,
+        _expected: ExpectedValue,
         _timeout: Option<Duration>,
     ) -> Result<u32, WaiterError> {
         Err(WaiterError::Unimplemented)
     }
 
     /// Notify waiters from the wait list. Return the number of waiters notified
-    fn do_notify(&mut self, _dst: NotifyLocation, _count: u32) -> u32 {
+    fn do_notify(&mut self, _dst: u32, _count: u32) -> u32 {
         0
     }
 
