@@ -15,6 +15,7 @@ pub trait StoreObject: Sized {
     /// List the objects in the store, mutably.
     fn list_mut(ctx: &mut StoreObjects) -> &mut Vec<Self>;
 }
+
 macro_rules! impl_context_object {
     ($($field:ident => $ty:ty,)*) => {
         $(
@@ -29,6 +30,7 @@ macro_rules! impl_context_object {
         )*
     };
 }
+
 impl_context_object! {
     functions => VMFunction,
     tables => VMTable,
@@ -115,7 +117,7 @@ impl StoreObjects {
     }
 
     /// Return an immutable iterator over all globals
-    pub fn iter_globals(&self) -> Iter<VMGlobal> {
+    pub fn iter_globals(&self) -> Iter<'_, VMGlobal> {
         self.globals.iter()
     }
 
@@ -239,7 +241,7 @@ pub struct InternalStoreHandle<T> {
 #[cfg(feature = "artifact-size")]
 impl<T> loupe::MemoryUsage for InternalStoreHandle<T> {
     fn size_of_val(&self, _tracker: &mut dyn loupe::MemoryUsageTracker) -> usize {
-        std::mem::size_of_val(&self)
+        std::mem::size_of_val(self)
     }
 }
 
