@@ -1,3 +1,4 @@
+#![allow(clippy::result_large_err)]
 use std::{
     collections::HashMap,
     future::Future,
@@ -8,6 +9,7 @@ use std::{
 };
 
 use anyhow::Context;
+use shared_buffer::OwnedBuffer;
 use virtual_fs::{AsyncReadExt, FileSystem};
 use wasmer::FunctionEnvMut;
 use wasmer_package::utils::from_bytes;
@@ -23,15 +25,16 @@ pub use self::{
     },
 };
 use crate::{
-    os::{command::Commands, task::TaskJoinHandle},
     Runtime, SpawnError, WasiEnv,
+    os::{command::Commands, task::TaskJoinHandle},
+    runtime::module_cache::HashedModuleData,
 };
 
 #[derive(Debug, Clone)]
 pub struct BinFactory {
     pub(crate) commands: Commands,
     runtime: Arc<dyn Runtime + Send + Sync + 'static>,
-    pub(crate) local: Arc<RwLock<HashMap<String, Option<BinaryPackage>>>>,
+    pub(crate) local: Arc<RwLock<HashMap<String, Option<Arc<BinaryPackage>>>>>,
 }
 
 impl BinFactory {
@@ -47,9 +50,9 @@ impl BinFactory {
         self.runtime.deref()
     }
 
-    pub fn set_binary(&self, name: &str, binary: BinaryPackage) {
+    pub fn set_binary(&self, name: &str, binary: &Arc<BinaryPackage>) {
         let mut cache = self.local.write().unwrap();
-        cache.insert(name.to_string(), Some(binary));
+        cache.insert(name.to_string(), Some(binary.clone()));
     }
 
     #[allow(clippy::await_holding_lock)]
@@ -57,7 +60,7 @@ impl BinFactory {
         &self,
         name: &str,
         fs: Option<&dyn FileSystem>,
-    ) -> Option<BinaryPackage> {
+    ) -> Option<Arc<BinaryPackage>> {
         self.get_executable(name, fs)
             .await
             .and_then(|executable| match executable {
@@ -84,15 +87,16 @@ impl BinFactory {
             // Execute
             match executable {
                 Executable::Wasm(bytes) => {
-                    spawn_exec_wasm(&bytes, name.as_str(), env, &self.runtime).await
+                    let data = HashedModuleData::new(bytes.clone());
+                    spawn_exec_wasm(data, name.as_str(), env, &self.runtime).await
                 }
                 Executable::BinaryPackage(pkg) => {
-                    // Get the command that is going to be executed
-                    let cmd = package_command_by_name(&pkg, name.as_str())?;
+                    {
+                        let cmd = package_command_by_name(&pkg, name.as_str())?;
+                        env.prepare_spawn(cmd);
+                    }
 
-                    env.prepare_spawn(cmd);
-
-                    spawn_exec(pkg, name.as_str(), env, &self.runtime).await
+                    spawn_exec(pkg.as_ref().clone(), name.as_str(), env, &self.runtime).await
                 }
             }
         })
@@ -125,7 +129,7 @@ impl BinFactory {
     ) -> Option<Executable> {
         let name = name.to_string();
 
-        // Fast path
+        // Return early if the path is already cached
         {
             let cache = self.local.read().unwrap();
             if let Some(data) = cache.get(&name) {
@@ -133,32 +137,31 @@ impl BinFactory {
             }
         }
 
-        // Slow path
         let mut cache = self.local.write().unwrap();
 
-        // Check the cache
+        // Check the cache again to avoid a race condition where the cache was populated inbetween the fast path and here
         if let Some(data) = cache.get(&name) {
             return data.clone().map(Executable::BinaryPackage);
         }
 
         // Check the filesystem for the file
-        if name.starts_with('/') {
-            if let Some(fs) = fs {
-                match load_executable_from_filesystem(fs, name.as_ref(), self.runtime()).await {
-                    Ok(executable) => {
-                        if let Executable::BinaryPackage(pkg) = &executable {
-                            cache.insert(name, Some(pkg.clone()));
-                        }
+        if name.starts_with('/')
+            && let Some(fs) = fs
+        {
+            match load_executable_from_filesystem(fs, name.as_ref(), self.runtime()).await {
+                Ok(executable) => {
+                    if let Executable::BinaryPackage(pkg) = &executable {
+                        cache.insert(name, Some(pkg.clone()));
+                    }
 
-                        return Some(executable);
-                    }
-                    Err(e) => {
-                        tracing::warn!(
-                            path = name,
-                            error = &*e,
-                            "Unable to load the package from disk"
-                        );
-                    }
+                    return Some(executable);
+                }
+                Err(e) => {
+                    tracing::warn!(
+                        path = name,
+                        error = &*e,
+                        "Unable to load the package from disk"
+                    );
                 }
             }
         }
@@ -170,8 +173,8 @@ impl BinFactory {
 }
 
 pub enum Executable {
-    Wasm(bytes::Bytes),
-    BinaryPackage(BinaryPackage),
+    Wasm(OwnedBuffer),
+    BinaryPackage(Arc<BinaryPackage>),
 }
 
 async fn load_executable_from_filesystem(
@@ -185,18 +188,35 @@ async fn load_executable_from_filesystem(
         .open(path)
         .context("Unable to open the file")?;
 
-    let mut data = Vec::with_capacity(f.size() as usize);
-    f.read_to_end(&mut data).await.context("Read failed")?;
+    // Fast path if the file is fully available in memory.
+    // Prevents redundant copying of the file data.
+    if let Some(buf) = f.as_owned_buffer() {
+        if wasmer_package::utils::is_container(buf.as_slice()) {
+            let bytes = buf.clone().into_bytes();
+            if let Ok(container) = from_bytes(bytes.clone()) {
+                let pkg = BinaryPackage::from_webc(&container, rt)
+                    .await
+                    .context("Unable to load the package")?;
 
-    let bytes: bytes::Bytes = data.into();
+                return Ok(Executable::BinaryPackage(Arc::new(pkg)));
+            }
+        }
 
-    if let Ok(container) = from_bytes(bytes.clone()) {
-        let pkg = BinaryPackage::from_webc(&container, rt)
-            .await
-            .context("Unable to load the package")?;
-
-        Ok(Executable::BinaryPackage(pkg))
+        Ok(Executable::Wasm(buf))
     } else {
-        Ok(Executable::Wasm(bytes))
+        let mut data = Vec::with_capacity(f.size() as usize);
+        f.read_to_end(&mut data).await.context("Read failed")?;
+
+        let bytes: bytes::Bytes = data.into();
+
+        if let Ok(container) = from_bytes(bytes.clone()) {
+            let pkg = BinaryPackage::from_webc(&container, rt)
+                .await
+                .context("Unable to load the package")?;
+
+            Ok(Executable::BinaryPackage(Arc::new(pkg)))
+        } else {
+            Ok(Executable::Wasm(OwnedBuffer::from_bytes(bytes)))
+        }
     }
 }
