@@ -2,23 +2,19 @@ use std::{
     borrow::Cow,
     collections::{HashMap, HashSet},
     path::PathBuf,
-    pin::Pin,
-    sync::{atomic::AtomicU64, Arc, RwLock, RwLockReadGuard, RwLockWriteGuard},
-    task::Context,
+    sync::{Arc, RwLock, RwLockReadGuard, RwLockWriteGuard, atomic::AtomicU64},
 };
 
-use futures::Future;
 use serde_derive::{Deserialize, Serialize};
 use std::sync::Mutex as StdMutex;
-use tokio::sync::{watch, Mutex as AsyncMutex};
+use tokio::sync::{Mutex as AsyncMutex, watch};
 use virtual_fs::{Pipe, PipeRx, PipeTx, VirtualFile};
 use wasmer_wasix_types::wasi::{EpollType, Fd as WasiFd, Fdflags, Fdflagsext, Filestat, Rights};
 
-use crate::{net::socket::InodeSocket, syscalls::EpollJoinWaker};
+use crate::net::socket::InodeSocket;
 
 use super::{
-    InodeGuard, InodeValFilePollGuard, InodeValFilePollGuardJoin, InodeValFilePollGuardMode,
-    InodeWeakGuard, NotificationInner,
+    InodeGuard, InodeValFilePollGuard, InodeValFilePollGuardMode, InodeWeakGuard, NotificationInner,
 };
 
 #[derive(Debug, Clone)]
@@ -78,11 +74,11 @@ pub struct InodeVal {
 }
 
 impl InodeVal {
-    pub fn read(&self) -> RwLockReadGuard<Kind> {
+    pub fn read(&self) -> RwLockReadGuard<'_, Kind> {
         self.kind.read().unwrap()
     }
 
-    pub fn write(&self) -> RwLockWriteGuard<Kind> {
+    pub fn write(&self) -> RwLockWriteGuard<'_, Kind> {
         self.kind.write().unwrap()
     }
 }
@@ -111,48 +107,32 @@ pub struct EpollInterest {
 
 /// Guard the cleans up the selector registrations
 #[derive(Debug)]
-pub enum EpollJoinGuard {
-    Join {
-        join_guard: InodeValFilePollGuardJoin,
-        epoll_waker: Arc<EpollJoinWaker>,
-    },
-    Handler {
-        fd_guard: InodeValFilePollGuard,
-    },
+pub struct EpollJoinGuard {
+    pub(crate) fd_guard: InodeValFilePollGuard,
 }
 impl Drop for EpollJoinGuard {
     fn drop(&mut self) {
-        if let Self::Handler { fd_guard, .. } = self {
-            if let InodeValFilePollGuardMode::Socket { inner } = &mut fd_guard.mode {
+        match &self.fd_guard.mode {
+            InodeValFilePollGuardMode::File(_) => {
+                // Intentionally ignored, epoll doesn't work with files
+            }
+            InodeValFilePollGuardMode::Socket { inner } => {
                 let mut inner = inner.protected.write().unwrap();
                 inner.remove_handler();
             }
-        }
-    }
-}
-impl EpollJoinGuard {
-    pub fn is_spent(&self) -> bool {
-        match self {
-            Self::Join { join_guard, .. } => join_guard.is_spent(),
-            Self::Handler { .. } => false,
-        }
-    }
-    pub fn renew(&mut self) {
-        if let Self::Join {
-            join_guard,
-            epoll_waker,
-        } = self
-        {
-            let fd = join_guard.fd();
-            join_guard.reset();
-
-            let waker = epoll_waker.as_waker();
-            let mut cx = Context::from_waker(&waker);
-            if Pin::new(join_guard).poll(&mut cx).is_ready() {
-                tracing::trace!(fd, "join renew already woken");
-                waker.wake();
-            } else {
-                tracing::trace!(fd, "join waker reinstalled");
+            InodeValFilePollGuardMode::EventNotifications(inner) => {
+                inner.remove_interest_handler();
+            }
+            InodeValFilePollGuardMode::DuplexPipe { pipe } => {
+                let inner = pipe.write().unwrap();
+                inner.remove_interest_handler();
+            }
+            InodeValFilePollGuardMode::PipeRx { rx } => {
+                let inner = rx.write().unwrap();
+                inner.remove_interest_handler();
+            }
+            InodeValFilePollGuardMode::PipeTx { .. } => {
+                // Intentionally ignored, the sending end of a pipe can't have an interest handler
             }
         }
     }
