@@ -15,11 +15,11 @@ use wasmer_wasix_types::{
     wasi::{Errno, EventFdReadwrite, Eventrwflags, Subscription},
 };
 
-use super::{notification::NotificationInner, InodeGuard, Kind};
+use super::{InodeGuard, Kind, notification::NotificationInner};
 use crate::{
     net::socket::{InodeSocketInner, InodeSocketKind},
-    state::{iterate_poll_events, PollEvent, PollEventSet, WasiState},
-    syscalls::{map_io_err, EventResult, EventResultType},
+    state::{PollEvent, PollEventSet, WasiState, iterate_poll_events},
+    syscalls::{EventResult, EventResultType, map_io_err},
     utils::{OwnedRwLockReadGuard, OwnedRwLockWriteGuard},
 };
 
@@ -91,11 +91,11 @@ impl std::fmt::Debug for InodeValFilePollGuard {
             }
             InodeValFilePollGuardMode::Socket { inner } => {
                 let inner = inner.protected.read().unwrap();
-                match inner.kind {
+                match &inner.kind {
                     InodeSocketKind::TcpListener { .. } => {
                         write!(f, "guard-tcp-listener(fd={}, peb={})", self.fd, self.peb)
                     }
-                    InodeSocketKind::TcpStream { ref socket, .. } => {
+                    InodeSocketKind::TcpStream { socket, .. } => {
                         if socket.is_closed() {
                             write!(
                                 f,
@@ -134,7 +134,6 @@ pub struct InodeValFilePollGuardJoin {
     fd: u32,
     peb: PollEventSet,
     subscription: Subscription,
-    spent: bool,
 }
 
 impl InodeValFilePollGuardJoin {
@@ -144,7 +143,6 @@ impl InodeValFilePollGuardJoin {
             fd: guard.fd,
             peb: guard.peb,
             subscription: guard.subscription,
-            spent: false,
         }
     }
     pub(crate) fn fd(&self) -> u32 {
@@ -153,22 +151,6 @@ impl InodeValFilePollGuardJoin {
     pub(crate) fn peb(&self) -> PollEventSet {
         self.peb
     }
-    pub fn is_spent(&self) -> bool {
-        self.spent
-    }
-    pub fn reset(&mut self) {
-        match &self.mode {
-            InodeValFilePollGuardMode::File(_) => {}
-            InodeValFilePollGuardMode::EventNotifications(inner) => {
-                inner.reset();
-            }
-            InodeValFilePollGuardMode::Socket { .. }
-            | InodeValFilePollGuardMode::PipeRx { .. }
-            | InodeValFilePollGuardMode::PipeTx { .. }
-            | InodeValFilePollGuardMode::DuplexPipe { .. } => {}
-        }
-        self.spent = false;
-    }
 }
 
 pub const POLL_GUARD_MAX_RET: usize = 4;
@@ -212,7 +194,7 @@ impl Future for InodeValFilePollGuardJoin {
                     file.poll_read_ready(cx)
                 }
                 InodeValFilePollGuardMode::EventNotifications(inner) => inner.poll(waker).map(Ok),
-                InodeValFilePollGuardMode::Socket { ref inner } => {
+                InodeValFilePollGuardMode::Socket { inner } => {
                     let mut guard = inner.protected.write().unwrap();
                     guard.poll_read_ready(cx)
                 }
@@ -245,7 +227,7 @@ impl Future for InodeValFilePollGuardJoin {
                             }))
                         }
                         Eventtype::Clock => Some(EventResultType::Clock(0)),
-                        Eventtype::Unknown => None,
+                        _ => None,
                     };
                     if let Some(inner) = inner {
                         ret.push((
@@ -281,7 +263,7 @@ impl Future for InodeValFilePollGuardJoin {
                             }))
                         }
                         Eventtype::Clock => Some(EventResultType::Clock(0)),
-                        Eventtype::Unknown => None,
+                        _ => None,
                     };
                     if let Some(inner) = inner {
                         ret.push((
@@ -311,7 +293,7 @@ impl Future for InodeValFilePollGuardJoin {
                     file.poll_write_ready(cx)
                 }
                 InodeValFilePollGuardMode::EventNotifications(inner) => inner.poll(waker).map(Ok),
-                InodeValFilePollGuardMode::Socket { ref inner } => {
+                InodeValFilePollGuardMode::Socket { inner } => {
                     let mut guard = inner.protected.write().unwrap();
                     guard.poll_write_ready(cx)
                 }
@@ -344,7 +326,7 @@ impl Future for InodeValFilePollGuardJoin {
                             }))
                         }
                         Eventtype::Clock => Some(EventResultType::Clock(0)),
-                        Eventtype::Unknown => None,
+                        _ => None,
                     };
                     if let Some(inner) = inner {
                         ret.push((
@@ -380,7 +362,7 @@ impl Future for InodeValFilePollGuardJoin {
                             }))
                         }
                         Eventtype::Clock => Some(EventResultType::Clock(0)),
-                        Eventtype::Unknown => None,
+                        _ => None,
                     };
                     if let Some(inner) = inner {
                         ret.push((
@@ -403,7 +385,6 @@ impl Future for InodeValFilePollGuardJoin {
             };
         }
         if !ret.is_empty() {
-            self.spent = true;
             return Poll::Ready(ret);
         }
         Poll::Pending
