@@ -9,6 +9,7 @@ mod fd;
 mod fd_list;
 mod inode_guard;
 mod notification;
+pub(crate) mod relative_path_hack;
 
 use std::{
     borrow::{Borrow, Cow},
@@ -17,8 +18,8 @@ use std::{
     path::{Component, Path, PathBuf},
     pin::Pin,
     sync::{
-        atomic::{AtomicBool, AtomicI32, AtomicU64, Ordering},
         Arc, Mutex, RwLock, Weak,
+        atomic::{AtomicBool, AtomicI32, AtomicU64, Ordering},
     },
     task::{Context, Poll},
 };
@@ -28,12 +29,15 @@ use crate::{
     net::socket::InodeSocketKind,
     state::{Stderr, Stdin, Stdout},
 };
-use futures::{future::BoxFuture, Future, TryStreamExt};
+use futures::{Future, TryStreamExt, future::BoxFuture};
 #[cfg(feature = "enable-serde")]
 use serde_derive::{Deserialize, Serialize};
 use tokio::io::AsyncWriteExt;
 use tracing::{debug, trace};
-use virtual_fs::{copy_reference, FileSystem, FsError, OpenOptions, VirtualFile};
+use virtual_fs::{
+    FileSystem, FsError, OpenOptions, UnionFileSystem, VirtualFile, copy_reference,
+    tmp_fs::TmpFileSystem,
+};
 use wasmer_config::package::PackageId;
 use wasmer_wasix_types::{
     types::{__WASI_STDERR_FILENO, __WASI_STDIN_FILENO, __WASI_STDOUT_FILENO},
@@ -46,11 +50,12 @@ use wasmer_wasix_types::{
 pub use self::fd::{EpollFd, EpollInterest, EpollJoinGuard, Fd, FdInner, InodeVal, Kind};
 pub(crate) use self::inode_guard::{
     InodeValFilePollGuard, InodeValFilePollGuardJoin, InodeValFilePollGuardMode,
-    InodeValFileReadGuard, InodeValFileWriteGuard, WasiStateFileGuard, POLL_GUARD_MAX_RET,
+    InodeValFileReadGuard, InodeValFileWriteGuard, POLL_GUARD_MAX_RET, WasiStateFileGuard,
 };
 pub use self::notification::NotificationInner;
+use self::relative_path_hack::RelativeOrAbsolutePathHack;
 use crate::syscalls::map_io_err;
-use crate::{bin_factory::BinaryPackage, state::PreopenedDir, ALL_RIGHTS};
+use crate::{ALL_RIGHTS, bin_factory::BinaryPackage, state::PreopenedDir};
 
 /// the fd value of the virtual root
 ///
@@ -362,91 +367,105 @@ impl Default for WasiInodes {
 
 #[derive(Debug, Clone)]
 pub enum WasiFsRoot {
-    Sandbox(Arc<virtual_fs::tmp_fs::TmpFileSystem>),
-    Backing(Arc<Box<dyn FileSystem>>),
-}
-
-impl WasiFsRoot {
-    /// Merge the contents of a filesystem into this one.
-    #[tracing::instrument(level = "debug", skip_all)]
-    pub(crate) async fn merge(
-        &self,
-        other: &Arc<dyn FileSystem + Send + Sync>,
-    ) -> Result<(), virtual_fs::FsError> {
-        match self {
-            WasiFsRoot::Sandbox(fs) => {
-                fs.union(other);
-                Ok(())
-            }
-            WasiFsRoot::Backing(fs) => {
-                merge_filesystems(other, fs).await?;
-                Ok(())
-            }
-        }
-    }
+    Sandbox(TmpFileSystem),
+    /// Dedicated canonical overlay representation.
+    ///
+    /// Overlays are the common form of the file system for WASIX packages.
+    /// Dependencies are all added to the overlay, with the regular file system
+    /// as the foundation.
+    ///
+    /// This dedicated variant is necessary to norm the behaviour and prevent
+    /// redundant recursive merging of filesystems when additional dependencies
+    /// are added.
+    Overlay(
+        Arc<
+            virtual_fs::OverlayFileSystem<
+                TmpFileSystem,
+                [RelativeOrAbsolutePathHack<UnionFileSystem>; 1],
+            >,
+        >,
+    ),
+    Backing(Arc<dyn FileSystem + Send + Sync>),
 }
 
 impl FileSystem for WasiFsRoot {
     fn readlink(&self, path: &Path) -> virtual_fs::Result<PathBuf> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.readlink(path),
-            WasiFsRoot::Backing(fs) => fs.readlink(path),
+            Self::Sandbox(fs) => fs.readlink(path),
+            Self::Overlay(overlay) => overlay.readlink(path),
+            Self::Backing(fs) => fs.readlink(path),
         }
     }
 
     fn read_dir(&self, path: &Path) -> virtual_fs::Result<virtual_fs::ReadDir> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.read_dir(path),
-            WasiFsRoot::Backing(fs) => fs.read_dir(path),
+            Self::Sandbox(fs) => fs.read_dir(path),
+            Self::Overlay(overlay) => overlay.read_dir(path),
+            Self::Backing(fs) => fs.read_dir(path),
         }
     }
+
     fn create_dir(&self, path: &Path) -> virtual_fs::Result<()> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.create_dir(path),
-            WasiFsRoot::Backing(fs) => fs.create_dir(path),
+            Self::Sandbox(fs) => fs.create_dir(path),
+            Self::Overlay(overlay) => overlay.create_dir(path),
+            Self::Backing(fs) => fs.create_dir(path),
         }
     }
+
     fn remove_dir(&self, path: &Path) -> virtual_fs::Result<()> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.remove_dir(path),
-            WasiFsRoot::Backing(fs) => fs.remove_dir(path),
+            Self::Sandbox(fs) => fs.remove_dir(path),
+            Self::Overlay(overlay) => overlay.remove_dir(path),
+            Self::Backing(fs) => fs.remove_dir(path),
         }
     }
+
     fn rename<'a>(&'a self, from: &Path, to: &Path) -> BoxFuture<'a, virtual_fs::Result<()>> {
         let from = from.to_owned();
         let to = to.to_owned();
         let this = self.clone();
         Box::pin(async move {
             match this {
-                WasiFsRoot::Sandbox(fs) => fs.rename(&from, &to).await,
-                WasiFsRoot::Backing(fs) => fs.rename(&from, &to).await,
+                Self::Sandbox(fs) => fs.rename(&from, &to).await,
+                Self::Overlay(overlay) => overlay.rename(&from, &to).await,
+                Self::Backing(fs) => fs.rename(&from, &to).await,
             }
         })
     }
+
     fn metadata(&self, path: &Path) -> virtual_fs::Result<virtual_fs::Metadata> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.metadata(path),
-            WasiFsRoot::Backing(fs) => fs.metadata(path),
+            Self::Sandbox(fs) => fs.metadata(path),
+            Self::Overlay(overlay) => overlay.metadata(path),
+            Self::Backing(fs) => fs.metadata(path),
         }
     }
+
     fn symlink_metadata(&self, path: &Path) -> virtual_fs::Result<virtual_fs::Metadata> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.symlink_metadata(path),
-            WasiFsRoot::Backing(fs) => fs.symlink_metadata(path),
+            Self::Sandbox(fs) => fs.symlink_metadata(path),
+            Self::Overlay(overlay) => overlay.symlink_metadata(path),
+            Self::Backing(fs) => fs.symlink_metadata(path),
         }
     }
+
     fn remove_file(&self, path: &Path) -> virtual_fs::Result<()> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.remove_file(path),
-            WasiFsRoot::Backing(fs) => fs.remove_file(path),
+            Self::Sandbox(fs) => fs.remove_file(path),
+            Self::Overlay(overlay) => overlay.remove_file(path),
+            Self::Backing(fs) => fs.remove_file(path),
         }
     }
-    fn new_open_options(&self) -> OpenOptions {
+
+    fn new_open_options(&self) -> OpenOptions<'_> {
         match self {
-            WasiFsRoot::Sandbox(fs) => fs.new_open_options(),
-            WasiFsRoot::Backing(fs) => fs.new_open_options(),
+            Self::Sandbox(fs) => fs.new_open_options(),
+            Self::Overlay(overlay) => overlay.new_open_options(),
+            Self::Backing(fs) => fs.new_open_options(),
         }
     }
+
     fn mount(
         &self,
         name: String,
@@ -454,20 +473,25 @@ impl FileSystem for WasiFsRoot {
         fs: Box<dyn FileSystem + Send + Sync>,
     ) -> virtual_fs::Result<()> {
         match self {
-            WasiFsRoot::Sandbox(f) => f.mount(name, path, fs),
-            WasiFsRoot::Backing(f) => f.mount(name, path, fs),
+            Self::Sandbox(root) => FileSystem::mount(root, name, path, fs),
+            Self::Overlay(overlay) => FileSystem::mount(overlay.primary(), name, path, fs),
+            Self::Backing(f) => f.mount(name, path, fs),
         }
     }
 }
 
 /// Merge the contents of one filesystem into another.
 ///
+/// NOTE: merging is a very expensive operation, since it requires copying
+/// many files in memory, even if the underlying files are immutable and
+/// mapped through mmap or similar mechanisms.
+/// Merging should be avoided when possible.
 #[tracing::instrument(level = "trace", skip_all)]
 async fn merge_filesystems(
     source: &dyn FileSystem,
     destination: &dyn FileSystem,
 ) -> Result<(), virtual_fs::FsError> {
-    tracing::debug!("Falling back to a recursive copy to merge filesystems");
+    tracing::warn!("Falling back to a recursive copy to merge filesystems");
     let files = futures::stream::FuturesUnordered::new();
 
     let mut to_check = VecDeque::new();
@@ -527,12 +551,12 @@ fn create_dir_all(fs: &dyn FileSystem, path: &Path) -> Result<(), virtual_fs::Fs
 pub struct WasiFs {
     //pub repo: Repo,
     pub preopen_fds: RwLock<Vec<u32>>,
-    pub fd_map: Arc<RwLock<FdList>>,
+    pub fd_map: RwLock<FdList>,
     pub current_dir: Mutex<String>,
     #[cfg_attr(feature = "enable-serde", serde(skip, default))]
     pub root_fs: WasiFsRoot,
     pub root_inode: InodeGuard,
-    pub has_unioned: Arc<Mutex<HashSet<PackageId>>>,
+    pub has_unioned: Mutex<HashSet<PackageId>>,
 
     // TODO: remove
     // using an atomic is a hack to enable customization after construction,
@@ -559,15 +583,14 @@ impl WasiFs {
 
     /// Forking the WasiState is used when either fork or vfork is called
     pub fn fork(&self) -> Self {
-        let fd_map = self.fd_map.read().unwrap().clone();
         Self {
             preopen_fds: RwLock::new(self.preopen_fds.read().unwrap().clone()),
-            fd_map: Arc::new(RwLock::new(fd_map)),
+            fd_map: RwLock::new(self.fd_map.read().unwrap().clone()),
             current_dir: Mutex::new(self.current_dir.lock().unwrap().clone()),
             is_wasix: AtomicBool::new(self.is_wasix.load(Ordering::Acquire)),
             root_fs: self.root_fs.clone(),
             root_inode: self.root_inode.clone(),
-            has_unioned: Arc::new(Mutex::new(HashSet::new())),
+            has_unioned: Mutex::new(self.has_unioned.lock().unwrap().clone()),
             init_preopens: self.init_preopens.clone(),
             init_vfs_preopens: self.init_vfs_preopens.clone(),
         }
@@ -639,15 +662,28 @@ impl WasiFs {
         &self,
         binary: &BinaryPackage,
     ) -> Result<(), virtual_fs::FsError> {
-        let needs_to_be_unioned = self.has_unioned.lock().unwrap().insert(binary.id.clone());
+        let Some(webc_fs) = &binary.webc_fs else {
+            return Ok(());
+        };
 
+        let needs_to_be_unioned = self.has_unioned.lock().unwrap().insert(binary.id.clone());
         if !needs_to_be_unioned {
             return Ok(());
         }
 
-        self.root_fs.merge(&binary.webc_fs).await?;
-
-        Ok(())
+        match &self.root_fs {
+            WasiFsRoot::Sandbox(fs) => {
+                // TODO: this can be changed to switch to Self::Overlay instead!
+                let fdyn: Arc<dyn FileSystem + Send + Sync> = webc_fs.clone();
+                fs.union(&fdyn);
+                Ok(())
+            }
+            WasiFsRoot::Overlay(overlay) => {
+                let union = &overlay.secondaries()[0];
+                union.0.merge(webc_fs, virtual_fs::UnionMergeMode::Skip)
+            }
+            WasiFsRoot::Backing(backing) => merge_filesystems(webc_fs, backing).await,
+        }
     }
 
     /// Created for the builder API. like `new` but with more information
@@ -665,15 +701,13 @@ impl WasiFs {
     }
 
     /// Converts a relative path into an absolute path
-    pub(crate) fn relative_path_to_absolute(&self, mut path: String) -> String {
-        if !path.starts_with("/") {
-            let current_dir = self.current_dir.lock().unwrap();
-            path = format!("{}{}", current_dir.as_str(), &path[1..]);
-            if path.contains("//") {
-                path = path.replace("//", "/");
-            }
+    pub(crate) fn relative_path_to_absolute(&self, path: String) -> String {
+        if path.starts_with('/') {
+            return path;
         }
-        path
+
+        let current_dir = self.current_dir.lock().unwrap();
+        format!("{}/{}", current_dir.trim_end_matches('/'), path)
     }
 
     /// Private helper function to init the filesystem, called in `new` and
@@ -702,12 +736,12 @@ impl WasiFs {
 
         let wasi_fs = Self {
             preopen_fds: RwLock::new(vec![]),
-            fd_map: Arc::new(RwLock::new(FdList::new())),
+            fd_map: RwLock::new(FdList::new()),
             current_dir: Mutex::new("/".to_string()),
             is_wasix: AtomicBool::new(false),
             root_fs: fs_backing,
             root_inode,
-            has_unioned: Arc::new(Mutex::new(HashSet::new())),
+            has_unioned: Mutex::new(HashSet::new()),
             init_preopens: Default::default(),
             init_vfs_preopens: Default::default(),
         };
@@ -750,7 +784,7 @@ impl WasiFs {
             let segment_name = c.as_os_str().to_string_lossy().to_string();
             let guard = cur_inode.read();
             match guard.deref() {
-                Kind::Dir { ref entries, .. } | Kind::Root { ref entries } => {
+                Kind::Dir { entries, .. } | Kind::Root { entries } => {
                     if let Some(_entry) = entries.get(&segment_name) {
                         // TODO: this should be fixed
                         return Err(FsError::AlreadyExists);
@@ -774,10 +808,7 @@ impl WasiFs {
                     {
                         let mut guard = cur_inode.write();
                         match guard.deref_mut() {
-                            Kind::Dir {
-                                ref mut entries, ..
-                            }
-                            | Kind::Root { ref mut entries } => {
+                            Kind::Dir { entries, .. } | Kind::Root { entries } => {
                                 entries.insert(segment_name, inode.clone());
                             }
                             _ => unreachable!("Dir or Root became not Dir or Root"),
@@ -824,7 +855,7 @@ impl WasiFs {
 
         let guard = base_inode.read();
         match guard.deref() {
-            Kind::Dir { ref entries, .. } | Kind::Root { ref entries } => {
+            Kind::Dir { entries, .. } | Kind::Root { entries } => {
                 if let Some(_entry) = entries.get(&name) {
                     // TODO: eventually change the logic here to allow overwrites
                     return Err(FsError::AlreadyExists);
@@ -844,10 +875,7 @@ impl WasiFs {
                 {
                     let mut guard = base_inode.write();
                     match guard.deref_mut() {
-                        Kind::Dir {
-                            ref mut entries, ..
-                        }
-                        | Kind::Root { ref mut entries } => {
+                        Kind::Dir { entries, .. } | Kind::Root { entries } => {
                             entries.insert(name, inode.clone());
                         }
                         _ => unreachable!("Dir or Root became not Dir or Root"),
@@ -868,10 +896,11 @@ impl WasiFs {
 
                 {
                     let mut guard = inode.kind.write().unwrap();
-                    if let Kind::File { ref mut fd, .. } = *guard {
-                        *fd = Some(real_fd);
-                    } else {
-                        unreachable!("We just created a Kind::File");
+                    match guard.deref_mut() {
+                        Kind::File { fd, .. } => {
+                            *fd = Some(real_fd);
+                        }
+                        _ => unreachable!("We just created a Kind::File"),
                     }
                 }
 
@@ -909,7 +938,7 @@ impl WasiFs {
                     // happy path
                     let guard = base_inode.read();
                     match guard.deref() {
-                        Kind::File { ref handle, .. } => {
+                        Kind::File { handle, .. } => {
                             if let Some(handle) = handle {
                                 let mut handle = handle.write().unwrap();
                                 std::mem::swap(handle.deref_mut(), &mut file);
@@ -922,7 +951,7 @@ impl WasiFs {
                 // slow path
                 let mut guard = base_inode.write();
                 match guard.deref_mut() {
-                    Kind::File { ref mut handle, .. } => {
+                    Kind::File { handle, .. } => {
                         if let Some(handle) = handle {
                             let mut handle = handle.write().unwrap();
                             std::mem::swap(handle.deref_mut(), &mut file);
@@ -1044,9 +1073,9 @@ impl WasiFs {
                 match guard.deref_mut() {
                     Kind::Buffer { .. } => unimplemented!("state::get_inode_at_path for buffers"),
                     Kind::Dir {
-                        ref mut entries,
-                        ref path,
-                        ref parent,
+                        entries,
+                        path,
+                        parent,
                         ..
                     } => {
                         match component.as_os_str().to_string_lossy().borrow() {
@@ -1134,7 +1163,9 @@ impl WasiFs {
                                         // a `SocketDgram`?
                                         Filetype::SocketStream
                                     } else {
-                                        unimplemented!("state::get_inode_at_path unknown file type: not file, directory, symlink, char device, block device, fifo, or socket");
+                                        unimplemented!(
+                                            "state::get_inode_at_path unknown file type: not file, directory, symlink, char device, block device, fifo, or socket"
+                                        );
                                     };
 
                                     let kind = Kind::File {
@@ -1160,10 +1191,7 @@ impl WasiFs {
                                     );
 
                                     let mut guard = cur_inode.write();
-                                    if let Kind::Dir {
-                                        ref mut entries, ..
-                                    } = guard.deref_mut()
-                                    {
+                                    if let Kind::Dir { entries, .. } = guard.deref_mut() {
                                         entries.insert(
                                             component.as_os_str().to_string_lossy().to_string(),
                                             new_inode.clone(),
@@ -1177,7 +1205,9 @@ impl WasiFs {
                                     return Ok(new_inode);
                                 }
                                 #[cfg(not(unix))]
-                                unimplemented!("state::get_inode_at_path unknown file type: not file, directory, or symlink");
+                                unimplemented!(
+                                    "state::get_inode_at_path unknown file type: not file, directory, or symlink"
+                                );
                             };
                             drop(guard);
 
@@ -1189,10 +1219,7 @@ impl WasiFs {
                             )?;
                             if should_insert {
                                 let mut guard = processing_cur_inode.write();
-                                if let Kind::Dir {
-                                    ref mut entries, ..
-                                } = guard.deref_mut()
-                                {
+                                if let Kind::Dir { entries, .. } = guard.deref_mut() {
                                     entries.insert(
                                         component.as_os_str().to_string_lossy().to_string(),
                                         new_inode.clone(),
@@ -1308,7 +1335,7 @@ impl WasiFs {
             },
         }
 
-        impl<'a> BaseFdAndRelPath<'a> {
+        impl BaseFdAndRelPath<'_> {
             const fn max_seen(&self) -> usize {
                 match self {
                     Self::None => 0,
@@ -1477,7 +1504,7 @@ impl WasiFs {
                     fs_flags: Fdflags::empty(),
                     fs_rights_base: STDIN_DEFAULT_RIGHTS,
                     fs_rights_inheriting: Rights::empty(),
-                })
+                });
             }
             __WASI_STDOUT_FILENO => {
                 return Ok(Fdstat {
@@ -1485,7 +1512,7 @@ impl WasiFs {
                     fs_flags: Fdflags::APPEND,
                     fs_rights_base: STDOUT_DEFAULT_RIGHTS,
                     fs_rights_inheriting: Rights::empty(),
-                })
+                });
             }
             __WASI_STDERR_FILENO => {
                 return Ok(Fdstat {
@@ -1493,7 +1520,7 @@ impl WasiFs {
                     fs_flags: Fdflags::APPEND,
                     fs_rights_base: STDERR_DEFAULT_RIGHTS,
                     fs_rights_inheriting: Rights::empty(),
-                })
+                });
             }
             VIRTUAL_ROOT_FD => {
                 return Ok(Fdstat {
@@ -1550,10 +1577,8 @@ impl WasiFs {
         Prestat {
             pr_type: Preopentype::Dir,
             u: PrestatEnum::Dir {
-                // REVIEW:
-                // no need for +1, because there is no 0 end-of-string marker
-                // john: removing the +1 seems cause regression issues
-                pr_name_len: inode_val.name.read().unwrap().len() as u32 + 1,
+                // WASI spec: pr_name_len is the length of the path string, NOT including null terminator
+                pr_name_len: inode_val.name.read().unwrap().len() as u32,
             }
             .untagged(),
         }
@@ -2139,9 +2164,10 @@ impl WasiFs {
                 let base_po_inode = &guard.get(*base_po_dir).unwrap().inode;
                 let guard = base_po_inode.read();
                 match guard.deref() {
-                    Kind::Root { .. } => {
-                        self.root_fs.symlink_metadata(path_to_symlink).map_err(fs_error_into_wasi_err)?
-                    }
+                    Kind::Root { .. } => self
+                        .root_fs
+                        .symlink_metadata(path_to_symlink)
+                        .map_err(fs_error_into_wasi_err)?,
                     Kind::Dir { path, .. } => {
                         let mut real_path = path.clone();
                         // PHASE 1: ignore all possible symlinks in `relative_path`
@@ -2151,10 +2177,14 @@ impl WasiFs {
                         // TODO: adjust size of symlink, too
                         //      for all paths adjusted think about this
                         real_path.push(path_to_symlink);
-                        self.root_fs.symlink_metadata(&real_path).map_err(fs_error_into_wasi_err)?
+                        self.root_fs
+                            .symlink_metadata(&real_path)
+                            .map_err(fs_error_into_wasi_err)?
                     }
                     // if this triggers, there's a bug in the symlink code
-                    _ => unreachable!("Symlink pointing to something that's not a directory as its base preopened directory"),
+                    _ => unreachable!(
+                        "Symlink pointing to something that's not a directory as its base preopened directory"
+                    ),
                 }
             }
             _ => return Err(Errno::Io),
@@ -2214,14 +2244,14 @@ impl std::fmt::Debug for WasiFs {
 }
 
 /// Returns the default filesystem backing
-pub fn default_fs_backing() -> Box<dyn virtual_fs::FileSystem + Send + Sync> {
+pub fn default_fs_backing() -> Arc<dyn virtual_fs::FileSystem + Send + Sync> {
     cfg_if::cfg_if! {
         if #[cfg(feature = "host-fs")] {
-            Box::new(virtual_fs::host_fs::FileSystem::new(tokio::runtime::Handle::current(), "/").unwrap())
+            Arc::new(virtual_fs::host_fs::FileSystem::new(tokio::runtime::Handle::current(), "/").unwrap())
         } else if #[cfg(not(feature = "host-fs"))] {
-            Box::<virtual_fs::mem_fs::FileSystem>::default()
+            Arc::<virtual_fs::mem_fs::FileSystem>::default()
         } else {
-            Box::<FallbackFileSystem>::default()
+            Arc::<FallbackFileSystem>::default()
         }
     }
 }
@@ -2231,7 +2261,9 @@ pub struct FallbackFileSystem;
 
 impl FallbackFileSystem {
     fn fail() -> ! {
-        panic!("No filesystem set for wasmer-wasi, please enable either the `host-fs` or `mem-fs` feature or set your custom filesystem with `WasiEnvBuilder::set_fs`");
+        panic!(
+            "No filesystem set for wasmer-wasi, please enable either the `host-fs` or `mem-fs` feature or set your custom filesystem with `WasiEnvBuilder::set_fs`"
+        );
     }
 }
 
@@ -2260,7 +2292,7 @@ impl FileSystem for FallbackFileSystem {
     fn remove_file(&self, _path: &Path) -> Result<(), FsError> {
         Self::fail();
     }
-    fn new_open_options(&self) -> virtual_fs::OpenOptions {
+    fn new_open_options(&self) -> virtual_fs::OpenOptions<'_> {
         Self::fail();
     }
     fn mount(
@@ -2342,3 +2374,94 @@ pub fn fs_error_into_wasi_err(fs_error: FsError) -> Errno {
         FsError::Unsupported => Errno::Notsup,
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[tokio::test]
+    async fn test_relative_path_to_absolute() {
+        let inodes = WasiInodes::new();
+        let fs_backing = WasiFsRoot::Sandbox(TmpFileSystem::new());
+        let wasi_fs = WasiFs::new_init(fs_backing, &inodes, FS_ROOT_INO).unwrap();
+
+        // Test absolute path (returned as-is, no normalization)
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("/foo/bar".to_string()),
+            "/foo/bar"
+        );
+        assert_eq!(wasi_fs.relative_path_to_absolute("/".to_string()), "/");
+
+        // Absolute paths with special components are not normalized
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("//foo//bar//".to_string()),
+            "//foo//bar//"
+        );
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("/a/b/./c".to_string()),
+            "/a/b/./c"
+        );
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("/a/b/../c".to_string()),
+            "/a/b/../c"
+        );
+
+        // Test relative path with root as current dir
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("foo/bar".to_string()),
+            "/foo/bar"
+        );
+        assert_eq!(wasi_fs.relative_path_to_absolute("foo".to_string()), "/foo");
+
+        // Test with different current directory
+        wasi_fs.set_current_dir("/home/user");
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("file.txt".to_string()),
+            "/home/user/file.txt"
+        );
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("dir/file.txt".to_string()),
+            "/home/user/dir/file.txt"
+        );
+
+        // Test relative paths with . and .. components
+        wasi_fs.set_current_dir("/a/b/c");
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("./file.txt".to_string()),
+            "/a/b/c/./file.txt"
+        );
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("../file.txt".to_string()),
+            "/a/b/c/../file.txt"
+        );
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("../../file.txt".to_string()),
+            "/a/b/c/../../file.txt"
+        );
+
+        // Test edge cases
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute(".".to_string()),
+            "/a/b/c/."
+        );
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("..".to_string()),
+            "/a/b/c/.."
+        );
+        assert_eq!(wasi_fs.relative_path_to_absolute("".to_string()), "/a/b/c/");
+
+        // Test current directory with trailing slash
+        wasi_fs.set_current_dir("/home/user/");
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("file.txt".to_string()),
+            "/home/user/file.txt"
+        );
+
+        // Test current directory without trailing slash
+        wasi_fs.set_current_dir("/home/user");
+        assert_eq!(
+            wasi_fs.relative_path_to_absolute("file.txt".to_string()),
+            "/home/user/file.txt"
+        );
+    }
+}
