@@ -16,11 +16,7 @@ impl JournalEffector {
     ) -> anyhow::Result<()> {
         crate::syscalls::fd_filestat_set_size_internal(ctx, fd, st_size).map_err(|err| {
             anyhow::format_err!(
-                "journal restore error: failed to set file size (fd={}, st_size={}) - {}",
-                fd,
-                st_size,
-                err
-            )
+                "journal restore error: failed to set file size (fd={fd}, st_size={st_size}) - {err}")
         })?;
         Ok(())
     }
