@@ -61,11 +61,7 @@ impl JournalEffector {
         )?
         .map_err(|err| {
             anyhow::format_err!(
-                "journal restore error: failed to write to descriptor (fd={}, offset={}) - {}",
-                fd,
-                offset,
-                err
-            )
+                "journal restore error: failed to write to descriptor (fd={fd}, offset={offset}) - {err}")
         })?;
         Ok(())
     }
