@@ -32,14 +32,7 @@ impl JournalEffector {
         crate::syscalls::path_link_internal(ctx, old_fd, old_flags, old_path, new_fd, new_path)
             .map_err(|err| {
                 anyhow::format_err!(
-                    "journal restore error: failed to create hard link (old_fd={}, old_flags={}, old_path={}, new_fd={}, new_path={}) - {}",
-                    old_fd,
-                    old_flags,
-                    old_path,
-                    new_fd,
-                    new_path,
-                    err
-                )
+                    "journal restore error: failed to create hard link (old_fd={old_fd}, old_flags={old_flags}, old_path={old_path}, new_fd={new_fd}, new_path={new_path}) - {err}")
             })?;
         Ok(())
     }
