@@ -30,10 +30,7 @@ impl JournalEffector {
         if fd == VIRTUAL_ROOT_FD {
             ctx.data().state.fs.root_fs.remove_dir(Path::new(path))?;
         } else if let Err(err) = crate::syscalls::path_remove_directory_internal(ctx, fd, path) {
-            bail!(
-                "journal restore error: failed to remove directory - {}",
-                err
-            );
+            bail!("journal restore error: failed to remove directory - {err}");
         }
         Ok(())
     }
