@@ -41,15 +41,7 @@ impl JournalEffector {
             crate::syscalls::path_filestat_set_times_internal(ctx, fd, flags, path, st_atim, st_mtim, fst_flags)
                 .map_err(|err| {
                     anyhow::format_err!(
-                        "journal restore error: failed to set path times (fd={}, flags={}, path={}, st_atim={}, st_mtim={}, fst_flags={:?}) - {}",
-                        fd,
-                        flags,
-                        path,
-                        st_atim,
-                        st_mtim,
-                        fst_flags,
-                        err
-                    )
+                        "journal restore error: failed to set path times (fd={fd}, flags={flags}, path={path}, st_atim={st_atim}, st_mtim={st_mtim}, fst_flags={fst_flags:?}) - {err}")
                 })?;
         }
         Ok(())
