@@ -21,11 +21,7 @@ impl JournalEffector {
             .unwrap_or_else(|err| Err(err.to_string()))
             .map_err(|err| {
                 anyhow::format_err!(
-                    "journal restore error: failed to bind socket to address (fd={}, addr={}) - {}",
-                    fd,
-                    addr,
-                    err
-                )
+                    "journal restore error: failed to bind socket to address (fd={fd}, addr={addr}) - {err}")
             })?;
         Ok(())
     }
