@@ -23,12 +23,7 @@ impl JournalEffector {
             .unwrap_or_else(|err| Err(err.to_string()))
             .map_err(|err| {
                 anyhow::format_err!(
-                    "journal restore error: failed to set socket option (fd={}, opt={:?}, flag={}) - {}",
-                    fd,
-                    opt,
-                    flag,
-                    err
-                )
+                    "journal restore error: failed to set socket option (fd={fd}, opt={opt:?}, flag={flag}) - {err}")
             })?;
         Ok(())
     }
