@@ -3,36 +3,43 @@ pub mod package_loader;
 pub mod resolver;
 pub mod task_manager;
 
-pub use self::task_manager::{SpawnMemoryType, VirtualTaskManager};
-use self::{module_cache::CacheError, task_manager::InlineWaker};
+use self::module_cache::CacheError;
+pub use self::task_manager::{SpawnType, VirtualTaskManager};
+use module_cache::HashedModuleData;
 use wasmer_config::package::SuggestedCompilerOptimizations;
 use wasmer_types::{
-    target::UserCompilerOptimizations as WasmerSuggestedCompilerOptimizations, ModuleHash,
+    CompilationProgressCallback, ModuleHash,
+    target::UserCompilerOptimizations as WasmerSuggestedCompilerOptimizations,
 };
 
 use std::{
+    borrow::Cow,
     fmt,
     ops::Deref,
     sync::{Arc, Mutex},
 };
 
 use futures::future::BoxFuture;
+use virtual_mio::block_on;
 use virtual_net::{DynVirtualNetworking, VirtualNetworking};
-use wasmer::{CompileError, Module, RuntimeError};
+use wasmer::{CompileError, Engine, Module, RuntimeError};
 use wasmer_wasix_types::wasi::ExitCode;
 
 #[cfg(feature = "journal")]
 use crate::journal::{DynJournal, DynReadableJournal};
 use crate::{
+    SpawnError, WasiTtyState,
     bin_factory::BinaryPackageCommand,
     http::{DynHttpClient, HttpClient},
     os::TtyBridge,
     runtime::{
-        module_cache::{ModuleCache, ThreadLocalCache},
+        module_cache::{
+            ModuleCache, ThreadLocalCache,
+            progress::{ModuleLoadProgress, ModuleLoadProgressReporter},
+        },
         package_loader::{PackageLoader, UnsupportedPackageLoader},
         resolver::{BackendSource, MultiSource, Source},
     },
-    SpawnError, WasiTtyState,
 };
 
 #[derive(Clone)]
@@ -40,6 +47,75 @@ pub enum TaintReason {
     UnknownWasiVersion,
     NonZeroExitCode(ExitCode),
     RuntimeError(RuntimeError),
+    DlSymbolResolutionFailed(String),
+}
+
+/// The input to load a module.
+///
+/// Exists because the semantics for resolving modules can vary between
+/// different sources.
+///
+/// All variants are wrapped in `Cow` to allow for zero-copy usage when possible.
+pub enum ModuleInput<'a> {
+    /// Raw bytes.
+    Bytes(Cow<'a, [u8]>),
+    /// Pre-hashed module data.
+    Hashed(Cow<'a, HashedModuleData>),
+    /// A binary package command.
+    Command(Cow<'a, BinaryPackageCommand>),
+}
+
+impl<'a> ModuleInput<'a> {
+    /// Convert to an owned version of the module input.
+    pub fn to_owned(&'a self) -> ModuleInput<'static> {
+        // The manual code below is needed due to compiler issues with the lifetime.
+        match self {
+            Self::Bytes(Cow::Borrowed(b)) => {
+                let v: Vec<u8> = (*b).to_owned();
+                let c: Cow<'static, [u8]> = Cow::from(v);
+                ModuleInput::Bytes(c)
+            }
+            Self::Bytes(Cow::Owned(b)) => ModuleInput::Bytes(Cow::Owned((*b).clone())),
+            Self::Hashed(Cow::Borrowed(h)) => ModuleInput::Hashed(Cow::Owned((*h).clone())),
+            Self::Hashed(Cow::Owned(h)) => ModuleInput::Hashed(Cow::Owned(h.clone())),
+            Self::Command(Cow::Borrowed(c)) => ModuleInput::Command(Cow::Owned((*c).clone())),
+            Self::Command(Cow::Owned(c)) => ModuleInput::Command(Cow::Owned(c.clone())),
+        }
+    }
+
+    /// Get the module hash.
+    ///
+    /// NOTE: may be expensive, depending on the variant.
+    pub fn hash(&self) -> ModuleHash {
+        match self {
+            Self::Bytes(b) => {
+                // Hash on the fly
+                ModuleHash::new(b)
+            }
+            Self::Hashed(hashed) => *hashed.hash(),
+            Self::Command(cmd) => *cmd.hash(),
+        }
+    }
+
+    /// Get the raw WebAssembly bytes.
+    pub fn wasm(&self) -> &[u8] {
+        match self {
+            Self::Bytes(b) => b,
+            Self::Hashed(hashed) => hashed.wasm().as_ref(),
+            Self::Command(cmd) => cmd.atom_ref().as_ref(),
+        }
+    }
+
+    /// Convert to a `HashedModuleData`.
+    ///
+    /// May involve cloning and hashing.
+    pub fn to_hashed(&self) -> HashedModuleData {
+        match self {
+            Self::Bytes(b) => HashedModuleData::new(b.as_ref()),
+            Self::Hashed(hashed) => hashed.as_ref().clone(),
+            Self::Command(cmd) => HashedModuleData::from_command(cmd),
+        }
+    }
 }
 
 /// Runtime components used when running WebAssembly programs.
@@ -76,14 +152,14 @@ where
     fn source(&self) -> Arc<dyn Source + Send + Sync>;
 
     /// Get a [`wasmer::Engine`] for module compilation.
-    fn engine(&self) -> wasmer::Engine {
-        wasmer::Engine::default()
+    fn engine(&self) -> Engine {
+        Engine::default()
     }
 
     fn engine_with_suggested_opts(
         &self,
         suggested_opts: &SuggestedCompilerOptimizations,
-    ) -> Result<wasmer::Engine, CompileError> {
+    ) -> Result<Engine, CompileError> {
         let mut engine = self.engine();
         engine.with_opts(&WasmerSuggestedCompilerOptimizations {
             pass_params: suggested_opts.pass_params,
@@ -112,56 +188,115 @@ where
         None
     }
 
+    /// The primary way to load a module given a module input.
+    ///
+    /// The engine to use can be optionally provided, otherwise the most appropriate engine
+    /// should be selected.
+    ///
+    /// An optional progress reporter callback can be provided to report progress during module loading.
+    fn resolve_module<'a>(
+        &'a self,
+        input: ModuleInput<'a>,
+        engine: Option<&Engine>,
+        on_progress: Option<ModuleLoadProgressReporter>,
+    ) -> BoxFuture<'a, Result<Module, SpawnError>> {
+        let data = input.to_hashed();
+
+        let engine = if let Some(e) = engine {
+            e.clone()
+        } else {
+            match &input {
+                ModuleInput::Bytes(_) => self.engine(),
+                ModuleInput::Hashed(_) => self.engine(),
+                ModuleInput::Command(cmd) => {
+                    match self
+                        .engine_with_suggested_opts(&cmd.as_ref().suggested_compiler_optimizations)
+                    {
+                        Ok(engine) => engine,
+                        Err(error) => {
+                            return Box::pin(async move {
+                                Err(SpawnError::CompileError {
+                                    module_hash: *data.hash(),
+                                    error,
+                                })
+                            });
+                        }
+                    }
+                }
+            }
+        };
+
+        let module_cache = self.module_cache();
+
+        let task = async move { load_module(&engine, &module_cache, input, on_progress).await };
+        Box::pin(task)
+    }
+
+    /// Sync variant of [`Self::resolve_module`].
+    fn resolve_module_sync(
+        &self,
+        input: ModuleInput<'_>,
+        engine: Option<&Engine>,
+        on_progress: Option<ModuleLoadProgressReporter>,
+    ) -> Result<Module, SpawnError> {
+        block_on(self.resolve_module(input, engine, on_progress))
+    }
+
     /// Load the module for a command.
     ///
+    /// Will load the module from the cache if possible, otherwise will compile.
+    ///
     /// NOTE: This always be preferred over [`Self::load_module`] to avoid
     /// re-hashing the module!
+    #[deprecated(since = "0.601.0", note = "Use `resolve_module` instead")]
     fn load_command_module(
         &self,
         cmd: &BinaryPackageCommand,
     ) -> BoxFuture<'_, Result<Module, SpawnError>> {
-        let hash = *cmd.hash();
-        let wasm = cmd.atom();
-        let module_cache = self.module_cache();
-
-        let engine = match self.engine_with_suggested_opts(&cmd.suggested_compiler_optimizations) {
-            Ok(engine) => engine,
-            Err(error) => {
-                return Box::pin(async move {
-                    Err(SpawnError::CompileError {
-                        module_hash: hash,
-                        error,
-                    })
-                })
-            }
-        };
-
-        let task = async move { load_module(&engine, &module_cache, &wasm, hash).await };
-
-        Box::pin(task)
+        self.resolve_module(ModuleInput::Command(Cow::Owned(cmd.clone())), None, None)
     }
 
     /// Sync version of [`Self::load_command_module`].
+    #[deprecated(since = "0.601.0", note = "Use `resolve_module_sync` instead")]
     fn load_command_module_sync(&self, cmd: &BinaryPackageCommand) -> Result<Module, SpawnError> {
-        InlineWaker::block_on(self.load_command_module(cmd))
+        block_on(self.resolve_module(ModuleInput::Command(Cow::Borrowed(cmd)), None, None))
     }
 
-    /// Load a a Webassembly module, trying to use a pre-compiled version if possible.
-    fn load_module<'a>(&'a self, wasm: &'a [u8]) -> BoxFuture<'a, Result<Module, SpawnError>> {
-        let engine = self.engine();
-        let module_cache = self.module_cache();
-        let hash = ModuleHash::xxhash(wasm);
-
-        let task = async move { load_module(&engine, &module_cache, wasm, hash).await };
-
-        Box::pin(task)
-    }
-
-    /// Load a a Webassembly module, trying to use a pre-compiled version if possible.
+    /// Load a WebAssembly module from raw bytes.
     ///
-    /// Non-async version of [`Self::load_module`].
+    /// Will load the module from the cache if possible, otherwise will compile.
+    #[deprecated(since = "0.601.0", note = "Use `resolve_module` instead")]
+    fn load_module<'a>(&'a self, wasm: &'a [u8]) -> BoxFuture<'a, Result<Module, SpawnError>> {
+        self.resolve_module(ModuleInput::Bytes(Cow::Borrowed(wasm)), None, None)
+    }
+
+    /// Synchronous version of [`Self::load_module`].
+    #[deprecated(
+        since = "0.601.0",
+        note = "Use `load_command_module` or `load_hashed_module` instead - this method can have high overhead"
+    )]
     fn load_module_sync(&self, wasm: &[u8]) -> Result<Module, SpawnError> {
-        InlineWaker::block_on(self.load_module(wasm))
+        block_on(self.resolve_module(ModuleInput::Bytes(Cow::Borrowed(wasm)), None, None))
+    }
+
+    /// Load a WebAssembly module from pre-hashed data.
+    ///
+    /// Will load the module from the cache if possible, otherwise will compile.
+    fn load_hashed_module(
+        &self,
+        module: HashedModuleData,
+        engine: Option<&Engine>,
+    ) -> BoxFuture<'_, Result<Module, SpawnError>> {
+        self.resolve_module(ModuleInput::Hashed(Cow::Owned(module)), engine, None)
+    }
+
+    /// Synchronous version of [`Self::load_hashed_module`].
+    fn load_hashed_module_sync(
+        &self,
+        wasm: HashedModuleData,
+        engine: Option<&Engine>,
+    ) -> Result<Module, SpawnError> {
+        block_on(self.resolve_module(ModuleInput::Hashed(Cow::Owned(wasm)), engine, None))
     }
 
     /// Callback thats invokes whenever the instance is tainted, tainting can occur
@@ -197,12 +332,20 @@ pub type DynRuntime = dyn Runtime + Send + Sync;
 // implementing [`Runtime::load_module`], so custom logic can be added on top.
 #[tracing::instrument(level = "debug", skip_all)]
 pub async fn load_module(
-    engine: &wasmer::Engine,
+    engine: &Engine,
     module_cache: &(dyn ModuleCache + Send + Sync),
-    wasm: &[u8],
-    wasm_hash: ModuleHash,
+    input: ModuleInput<'_>,
+    on_progress: Option<ModuleLoadProgressReporter>,
 ) -> Result<Module, crate::SpawnError> {
-    let result = module_cache.load(wasm_hash, engine).await;
+    let wasm_hash = input.hash();
+
+    let result = if let Some(on_progress) = &on_progress {
+        module_cache
+            .load_with_progress(wasm_hash, engine, on_progress.clone())
+            .await
+    } else {
+        module_cache.load(wasm_hash, engine).await
+    };
 
     match result {
         Ok(module) => return Ok(module),
@@ -216,11 +359,34 @@ pub async fn load_module(
         }
     }
 
-    let module = Module::new(&engine, wasm).map_err(|err| crate::SpawnError::CompileError {
+    let res = if let Some(progress) = on_progress {
+        #[allow(unused_variables)]
+        let p = CompilationProgressCallback::new(move |p| {
+            progress.notify(ModuleLoadProgress::CompilingModule(p))
+        });
+        #[cfg(feature = "sys-default")]
+        {
+            if engine.is_sys() {
+                use wasmer::sys::NativeEngineExt;
+                engine.new_module_with_progress(input.wasm(), p)
+            } else {
+                Module::new(&engine, input.wasm())
+            }
+        }
+        #[cfg(not(feature = "sys-default"))]
+        {
+            Module::new(&engine, input.wasm())
+        }
+    } else {
+        Module::new(&engine, input.wasm())
+    };
+
+    let module = res.map_err(|err| crate::SpawnError::CompileError {
         module_hash: wasm_hash,
         error: err,
     })?;
 
+    // TODO: pass a [`HashedModule`] struct that is safe by construction.
     if let Err(e) = module_cache.save(wasm_hash, engine, &module).await {
         tracing::warn!(
             %wasm_hash,
@@ -263,7 +429,7 @@ pub struct PluggableRuntime {
     pub http_client: Option<DynHttpClient>,
     pub package_loader: Arc<dyn PackageLoader + Send + Sync>,
     pub source: Arc<dyn Source + Send + Sync>,
-    pub engine: Option<wasmer::Engine>,
+    pub engine: Engine,
     pub module_cache: Arc<dyn ModuleCache + Send + Sync>,
     pub tty: Option<Arc<dyn TtyBridge + Send + Sync>>,
     #[cfg(feature = "journal")]
@@ -299,7 +465,7 @@ impl PluggableRuntime {
             rt,
             networking,
             http_client,
-            engine: None,
+            engine: Default::default(),
             tty: None,
             source: Arc::new(source),
             package_loader: Arc::new(loader),
@@ -319,7 +485,7 @@ impl PluggableRuntime {
         self
     }
 
-    pub fn set_engine(&mut self, engine: Option<wasmer::Engine>) -> &mut Self {
+    pub fn set_engine(&mut self, engine: Engine) -> &mut Self {
         self.engine = engine;
         self
     }
@@ -388,15 +554,12 @@ impl Runtime for PluggableRuntime {
         Arc::clone(&self.source)
     }
 
-    fn engine(&self) -> wasmer::Engine {
-        self.engine.clone().unwrap_or_default()
+    fn engine(&self) -> Engine {
+        self.engine.clone()
     }
 
     fn new_store(&self) -> wasmer::Store {
-        self.engine
-            .clone()
-            .map(wasmer::Store::new)
-            .unwrap_or_default()
+        wasmer::Store::new(self.engine.clone())
     }
 
     fn task_manager(&self) -> &Arc<dyn VirtualTaskManager> {
@@ -437,7 +600,7 @@ pub struct OverriddenRuntime {
     http_client: Option<DynHttpClient>,
     package_loader: Option<Arc<dyn PackageLoader + Send + Sync>>,
     source: Option<Arc<dyn Source + Send + Sync>>,
-    engine: Option<wasmer::Engine>,
+    engine: Option<Engine>,
     module_cache: Option<Arc<dyn ModuleCache + Send + Sync>>,
     tty: Option<Arc<dyn TtyBridge + Send + Sync>>,
     #[cfg(feature = "journal")]
@@ -493,7 +656,7 @@ impl OverriddenRuntime {
         self
     }
 
-    pub fn with_engine(mut self, engine: wasmer::Engine) -> Self {
+    pub fn with_engine(mut self, engine: Engine) -> Self {
         self.engine.replace(engine);
         self
     }
@@ -562,7 +725,7 @@ impl Runtime for OverriddenRuntime {
         }
     }
 
-    fn engine(&self) -> wasmer::Engine {
+    fn engine(&self) -> Engine {
         if let Some(engine) = self.engine.clone() {
             engine
         } else {
@@ -620,25 +783,4 @@ impl Runtime for OverriddenRuntime {
             self.inner.active_journal()
         }
     }
-
-    fn load_module<'a>(&'a self, wasm: &'a [u8]) -> BoxFuture<'a, Result<Module, SpawnError>> {
-        if self.engine.is_some() || self.module_cache.is_some() {
-            let engine = self.engine();
-            let module_cache = self.module_cache();
-            let hash = ModuleHash::xxhash(wasm);
-
-            let task = async move { load_module(&engine, &module_cache, wasm, hash).await };
-            Box::pin(task)
-        } else {
-            self.inner.load_module(wasm)
-        }
-    }
-
-    fn load_module_sync(&self, wasm: &[u8]) -> Result<Module, SpawnError> {
-        if self.engine.is_some() || self.module_cache.is_some() {
-            InlineWaker::block_on(self.load_module(wasm))
-        } else {
-            self.inner.load_module_sync(wasm)
-        }
-    }
 }
