@@ -85,6 +85,14 @@ where
         Err(primary_error)
     }
 
+    async fn contains(&self, key: ModuleHash, engine: &Engine) -> Result<bool, CacheError> {
+        if self.primary.contains(key, engine).await? {
+            return Ok(true);
+        }
+
+        self.fallback.contains(key, engine).await
+    }
+
     async fn save(
         &self,
         key: ModuleHash,
@@ -156,6 +164,10 @@ mod tests {
             }
         }
 
+        async fn contains(&self, key: ModuleHash, engine: &Engine) -> Result<bool, CacheError> {
+            self.inner.contains(key, engine).await
+        }
+
         async fn save(
             &self,
             key: ModuleHash,
@@ -179,7 +191,7 @@ mod tests {
     async fn load_from_primary() {
         let engine = Engine::default();
         let module = Module::new(&engine, ADD_WAT).unwrap();
-        let key = ModuleHash::from_bytes([0; 8]);
+        let key = ModuleHash::from_bytes([0; _]);
         let primary = SharedCache::default();
         let fallback = SharedCache::default();
         primary.save(key, &engine, &module).await.unwrap();
@@ -204,7 +216,7 @@ mod tests {
     async fn loading_from_fallback_also_populates_primary() {
         let engine = Engine::default();
         let module = Module::new(&engine, ADD_WAT).unwrap();
-        let key = ModuleHash::from_bytes([0; 8]);
+        let key = ModuleHash::from_bytes([0; _]);
         let primary = SharedCache::default();
         let fallback = SharedCache::default();
         fallback.save(key, &engine, &module).await.unwrap();
@@ -230,7 +242,7 @@ mod tests {
     async fn saving_will_update_both() {
         let engine = Engine::default();
         let module = Module::new(&engine, ADD_WAT).unwrap();
-        let key = ModuleHash::from_bytes([0; 8]);
+        let key = ModuleHash::from_bytes([0; _]);
         let primary = SharedCache::default();
         let fallback = SharedCache::default();
         let cache = FallbackCache::new(&primary, &fallback);
