@@ -1,12 +1,11 @@
 use std::{
     collections::HashMap,
-    fmt::Write as _,
-    io::{ErrorKind, Write as _},
+    io::{ErrorKind, Read, Write as _},
     path::PathBuf,
     sync::{Arc, RwLock},
 };
 
-use anyhow::{Context, Error};
+use anyhow::{Context, Error, bail};
 use bytes::Bytes;
 use http::{HeaderMap, Method};
 use tempfile::NamedTempFile;
@@ -80,6 +79,10 @@ impl BuiltinPackageLoader {
         }
     }
 
+    pub fn cache(&self) -> Option<&FileSystemCache> {
+        self.cache.as_ref()
+    }
+
     pub fn validate_cache(
         &self,
         mode: CacheValidationMode,
@@ -161,13 +164,13 @@ impl BuiltinPackageLoader {
             return Ok(Some(cached));
         }
 
-        if let Some(cache) = self.cache.as_ref() {
-            if let Some(cached) = cache.lookup(hash).await? {
-                // Note: We want to propagate it to the in-memory cache, too
-                tracing::debug!("Copying from the filesystem cache to the in-memory cache");
-                self.in_memory.save(&cached, *hash);
-                return Ok(Some(cached));
-            }
+        if let Some(cache) = self.cache.as_ref()
+            && let Some(cached) = cache.lookup(hash).await?
+        {
+            // Note: We want to propagate it to the in-memory cache, too
+            tracing::debug!("Copying from the filesystem cache to the in-memory cache");
+            self.in_memory.save(&cached, *hash);
+            return Ok(Some(cached));
         }
 
         Ok(None)
@@ -280,6 +283,8 @@ impl BuiltinPackageLoader {
         }
 
         let body = response.body.context("package download failed")?;
+        let body = Self::decode_response_body(&response.headers, body)
+            .context("package download failed: could not decode response body")?;
         tracing::debug!(%url, "package_download_succeeded");
 
         let body = bytes::Bytes::from(body);
@@ -294,25 +299,90 @@ impl BuiltinPackageLoader {
         headers.insert("Accept", "application/webc".parse().unwrap());
         headers.insert("User-Agent", USER_AGENT.parse().unwrap());
 
-        if url.has_authority() {
-            if let Some(token) = self.tokens.get(url.authority()) {
-                let header = format!("Bearer {token}");
-                match header.parse() {
-                    Ok(header) => {
-                        headers.insert(http::header::AUTHORIZATION, header);
-                    }
-                    Err(e) => {
-                        tracing::warn!(
-                            error = &e as &dyn std::error::Error,
-                            "An error occurred while parsing the authorization header",
-                        );
-                    }
+        // Accept compressed responses.
+        // NOTE: gzip and zstd decoding is available on native platforms.
+        // In browser platforms, the fetch implementation should automatically
+        // handle decoding of gzip/zstd responses transparently.
+        headers.insert(
+            http::header::ACCEPT_ENCODING,
+            "zstd;q=1.0, gzip;q=0.8".parse().unwrap(),
+        );
+
+        if url.has_authority()
+            && let Some(token) = self.tokens.get(url.authority())
+        {
+            let header = format!("Bearer {token}");
+            match header.parse() {
+                Ok(header) => {
+                    headers.insert(http::header::AUTHORIZATION, header);
+                }
+                Err(e) => {
+                    tracing::warn!(
+                        error = &e as &dyn std::error::Error,
+                        "An error occurred while parsing the authorization header",
+                    );
                 }
             }
         }
 
         headers
     }
+
+    /// Decode the response body according to the `Content-Encoding` header.
+    ///
+    /// * Supports `gzip` and `zstd` encodings
+    /// * Supports nested encodings (e.g. `gzip, zstd`)
+    /// * Passes through unencoded bodies or "identity" encoding unchanged
+    fn decode_response_body(headers: &HeaderMap, body: Vec<u8>) -> Result<Vec<u8>, anyhow::Error> {
+        let encodings = match headers.get(http::header::CONTENT_ENCODING) {
+            Some(header) => header
+                .to_str()
+                .context("non-utf8 content-encoding header")?
+                .split(',')
+                .map(|encoding| encoding.trim().to_ascii_lowercase())
+                .filter(|encoding| !encoding.is_empty())
+                .collect::<Vec<_>>(),
+            None => Vec::new(),
+        };
+
+        // Check if there is nothing to decode, return early.
+        // "identity" is the default encoding meaning "no encoding" (See RFC 2616 / RFC 7231)
+        if encodings.is_empty() || (encodings.len() == 1 && encodings[0] == "identity") {
+            return Ok(body);
+        }
+
+        let mut reader: Box<dyn Read> = Box::new(std::io::Cursor::new(body));
+        for encoding in encodings.iter().rev() {
+            match encoding.as_str() {
+                "gzip" => {
+                    reader = Box::new(flate2::read::GzDecoder::new(reader));
+                }
+                "zstd" => {
+                    #[cfg(not(target_arch = "wasm32"))]
+                    {
+                        reader = Box::new(
+                            zstd::stream::read::Decoder::new(reader)
+                                .context("failed to initialize zstd decoder")?,
+                        );
+                    }
+                    #[cfg(target_arch = "wasm32")]
+                    {
+                        // NOTE: in browsers this code will not be hit because
+                        // the fetch API automatically handles content decoding.
+                        bail!("zstd content-encoding is not supported on wasm32");
+                    }
+                }
+                "identity" => {}
+                other => bail!("unsupported content-encoding: {other}"),
+            }
+        }
+
+        let mut decoded = Vec::new();
+        reader
+            .read_to_end(&mut decoded)
+            .context("failed to decode response body")?;
+        Ok(decoded)
+    }
 }
 
 impl Default for BuiltinPackageLoader {
@@ -419,13 +489,17 @@ pub enum CacheValidationMode {
 // FIXME: This implementation will block the async runtime and should use
 // some sort of spawn_blocking() call to run it in the background.
 #[derive(Debug)]
-struct FileSystemCache {
+pub struct FileSystemCache {
     cache_dir: PathBuf,
 }
 
 impl FileSystemCache {
     const FILE_SUFFIX: &'static str = ".bin";
 
+    fn temp_dir(&self) -> PathBuf {
+        self.cache_dir.join("__temp__")
+    }
+
     /// Validate that the cached image file names correspond to their actual
     /// file content hashes.
     fn validate_hashes(&self) -> Result<Vec<(PathBuf, ImageHashMismatchError)>, anyhow::Error> {
@@ -506,20 +580,25 @@ impl FileSystemCache {
         }
     }
 
-    async fn save(&self, webc: Bytes, dist: &DistributionInfo) -> Result<(), Error> {
+    async fn save(&self, webc: Bytes, dist: &DistributionInfo) -> Result<PathBuf, Error> {
         let path = self.path(&dist.webc_sha256);
         let dist = dist.clone();
+        let temp_dir = self.temp_dir();
 
+        let path2 = path.clone();
         crate::spawn_blocking(move || {
-            let parent = path.parent().expect("Always within cache_dir");
+            // Keep files in a temporary directory until they are fully written
+            // to prevent temp files being included in [`Self::scan`] or `[Self::retain]`.
 
-            std::fs::create_dir_all(parent)
-                .with_context(|| format!("Unable to create \"{}\"", parent.display()))?;
+            std::fs::create_dir_all(&temp_dir)
+                .with_context(|| format!("Unable to create directory '{}'", temp_dir.display()))?;
 
-            let mut temp = NamedTempFile::new_in(parent)?;
+            let mut temp = NamedTempFile::new_in(&temp_dir)?;
             temp.write_all(&webc)?;
             temp.flush()?;
             temp.as_file_mut().sync_all()?;
+
+            // Move the temporary file to the final location.
             temp.persist(&path)?;
 
             tracing::debug!(
@@ -533,7 +612,7 @@ impl FileSystemCache {
         })
         .await??;
 
-        Ok(())
+        Ok(path2)
     }
 
     #[tracing::instrument(level = "debug", skip_all)]
@@ -559,14 +638,102 @@ impl FileSystemCache {
     }
 
     fn path(&self, hash: &WebcHash) -> PathBuf {
-        let hash = hash.as_bytes();
-        let mut filename = String::with_capacity(hash.len() * 2);
-        for b in hash {
-            write!(filename, "{b:02x}").unwrap();
-        }
-        filename.push_str(Self::FILE_SUFFIX);
+        self.cache_dir.join(format!(
+            "{}{}",
+            hex::encode(hash.as_bytes()),
+            Self::FILE_SUFFIX
+        ))
+    }
 
-        self.cache_dir.join(filename)
+    /// Scan all the cached webc files and invoke the callback for each.
+    pub async fn scan<S, F>(&self, state: S, callback: F) -> Result<S, Error>
+    where
+        S: Send + 'static,
+        F: Fn(&mut S, &std::fs::DirEntry) -> Result<(), Error> + Send + 'static,
+    {
+        let cache_dir = self.cache_dir.clone();
+        tokio::task::spawn_blocking(move || -> Result<S, anyhow::Error> {
+            let mut state = state;
+
+            let iter = match std::fs::read_dir(&cache_dir) {
+                Ok(v) => v,
+                Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
+                    // path does not exist, so nothing to scan.
+                    return Ok(state);
+                }
+                Err(err) => {
+                    return Err(err).with_context(|| {
+                        format!("Could not read image cache dir: '{}'", cache_dir.display())
+                    });
+                }
+            };
+
+            for res in iter {
+                let entry = res?;
+                if !entry.file_type()?.is_file() {
+                    continue;
+                }
+
+                callback(&mut state, &entry)?;
+            }
+
+            Ok(state)
+        })
+        .await?
+        .context("tokio runtime failed")
+    }
+
+    /// Remove entries from the cache that do not pass the callback.
+    pub async fn retain<S, F>(&self, state: S, filter: F) -> Result<S, Error>
+    where
+        S: Send + 'static,
+        F: Fn(&mut S, &std::fs::DirEntry) -> Result<bool, anyhow::Error> + Send + 'static,
+    {
+        let cache_dir = self.cache_dir.clone();
+        tokio::task::spawn_blocking(move || {
+            let iter = match std::fs::read_dir(&cache_dir) {
+                Ok(v) => v,
+                Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
+                    // path does not exist, so nothing to scan.
+                    return Ok(state);
+                }
+                Err(err) => {
+                    return Err(err).with_context(|| {
+                        format!("Could not read image cache dir: '{}'", cache_dir.display())
+                    });
+                }
+            };
+
+            let mut state = state;
+            for res in iter {
+                let entry = res?;
+                if !entry.file_type()?.is_file() {
+                    continue;
+                }
+
+                if !filter(&mut state, &entry)? {
+                    tracing::debug!(
+                        path=%entry.path().display(),
+                        "Removing cached image file - does not pass the filter",
+                    );
+                    match std::fs::remove_file(entry.path()) {
+                        Ok(()) => {}
+                        Err(error) if error.kind() == std::io::ErrorKind::NotFound => {}
+                        Err(fs_err) => {
+                            tracing::warn!(
+                                path=%entry.path().display(),
+                                ?fs_err,
+                                "Could not delete cached image file",
+                            );
+                        }
+                    }
+                }
+            }
+
+            Ok(state)
+        })
+        .await?
+        .context("tokio runtime failed")
     }
 }
 
@@ -586,10 +753,10 @@ impl InMemoryCache {
 
 #[cfg(test)]
 mod tests {
-    use std::{collections::VecDeque, sync::Mutex};
+    use std::{collections::VecDeque, io::Write, sync::Mutex};
 
     use futures::future::BoxFuture;
-    use http::{HeaderMap, StatusCode};
+    use http::{HeaderMap, HeaderValue, StatusCode};
     use tempfile::TempDir;
     use wasmer_config::package::PackageId;
 
@@ -660,7 +827,16 @@ mod tests {
         let request = &requests[0];
         assert_eq!(request.url, summary.dist.webc);
         assert_eq!(request.method, "GET");
-        assert_eq!(request.headers.len(), 2);
+        #[cfg(not(target_arch = "wasm32"))]
+        {
+            assert_eq!(request.headers.len(), 3);
+            assert_eq!(request.headers["Accept-Encoding"], "zstd;q=1.0, gzip;q=0.8");
+        }
+        #[cfg(target_arch = "wasm32")]
+        {
+            assert_eq!(request.headers.len(), 2);
+            assert!(!request.headers.contains_key(http::header::ACCEPT_ENCODING));
+        }
         assert_eq!(request.headers["Accept"], "application/webc");
         assert_eq!(request.headers["User-Agent"], USER_AGENT);
         // Make sure we got the right package
@@ -690,11 +866,165 @@ mod tests {
     async fn cache_misses_will_trigger_a_download() {
         cache_misses_will_trigger_a_download_internal().await
     }
-}
 
-#[cfg(test)]
-mod test {
-    use super::*;
+    /// Small helper to construct headers with a given content-encoding.
+    fn headers_with_encoding(content_encoding: Option<&str>) -> HeaderMap {
+        let mut headers = HeaderMap::new();
+        if let Some(value) = content_encoding {
+            headers.insert(http::header::CONTENT_ENCODING, value.parse().unwrap());
+        }
+        headers
+    }
+
+    /// Small helper to construct headers with a raw content-encoding value.
+    fn headers_with_raw_encoding(value: &[u8]) -> HeaderMap {
+        let mut headers = HeaderMap::new();
+        headers.insert(
+            http::header::CONTENT_ENCODING,
+            HeaderValue::from_bytes(value).unwrap(),
+        );
+        headers
+    }
+
+    /// Confirm decode_response_body passes through unencoded bodies unchanged.
+    #[test]
+    fn decode_response_body_passthrough() {
+        let body = b"plain-bytes".to_vec();
+
+        let decoded =
+            BuiltinPackageLoader::decode_response_body(&headers_with_encoding(None), body.clone())
+                .unwrap();
+        assert_eq!(decoded, body);
+
+        let decoded = BuiltinPackageLoader::decode_response_body(
+            &headers_with_encoding(Some("identity")),
+            body.clone(),
+        )
+        .unwrap();
+        assert_eq!(decoded, body);
+    }
+
+    /// Confirm decode_response_body treats empty/whitespace encoding lists as no encoding.
+    #[test]
+    fn decode_response_body_empty_encoding_list() {
+        let body = b"plain-bytes".to_vec();
+        let decoded = BuiltinPackageLoader::decode_response_body(
+            &headers_with_encoding(Some(" , , ")),
+            body.clone(),
+        )
+        .unwrap();
+        assert_eq!(decoded, body);
+    }
+
+    /// Confirm decode_response_body errors on non-utf8 content-encoding headers.
+    #[test]
+    fn decode_response_body_non_utf8_encoding_header() {
+        let body = b"bytes".to_vec();
+        let err =
+            BuiltinPackageLoader::decode_response_body(&headers_with_raw_encoding(&[0xff]), body)
+                .unwrap_err();
+        let msg = err.to_string();
+        assert!(msg.contains("non-utf8 content-encoding"));
+    }
+
+    /// Confirm decode_response_body decodes gzip-encoded bodies.
+    #[test]
+    fn decode_response_body_gzip() {
+        let body = b"gzip-bytes".to_vec();
+        let mut encoder = flate2::write::GzEncoder::new(Vec::new(), flate2::Compression::default());
+        encoder.write_all(&body).unwrap();
+        let encoded = encoder.finish().unwrap();
+
+        let decoded = BuiltinPackageLoader::decode_response_body(
+            &headers_with_encoding(Some("gzip")),
+            encoded,
+        )
+        .unwrap();
+        assert_eq!(decoded, body);
+    }
+
+    /// Confirm decode_response_body ignores identity when combined with other encodings.
+    #[test]
+    fn decode_response_body_identity_and_gzip() {
+        let body = b"gzip-bytes".to_vec();
+        let mut encoder = flate2::write::GzEncoder::new(Vec::new(), flate2::Compression::default());
+        encoder.write_all(&body).unwrap();
+        let encoded = encoder.finish().unwrap();
+
+        let decoded = BuiltinPackageLoader::decode_response_body(
+            &headers_with_encoding(Some("identity, gzip")),
+            encoded,
+        )
+        .unwrap();
+        assert_eq!(decoded, body);
+    }
+
+    /// Confirm decode_response_body errors on invalid gzip payloads.
+    #[test]
+    fn decode_response_body_gzip_invalid_payload() {
+        let body = b"not-gzip".to_vec();
+        let err =
+            BuiltinPackageLoader::decode_response_body(&headers_with_encoding(Some("gzip")), body)
+                .unwrap_err();
+        let msg = err.to_string();
+        assert!(msg.contains("failed to decode response body"));
+    }
+
+    /// Confirm decode_response_body decodes zstd-encoded bodies.
+    #[cfg(not(target_arch = "wasm32"))]
+    #[test]
+    fn decode_response_body_zstd() {
+        let body = b"zstd-bytes".to_vec();
+        let encoded = zstd::stream::encode_all(std::io::Cursor::new(&body), 0).unwrap();
+
+        let decoded = BuiltinPackageLoader::decode_response_body(
+            &headers_with_encoding(Some("zstd")),
+            encoded,
+        )
+        .unwrap();
+        assert_eq!(decoded, body);
+    }
+
+    /// Confirm decode_response_body errors on invalid zstd payloads.
+    #[cfg(not(target_arch = "wasm32"))]
+    #[test]
+    fn decode_response_body_zstd_invalid_payload() {
+        let body = b"not-zstd".to_vec();
+        let err =
+            BuiltinPackageLoader::decode_response_body(&headers_with_encoding(Some("zstd")), body)
+                .unwrap_err();
+        let msg = err.to_string();
+        assert!(msg.contains("failed to decode response body"));
+    }
+
+    /// Confirm decode_response_body decodes layered gzip+zstd-encoded bodies.
+    #[cfg(not(target_arch = "wasm32"))]
+    #[test]
+    fn decode_response_body_zstd_and_gzip() {
+        let body = b"layered-bytes".to_vec();
+        let mut encoder = flate2::write::GzEncoder::new(Vec::new(), flate2::Compression::default());
+        encoder.write_all(&body).unwrap();
+        let gzipped = encoder.finish().unwrap();
+        let encoded = zstd::stream::encode_all(std::io::Cursor::new(gzipped), 0).unwrap();
+
+        let decoded = BuiltinPackageLoader::decode_response_body(
+            &headers_with_encoding(Some("gzip, zstd")),
+            encoded,
+        )
+        .unwrap();
+        assert_eq!(decoded, body);
+    }
+
+    /// Confirm decode_response_body errors on unknown encodings.
+    #[test]
+    fn decode_response_body_unknown_encoding() {
+        let body = b"weird".to_vec();
+        let err =
+            BuiltinPackageLoader::decode_response_body(&headers_with_encoding(Some("br")), body)
+                .unwrap_err();
+        let msg = err.to_string();
+        assert!(msg.contains("unsupported content-encoding"));
+    }
 
     // NOTE: must be a tokio test because the BuiltinPackageLoader::new()
     // constructor requires a runtime...
@@ -723,4 +1053,90 @@ mod test {
 
         assert_eq!(file_path.exists(), false);
     }
+
+    #[tokio::test]
+    async fn test_file_cache_scan_retain() {
+        let dir = tempfile::tempdir().unwrap();
+        let path = dir.path();
+
+        let cache = FileSystemCache {
+            cache_dir: path.to_path_buf(),
+        };
+
+        {
+            let state = cache
+                .scan(0u64, |state: &mut u64, _entry| {
+                    *state += 1;
+                    Ok(())
+                })
+                .await
+                .unwrap();
+
+            assert_eq!(state, 0);
+        }
+
+        let path1 = cache
+            .save(
+                Bytes::from_static(b"test1"),
+                &DistributionInfo {
+                    webc: Url::parse("file:///test1.webc").unwrap(),
+                    webc_sha256: WebcHash::sha256(b"test1"),
+                },
+            )
+            .await
+            .unwrap();
+        let path2 = cache
+            .save(
+                Bytes::from_static(b"test2"),
+                &DistributionInfo {
+                    webc: Url::parse("file:///test2.webc").unwrap(),
+                    webc_sha256: WebcHash::sha256(b"test2"),
+                },
+            )
+            .await
+            .unwrap();
+
+        {
+            let path1 = path1.clone();
+            let path2 = path2.clone();
+            let state = cache
+                .scan(0u64, move |state: &mut u64, entry| {
+                    *state += 1;
+                    assert!(entry.path() == path1 || entry.path() == path2);
+                    Ok(())
+                })
+                .await
+                .unwrap();
+
+            assert_eq!(state, 2);
+        }
+
+        {
+            let path1 = path1.clone();
+            let state = cache
+                .retain(0u64, move |state: &mut u64, entry| {
+                    *state += 1;
+                    Ok(entry.path() == path1)
+                })
+                .await
+                .unwrap();
+            assert_eq!(state, 2);
+        }
+
+        assert!(path1.exists());
+        assert!(!path2.exists(), "Path 2 should have been deleted");
+
+        {
+            let path1 = path1.clone();
+            let state = cache
+                .scan(0u64, move |state: &mut u64, entry| {
+                    *state += 1;
+                    assert!(entry.path() == path1);
+                    Ok(())
+                })
+                .await
+                .unwrap();
+            assert_eq!(state, 1);
+        }
+    }
 }
