@@ -6,10 +6,10 @@ use std::{
 };
 
 use anyhow::{Context, Error};
-use futures::{future::BoxFuture, StreamExt, TryStreamExt};
+use futures::{StreamExt, TryStreamExt, future::BoxFuture};
 use once_cell::sync::OnceCell;
 use petgraph::visit::EdgeRef;
-use virtual_fs::{FileSystem, OverlayFileSystem, UnionFileSystem, WebcVolumeFileSystem};
+use virtual_fs::{FileSystem, UnionFileSystem, WebcVolumeFileSystem};
 use wasmer_config::package::{PackageId, SuggestedCompilerOptimizations};
 use wasmer_package::utils::wasm_annotations_to_features;
 use webc::metadata::annotations::Atom as AtomAnnotation;
@@ -62,13 +62,17 @@ pub async fn load_package_tree(
     let mut containers = fetch_dependencies(loader, &resolution.package, &resolution.graph).await?;
     containers.insert(resolution.package.root_package.clone(), root.clone());
     let package_ids = containers.keys().cloned().collect();
-    let fs = filesystem(&containers, &resolution.package, root_is_local_dir)?;
+    let fs_opt = filesystem(&containers, &resolution.package, root_is_local_dir)?;
 
     let root = &resolution.package.root_package;
     let commands: Vec<BinaryPackageCommand> =
         commands(&resolution.package.commands, &containers, resolution)?;
 
-    let file_system_memory_footprint = count_file_system(&fs, Path::new("/"));
+    let file_system_memory_footprint = if let Some(fs) = &fs_opt {
+        count_file_system(fs, Path::new("/"))
+    } else {
+        0
+    };
 
     let loaded = BinaryPackage {
         id: root.clone(),
@@ -81,7 +85,7 @@ pub async fn load_package_tree(
         .map(|ts| ts as u128),
         hash: OnceCell::new(),
         entrypoint_cmd: resolution.package.entrypoint.clone(),
-        webc_fs: Arc::new(fs),
+        webc_fs: fs_opt.map(Arc::new),
         commands,
         uses: Vec::new(),
         file_system_memory_footprint,
@@ -235,15 +239,14 @@ fn extract_suggested_compiler_opts_from_atom_metadata(
     if let Some(sco) = atom_metadata
         .annotations
         .get(SuggestedCompilerOptimizations::KEY)
-    {
-        if let Some((_, v)) = sco.as_map().and_then(|v| {
+        && let Some((_, v)) = sco.as_map().and_then(|v| {
             v.iter().find(|(k, _)| {
                 k.as_text()
                     .is_some_and(|v| v == SuggestedCompilerOptimizations::PASS_PARAMS_KEY)
             })
-        }) {
-            ret.pass_params = v.as_bool()
-        }
+        })
+    {
+        ret.pass_params = v.as_bool()
     }
 
     ret
@@ -399,18 +402,15 @@ fn count_file_system(fs: &dyn FileSystem, path: &Path) -> u64 {
 
 /// Given a set of [`ResolvedFileSystemMapping`]s and the [`Container`] for each
 /// package in a dependency tree, construct the resulting filesystem.
+///
+/// Returns `Ok(None)` if no filesystem mappings were specified.
 fn filesystem(
     packages: &HashMap<PackageId, Container>,
     pkg: &ResolvedPackage,
     root_is_local_dir: bool,
-) -> Result<Box<dyn FileSystem + Send + Sync>, Error> {
+) -> Result<Option<UnionFileSystem>, Error> {
     if pkg.filesystem.is_empty() {
-        return Ok(Box::new(OverlayFileSystem::<
-            virtual_fs::EmptyFileSystem,
-            Vec<WebcVolumeFileSystem>,
-        >::new(
-            virtual_fs::EmptyFileSystem::default(), vec![]
-        )));
+        return Ok(None);
     }
 
     let mut found_v2 = None;
@@ -424,17 +424,31 @@ fn filesystem(
             )
         })?;
 
-        if container.version() == webc::Version::V2 && found_v2.is_none() {
-            found_v2 = Some(package.clone());
-        }
-        if container.version() == webc::Version::V3 && found_v3.is_none() {
-            found_v3 = Some(package.clone());
+        match container.version() {
+            webc::Version::V1 => {
+                anyhow::bail!(
+                    "the package '{package}' is a webc v1 package, but webc v1 support was removed"
+                );
+            }
+            webc::Version::V2 => {
+                if found_v2.is_none() {
+                    found_v2 = Some(package.clone());
+                }
+            }
+            webc::Version::V3 => {
+                if found_v3.is_none() {
+                    found_v3 = Some(package.clone());
+                }
+            }
+            other => {
+                anyhow::bail!("the package '{package}' has an unknown webc version: {other}");
+            }
         }
     }
 
     match (found_v2, found_v3) {
-        (None, Some(_)) => filesystem_v3(packages, pkg, root_is_local_dir),
-        (Some(_), None) => filesystem_v2(packages, pkg, root_is_local_dir),
+        (None, Some(_)) => filesystem_v3(packages, pkg, root_is_local_dir).map(Some),
+        (Some(_), None) => filesystem_v2(packages, pkg, root_is_local_dir).map(Some),
         (Some(v2), Some(v3)) => {
             anyhow::bail!(
                 "Mix of webc v2 and v3 in the same dependency tree is not supported; v2: {v2}, v3: {v3}"
@@ -449,12 +463,9 @@ fn filesystem_v3(
     packages: &HashMap<PackageId, Container>,
     pkg: &ResolvedPackage,
     root_is_local_dir: bool,
-) -> Result<Box<dyn FileSystem + Send + Sync>, Error> {
+) -> Result<UnionFileSystem, Error> {
     let mut volumes: HashMap<&PackageId, BTreeMap<String, Volume>> = HashMap::new();
 
-    let mut mountings: Vec<_> = pkg.filesystem.iter().collect();
-    mountings.sort_by_key(|m| std::cmp::Reverse(m.mount_path.as_path()));
-
     let union_fs = UnionFileSystem::new();
 
     for ResolvedFileSystemMapping {
@@ -468,6 +479,12 @@ fn filesystem_v3(
             continue;
         }
 
+        if mount_path.as_path() == Path::new("/") {
+            tracing::warn!(
+                "The \"{package}\" package wants to mount a volume at \"/\", which breaks WASIX modules' filesystems",
+            );
+        }
+
         // Note: We want to reuse existing Volume instances if we can. That way
         // we can keep the memory usage down. A webc::compat::Volume is
         // reference-counted, anyway.
@@ -491,9 +508,7 @@ fn filesystem_v3(
         union_fs.mount(volume_name.clone(), mount_path, Box::new(webc_vol))?;
     }
 
-    let fs = OverlayFileSystem::new(virtual_fs::EmptyFileSystem::default(), [union_fs]);
-
-    Ok(Box::new(fs))
+    Ok(union_fs)
 }
 
 /// Build the filesystem for webc v2 packages.
@@ -523,12 +538,10 @@ fn filesystem_v2(
     packages: &HashMap<PackageId, Container>,
     pkg: &ResolvedPackage,
     root_is_local_dir: bool,
-) -> Result<Box<dyn FileSystem + Send + Sync>, Error> {
-    let mut filesystems = Vec::new();
+) -> Result<UnionFileSystem, Error> {
     let mut volumes: HashMap<&PackageId, BTreeMap<String, Volume>> = HashMap::new();
 
-    let mut mountings: Vec<_> = pkg.filesystem.iter().collect();
-    mountings.sort_by_key(|m| std::cmp::Reverse(m.mount_path.as_path()));
+    let union_fs = UnionFileSystem::new();
 
     for ResolvedFileSystemMapping {
         mount_path,
@@ -541,6 +554,12 @@ fn filesystem_v2(
             continue;
         }
 
+        if mount_path.as_path() == Path::new("/") {
+            tracing::warn!(
+                "The \"{package}\" package wants to mount a volume at \"/\", which breaks WASIX modules' filesystems",
+            );
+        }
+
         // Note: We want to reuse existing Volume instances if we can. That way
         // we can keep the memory usage down. A webc::compat::Volume is
         // reference-counted, anyway.
@@ -562,46 +581,36 @@ fn filesystem_v2(
             format!("The \"{package}\" package doesn't have a \"{volume_name}\" volume")
         })?;
 
-        let mount_path = mount_path.clone();
-        // Get a filesystem which will map "$mount_dir/some-path" to
-        // "$original_path/some-path" on the original volume
+        // UnionFileSystem strips the mount point before forwarding paths to the
+        // mounted filesystem. That means paths are already relative to the
+        // mount root and shouldn't be stripped by mount_path.
         let fs = if let Some(original) = original_path {
             let original = PathBuf::from(original);
 
             MappedPathFileSystem::new(
                 WebcVolumeFileSystem::new(volume.clone()),
-                Box::new(move |path: &Path| {
-                    let without_mount_dir = path
-                        .strip_prefix(&mount_path)
-                        .map_err(|_| virtual_fs::FsError::BaseNotDirectory)?;
-                    Ok(original.join(without_mount_dir))
-                }) as DynPathMapper,
+                Box::new(move |path: &Path| Ok(original.join(strip_root_prefix(path))))
+                    as DynPathMapper,
             )
         } else {
             MappedPathFileSystem::new(
                 WebcVolumeFileSystem::new(volume.clone()),
-                Box::new(move |path: &Path| {
-                    let without_mount_dir = path
-                        .strip_prefix(&mount_path)
-                        .map_err(|_| virtual_fs::FsError::BaseNotDirectory)?;
-                    Ok(without_mount_dir.to_owned())
-                }) as DynPathMapper,
+                Box::new(move |path: &Path| Ok(strip_root_prefix(path))) as DynPathMapper,
             )
         };
 
-        filesystems.push(fs);
+        union_fs.mount(volume_name.clone(), mount_path, Box::new(fs))?;
     }
 
-    let fs = OverlayFileSystem::new(virtual_fs::EmptyFileSystem::default(), filesystems);
+    Ok(union_fs)
+}
 
-    Ok(Box::new(fs))
+fn strip_root_prefix(path: &Path) -> PathBuf {
+    path.strip_prefix("/").unwrap_or(path).to_owned()
 }
 
 type DynPathMapper = Box<dyn Fn(&Path) -> Result<PathBuf, virtual_fs::FsError> + Send + Sync>;
 
-/// A [`FileSystem`] implementation that lets you map the [`Path`] to something
-/// else.
-#[derive(Clone, PartialEq)]
 struct MappedPathFileSystem<F, M> {
     inner: F,
     map: M,
@@ -673,7 +682,7 @@ where
         self.inner.remove_file(&path)
     }
 
-    fn new_open_options(&self) -> virtual_fs::OpenOptions {
+    fn new_open_options(&self) -> virtual_fs::OpenOptions<'_> {
         virtual_fs::OpenOptions::new(self)
     }
 
@@ -717,3 +726,96 @@ where
             .finish()
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use std::{
+        collections::{BTreeMap, HashMap},
+        path::{Path, PathBuf},
+    };
+
+    use ciborium::value::Value;
+    use virtual_fs::FileSystem;
+    use wasmer_config::package::PackageId;
+    use webc::{
+        Container,
+        indexmap::IndexMap,
+        metadata::{
+            Manifest,
+            annotations::{FileSystemMapping, FileSystemMappings},
+        },
+        v2::{
+            SignatureAlgorithm,
+            read::OwnedReader,
+            write::{DirEntry, Directory, FileEntry, Writer},
+        },
+    };
+
+    use super::{ResolvedFileSystemMapping, ResolvedPackage, filesystem_v2};
+
+    #[test]
+    fn v2_filesystem_mapping_resolves_mount_paths() {
+        // Regression test: v2 fs mounts are already relative to the mount root,
+        // so stripping the mount path again breaks lookups like /public.
+        let mut manifest = Manifest::default();
+        let fs = FileSystemMappings(vec![FileSystemMapping {
+            from: None,
+            volume_name: "atom".to_string(),
+            host_path: Some("/public".to_string()),
+            mount_path: "/public".to_string(),
+        }]);
+        let mut package = IndexMap::new();
+        package.insert(
+            FileSystemMappings::KEY.to_string(),
+            Value::serialized(&fs).unwrap(),
+        );
+        manifest.package = package;
+
+        let mut public_children = BTreeMap::new();
+        public_children.insert(
+            "index.html".parse().unwrap(),
+            DirEntry::File(FileEntry::from(b"ok".as_slice())),
+        );
+        let public_dir = Directory {
+            children: public_children,
+        };
+        let mut root_children = BTreeMap::new();
+        root_children.insert("public".parse().unwrap(), DirEntry::Dir(public_dir));
+        let atom_dir = Directory {
+            children: root_children,
+        };
+
+        let writer = Writer::default().write_manifest(&manifest).unwrap();
+        let writer = writer.write_atoms(BTreeMap::new()).unwrap();
+        let writer = writer.with_volume("atom", atom_dir).unwrap();
+        let bytes = writer.finish(SignatureAlgorithm::None).unwrap();
+
+        let reader = OwnedReader::parse(bytes).unwrap();
+        let container = Container::from(reader);
+
+        let pkg_id = PackageId::new_named("ns/pkg", "0.1.0".parse().unwrap());
+        let mut packages = HashMap::new();
+        packages.insert(pkg_id.clone(), container);
+
+        let pkg = ResolvedPackage {
+            root_package: pkg_id.clone(),
+            commands: BTreeMap::new(),
+            entrypoint: None,
+            filesystem: vec![ResolvedFileSystemMapping {
+                mount_path: PathBuf::from("/public"),
+                volume_name: "atom".to_string(),
+                original_path: Some("/public".to_string()),
+                package: pkg_id,
+            }],
+        };
+
+        let union_fs = filesystem_v2(&packages, &pkg, false).unwrap();
+        assert!(union_fs.metadata(Path::new("/public")).unwrap().is_dir());
+        assert!(
+            union_fs
+                .metadata(Path::new("/public/index.html"))
+                .unwrap()
+                .is_file()
+        );
+    }
+}
