@@ -5,5 +5,5 @@ mod unsupported;
 
 pub use self::{
     builtin_loader::BuiltinPackageLoader, load_package_tree::load_package_tree,
-    types::to_module_hash, types::PackageLoader, unsupported::UnsupportedPackageLoader,
+    types::PackageLoader, types::to_module_hash, unsupported::UnsupportedPackageLoader,
 };
