@@ -1,5 +1,5 @@
 use std::{
-    path::PathBuf,
+    path::{MAIN_SEPARATOR_STR, PathBuf},
     sync::Arc,
     time::{Duration, SystemTime},
 };
@@ -257,7 +257,7 @@ impl Source for BackendSource {
             _ => {
                 return Err(QueryError::Unsupported {
                     query: package.clone(),
-                })
+                });
             }
         };
 
@@ -290,14 +290,14 @@ impl Source for BackendSource {
             .await
             .map_err(|error| QueryError::new_other(error, package))?;
 
-        if let Some(cache) = &self.cache {
-            if let Err(e) = cache.update(&package_name, &response) {
-                tracing::warn!(
-                    package_name,
-                    error = &*e,
-                    "An error occurred while caching the GraphQL response",
-                );
-            }
+        if let Some(cache) = &self.cache
+            && let Err(e) = cache.update(&package_name, &response)
+        {
+            tracing::warn!(
+                package_name,
+                error = &*e,
+                "An error occurred while caching the GraphQL response",
+            );
         }
 
         matching_package_summaries(
@@ -460,7 +460,8 @@ impl FileSystemCache {
     }
 
     fn path(&self, package_name: &str) -> PathBuf {
-        self.cache_dir.join(package_name)
+        self.cache_dir
+            .join(package_name.replace(MAIN_SEPARATOR_STR, "#"))
     }
 
     fn lookup_cached_query(&self, package_name: &str) -> Result<Option<WebQuery>, Error> {
@@ -565,14 +566,6 @@ struct CacheEntry {
     response: WebQuery,
 }
 
-/// Cache entry for a webc lookup by hash.
-#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
-struct HashCacheEntry {
-    unix_timestamp: u64,
-    hash: String,
-    response: WebQuery,
-}
-
 impl CacheEntry {
     fn is_still_valid(&self, timeout: Duration) -> bool {
         let timestamp = SystemTime::UNIX_EPOCH + Duration::from_secs(self.unix_timestamp);
@@ -705,18 +698,13 @@ pub struct WebQueryGetPackageVersion {
     pub v3: WebQueryGetPackageVersionDistribution,
 }
 
-#[derive(Debug, serde::Serialize, serde::Deserialize, Clone)]
+#[derive(Debug, serde::Serialize, serde::Deserialize, Clone, Default)]
 pub enum WebCVersion {
+    #[default]
     V2,
     V3,
 }
 
-impl Default for WebCVersion {
-    fn default() -> Self {
-        Self::V2
-    }
-}
-
 impl From<WebCVersion> for webc::Version {
     fn from(val: WebCVersion) -> Self {
         match val {
