@@ -39,18 +39,17 @@ pub(crate) fn webc_headers() -> HeaderMap {
 pub(crate) fn http_error(response: &HttpResponse) -> Error {
     let status = response.status;
 
-    if status == StatusCode::SERVICE_UNAVAILABLE {
-        if let Some(retry_after) = response
+    if status == StatusCode::SERVICE_UNAVAILABLE
+        && let Some(retry_after) = response
             .headers
             .get("Retry-After")
             .and_then(|retry_after| retry_after.to_str().ok())
-        {
-            tracing::debug!(
-                %retry_after,
-                "Received 503 Service Unavailable while looking up a package. The backend may still be generating the *.webc file.",
-            );
-            return anyhow::anyhow!("{status} (Retry After: {retry_after})");
-        }
+    {
+        tracing::debug!(
+            %retry_after,
+            "Received 503 Service Unavailable while looking up a package. The backend may still be generating the *.webc file.",
+        );
+        return anyhow::anyhow!("{status} (Retry After: {retry_after})");
     }
 
     Error::msg(status)
