@@ -1,13 +1,13 @@
 use std::sync::Mutex;
 use std::{num::NonZeroUsize, pin::Pin, sync::Arc, time::Duration};
 
-use futures::{future::BoxFuture, Future};
+use futures::{Future, future::BoxFuture};
 use tokio::runtime::{Handle, Runtime};
-use virtual_mio::InlineWaker;
+use virtual_mio::block_on;
 use wasmer::AsStoreMut;
 
-use crate::runtime::SpawnMemoryType;
-use crate::{os::task::thread::WasiThreadError, WasiFunctionEnv};
+use crate::runtime::SpawnType;
+use crate::{WasiFunctionEnv, os::task::thread::WasiThreadError};
 
 use super::{SpawnMemoryTypeOrStore, TaskWasm, TaskWasmRunProperties, VirtualTaskManager};
 
@@ -29,10 +29,10 @@ impl From<Runtime> for RuntimeOrHandle {
 
 impl Drop for RuntimeOrHandle {
     fn drop(&mut self) {
-        if let Self::Runtime(_, runtime) = self {
-            if let Some(h) = runtime.lock().unwrap().take() {
-                h.shutdown_timeout(Duration::from_secs(0))
-            }
+        if let Self::Runtime(_, runtime) = self
+            && let Some(h) = runtime.lock().unwrap().take()
+        {
+            h.shutdown_timeout(Duration::from_secs(0))
         }
     }
 }
@@ -113,15 +113,6 @@ impl Default for TokioTaskManager {
     }
 }
 
-#[allow(dead_code)]
-struct TokioRuntimeGuard<'g> {
-    #[allow(unused)]
-    inner: tokio::runtime::EnterGuard<'g>,
-}
-impl<'g> Drop for TokioRuntimeGuard<'g> {
-    fn drop(&mut self) {}
-}
-
 impl VirtualTaskManager for TokioTaskManager {
     /// See [`VirtualTaskManager::sleep_now`].
     fn sleep_now(&self, time: Duration) -> Pin<Box<dyn Future<Output = ()> + Send + Sync>> {
@@ -154,16 +145,47 @@ impl VirtualTaskManager for TokioTaskManager {
         let env = task.env;
         let pre_run = task.pre_run;
 
-        let make_memory: SpawnMemoryTypeOrStore = match task.spawn_type {
-            SpawnMemoryType::CreateMemory => SpawnMemoryTypeOrStore::New,
-            SpawnMemoryType::CreateMemoryOfType(t) => SpawnMemoryTypeOrStore::Type(t),
-            SpawnMemoryType::ShareMemory(_, _) | SpawnMemoryType::CopyMemory(_, _) => {
+        let make_memory: SpawnMemoryTypeOrStore = match &task.spawn_type {
+            SpawnType::CreateMemory | SpawnType::NewLinkerInstanceGroup(..) => {
+                SpawnMemoryTypeOrStore::New
+            }
+            SpawnType::CreateMemoryOfType(t) => SpawnMemoryTypeOrStore::Type(*t),
+            SpawnType::ShareMemory(_, _) | SpawnType::CopyMemory(_, _) => {
                 let mut store = env.runtime().new_store();
-                let memory = self.build_memory(&mut store.as_store_mut(), task.spawn_type)?;
+                let memory = self.build_memory(&mut store.as_store_mut(), &task.spawn_type)?;
                 SpawnMemoryTypeOrStore::StoreAndMemory(store, memory)
             }
         };
 
+        // This should actually run in the blocking thread, just like the task itself.
+        // See the comment below for why we can't do it there yet.
+        //
+        // For now block_in_place at least ensures that we don't block the async runtime
+        let ret = tokio::task::block_in_place(move || {
+            if let SpawnType::NewLinkerInstanceGroup(linker, func_env, mut store) = task.spawn_type
+            {
+                WasiFunctionEnv::new_with_store(
+                    task.module,
+                    env,
+                    task.globals,
+                    make_memory,
+                    task.update_layout,
+                    task.call_initialize,
+                    Some((linker, &mut func_env.into_mut(&mut store))),
+                )
+            } else {
+                WasiFunctionEnv::new_with_store(
+                    task.module,
+                    env,
+                    task.globals,
+                    make_memory,
+                    task.update_layout,
+                    task.call_initialize,
+                    None,
+                )
+            }
+        });
+
         if let Some(trigger) = task.trigger {
             tracing::trace!("spawning task_wasm trigger in async pool");
             // In principle, we'd need to create this in the `pool.execute` function below, that is
@@ -195,13 +217,7 @@ impl VirtualTaskManager for TokioTaskManager {
             // pool's one), and let it fail for runtimes that don't support entities created in a
             // thread that's not the one in which execution happens in; this until we can clone
             // stores.
-            let (mut ctx, mut store) = WasiFunctionEnv::new_with_store(
-                task.module,
-                env,
-                task.globals,
-                make_memory,
-                task.update_layout,
-            )?;
+            let (mut ctx, mut store) = ret?;
 
             let mut trigger = trigger();
             let pool = self.pool.clone();
@@ -214,7 +230,14 @@ impl VirtualTaskManager for TokioTaskManager {
                         _ = env.thread.wait_for_signal() => {
                             tracing::debug!("wait-for-signal(triggered)");
                             let mut ctx = ctx.env.clone().into_mut(&mut store);
-                            if let Err(err) = crate::WasiEnv::process_signals_and_exit(&mut ctx) {
+                            if let Err(err) =
+                                crate::WasiEnv::do_pending_link_operations(
+                                    &mut ctx,
+                                    false
+                                ).and_then(|()|
+                                    crate::WasiEnv::process_signals_and_exit(&mut ctx)
+                                )
+                            {
                                 match err {
                                     crate::WasiError::Exit(code) => Err(code),
                                     err => {
@@ -258,14 +281,6 @@ impl VirtualTaskManager for TokioTaskManager {
             // Run the callback on a dedicated thread
             self.pool.execute(move || {
                 tracing::trace!("task_wasm started in blocking thread");
-                let ret = WasiFunctionEnv::new_with_store(
-                    task.module,
-                    env,
-                    task.globals,
-                    make_memory,
-                    task.update_layout,
-                );
-
                 let (mut ctx, mut store) = match ret {
                     Ok(x) => {
                         sx.send(Ok(())).unwrap();
@@ -278,7 +293,7 @@ impl VirtualTaskManager for TokioTaskManager {
                 };
 
                 if let Some(pre_run) = pre_run {
-                    InlineWaker::block_on(pre_run(&mut ctx, &mut store));
+                    block_on(pre_run(&mut ctx, &mut store));
                 }
 
                 // Invoke the callback
