@@ -9,29 +9,27 @@ use std::{
 use rand::Rng;
 use thiserror::Error;
 use virtual_fs::{ArcFile, FileSystem, FsError, TmpFileSystem, VirtualFile};
-use wasmer::{AsStoreMut, Extern, Imports, Instance, Module, Store};
+use wasmer::{AsStoreMut, Engine, Instance, Module};
 use wasmer_config::package::PackageId;
 
 #[cfg(feature = "journal")]
 use crate::journal::{DynJournal, DynReadableJournal, SnapshotTrigger};
 use crate::{
+    Runtime, WasiEnv, WasiFunctionEnv, WasiRuntimeError, WasiThreadError,
     bin_factory::{BinFactory, BinaryPackage},
     capabilities::Capabilities,
     fs::{WasiFs, WasiFsRoot, WasiInodes},
     os::task::control_plane::{ControlPlaneConfig, ControlPlaneError, WasiControlPlane},
     state::WasiState,
-    syscalls::{
-        rewind_ext2,
-        types::{__WASI_STDERR_FILENO, __WASI_STDIN_FILENO, __WASI_STDOUT_FILENO},
-    },
-    utils::xxhash_random,
-    Runtime, WasiEnv, WasiError, WasiFunctionEnv, WasiRuntimeError,
+    syscalls::types::{__WASI_STDERR_FILENO, __WASI_STDIN_FILENO, __WASI_STDOUT_FILENO},
 };
 use wasmer_types::ModuleHash;
 use wasmer_wasix_types::wasi::SignalDisposition;
 
 use super::env::WasiEnvInit;
 
+// FIXME: additional import support was broken and has been removed. We need to re-introduce
+// it in a way that works with multi-threaded WASIX apps.
 /// Builder API for configuring a [`WasiEnv`] environment needed to run WASI modules.
 ///
 /// Usage:
@@ -69,6 +67,7 @@ pub struct WasiEnvBuilder {
     pub(super) stderr: Option<Box<dyn VirtualFile + Send + Sync + 'static>>,
     pub(super) stdin: Option<Box<dyn VirtualFile + Send + Sync + 'static>>,
     pub(super) fs: Option<WasiFsRoot>,
+    pub(super) engine: Option<Engine>,
     pub(super) runtime: Option<Arc<dyn crate::Runtime + Send + Sync + 'static>>,
     pub(super) current_dir: Option<PathBuf>,
 
@@ -83,7 +82,6 @@ pub struct WasiEnvBuilder {
     pub(super) map_commands: HashMap<String, PathBuf>,
 
     pub(super) capabilites: Capabilities,
-    pub(super) additional_imports: Imports,
 
     #[cfg(feature = "journal")]
     pub(super) snapshot_on: Vec<SnapshotTrigger>,
@@ -120,6 +118,7 @@ impl std::fmt::Debug for WasiEnvBuilder {
             .field("stdout_override exists", &self.stdout.is_some())
             .field("stderr_override exists", &self.stderr.is_some())
             .field("stdin_override exists", &self.stdin.is_some())
+            .field("engine_override_exists", &self.engine.is_some())
             .field("runtime_override_exists", &self.runtime.is_some())
             .finish()
     }
@@ -699,20 +698,24 @@ impl WasiEnvBuilder {
     /// Sets the FileSystem to be used with this WASI instance.
     ///
     /// This is usually used in case a custom `virtual_fs::FileSystem` is needed.
-    pub fn fs(mut self, fs: Box<dyn virtual_fs::FileSystem + Send + Sync>) -> Self {
+    pub fn fs(mut self, fs: impl Into<Arc<dyn virtual_fs::FileSystem + Send + Sync>>) -> Self {
         self.set_fs(fs);
         self
     }
 
-    pub fn set_fs(&mut self, fs: Box<dyn virtual_fs::FileSystem + Send + Sync>) {
-        self.fs = Some(WasiFsRoot::Backing(Arc::new(fs)));
+    pub fn set_fs(&mut self, fs: impl Into<Arc<dyn virtual_fs::FileSystem + Send + Sync>>) {
+        self.fs = Some(WasiFsRoot::Backing(fs.into()));
+    }
+
+    pub(crate) fn set_fs_root(&mut self, fs: WasiFsRoot) {
+        self.fs = Some(fs);
     }
 
     /// Sets a new sandbox FileSystem to be used with this WASI instance.
     ///
     /// This is usually used in case a custom `virtual_fs::FileSystem` is needed.
     pub fn sandbox_fs(mut self, fs: TmpFileSystem) -> Self {
-        self.fs = Some(WasiFsRoot::Sandbox(Arc::new(fs)));
+        self.fs = Some(WasiFsRoot::Sandbox(fs));
         self
     }
 
@@ -724,6 +727,17 @@ impl WasiEnvBuilder {
         self
     }
 
+    /// Sets the wasmer engine and overrides the default; only used if
+    /// a runtime override is not provided.
+    pub fn engine(mut self, engine: Engine) -> Self {
+        self.set_engine(engine);
+        self
+    }
+
+    pub fn set_engine(&mut self, engine: Engine) {
+        self.engine = Some(engine);
+    }
+
     /// Sets the WASI runtime implementation and overrides the default
     /// implementation
     pub fn runtime(mut self, runtime: Arc<dyn Runtime + Send + Sync>) -> Self {
@@ -767,58 +781,13 @@ impl WasiEnvBuilder {
         self.skip_stdio_during_bootstrap = skip;
     }
 
-    /// Add an item to the list of importable items provided to the instance.
-    pub fn import(
-        mut self,
-        namespace: impl Into<String>,
-        name: impl Into<String>,
-        value: impl Into<Extern>,
-    ) -> Self {
-        self.add_imports([((namespace, name), value)]);
-        self
-    }
-
-    /// Add an item to the list of importable items provided to the instance.
-    pub fn add_import(
-        &mut self,
-        namespace: impl Into<String>,
-        name: impl Into<String>,
-        value: impl Into<Extern>,
-    ) {
-        self.add_imports([((namespace, name), value)]);
-    }
-
-    pub fn add_imports<I, S1, S2, E>(&mut self, imports: I)
-    where
-        I: IntoIterator<Item = ((S1, S2), E)>,
-        S1: Into<String>,
-        S2: Into<String>,
-        E: Into<Extern>,
-    {
-        let imports = imports
-            .into_iter()
-            .map(|((ns, n), e)| ((ns.into(), n.into()), e.into()));
-        self.additional_imports.extend(imports);
-    }
-
-    pub fn imports<I, S1, S2, E>(mut self, imports: I) -> Self
-    where
-        I: IntoIterator<Item = ((S1, S2), E)>,
-        S1: Into<String>,
-        S2: Into<String>,
-        E: Into<Extern>,
-    {
-        self.add_imports(imports);
-        self
-    }
-
     /// Consumes the [`WasiEnvBuilder`] and produces a [`WasiEnvInit`], which
     /// can be used to construct a new [`WasiEnv`].
     ///
     /// Returns the error from `WasiFs::new` if there's an error
     ///
     /// NOTE: You should prefer to not work directly with [`WasiEnvInit`].
-    /// Use [`WasiEnvBuilder::run`] or [`WasiEnvBuilder::run_with_store`] instead
+    /// Use [`WasiEnvBuilder::build`] or [`WasiEnvBuilder::instantiate`] instead
     /// to ensure proper invokation of WASI modules.
     pub fn build_init(mut self) -> Result<WasiEnvInit, WasiStateCreationError> {
         for arg in self.args.iter() {
@@ -847,19 +816,19 @@ impl WasiEnvBuilder {
                 Some(InvalidCharacter::Nul) => {
                     return Err(WasiStateCreationError::EnvironmentVariableFormatError(
                         format!("found nul byte in env var key \"{env_key}\" (key=value)"),
-                    ))
+                    ));
                 }
 
                 Some(InvalidCharacter::Equal) => {
                     return Err(WasiStateCreationError::EnvironmentVariableFormatError(
                         format!("found equal sign in env var key \"{env_key}\" (key=value)"),
-                    ))
+                    ));
                 }
 
                 None => (),
             }
 
-            if env_value.iter().any(|&ch| ch == 0) {
+            if env_value.contains(&0) {
                 return Err(WasiStateCreationError::EnvironmentVariableFormatError(
                     format!(
                         "found nul byte in env var value \"{}\" (key=value)",
@@ -869,13 +838,6 @@ impl WasiEnvBuilder {
             }
         }
 
-        // TODO: must be used! (runtime was removed from env, must ensure configured runtime is used)
-        // // Get a reference to the runtime
-        // let runtime = self
-        //     .runtime
-        //     .clone()
-        //     .unwrap_or_else(|| Arc::new(PluggableRuntimeImplementation::default()));
-
         // Determine the STDIN
         let stdin: Box<dyn VirtualFile + Send + Sync + 'static> = self
             .stdin
@@ -885,7 +847,7 @@ impl WasiEnvBuilder {
         let fs_backing = self
             .fs
             .take()
-            .unwrap_or_else(|| WasiFsRoot::Sandbox(Arc::new(TmpFileSystem::new())));
+            .unwrap_or_else(|| WasiFsRoot::Sandbox(TmpFileSystem::new()));
 
         if let Some(dir) = &self.current_dir {
             match fs_backing.read_dir(dir) {
@@ -902,7 +864,7 @@ impl WasiEnvBuilder {
                 }
                 Err(err) => {
                     return Err(WasiStateCreationError::WasiFsSetupError(format!(
-                        "Could check specified current directory at '{}': {err}",
+                        "Could not check specified current directory at '{}': {err}",
                         dir.display()
                     )));
                 }
@@ -956,7 +918,7 @@ impl WasiEnvBuilder {
 
         let state = WasiState {
             fs: wasi_fs,
-            secret: rand::thread_rng().gen::<[u8; 32]>(),
+            secret: rand::thread_rng().r#gen::<[u8; 32]>(),
             inodes,
             args: std::sync::Mutex::new(self.args.clone()),
             preopen: self.vfs_preopens.clone(),
@@ -971,6 +933,19 @@ impl WasiEnvBuilder {
             {
                 #[allow(unused_mut)]
                 let mut runtime = crate::runtime::PluggableRuntime::new(Arc::new(crate::runtime::task_manager::tokio::TokioTaskManager::default()));
+                runtime.set_engine(
+                    self
+                        .engine
+                        .as_ref()
+                        .expect(
+                            "Neither a runtime nor an engine was provided to WasiEnvBuilder. \
+                            This is not supported because it means the module that's going to \
+                            run with the resulting WasiEnv will have been loaded using a \
+                            different engine than the one that will exist within the WasiEnv. \
+                            Use either `set_runtime` or `set_engine` before calling `build_init`.",
+                        )
+                        .clone()
+                );
                 #[cfg(feature = "journal")]
                 for journal in self.read_only_journals.clone() {
                     runtime.add_read_only_journal(journal);
@@ -1025,7 +1000,6 @@ impl WasiEnvBuilder {
             #[cfg(feature = "journal")]
             stop_running_after_snapshot: self.stop_running_after_snapshot,
             skip_stdio_during_bootstrap: self.skip_stdio_during_bootstrap,
-            additional_imports: self.additional_imports,
         };
 
         Ok(init)
@@ -1033,7 +1007,7 @@ impl WasiEnvBuilder {
 
     #[allow(clippy::result_large_err)]
     pub fn build(self) -> Result<WasiEnv, WasiRuntimeError> {
-        let module_hash = self.module_hash.unwrap_or_else(xxhash_random);
+        let module_hash = self.module_hash.unwrap_or_else(ModuleHash::random);
         let init = self.build_init()?;
         WasiEnv::from_init(init, module_hash)
     }
@@ -1048,7 +1022,7 @@ impl WasiEnvBuilder {
         self,
         store: &mut impl AsStoreMut,
     ) -> Result<WasiFunctionEnv, WasiRuntimeError> {
-        let module_hash = self.module_hash.unwrap_or_else(xxhash_random);
+        let module_hash = self.module_hash.unwrap_or_else(ModuleHash::random);
         let init = self.build_init()?;
         let env = WasiEnv::from_init(init, module_hash)?;
         let func_env = WasiFunctionEnv::new(store, env);
@@ -1066,7 +1040,7 @@ impl WasiEnvBuilder {
         module: Module,
         store: &mut impl AsStoreMut,
     ) -> Result<(Instance, WasiFunctionEnv), WasiRuntimeError> {
-        self.instantiate_ext(module, xxhash_random(), store)
+        self.instantiate_ext(module, ModuleHash::random(), store)
     }
 
     #[allow(clippy::result_large_err)]
@@ -1077,119 +1051,18 @@ impl WasiEnvBuilder {
         store: &mut impl AsStoreMut,
     ) -> Result<(Instance, WasiFunctionEnv), WasiRuntimeError> {
         let init = self.build_init()?;
-        WasiEnv::instantiate(init, module, module_hash, store)
-    }
-
-    #[allow(clippy::result_large_err)]
-    pub fn run(self, module: Module) -> Result<(), WasiRuntimeError> {
-        self.run_ext(module, xxhash_random())
-    }
-
-    #[allow(clippy::result_large_err)]
-    pub fn run_ext(self, module: Module, module_hash: ModuleHash) -> Result<(), WasiRuntimeError> {
-        let mut store = wasmer::Store::default();
-        self.run_with_store_ext(module, module_hash, &mut store)
-    }
-
-    #[allow(clippy::result_large_err)]
-    #[tracing::instrument(level = "debug", skip_all)]
-    pub fn run_with_store(self, module: Module, store: &mut Store) -> Result<(), WasiRuntimeError> {
-        self.run_with_store_ext(module, xxhash_random(), store)
-    }
-
-    #[allow(clippy::result_large_err)]
-    pub fn run_with_store_ext(
-        self,
-        module: Module,
-        module_hash: ModuleHash,
-        store: &mut Store,
-    ) -> Result<(), WasiRuntimeError> {
-        // If no handle or runtime exists then create one
-        #[cfg(feature = "sys-thread")]
-        let _guard = if tokio::runtime::Handle::try_current().is_err() {
-            let runtime = tokio::runtime::Builder::new_multi_thread()
-                .enable_all()
-                .build()
-                .unwrap();
-            Some(runtime)
-        } else {
-            None
-        };
-        #[cfg(feature = "sys-thread")]
-        let _guard = _guard.as_ref().map(|r| r.enter());
-
-        if self.capabilites.threading.enable_asynchronous_threading {
-            tracing::warn!(
-                "The enable_asynchronous_threading capability is enabled. Use WasiEnvBuilder::run_with_store_async() to avoid spurious errors.",
-            );
-        }
-
-        let entry_function = self.entry_function.clone();
-
-        let (instance, env) = self.instantiate_ext(module, module_hash, store)?;
-
-        // Bootstrap the process
-        // Unsafe: The bootstrap must be executed in the same thread that runs the
-        //         actual WASM code
-        let rewind_state = unsafe { env.bootstrap(store)? };
-        if rewind_state.is_some() {
-            let mut ctx = env.env.clone().into_mut(store);
-            rewind_ext2(&mut ctx, rewind_state)
-                .map_err(|exit| WasiRuntimeError::Wasi(WasiError::Exit(exit)))?;
-        }
-
-        let start = instance
-            .exports
-            .get_function(entry_function.as_deref().unwrap_or("_start"))?;
-        env.data(&store).thread.set_status_running();
-
-        let result = crate::run_wasi_func_start(start, store);
-        let (result, exit_code) = super::wasi_exit_code(result);
-
-        let pid = env.data(&store).pid();
-        let tid = env.data(&store).tid();
-        tracing::trace!(
-            %pid,
-            %tid,
-            %exit_code,
-            error=result.as_ref().err().map(|e| e as &dyn std::error::Error),
-            "main exit",
-        );
-
-        env.on_exit(store, Some(exit_code));
-
-        result
-    }
-
-    /// Start the WASI executable with async threads enabled.
-    #[allow(clippy::result_large_err)]
-    #[tracing::instrument(level = "debug", skip_all)]
-    pub fn run_with_store_async(
-        self,
-        module: Module,
-        module_hash: ModuleHash,
-        mut store: Store,
-    ) -> Result<(), WasiRuntimeError> {
-        #[cfg(feature = "ctrlc")]
-        let attach_ctrl_c = self.attach_ctrl_c;
-
-        let (_, env) = self.instantiate_ext(module, module_hash, &mut store)?;
-
-        // Install the ctrl-c handler
-        #[cfg(feature = "ctrlc")]
-        if attach_ctrl_c {
-            tokio::spawn({
-                let process = env.data(&store).process.clone();
-                async move {
-                    while tokio::signal::ctrl_c().await.is_ok() {
-                        process.signal_process(wasmer_wasix_types::wasi::Signal::Sigint);
-                    }
-                }
-            });
-        }
-
-        env.run_async(store)?;
-        Ok(())
+        let call_init = init.call_initialize;
+        let env = WasiEnv::from_init(init, module_hash)?;
+        let memory = module
+            .imports()
+            .find_map(|i| match i.ty() {
+                wasmer::ExternType::Memory(ty) => Some(*ty),
+                _ => None,
+            })
+            .map(|ty| wasmer::Memory::new(store, ty))
+            .transpose()
+            .map_err(WasiThreadError::MemoryCreateFailed)?;
+        Ok(env.instantiate(module, store, memory, true, call_init, None)?)
     }
 }
 
@@ -1346,15 +1219,6 @@ mod test {
             "nul in key must be invalid"
         );
 
-        // `=` in the value is valid.
-        assert!(
-            WasiEnvBuilder::new("test_prog")
-                .env("HOME", "/home/home=home")
-                .build_init()
-                .is_ok(),
-            "equal sign in the value must be valid"
-        );
-
         // `\0` in the value is invalid.
         assert!(
             WasiEnvBuilder::new("test_prog")
@@ -1363,6 +1227,16 @@ mod test {
                 .is_err(),
             "nul in value must be invalid"
         );
+
+        // `=` in the value is valid.
+        assert!(
+            WasiEnvBuilder::new("test_prog")
+                .env("HOME", "/home/home=home")
+                .engine(Engine::default())
+                .build_init()
+                .is_ok(),
+            "equal sign in the value must be valid"
+        );
     }
 
     #[test]
