@@ -1,30 +1,8 @@
-use std::{
-    collections::HashMap,
-    ops::Deref,
-    path::{Path, PathBuf},
-    sync::Arc,
-    time::Duration,
-};
-
-use futures::future::BoxFuture;
-use rand::Rng;
-use virtual_fs::{FileSystem, FsError, VirtualFile};
-use virtual_net::DynVirtualNetworking;
-use wasmer::{
-    AsStoreMut, AsStoreRef, FunctionEnvMut, Global, Imports, Instance, Memory, MemoryType,
-    MemoryView, Module, TypedFunction,
-};
-use wasmer_config::package::PackageSource;
-use wasmer_wasix_types::{
-    types::Signal,
-    wasi::{Errno, ExitCode, Snapshot0Clockid},
-    wasix::ThreadStartType,
-};
-use webc::metadata::annotations::Wasi;
-
 #[cfg(feature = "journal")]
 use crate::journal::{DynJournal, JournalEffector, SnapshotTrigger};
 use crate::{
+    Runtime, VirtualTaskManager, WasiControlPlane, WasiEnvBuilder, WasiError, WasiFunctionEnv,
+    WasiResult, WasiRuntimeError, WasiStateCreationError, WasiThreadError, WasiVFork,
     bin_factory::{BinFactory, BinaryPackage, BinaryPackageCommand},
     capabilities::Capabilities,
     fs::{WasiFsRoot, WasiInodes},
@@ -34,188 +12,36 @@ use crate::{
         process::{WasiProcess, WasiProcessId},
         thread::{WasiMemoryLayout, WasiThread, WasiThreadHandle, WasiThreadId},
     },
-    runtime::{task_manager::InlineWaker, SpawnMemoryType},
     syscalls::platform_clock_time_get,
-    Runtime, VirtualTaskManager, WasiControlPlane, WasiEnvBuilder, WasiError, WasiFunctionEnv,
-    WasiResult, WasiRuntimeError, WasiStateCreationError, WasiVFork,
 };
+use futures::future::BoxFuture;
+use rand::Rng;
+use std::{
+    collections::HashMap,
+    ops::Deref,
+    path::{Path, PathBuf},
+    str,
+    sync::Arc,
+    time::Duration,
+};
+use virtual_fs::{FileSystem, FsError, VirtualFile};
+use virtual_mio::block_on;
+use virtual_net::DynVirtualNetworking;
+use wasmer::{
+    AsStoreMut, AsStoreRef, ExportError, FunctionEnvMut, Instance, Memory, MemoryType, MemoryView,
+    Module,
+};
+use wasmer_config::package::PackageSource;
 use wasmer_types::ModuleHash;
+use wasmer_wasix_types::{
+    types::Signal,
+    wasi::{Errno, ExitCode, Snapshot0Clockid},
+    wasix::ThreadStartType,
+};
+use webc::metadata::annotations::Wasi;
 
-pub(crate) use super::handles::*;
-use super::{conv_env_vars, WasiState};
-
-/// Various [`TypedFunction`] and [`Global`] handles for an active WASI(X) instance.
-///
-/// Used to access and modify runtime state.
-// TODO: make fields private
-#[derive(Debug, Clone)]
-pub struct WasiInstanceHandles {
-    // TODO: the two fields below are instance specific, while all others are module specific.
-    // Should be split up.
-    /// Represents a reference to the memory
-    pub(crate) memory: Memory,
-    pub(crate) instance: wasmer::Instance,
-
-    /// Points to the current location of the memory stack pointer
-    pub(crate) stack_pointer: Option<Global>,
-
-    /// Points to the end of the data section
-    pub(crate) data_end: Option<Global>,
-
-    /// Points to the lower end of the stack
-    pub(crate) stack_low: Option<Global>,
-
-    /// Points to the higher end of the stack
-    pub(crate) stack_high: Option<Global>,
-
-    /// Main function that will be invoked (name = "_start")
-    pub(crate) start: Option<TypedFunction<(), ()>>,
-
-    /// Function thats invoked to initialize the WASM module (name = "_initialize")
-    // TODO: review allow...
-    #[allow(dead_code)]
-    pub(crate) initialize: Option<TypedFunction<(), ()>>,
-
-    /// Represents the callback for spawning a thread (name = "wasi_thread_start")
-    /// (due to limitations with i64 in browsers the parameters are broken into i32 pairs)
-    /// [this takes a user_data field]
-    pub(crate) thread_spawn: Option<TypedFunction<(i32, i32), ()>>,
-
-    /// Represents the callback for signals (name = "__wasm_signal")
-    /// Signals are triggered asynchronously at idle times of the process
-    pub(crate) signal: Option<TypedFunction<i32, ()>>,
-
-    /// Flag that indicates if the signal callback has been set by the WASM
-    /// process - if it has not been set then the runtime behaves differently
-    /// when a CTRL-C is pressed.
-    pub(crate) signal_set: bool,
-
-    /// Flag that indicates if the stack capture exports are being used by
-    /// this WASM process which means that it will be using asyncify
-    pub(crate) has_stack_checkpoint: bool,
-
-    /// asyncify_start_unwind(data : i32): call this to start unwinding the
-    /// stack from the current location. "data" must point to a data
-    /// structure as described above (with fields containing valid data).
-    // TODO: review allow...
-    #[allow(dead_code)]
-    pub(crate) asyncify_start_unwind: Option<TypedFunction<i32, ()>>,
-
-    /// asyncify_stop_unwind(): call this to note that unwinding has
-    /// concluded. If no other code will run before you start to rewind,
-    /// this is not strictly necessary, however, if you swap between
-    /// coroutines, or even just want to run some normal code during a
-    /// "sleep", then you must call this at the proper time. Otherwise,
-    /// the code will think it is still unwinding when it should not be,
-    /// which means it will keep unwinding in a meaningless way.
-    // TODO: review allow...
-    #[allow(dead_code)]
-    pub(crate) asyncify_stop_unwind: Option<TypedFunction<(), ()>>,
-
-    /// asyncify_start_rewind(data : i32): call this to start rewinding the
-    /// stack vack up to the location stored in the provided data. This prepares
-    /// for the rewind; to start it, you must call the first function in the
-    /// call stack to be unwound.
-    // TODO: review allow...
-    #[allow(dead_code)]
-    pub(crate) asyncify_start_rewind: Option<TypedFunction<i32, ()>>,
-
-    /// asyncify_stop_rewind(): call this to note that rewinding has
-    /// concluded, and normal execution can resume.
-    // TODO: review allow...
-    #[allow(dead_code)]
-    pub(crate) asyncify_stop_rewind: Option<TypedFunction<(), ()>>,
-
-    /// asyncify_get_state(): call this to get the current value of the
-    /// internal "__asyncify_state" variable as described above.
-    /// It can be used to distinguish between unwinding/rewinding and normal
-    /// calls, so that you know when to start an asynchronous operation and
-    /// when to propagate results back.
-    #[allow(dead_code)]
-    pub(crate) asyncify_get_state: Option<TypedFunction<(), i32>>,
-}
-
-impl WasiInstanceHandles {
-    pub fn new(memory: Memory, store: &impl AsStoreRef, instance: Instance) -> Self {
-        let has_stack_checkpoint = instance
-            .module()
-            .imports()
-            .any(|f| f.name() == "stack_checkpoint");
-        WasiInstanceHandles {
-            memory,
-            stack_pointer: instance.exports.get_global("__stack_pointer").cloned().ok(),
-            data_end: instance.exports.get_global("__data_end").cloned().ok(),
-            stack_low: instance.exports.get_global("__stack_low").cloned().ok(),
-            stack_high: instance.exports.get_global("__stack_high").cloned().ok(),
-            start: instance.exports.get_typed_function(store, "_start").ok(),
-            initialize: instance
-                .exports
-                .get_typed_function(store, "_initialize")
-                .ok(),
-            thread_spawn: instance
-                .exports
-                .get_typed_function(store, "wasi_thread_start")
-                .ok(),
-            signal: instance
-                .exports
-                .get_typed_function(&store, "__wasm_signal")
-                .ok(),
-            has_stack_checkpoint,
-            signal_set: false,
-            asyncify_start_unwind: instance
-                .exports
-                .get_typed_function(store, "asyncify_start_unwind")
-                .ok(),
-            asyncify_stop_unwind: instance
-                .exports
-                .get_typed_function(store, "asyncify_stop_unwind")
-                .ok(),
-            asyncify_start_rewind: instance
-                .exports
-                .get_typed_function(store, "asyncify_start_rewind")
-                .ok(),
-            asyncify_stop_rewind: instance
-                .exports
-                .get_typed_function(store, "asyncify_stop_rewind")
-                .ok(),
-            asyncify_get_state: instance
-                .exports
-                .get_typed_function(store, "asyncify_get_state")
-                .ok(),
-            instance,
-        }
-    }
-
-    pub fn module(&self) -> &Module {
-        self.instance.module()
-    }
-
-    pub fn module_clone(&self) -> Module {
-        self.instance.module().clone()
-    }
-
-    /// Providers safe access to the memory
-    /// (it must be initialized before it can be used)
-    pub fn memory_view<'a>(&'a self, store: &'a (impl AsStoreRef + ?Sized)) -> MemoryView<'a> {
-        self.memory.view(store)
-    }
-
-    /// Providers safe access to the memory
-    /// (it must be initialized before it can be used)
-    pub fn memory(&self) -> &Memory {
-        &self.memory
-    }
-
-    /// Copy the lazy reference so that when it's initialized during the
-    /// export phase, all the other references get a copy of it
-    pub fn memory_clone(&self) -> Memory {
-        self.memory.clone()
-    }
-
-    pub fn instance(&self) -> &Instance {
-        &self.instance
-    }
-}
+pub use super::handles::*;
+use super::{Linker, WasiState, context_switching::ContextSwitchingEnvironment, conv_env_vars};
 
 /// Data required to construct a [`WasiEnv`].
 #[derive(Debug)]
@@ -242,10 +68,6 @@ pub struct WasiEnvInit {
     /// Indicates if extra tracing should be output
     pub extra_tracing: bool,
 
-    /// Additional functionality provided to the WASIX instance, besides the
-    /// normal WASIX syscalls.
-    pub additional_imports: Imports,
-
     /// Indicates triggers that will cause a snapshot to be taken
     #[cfg(feature = "journal")]
     pub snapshot_on: Vec<SnapshotTrigger>,
@@ -269,7 +91,7 @@ impl WasiEnvInit {
 
         Self {
             state: WasiState {
-                secret: rand::thread_rng().gen::<[u8; 32]>(),
+                secret: rand::thread_rng().r#gen::<[u8; 32]>(),
                 inodes,
                 fs,
                 futexs: Default::default(),
@@ -298,7 +120,6 @@ impl WasiEnvInit {
             #[cfg(feature = "journal")]
             stop_running_after_snapshot: self.stop_running_after_snapshot,
             skip_stdio_during_bootstrap: self.skip_stdio_during_bootstrap,
-            additional_imports: self.additional_imports.clone(),
         }
     }
 }
@@ -356,6 +177,13 @@ pub struct WasiEnv {
     ///  not be cloned when `WasiEnv` is cloned)
     /// TODO: We should move this outside of `WasiEnv` with some refactoring
     inner: WasiInstanceHandlesPointer,
+
+    /// Tracks the active contexts of the WASIX context switching API
+    ///
+    /// This is `None` when the main function was not launched with context switching
+    ///
+    /// Should probably only be set by [`ContextSwitchingContext::run_main_context`]
+    pub(crate) context_switching_environment: Option<ContextSwitchingEnvironment>,
 }
 
 impl std::fmt::Debug for WasiEnv {
@@ -385,6 +213,7 @@ impl Clone for WasiEnv {
             replaying_journal: self.replaying_journal,
             skip_stdio_during_bootstrap: self.skip_stdio_during_bootstrap,
             disable_fs_cleanup: self.disable_fs_cleanup,
+            context_switching_environment: None,
         }
     }
 }
@@ -426,6 +255,7 @@ impl WasiEnv {
             replaying_journal: false,
             skip_stdio_during_bootstrap: self.skip_stdio_during_bootstrap,
             disable_fs_cleanup: self.disable_fs_cleanup,
+            context_switching_environment: None,
         };
         Ok((new_env, handle))
     }
@@ -441,7 +271,10 @@ impl WasiEnv {
     /// Returns true if this WASM process will need and try to use
     /// asyncify while its running which normally means.
     pub fn will_use_asyncify(&self) -> bool {
-        self.enable_deep_sleep || unsafe { self.inner().has_stack_checkpoint }
+        self.inner()
+            .static_module_instance_handles()
+            .map(|handles| self.enable_deep_sleep || handles.has_stack_checkpoint)
+            .unwrap_or(false)
     }
 
     /// Re-initializes this environment so that it can be executed again
@@ -492,7 +325,7 @@ impl WasiEnv {
     }
 
     /// Returns true if this module is capable of deep sleep
-    /// (needs asyncify to unwind and rewin)
+    /// (needs asyncify to unwind and rewind)
     ///
     /// # Safety
     ///
@@ -502,10 +335,14 @@ impl WasiEnv {
         if !self.control_plane.config().enable_asynchronous_threading {
             return false;
         }
-        let inner = self.inner();
-        inner.asyncify_get_state.is_some()
-            && inner.asyncify_start_rewind.is_some()
-            && inner.asyncify_start_unwind.is_some()
+        self.inner()
+            .static_module_instance_handles()
+            .map(|handles| {
+                handles.asyncify_get_state.is_some()
+                    && handles.asyncify_start_rewind.is_some()
+                    && handles.asyncify_start_unwind.is_some()
+            })
+            .unwrap_or(false)
     }
 
     /// Returns true if this thread can go into a deep sleep
@@ -563,6 +400,7 @@ impl WasiEnv {
             bin_factory: init.bin_factory,
             capabilities: init.capabilities,
             disable_fs_cleanup: false,
+            context_switching_environment: None,
         };
         env.owned_handles.push(thread);
 
@@ -580,56 +418,81 @@ impl WasiEnv {
     // FIXME: use custom error type
     #[allow(clippy::result_large_err)]
     pub(crate) fn instantiate(
-        mut init: WasiEnvInit,
+        self,
         module: Module,
-        module_hash: ModuleHash,
         store: &mut impl AsStoreMut,
-    ) -> Result<(Instance, WasiFunctionEnv), WasiRuntimeError> {
-        let call_initialize = init.call_initialize;
-        let spawn_type = init.memory_ty.take();
-
-        if init.extra_tracing {
-            for import in module.imports() {
-                tracing::trace!("import {}.{}", import.module(), import.name());
-            }
-        }
-
-        let additional_imports = init.additional_imports.clone();
-
-        let env = Self::from_init(init, module_hash)?;
-        let pid = env.process.pid();
+        memory: Option<Memory>,
+        update_layout: bool,
+        call_initialize: bool,
+        parent_linker_and_ctx: Option<(Linker, &mut FunctionEnvMut<WasiEnv>)>,
+    ) -> Result<(Instance, WasiFunctionEnv), WasiThreadError> {
+        let pid = self.process.pid();
 
         let mut store = store.as_store_mut();
+        let engine = self.runtime().engine();
+        let mut func_env = WasiFunctionEnv::new(&mut store, self);
 
-        let tasks = env.runtime.task_manager().clone();
-        let mut func_env = WasiFunctionEnv::new(&mut store, env);
+        let is_dl = super::linker::is_dynamically_linked(&module);
+        if is_dl {
+            let linker = match parent_linker_and_ctx {
+                Some((linker, ctx)) => linker.create_instance_group(ctx, &mut store, &mut func_env),
+                None => {
+                    // FIXME: should we be storing envs as raw byte arrays?
+                    let ld_library_path_owned;
+                    let ld_library_path = {
+                        let envs = func_env.data(&store).state.envs.lock().unwrap();
+                        ld_library_path_owned = match envs
+                            .iter()
+                            .find_map(|env| env.strip_prefix(b"LD_LIBRARY_PATH="))
+                        {
+                            Some(path) => path
+                                .split(|b| *b == b':')
+                                .filter_map(|p| str::from_utf8(p).ok())
+                                .map(PathBuf::from)
+                                .collect::<Vec<_>>(),
+                            None => vec![],
+                        };
+                        ld_library_path_owned
+                            .iter()
+                            .map(AsRef::as_ref)
+                            .collect::<Vec<_>>()
+                    };
 
-        // Determine if shared memory needs to be created and imported
-        let shared_memory = module.imports().memories().next().map(|a| *a.ty());
+                    // TODO: make stack size configurable
+                    Linker::new(
+                        engine,
+                        &module,
+                        &mut store,
+                        memory,
+                        &mut func_env,
+                        8 * 1024 * 1024,
+                        &ld_library_path,
+                    )
+                }
+            };
 
-        // Determine if we are going to create memory and import it or just rely on self creation of memory
-        let spawn_type = if let Some(t) = spawn_type {
-            SpawnMemoryType::CreateMemoryOfType(t)
-        } else {
-            match shared_memory {
-                Some(ty) => SpawnMemoryType::CreateMemoryOfType(ty),
-                None => SpawnMemoryType::CreateMemory,
-            }
-        };
-        let memory = tasks.build_memory(&mut store, spawn_type)?;
-
-        // Let's instantiate the module with the imports.
-        let (mut import_object, instance_init_callback) =
-            import_object_for_all_wasi_versions(&module, &mut store, &func_env.env);
-
-        for ((namespace, name), value) in &additional_imports {
-            // Note: We don't want to let downstream users override WASIX
-            // syscalls
-            if !import_object.exists(&namespace, &name) {
-                import_object.define(&namespace, &name, value);
+            match linker {
+                Ok((_, linked_module)) => {
+                    return Ok((linked_module.instance, func_env));
+                }
+                Err(e) => {
+                    tracing::error!(
+                        %pid,
+                        error = &e as &dyn std::error::Error,
+                        "Failed to link DL main module",
+                    );
+                    func_env
+                        .data(&store)
+                        .blocking_on_exit(Some(Errno::Noexec.into()));
+                    return Err(WasiThreadError::LinkError(Arc::new(e)));
+                }
             }
         }
 
+        // Let's instantiate the module with the imports.
+        let mut import_object =
+            import_object_for_all_wasi_versions(&module, &mut store, &func_env.env);
+
         let imported_memory = if let Some(memory) = memory {
             import_object.define("env", "memory", memory.clone());
             Some(memory)
@@ -649,17 +512,49 @@ impl WasiEnv {
                 func_env
                     .data(&store)
                     .blocking_on_exit(Some(Errno::Noexec.into()));
-                return Err(err.into());
+                return Err(WasiThreadError::InstanceCreateFailed(Box::new(err)));
             }
         };
 
-        // Run initializers.
-        instance_init_callback(&instance, &store).unwrap();
+        let handles = match imported_memory {
+            Some(memory) => WasiModuleTreeHandles::Static(WasiModuleInstanceHandles::new(
+                memory,
+                &store,
+                instance.clone(),
+                None,
+            )),
+            None => {
+                let exported_memory = instance
+                    .exports
+                    .iter()
+                    .filter_map(|(_, export)| {
+                        if let wasmer::Extern::Memory(memory) = export {
+                            Some(memory.clone())
+                        } else {
+                            None
+                        }
+                    })
+                    .next()
+                    .ok_or(WasiThreadError::ExportError(ExportError::Missing(
+                        "No imported or exported memory found".to_owned(),
+                    )))?;
+                WasiModuleTreeHandles::Static(WasiModuleInstanceHandles::new(
+                    exported_memory,
+                    &store,
+                    instance.clone(),
+                    None,
+                ))
+            }
+        };
 
         // Initialize the WASI environment
-        if let Err(err) =
-            func_env.initialize_with_memory(&mut store, instance.clone(), imported_memory, true)
-        {
+        if let Err(err) = func_env.initialize_handles_and_layout(
+            &mut store,
+            instance.clone(),
+            handles,
+            None,
+            update_layout,
+        ) {
             tracing::error!(
                 %pid,
                 error = &err as &dyn std::error::Error,
@@ -668,18 +563,19 @@ impl WasiEnv {
             func_env
                 .data(&store)
                 .blocking_on_exit(Some(Errno::Noexec.into()));
-            return Err(err.into());
+            return Err(WasiThreadError::ExportError(err));
         }
 
         // If this module exports an _initialize function, run that first.
-        if call_initialize {
-            if let Ok(initialize) = instance.exports.get_function("_initialize") {
-                if let Err(err) = crate::run_wasi_func_start(initialize, &mut store) {
-                    func_env
-                        .data(&store)
-                        .blocking_on_exit(Some(Errno::Noexec.into()));
-                    return Err(err);
-                }
+        if call_initialize && let Ok(initialize) = instance.exports.get_function("_initialize") {
+            let initialize_result = initialize.call(&mut store, &[]);
+            if let Err(err) = initialize_result {
+                func_env
+                    .data(&store)
+                    .blocking_on_exit(Some(Errno::Noexec.into()));
+                return Err(WasiThreadError::InitFailed(Arc::new(anyhow::Error::from(
+                    err,
+                ))));
             }
         }
 
@@ -713,14 +609,36 @@ impl WasiEnv {
         self.process.active_threads()
     }
 
+    /// Called by most (if not all) syscalls to process pending operations that are
+    /// cross-cutting, such as signals, thread/process exit, DL operations, etc.
+    pub fn do_pending_operations(ctx: &mut FunctionEnvMut<'_, Self>) -> Result<(), WasiError> {
+        Self::do_pending_link_operations(ctx, true)?;
+        _ = Self::process_signals_and_exit(ctx)?;
+        Ok(())
+    }
+
+    pub fn do_pending_link_operations(
+        ctx: &mut FunctionEnvMut<'_, Self>,
+        fast: bool,
+    ) -> Result<(), WasiError> {
+        if let Some(linker) = ctx.data().inner().linker().cloned()
+            && let Err(e) = linker.do_pending_link_operations(ctx, fast)
+        {
+            tracing::warn!(err = ?e, "Failed to process pending link operations");
+            return Err(WasiError::Exit(Errno::Noexec.into()));
+        }
+        Ok(())
+    }
+
     /// Porcesses any signals that are batched up or any forced exit codes
     pub fn process_signals_and_exit(ctx: &mut FunctionEnvMut<'_, Self>) -> WasiResult<bool> {
         // If a signal handler has never been set then we need to handle signals
         // differently
         let env = ctx.data();
-        let inner = env
+        let env_inner = env
             .try_inner()
             .ok_or_else(|| WasiError::Exit(Errno::Fault.into()))?;
+        let inner = env_inner.main_module_instance_handles();
         if !inner.signal_set {
             let signals = env.thread.pop_signals();
             if !signals.is_empty() {
@@ -754,9 +672,10 @@ impl WasiEnv {
         // If a signal handler has never been set then we need to handle signals
         // differently
         let env = ctx.data();
-        let inner = env
+        let env_inner = env
             .try_inner()
             .ok_or_else(|| WasiError::Exit(Errno::Fault.into()))?;
+        let inner = env_inner.main_module_instance_handles();
         if !inner.signal_set {
             return Ok(Ok(false));
         }
@@ -778,9 +697,10 @@ impl WasiEnv {
         mut signals: Vec<Signal>,
     ) -> Result<bool, WasiError> {
         let env = ctx.data();
-        let inner = env
+        let env_inner = env
             .try_inner()
             .ok_or_else(|| WasiError::Exit(Errno::Fault.into()))?;
+        let inner = env_inner.main_module_instance_handles();
         if let Some(handler) = inner.signal.clone() {
             // We might also have signals that trigger on timers
             let mut now = 0;
@@ -811,6 +731,11 @@ impl WasiEnv {
             }
 
             for signal in signals {
+                // Skip over Sigwakeup, which is host-side-only
+                if matches!(signal, Signal::Sigwakeup) {
+                    continue;
+                }
+
                 tracing::trace!(
                     pid=%ctx.data().pid(),
                     ?signal,
@@ -883,14 +808,20 @@ impl WasiEnv {
 
     /// Providers safe access to the initialized part of WasiEnv
     /// (it must be initialized before it can be used)
-    /// This has been marked as unsafe as it will panic if its executed
-    /// on the wrong thread or before the inner is set
-    pub(crate) unsafe fn inner(&self) -> WasiInstanceGuard<'_> {
+    pub(crate) fn inner(&self) -> WasiInstanceGuard<'_> {
         self.inner.get().expect(
             "You must initialize the WasiEnv before using it and can not pass it between threads",
         )
     }
 
+    /// Provides safe access to the initialized part of WasiEnv
+    /// (it must be initialized before it can be used)
+    pub(crate) fn inner_mut(&mut self) -> WasiInstanceGuardMut<'_> {
+        self.inner.get_mut().expect(
+            "You must initialize the WasiEnv before using it and can not pass it between threads",
+        )
+    }
+
     /// Providers safe access to the initialized part of WasiEnv
     pub(crate) fn try_inner(&self) -> Option<WasiInstanceGuard<'_>> {
         self.inner.get()
@@ -898,6 +829,7 @@ impl WasiEnv {
 
     /// Providers safe access to the initialized part of WasiEnv
     /// (it must be initialized before it can be used)
+    #[allow(dead_code)]
     pub(crate) fn try_inner_mut(&mut self) -> Option<WasiInstanceGuardMut<'_>> {
         self.inner.get_mut()
     }
@@ -906,7 +838,7 @@ impl WasiEnv {
     /// creating the instance and eventually should be moved out
     /// of the WasiEnv)
     #[doc(hidden)]
-    pub(crate) fn set_inner(&mut self, handles: WasiInstanceHandles) {
+    pub(crate) fn set_inner(&mut self, handles: WasiModuleTreeHandles) {
         self.inner.set(handles)
     }
 
@@ -918,9 +850,23 @@ impl WasiEnv {
         std::mem::swap(&mut self.inner, &mut other.inner);
     }
 
-    /// Tries to clone the instance from this environment
+    /// Helper function to ensure the module isn't dynamically linked, needed since
+    /// we only support a subset of WASIX functionality for dynamically linked modules.
+    /// Specifically, anything that requires asyncify is not supported right now.
+    pub(crate) fn ensure_static_module(&self) -> Result<(), ()> {
+        self.inner.get().unwrap().ensure_static_module()
+    }
+
+    /// Tries to clone the instance from this environment, but only if it's a static
+    /// module, since dynamically linked modules are made up of multiple instances.
     pub fn try_clone_instance(&self) -> Option<Instance> {
-        self.inner.get().map(|i| i.instance.clone())
+        let guard = self.inner.get();
+        match guard {
+            Some(guard) => guard
+                .static_module_instance_handles()
+                .map(|instance| instance.instance.clone()),
+            None => None,
+        }
     }
 
     /// Providers safe access to the memory
@@ -966,7 +912,8 @@ impl WasiEnv {
     /// export phase, all the other references get a copy of it
     #[allow(dead_code)]
     pub(crate) fn try_memory_clone(&self) -> Option<Memory> {
-        self.try_inner().map(|i| i.memory_clone())
+        self.try_inner()
+            .map(|i| i.main_module_instance_handles().memory_clone())
     }
 
     /// Get the WASI state
@@ -1045,8 +992,8 @@ impl WasiEnv {
         &'a self,
         store: &'a impl AsStoreRef,
         _mem_index: u32,
-    ) -> (MemoryView<'a>, &WasiState) {
-        let memory = self.memory_view(store);
+    ) -> (MemoryView<'a>, &'a WasiState) {
+        let memory = unsafe { self.memory_view(store) };
         let state = self.state.deref();
         (memory, state)
     }
@@ -1060,8 +1007,8 @@ impl WasiEnv {
         &'a self,
         store: &'a impl AsStoreRef,
         _mem_index: u32,
-    ) -> (MemoryView<'a>, &WasiState, &WasiInodes) {
-        let memory = self.memory_view(store);
+    ) -> (MemoryView<'a>, &'a WasiState, &'a WasiInodes) {
+        let memory = unsafe { self.memory_view(store) };
         let state = self.state.deref();
         let inodes = &state.inodes;
         (memory, state, inodes)
@@ -1074,7 +1021,7 @@ impl WasiEnv {
     }
 
     pub fn use_package(&self, pkg: &BinaryPackage) -> Result<(), WasiStateCreationError> {
-        InlineWaker::block_on(self.use_package_async(pkg))
+        block_on(self.use_package_async(pkg))
     }
 
     /// Make all the commands in a [`BinaryPackage`] available to the WASI
@@ -1144,6 +1091,32 @@ impl WasiEnv {
                             continue;
                         }
                     }
+                    WasiFsRoot::Overlay(ofs) => {
+                        let root_fs = ofs.primary();
+
+                        if let Err(err) = root_fs
+                            .new_open_options_ext()
+                            .insert_ro_file(path, atom.clone())
+                        {
+                            tracing::debug!(
+                                "failed to add package [{}] command [{}] - {}",
+                                pkg.id,
+                                command.name(),
+                                err
+                            );
+                            continue;
+                        }
+                        if let Err(err) = root_fs.new_open_options_ext().insert_ro_file(path2, atom)
+                        {
+                            tracing::debug!(
+                                "failed to add package [{}] command [{}] - {}",
+                                pkg.id,
+                                command.name(),
+                                err
+                            );
+                            continue;
+                        }
+                    }
                     WasiFsRoot::Backing(fs) => {
                         // FIXME: we're counting on the fs being a mem_fs here. Otherwise, memory
                         // usage will be very high.
@@ -1166,8 +1139,11 @@ impl WasiEnv {
 
                 let mut package = pkg.clone();
                 package.entrypoint_cmd = Some(command.name().to_string());
+                let package_arc = Arc::new(package);
                 self.bin_factory
-                    .set_binary(path.as_os_str().to_string_lossy().as_ref(), package);
+                    .set_binary(path.to_string_lossy().as_ref(), &package_arc);
+                self.bin_factory
+                    .set_binary(path2.to_string_lossy().as_ref(), &package_arc);
 
                 tracing::debug!(
                     package=%pkg.id,
@@ -1195,13 +1171,11 @@ impl WasiEnv {
                     "package_name={package_name}, {e}",
                 ))
             })?;
-            let pkg = InlineWaker::block_on(BinaryPackage::from_registry(&specifier, rt)).map_err(
-                |e| {
-                    WasiStateCreationError::WasiIncludePackageError(format!(
-                        "package_name={package_name}, {e}",
-                    ))
-                },
-            )?;
+            let pkg = block_on(BinaryPackage::from_registry(&specifier, rt)).map_err(|e| {
+                WasiStateCreationError::WasiIncludePackageError(format!(
+                    "package_name={package_name}, {e}",
+                ))
+            })?;
             self.use_package(&pkg)?;
         }
 
@@ -1254,7 +1228,10 @@ impl WasiEnv {
                     continue;
                 }
             } else {
-                tracing::debug!("failed to add atom command [{}] to the root file system as it is not sandboxed", command);
+                tracing::debug!(
+                    "failed to add atom command [{}] to the root file system as it is not sandboxed",
+                    command
+                );
                 continue;
             }
         }
@@ -1265,7 +1242,7 @@ impl WasiEnv {
     #[allow(clippy::await_holding_lock)]
     pub fn blocking_on_exit(&self, process_exit_code: Option<ExitCode>) {
         let cleanup = self.on_exit(process_exit_code);
-        InlineWaker::block_on(cleanup);
+        block_on(cleanup);
     }
 
     /// Cleans up all the open files (if this is the main thread)
@@ -1281,10 +1258,10 @@ impl WasiEnv {
                 tracing::warn!("failed to save snapshot event for thread exit - {}", err);
             }
 
-            if self.thread.is_main() {
-                if let Err(err) = JournalEffector::save_process_exit(self, process_exit_code) {
-                    tracing::warn!("failed to save snapshot event for process exit - {}", err);
-                }
+            if self.thread.is_main()
+                && let Err(err) = JournalEffector::save_process_exit(self, process_exit_code)
+            {
+                tracing::warn!("failed to save snapshot event for process exit - {}", err);
             }
         }
 
@@ -1349,12 +1326,11 @@ impl WasiEnv {
                     .extend_from_slice(env_vars.as_slice());
             }
 
-            if let Some(args) = main_args {
-                self.state
-                    .args
-                    .lock()
-                    .unwrap()
-                    .extend_from_slice(args.as_slice());
+            if let Some(main_args) = main_args {
+                let mut args: std::sync::MutexGuard<'_, Vec<String>> =
+                    self.state.args.lock().unwrap();
+                // Insert main-args before user args
+                args.splice(1..1, main_args);
             }
 
             if let Some(exec_name) = exec_name {
