@@ -1,6 +1,6 @@
 use super::*;
 
-impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
+impl JournalSyscallPlayer<'_, '_> {
     #[allow(clippy::result_large_err)]
     pub(crate) unsafe fn action_fd_close(&mut self, fd: u32) -> Result<(), WasiRuntimeError> {
         tracing::trace!(%fd, "Replay journal - FdClose");
