@@ -1,6 +1,6 @@
 use super::*;
 
-impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
+impl<'a> JournalSyscallPlayer<'a, '_> {
     #[allow(clippy::result_large_err)]
     pub(crate) unsafe fn action_snapshot(
         &mut self,
@@ -30,7 +30,7 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
         if let Some(ethereal_events) = differ_ethereal {
             for next in ethereal_events.drain(..) {
                 tracing::trace!("Replay(ether) snapshot event - {next:?}");
-                if let Err(err) = self.play_event(next, None) {
+                if let Err(err) = unsafe { self.play_event(next, None) } {
                     tracing::warn!("failed to replay event - {}", err);
                     return Err(err);
                 }
