@@ -20,13 +20,15 @@ pub fn args_sizes_get<M: MemorySize>(
     let argc = argc.deref(&memory);
     let argv_buf_size = argv_buf_size.deref(&memory);
 
-    let argc_val: M::Offset = wasi_try!(state
-        .args
-        .lock()
-        .unwrap()
-        .len()
-        .try_into()
-        .map_err(|_| Errno::Overflow));
+    let argc_val: M::Offset = wasi_try!(
+        state
+            .args
+            .lock()
+            .unwrap()
+            .len()
+            .try_into()
+            .map_err(|_| Errno::Overflow)
+    );
     let argv_buf_size_val: usize = state.args.lock().unwrap().iter().map(|v| v.len() + 1).sum();
     let argv_buf_size_val: M::Offset =
         wasi_try!(argv_buf_size_val.try_into().map_err(|_| Errno::Overflow));
