@@ -6,14 +6,18 @@ use crate::syscalls::*;
 
 /// ### `clock_time_get()`
 /// Get the time of the specified clock
+///
 /// Inputs:
+///
 /// - `Clockid clock_id`
-///     The ID of the clock to query
+///   The ID of the clock to query
 /// - `Timestamp precision`
-///     The maximum amount of error the reading may have
+///   The maximum amount of error the reading may have
+///
 /// Output:
+///
 /// - `Timestamp *time`
-///     The value of the clock in nanoseconds
+///   The value of the clock in nanoseconds
 #[cfg_attr(
     feature = "extra-logging",
     tracing::instrument(level = "trace", skip_all, ret)
@@ -24,6 +28,8 @@ pub fn clock_time_get<M: MemorySize>(
     precision: Timestamp,
     time: WasmPtr<Timestamp, M>,
 ) -> Result<Errno, WasiError> {
+    WasiEnv::do_pending_operations(&mut ctx)?;
+
     ctx = wasi_try_ok!(maybe_backoff::<M>(ctx)?);
 
     let env = ctx.data();
