@@ -14,7 +14,7 @@ use crate::syscalls::*;
 ///     If `fd` is invalid or not open
 #[instrument(level = "trace", skip_all, fields(pid = ctx.data().process.pid().raw(), %fd), ret)]
 pub fn fd_close(mut ctx: FunctionEnvMut<'_, WasiEnv>, fd: WasiFd) -> Result<Errno, WasiError> {
-    wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
+    WasiEnv::do_pending_operations(&mut ctx)?;
 
     let env = ctx.data();
     let (_, mut state) = unsafe { env.get_memory_and_wasi_state(&ctx, 0) };
@@ -22,11 +22,12 @@ pub fn fd_close(mut ctx: FunctionEnvMut<'_, WasiEnv>, fd: WasiFd) -> Result<Errn
     // We don't want to allow programs that blindly close all FDs in a loop
     // to be able to close pre-opens, as that breaks wasix-libc in rather
     // spectacular fashion.
-    if let Ok(pfd) = state.fs.get_fd(fd) {
-        if !pfd.is_stdio && pfd.inode.is_preopened {
-            trace!("Skipping fd_close for pre-opened FD ({})", fd);
-            return Ok(Errno::Success);
-        }
+    if let Ok(pfd) = state.fs.get_fd(fd)
+        && !pfd.is_stdio
+        && pfd.inode.is_preopened
+    {
+        trace!("Skipping fd_close for pre-opened FD ({})", fd);
+        return Ok(Errno::Success);
     }
     // HACK: we use tokio files to back WASI file handles. Since tokio
     // does writes in the background, it may miss writes if the file is
