@@ -14,6 +14,8 @@ pub fn fd_fdstat_set_flags(
     fd: WasiFd,
     flags: Fdflags,
 ) -> Result<Errno, WasiError> {
+    WasiEnv::do_pending_operations(&mut ctx)?;
+
     let ret = fd_fdstat_set_flags_internal(&mut ctx, fd, flags)?;
     let env = ctx.data();
 
@@ -38,9 +40,8 @@ pub(crate) fn fd_fdstat_set_flags_internal(
     {
         let env = ctx.data();
         let (_, mut state, inodes) = unsafe { env.get_memory_and_wasi_state_and_inodes(&ctx, 0) };
-        let mut fd_map = state.fs.fd_map.write().unwrap();
+        let fd_map = state.fs.fd_map.read().unwrap();
         let fd_entry = wasi_try_ok!(fd_map.get(fd).ok_or(Errno::Badf));
-        let inode = fd_entry.inode.clone();
 
         if !fd_entry.inner.rights.contains(Rights::FD_FDSTAT_SET_FLAGS) {
             return Ok(Errno::Access);
