@@ -26,6 +26,8 @@ pub fn path_create_directory<M: MemorySize>(
     path: WasmPtr<u8, M>,
     path_len: M::Offset,
 ) -> Result<Errno, WasiError> {
+    WasiEnv::do_pending_operations(&mut ctx)?;
+
     let env = ctx.data();
     let (memory, state, inodes) = unsafe { env.get_memory_and_wasi_state_and_inodes(&ctx, 0) };
 
@@ -71,11 +73,7 @@ pub(crate) fn path_create_directory_internal(
 
     let mut guard = parent_inode.write();
     match guard.deref_mut() {
-        Kind::Dir {
-            ref entries,
-            ref path,
-            ..
-        } => {
+        Kind::Dir { entries, path, .. } => {
             if let Some(child) = entries.get(&dir_name) {
                 return Err(Errno::Exist);
             }
@@ -114,10 +112,7 @@ pub(crate) fn path_create_directory_internal(
             // reborrow to insert
             {
                 let mut guard = parent_inode.write();
-                let Kind::Dir {
-                    ref mut entries, ..
-                } = guard.deref_mut()
-                else {
+                let Kind::Dir { entries, .. } = guard.deref_mut() else {
                     unreachable!();
                 };
 
