@@ -12,7 +12,7 @@ pub fn proc_raise(mut ctx: FunctionEnvMut<'_, WasiEnv>, sig: Signal) -> Result<E
     let env = ctx.data();
     env.process.signal_process(sig);
 
-    wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
+    WasiEnv::do_pending_operations(&mut ctx)?;
 
     Ok(Errno::Success)
 }
@@ -37,7 +37,7 @@ pub fn proc_raise_interval(
     let repeat = matches!(repeat, Bool::True);
     env.process.signal_interval(sig, interval, repeat);
 
-    wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
+    WasiEnv::do_pending_operations(&mut ctx)?;
 
     Ok(Errno::Success)
 }
