@@ -1,5 +1,15 @@
+mod call_dynamic;
 mod callback_signal;
 mod chdir;
+mod closure_allocate;
+mod closure_free;
+mod closure_prepare;
+mod context_create;
+mod context_destroy;
+mod context_switch;
+mod dl_invalid_handle;
+mod dlopen;
+mod dlsym;
 mod epoll_create;
 mod epoll_ctl;
 mod epoll_wait;
@@ -28,7 +38,9 @@ mod port_unbridge;
 mod proc_exec;
 mod proc_exec2;
 mod proc_exec3;
+mod proc_exit2;
 mod proc_fork;
+mod proc_fork_env;
 mod proc_id;
 mod proc_join;
 mod proc_parent;
@@ -38,6 +50,7 @@ mod proc_signals_sizes_get;
 mod proc_snapshot;
 mod proc_spawn;
 mod proc_spawn2;
+mod reflect_signature;
 mod resolve;
 mod sched_yield;
 mod sock_accept;
@@ -77,8 +90,18 @@ mod thread_spawn;
 mod tty_get;
 mod tty_set;
 
+pub use call_dynamic::*;
 pub use callback_signal::*;
 pub use chdir::*;
+pub use closure_allocate::*;
+pub use closure_free::*;
+pub use closure_prepare::*;
+pub use context_create::*;
+pub use context_destroy::*;
+pub use context_switch::*;
+pub use dl_invalid_handle::*;
+pub use dlopen::*;
+pub use dlsym::*;
 pub use epoll_create::*;
 pub use epoll_ctl::*;
 pub use epoll_wait::*;
@@ -107,7 +130,9 @@ pub use port_unbridge::*;
 pub use proc_exec::*;
 pub use proc_exec2::*;
 pub use proc_exec3::*;
+pub use proc_exit2::*;
 pub use proc_fork::*;
+pub use proc_fork_env::*;
 pub use proc_id::*;
 pub use proc_join::*;
 pub use proc_parent::*;
@@ -117,6 +142,7 @@ pub use proc_signals_sizes_get::*;
 pub use proc_snapshot::*;
 pub use proc_spawn::*;
 pub use proc_spawn2::*;
+pub use reflect_signature::*;
 pub use resolve::*;
 pub use sched_yield::*;
 pub use sock_accept::*;
@@ -156,4 +182,4 @@ pub use thread_spawn::*;
 pub use tty_get::*;
 pub use tty_set::*;
 
-use tracing::{debug_span, field, instrument, trace_span, Span};
+use tracing::{Span, debug_span, field, instrument, trace_span};
