@@ -9,7 +9,7 @@ use crate::syscalls::*;
 /// * `pid` - Handle of the child process to wait on
 /// * `sig` - Signal to send the child process
 #[instrument(level = "trace", skip_all, fields(%pid, ?sig), ret)]
-pub fn proc_signal<M: MemorySize>(
+pub fn proc_signal(
     mut ctx: FunctionEnvMut<'_, WasiEnv>,
     pid: Pid,
     sig: Signal,
@@ -22,7 +22,7 @@ pub fn proc_signal<M: MemorySize>(
         process.signal_process(sig);
     }
 
-    wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
+    WasiEnv::do_pending_operations(&mut ctx)?;
 
     Ok(Errno::Success)
 }
