@@ -40,6 +40,8 @@ pub fn proc_spawn<M: MemorySize>(
     working_dir_len: M::Offset,
     ret_handles: WasmPtr<ProcessHandles, M>,
 ) -> Result<Errno, WasiError> {
+    WasiEnv::do_pending_operations(&mut ctx)?;
+
     let env = ctx.data();
     let control_plane = &env.control_plane;
     let memory = unsafe { env.memory_view(&ctx) };
@@ -123,16 +125,16 @@ pub fn proc_spawn_internal(
     let env = ctx.data();
 
     // Preopen
-    if let Some(preopen) = preopen {
-        if !preopen.is_empty() {
-            for preopen in preopen {
-                warn!(
-                    "preopens are not yet supported for spawned processes [{}]",
-                    preopen
-                );
-            }
-            return Ok(Err(Errno::Notsup));
+    if let Some(preopen) = preopen
+        && !preopen.is_empty()
+    {
+        for preopen in preopen {
+            warn!(
+                "preopens are not yet supported for spawned processes [{}]",
+                preopen
+            );
         }
+        return Ok(Err(Errno::Notsup));
     }
 
     // Change the current directory
