@@ -22,7 +22,7 @@ pub fn sock_accept<M: MemorySize>(
     fd_flags: Fdflags,
     ro_fd: WasmPtr<WasiFd, M>,
 ) -> Result<Errno, WasiError> {
-    wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
+    WasiEnv::do_pending_operations(&mut ctx)?;
 
     ctx = wasi_try_ok!(maybe_snapshot::<M>(ctx)?);
 
@@ -65,7 +65,7 @@ pub fn sock_accept_v2<M: MemorySize>(
     ro_fd: WasmPtr<WasiFd, M>,
     ro_addr: WasmPtr<__wasi_addr_port_t, M>,
 ) -> Result<Errno, WasiError> {
-    wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
+    WasiEnv::do_pending_operations(&mut ctx)?;
 
     let env = ctx.data();
     let (memory, state, _) = unsafe { env.get_memory_and_wasi_state_and_inodes(&ctx, 0) };
