@@ -1,3 +1,5 @@
+use bincode::config;
+
 use super::*;
 use crate::syscalls::*;
 
@@ -14,6 +16,8 @@ pub fn stack_restore<M: MemorySize>(
     snapshot_ptr: WasmPtr<StackSnapshot, M>,
     mut val: Longsize,
 ) -> Result<(), WasiError> {
+    WasiEnv::do_pending_operations(&mut ctx)?;
+
     // Read the snapshot from the stack
     let env = ctx.data();
     let memory = unsafe { env.memory_view(&ctx) };
@@ -43,7 +47,9 @@ pub fn stack_restore<M: MemorySize>(
             let pid = ctx.data().pid();
             let tid = ctx.data().tid();
 
-            let rewind_result = bincode::serialize(&val).unwrap().into();
+            let rewind_result = bincode::encode_to_vec(val, config::legacy())
+                .unwrap()
+                .into();
             let ret = rewind_ext::<M>(
                 &mut ctx,
                 None, // we do not restore the thread memory as `longjmp`` is not meant to do this
