@@ -13,8 +13,8 @@ static INIT: Once = Once::new();
 
 use reqwest::Client;
 use tokio::runtime::Handle;
-use wasmer::Engine;
 use wasmer_wasix::{
+    PluggableRuntime, Runtime,
     http::HttpClient,
     runners::Runner,
     runtime::{
@@ -22,13 +22,16 @@ use wasmer_wasix::{
         package_loader::BuiltinPackageLoader,
         task_manager::tokio::TokioTaskManager,
     },
-    PluggableRuntime, Runtime,
 };
 
 mod wasi {
     use virtual_fs::{AsyncReadExt, AsyncSeekExt};
     use wasmer_package::utils::from_bytes;
-    use wasmer_wasix::{bin_factory::BinaryPackage, runners::wasi::WasiRunner, WasiError};
+    use wasmer_wasix::{
+        WasiError,
+        bin_factory::BinaryPackage,
+        runners::wasi::{RuntimeOrEngine, WasiRunner},
+    };
 
     use super::*;
 
@@ -67,7 +70,7 @@ mod wasi {
                 .with_stdin(Box::<virtual_fs::NullFile>::default())
                 .with_stdout(Box::new(stdout_2) as Box<_>)
                 .with_stderr(Box::new(stderr_2) as Box<_>)
-                .run_command("wat2wasm", &pkg, Arc::new(rt))
+                .run_command("wat2wasm", &pkg, RuntimeOrEngine::Runtime(Arc::new(rt)))
         });
 
         handle
@@ -111,7 +114,7 @@ mod wasi {
                 .with_stdin(Box::<virtual_fs::NullFile>::default())
                 .with_stdout(Box::new(stdout_2) as Box<_>)
                 .with_stderr(Box::new(stderr_2) as Box<_>)
-                .run_command("python", &pkg, Arc::new(rt))
+                .run_command("python", &pkg, RuntimeOrEngine::Runtime(Arc::new(rt)))
         });
 
         stdout.rewind().await.unwrap();
@@ -141,9 +144,10 @@ mod wasi {
 mod wcgi {
     use std::{future::Future, sync::Arc};
 
-    use futures::{channel::mpsc::Sender, future::AbortHandle, SinkExt, StreamExt};
+    use futures::{SinkExt, StreamExt, channel::mpsc::Sender, future::AbortHandle};
     use rand::Rng;
     use tokio::runtime::Handle;
+    use wasmer_package::utils::from_bytes;
     use wasmer_wasix::{
         bin_factory::BinaryPackage,
         runners::wcgi::{NoOpWcgiCallbacks, WcgiRunner},
@@ -214,6 +218,7 @@ mod wcgi {
         (cb, fut)
     }
 
+    #[derive(Debug)]
     struct Callbacks {
         sender: Sender<AbortHandle>,
         handle: Handle,
@@ -320,8 +325,7 @@ fn runtime() -> (impl Runtime + Send + Sync, Arc<TokioTaskManager>) {
     let http_client: Arc<dyn HttpClient + Send + Sync> =
         Arc::new(wasmer_wasix::http::default_http_client().unwrap());
 
-    rt.set_engine(Some(Engine::default()))
-        .set_module_cache(cache)
+    rt.set_module_cache(cache)
         .set_package_loader(
             BuiltinPackageLoader::new()
                 .with_cache_dir(cache_dir)
