@@ -37,9 +37,6 @@ pub fn run_wast(mut config: crate::Config, wast_path: &str) -> anyhow::Result<()
     if is_exception_handling {
         features.exceptions(true);
     }
-    if config.compiler == crate::Compiler::Singlepass {
-        features.multi_value(false);
-    }
     config.set_features(features);
     config.set_nan_canonicalization(try_nan_canonicalization);
 
@@ -54,26 +51,21 @@ pub fn run_wast(mut config: crate::Config, wast_path: &str) -> anyhow::Result<()
     if cfg!(feature = "coverage") {
         wast.disable_assert_and_exhaustion();
     }
+
+    wast.allow_instantiation_failures(&["Validation error: memory size must be at most"]);
     if is_simd {
         // We allow this, so tests can be run properly for `simd_const` test.
         wast.allow_instantiation_failures(&[
             "Validation error: multiple tables",
             "Validation error: unknown memory 0",
             "Validation error: Invalid var_u32",
+            "Validation error: SIMD index out of bounds",
         ]);
     }
     if is_threads {
         // We allow this, so tests can be run properly for `simd_const` test.
         wast.allow_instantiation_failures(&["Validation error: multiple tables"]);
     }
-    if config.compiler == crate::Compiler::Singlepass {
-        // We don't support multivalue yet in singlepass
-        wast.allow_instantiation_failures(&[
-            "Validation error: invalid result arity: func type returns multiple values",
-            "Validation error: blocks, loops, and ifs may only produce a resulttype when multi-value is not enabled",
-            "Validation error: func type returns multiple values but the multi-value feature is not enabled",
-        ]);
-    }
     wast.fail_fast = false;
     let path = Path::new(wast_path);
     wast.run_file(path)
