@@ -7,7 +7,7 @@ use std::{
     process::Command,
 };
 
-use anyhow::{bail, Context};
+use anyhow::{Context, bail};
 use assert_cmd::prelude::OutputAssertExt;
 use tempfile::TempDir;
 use wasmer_integration_tests_cli::*;
@@ -170,11 +170,19 @@ fn test_create_exe_with_pirita_works_1() {
 
     let stderr = String::from_utf8_lossy(&cmd.stderr);
 
-    assert_eq!(stderr.lines().map(|s| s.trim().to_string()).collect::<Vec<_>>(), vec![
-        format!("error: cannot compile more than one atom at a time"),
-        format!("│   1: note: use --atom <ATOM> to specify which atom to compile"),
-        format!("╰─▶ 2: where <ATOM> is one of: wabt, wasm-interp, wasm-strip, wasm-validate, wasm2wat, wast2json, wat2wasm"),
-    ]);
+    assert_eq!(
+        stderr
+            .lines()
+            .map(|s| s.trim().to_string())
+            .collect::<Vec<_>>(),
+        vec![
+            format!("error: cannot compile more than one atom at a time"),
+            format!("│   1: note: use --atom <ATOM> to specify which atom to compile"),
+            format!(
+                "╰─▶ 2: where <ATOM> is one of: wabt, wasm-interp, wasm-strip, wasm-validate, wasm2wat, wast2json, wat2wasm"
+            ),
+        ]
+    );
 
     assert!(!cmd.status.success());
 
@@ -548,12 +556,12 @@ fn create_exe_works_with_file() -> anyhow::Result<()> {
     let result_lines = result.lines().collect::<Vec<&str>>();
     assert_eq!(result_lines, vec!["\"Hello, World\""]);
 
-    // test with `--mapdir`
+    // test with `--volume`
     let result = run_code(
         &operating_dir,
         &executable_path,
         &[
-            "--mapdir=abc:.".to_string(),
+            "--volume=.:abc".to_string(),
             "--script".to_string(),
             "abc/test.js".to_string(),
         ],
