@@ -11,7 +11,7 @@ use assert_cmd::{assert::Assert, prelude::OutputAssertExt};
 use once_cell::sync::Lazy;
 use predicates::str::{contains, is_match};
 use rand::Rng;
-use reqwest::{blocking::Client, IntoUrl};
+use reqwest::{IntoUrl, blocking::Client};
 use tempfile::TempDir;
 use wasmer_integration_tests_cli::{
     asset_path,
@@ -181,8 +181,8 @@ fn run_python_create_temp_dir_in_subprocess() {
     let output = Command::new(get_wasmer_path())
         .arg("run")
         .arg("python/python")
-        .arg("--mapdir")
-        .arg(format!("/code:{}", resources.display()))
+        .arg("--volume")
+        .arg(format!("{}:/code", resources.display()))
         .arg("--")
         .arg("/code/main.py")
         .output()
@@ -204,10 +204,10 @@ fn run_php_with_sqlite() {
         .arg("-q")
         .arg("run")
         .arg(php_wasm)
-        .arg("--mapdir")
-        .arg(format!("/db:{}", db.display()))
-        .arg("--mapdir")
-        .arg(format!("/app:{}", app_dir.display()))
+        .arg("--volume")
+        .arg(format!("{}:/db", db.display()))
+        .arg("--volume")
+        .arg(format!("{}:/app", app_dir.display()))
         .arg("--")
         .arg("/app/test.php")
         .output()
@@ -355,7 +355,7 @@ fn test_wasmer_run_works_with_dir() {
 fn test_wasmer_run_works() {
     let assert = Command::new(get_wasmer_path())
         .arg("https://wasmer.io/python/python@0.2.0")
-        .arg(format!("--mapdir=.:{}", asset_path().display()))
+        .arg(format!("--volume={}:.", asset_path().display()))
         .arg("test.py")
         .assert()
         .success();
@@ -372,7 +372,7 @@ fn test_wasmer_run_works() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg("https://wasmer.io/python/python@0.2.0")
-        .arg(format!("--mapdir=.:{}", asset_path().display()))
+        .arg(format!("--volume={}:.", asset_path().display()))
         .arg("test.py")
         .assert()
         .success();
@@ -388,7 +388,7 @@ fn test_wasmer_run_works() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg("python/python@0.2.0")
-        .arg(format!("--mapdir=.:{}", asset_path().display()))
+        .arg(format!("--volume={}:.", asset_path().display()))
         .arg("--registry=wasmer.io")
         .arg("test.py")
         .assert()
@@ -405,7 +405,7 @@ fn test_wasmer_run_works() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg("_/python")
-        .arg(format!("--mapdir=.:{}", asset_path().display()))
+        .arg(format!("--volume={}:.", asset_path().display()))
         .arg("--registry=wasmer.io")
         .arg("test.py")
         .assert()
@@ -453,7 +453,7 @@ fn run_test_caching_works_for_packages() {
 
     let assert = Command::new(get_wasmer_path())
         .arg("python/python@0.1.0")
-        .arg(format!("--mapdir=/app:{}", asset_path().display()))
+        .arg(format!("--volume={}:/app", asset_path().display()))
         .arg("--registry=wasmer.io")
         .arg("/app/test.py")
         .env("WASMER_CACHE_DIR", wasmer_dir.path())
@@ -468,7 +468,7 @@ fn run_test_caching_works_for_packages() {
 
     let assert = Command::new(get_wasmer_path())
         .arg("python/python@0.1.0")
-        .arg(format!("--mapdir=/app:{}", asset_path().display()))
+        .arg(format!("--volume={}:/app", asset_path().display()))
         .arg("--registry=wasmer.io")
         .arg("/app/test.py")
         .env("WASMER_CACHE_DIR", wasmer_dir.path())
@@ -488,7 +488,7 @@ fn run_test_caching_works_for_packages_with_versions() {
 
     let assert = Command::new(get_wasmer_path())
         .arg("python/python@0.1.0")
-        .arg(format!("--mapdir=/app:{}", asset_path().display()))
+        .arg(format!("--volume={}:/app", asset_path().display()))
         .arg("--registry=wasmer.io")
         .arg("/app/test.py")
         .env("RUST_LOG", &*CACHE_RUST_LOG)
@@ -504,7 +504,7 @@ fn run_test_caching_works_for_packages_with_versions() {
 
     let assert = Command::new(get_wasmer_path())
         .arg("python/python@0.1.0")
-        .arg(format!("--mapdir=/app:{}", asset_path().display()))
+        .arg(format!("--volume={}:/app", asset_path().display()))
         .arg("--registry=wasmer.io")
         .arg("/app/test.py")
         .env("RUST_LOG", &*CACHE_RUST_LOG)
@@ -525,7 +525,7 @@ fn run_test_caching_works_for_urls() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg("https://wasmer.io/python/python@0.1.0")
-        .arg(format!("--mapdir=/app:{}", asset_path().display()))
+        .arg(format!("--volume={}:/app", asset_path().display()))
         .arg("/app/test.py")
         .env("RUST_LOG", &*CACHE_RUST_LOG)
         .env("WASMER_CACHE_DIR", wasmer_dir.path())
@@ -540,7 +540,7 @@ fn run_test_caching_works_for_urls() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg("https://wasmer.io/python/python@0.1.0")
-        .arg(format!("--mapdir=/app:{}", asset_path().display()))
+        .arg(format!("--volume={}:/app", asset_path().display()))
         .arg("/app/test.py")
         .env("RUST_LOG", &*CACHE_RUST_LOG)
         .env("WASMER_CACHE_DIR", wasmer_dir.path())
@@ -663,7 +663,7 @@ fn wasi_runner_on_disk_mount_using_relative_directory_on_the_host() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg(fixtures::python())
-        .arg("--mapdir=/app:.")
+        .arg("--volume=.:/app")
         .arg("--")
         .arg("/app/main.py")
         .env("RUST_LOG", &*RUST_LOG)
@@ -685,7 +685,7 @@ fn wasi_runner_on_disk_with_mounted_directories() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg(fixtures::qjs())
-        .arg(format!("--mapdir=/app:{}", temp.path().display()))
+        .arg(format!("--volume={}:/app", temp.path().display()))
         .arg("--")
         .arg("/app/index.js")
         .env("RUST_LOG", &*RUST_LOG)
@@ -706,7 +706,7 @@ fn wasi_runner_on_disk_with_mounted_directories_and_webc_volumes() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg(fixtures::python())
-        .arg(format!("--mapdir=/app:{}", temp.path().display()))
+        .arg(format!("--volume={}:/app", temp.path().display()))
         .arg("--")
         .arg("-B")
         .arg("/app/main.py")
@@ -836,7 +836,7 @@ fn wcgi_runner_on_disk_with_mounted_directories() {
     let mut cmd = Command::new(get_wasmer_path());
     cmd.arg("run")
         .arg(format!("--addr=127.0.0.1:{port}"))
-        .arg(format!("--mapdir=/path/to:{}", temp.path().display()))
+        .arg(format!("--volume={}:/path/to", temp.path().display()))
         .arg(fixtures::static_server())
         .env("RUST_LOG", &*RUST_LOG);
 
@@ -871,7 +871,7 @@ fn issue_3794_unable_to_mount_relative_paths() {
     let assert = Command::new(get_wasmer_path())
         .arg("run")
         .arg(fixtures::coreutils())
-        .arg(format!("--mapdir=./some-dir/:{}", temp.path().display()))
+        .arg(format!("--volume={}:./some-dir/", temp.path().display()))
         .arg("--command-name=cat")
         .arg("--")
         .arg("./some-dir/message.txt")
@@ -1055,6 +1055,7 @@ fn run_quickjs_via_url() {
 }
 
 #[test]
+#[allow(unused_attributes)]
 #[cfg_attr(
     all(target_env = "musl", target_os = "linux"),
     ignore = "wasmer run-unstable segfaults on musl"
@@ -1259,7 +1260,7 @@ fn read_line(reader: &mut dyn Read) -> Result<String, std::io::Error> {
         }
     }
 
-    let line = String::from_utf8(line).map_err(|e| std::io::Error::new(ErrorKind::Other, e))?;
+    let line = String::from_utf8(line).map_err(std::io::Error::other)?;
     Ok(line)
 }
 
@@ -1316,5 +1317,5 @@ fn http_get(url: impl IntoUrl) -> Result<String, reqwest::Error> {
 }
 
 fn random_port() -> u16 {
-    rand::thread_rng().gen_range(10_000_u16..u16::MAX)
+    rand::rng().random_range(10_000_u16..u16::MAX)
 }
