@@ -1,7 +1,7 @@
 set -e
 
 # step 1: tests involving the flag itself
-$WASMER -q run main.wasm -- flag_tests
+$WASMER_RUN main.wasm -- flag_tests
 
 # step 2: tests involving exec
-$WASMER -q run main.wasm --dir . -- exec_tests
+$WASMER_RUN main.wasm --volume . -- exec_tests
