@@ -1,6 +1,6 @@
 TEMP_DIR1=$(mktemp -d)
 TEMP_DIR2=$(mktemp -d)
 
-$WASMER -q run main.wasm --mapdir /temp1:$TEMP_DIR1 --mapdir /temp2:$TEMP_DIR2 > output
+$WASMER_RUN main.wasm --volume $TEMP_DIR1:/temp1 --volume $TEMP_DIR2:/temp2 > output
 
 printf "0" | diff -u output - 1>/dev/null
