@@ -1,5 +1,5 @@
 #!/bin/bash
 
-$WASMER -q run main.wasm --dir=. > output
+$WASMER_RUN main.wasm --volume . > output
 
-printf "0" | diff -u output - 1>/dev/null
\ No newline at end of file
+printf "0" | diff -u output - 1>/dev/null
