@@ -1,7 +1,7 @@
 #!/bin/bash
 
-$WASMER -q run main.wasm --mapdir=/data:. > output
+$WASMER_RUN main.wasm --volume=.:/data > output
 
 printf "0" | diff -u output - 1>/dev/null
 
-rm my_file.txt
\ No newline at end of file
+rm my_file.txt
