@@ -1,3 +1,3 @@
-$WASMER -q run main.wasm > output
+$WASMER_RUN main.wasm > output
 
-printf "0" | diff -u output - 1>/dev/null
\ No newline at end of file
+printf "0" | diff -u output - 1>/dev/null
