@@ -2,7 +2,7 @@ use super::Argus;
 use crate::ArgusConfig;
 use futures::StreamExt;
 use indicatif::{ProgressBar, ProgressStyle};
-use reqwest::{header, Client};
+use reqwest::{Client, header};
 use std::{path::PathBuf, sync::Arc, time::Duration};
 use tokio::{
     fs::File,
@@ -64,13 +64,13 @@ impl Argus {
             count += pkgs.len();
 
             for pkg in pkgs {
-                if self.to_test(&pkg).await {
-                    if let Err(e) = s.send(pkg) {
-                        error!("failed to send packages: {e}");
-                        p.finish_and_clear();
-                        anyhow::bail!("failed to send packages: {e}")
-                    };
-                }
+                if self.to_test(&pkg).await
+                    && let Err(e) = s.send(pkg)
+                {
+                    error!("failed to send packages: {e}");
+                    p.finish_and_clear();
+                    anyhow::bail!("failed to send packages: {e}")
+                };
             }
         }
 
@@ -109,7 +109,7 @@ impl Argus {
         if !dir_path.exists() {
             tokio::fs::create_dir_all(dir_path).await?;
         } else if dir_path.exists() && !dir_path.is_dir() {
-            anyhow::bail!("path {:?} exists, but it is not a directory!", path)
+            anyhow::bail!("path {path:?} exists, but it is not a directory!")
         }
 
         let client = Client::builder().user_agent(APP_USER_AGENT).build()?;
@@ -123,11 +123,8 @@ impl Argus {
                     .and_then(|ct_len| ct_len.parse().ok())
                     .unwrap_or(0) // Fallback to 0
             } else {
-                anyhow::bail!(
-                    "Couldn't fetch head from URL {}. Error: {:?}",
-                    url,
-                    resp.status()
-                )
+                let status = resp.status();
+                anyhow::bail!("Couldn't fetch head from URL {url}. Error: {status:?}")
             }
         };
 
@@ -156,10 +153,8 @@ impl Argus {
 
                 p.finish_and_clear();
 
-                anyhow::bail!(
-                    "[{test_id}] failed to create file at {:?}. Error: {e}",
-                    path.display()
-                );
+                let path_display = path.display();
+                anyhow::bail!("[{test_id}] failed to create file at {path_display}. Error: {e}");
             }
         };
         let mut download = match request.send().await {
@@ -180,8 +175,7 @@ impl Argus {
                     );
                     p.finish_and_clear();
                     anyhow::bail!(
-                        "[{test_id}] failed to download chunk from {:?}. Error: {e}",
-                        download
+                        "[{test_id}] failed to download chunk from {download:?}. Error: {e}"
                     );
                 }
                 Ok(chunk) => {
@@ -194,8 +188,7 @@ impl Argus {
                             );
                             p.finish_and_clear();
                             anyhow::bail!(
-                                "[{test_id}] failed to write chunk to file {:?}. Error: {e}",
-                                outfile
+                                "[{test_id}] failed to write chunk to file {outfile:?}. Error: {e}"
                             );
                         };
                     } else {
