@@ -4,7 +4,7 @@ use std::{fs::File, io::BufReader, path::Path, process::Command, sync::Arc};
 use tokio::time::{self, Instant};
 use tracing::*;
 use wasmer_backend_api::types::PackageVersionWithPackage;
-use webc::{v2::read::OwnedReader, v3::read::OwnedReader as OwnedReaderV3, Container, Version};
+use webc::{Container, Version, v2::read::OwnedReader, v3::read::OwnedReader as OwnedReaderV3};
 
 use super::{TestReport, Tester};
 
@@ -38,10 +38,10 @@ impl<'a> CLIRunner<'a> {
         dir_path: &Path,
         atom_id: usize,
     ) -> anyhow::Result<Result<(), String>> {
-        if let Err(e) = Command::new(cli_path).arg("-V").output() {
-            if let std::io::ErrorKind::NotFound = e.kind() {
-                anyhow::bail!("the command '{cli_path}' was not found");
-            }
+        if let Err(e) = Command::new(cli_path).arg("-V").output()
+            && let std::io::ErrorKind::NotFound = e.kind()
+        {
+            anyhow::bail!("the command '{cli_path}' was not found");
         }
 
         let atom_path = dir_path.join(format!("atom_{atom_id}.wasm"));
@@ -117,7 +117,7 @@ impl<'a> CLIRunner<'a> {
 
     async fn get_version(&self) -> anyhow::Result<String> {
         let cli_path = match &self.config.cli_path {
-            Some(ref p) => p.clone(),
+            Some(p) => p.clone(),
             None => String::from("wasmer"),
         };
 
@@ -143,12 +143,12 @@ impl<'a> CLIRunner<'a> {
 }
 
 #[async_trait::async_trait]
-impl<'a> Tester for CLIRunner<'a> {
+impl Tester for CLIRunner<'_> {
     async fn run_test(&self) -> anyhow::Result<TestReport> {
         let start_time = time::Instant::now();
         let version = self.get_version().await?;
         let cli_path = match &self.config.cli_path {
-            Some(ref p) => p.clone(),
+            Some(p) => p.clone(),
             None => String::from("wasmer"),
         };
 
@@ -164,7 +164,7 @@ impl<'a> Tester for CLIRunner<'a> {
         let webc_v2 = match webc::detect(v2_bytes.as_slice()) {
             Ok(Version::V2) => Container::from(OwnedReader::parse(v2_bytes)?),
             Ok(other) => {
-                return self.err(version, start_time, format!("Unsupported version, {other}"))
+                return self.err(version, start_time, format!("Unsupported version, {other}"));
             }
             Err(e) => return self.err(version, start_time, format!("An error occurred: {e}")),
         };
@@ -189,7 +189,7 @@ impl<'a> Tester for CLIRunner<'a> {
         let webc_v3 = match webc::detect(v3_bytes.as_slice()) {
             Ok(Version::V3) => Container::from(OwnedReaderV3::parse(v3_bytes)?),
             Ok(other) => {
-                return self.err(version, start_time, format!("Unsupported version, {other}"))
+                return self.err(version, start_time, format!("Unsupported version, {other}"));
             }
             Err(e) => return self.err(version, start_time, format!("An error occurred: {e}")),
         };
